
library IEEE;

use IEEE.std_logic_1164.all;

package CONV_PACK_mlite_cpu is

-- define attributes
attribute ENUM_ENCODING : STRING;

-- define any necessary types
subtype alu_function_type is std_logic_vector (3 downto 0);
subtype shift_function_type is std_logic_vector (1 downto 0);
subtype mult_function_type is std_logic_vector (3 downto 0);
subtype pc_source_type is std_logic_vector (1 downto 0);
subtype mem_source_type is std_logic_vector (3 downto 0);
subtype a_source_type is std_logic_vector (1 downto 0);
subtype b_source_type is std_logic_vector (1 downto 0);
subtype c_source_type is std_logic_vector (2 downto 0);
subtype branch_function_type is std_logic_vector (2 downto 0);

end CONV_PACK_mlite_cpu;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity mult_mult_typeDEFAULT_DW01_dec_0 is

   port( A : in std_logic_vector (5 downto 0);  SUM : out std_logic_vector (5 
         downto 0));

end mult_mult_typeDEFAULT_DW01_dec_0;

architecture SYN_rpl of mult_mult_typeDEFAULT_DW01_dec_0 is

   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component XOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component XNOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   signal SUMx5x, SUMx4x, SUMx3x, SUMx2x, SUMx1x, SUMx0x, n1, n2, n3, n4 : 
      std_logic;

begin
   SUM <= ( SUMx5x, SUMx4x, SUMx3x, SUMx2x, SUMx1x, SUMx0x );
   
   U1 : INVXL port map( A => A(0), Y => SUMx0x);
   U2 : NAND2BXL port map( AN => A(1), B => SUMx0x, Y => n4);
   U3 : OR2XL port map( A => n4, B => A(2), Y => n3);
   U4 : OR2XL port map( A => n3, B => A(3), Y => n2);
   U5 : XNOR2XL port map( A => A(4), B => n2, Y => SUMx4x);
   U6 : OAI2BB1XL port map( A0N => n4, A1N => A(2), B0 => n3, Y => SUMx2x);
   U7 : OAI2BB1XL port map( A0N => A(0), A1N => A(1), B0 => n4, Y => SUMx1x);
   U8 : OAI2BB1XL port map( A0N => n3, A1N => A(3), B0 => n2, Y => SUMx3x);
   U9 : XOR2XL port map( A => A(5), B => n1, Y => SUMx5x);
   U10 : NOR2XL port map( A => A(4), B => n2, Y => n1);

end SYN_rpl;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity pipeline is

   port( clk, reset : in std_logic;  a_bus : in std_logic_vector (31 downto 0);
         a_busD : out std_logic_vector (31 downto 0);  b_bus : in 
         std_logic_vector (31 downto 0);  b_busD : out std_logic_vector (31 
         downto 0);  alu_func : in std_logic_vector (3 downto 0);  alu_funcD : 
         out std_logic_vector (3 downto 0);  shift_func : in std_logic_vector 
         (1 downto 0);  shift_funcD : out std_logic_vector (1 downto 0);  
         mult_func : in std_logic_vector (3 downto 0);  mult_funcD : out 
         std_logic_vector (3 downto 0);  reg_dest : in std_logic_vector (31 
         downto 0);  reg_destD : out std_logic_vector (31 downto 0);  rd_index 
         : in std_logic_vector (5 downto 0);  rd_indexD : out std_logic_vector 
         (5 downto 0);  rs_index, rt_index : in std_logic_vector (5 downto 0); 
         pc_source : in std_logic_vector (1 downto 0);  mem_source : in 
         std_logic_vector (3 downto 0);  a_source, b_source : in 
         std_logic_vector (1 downto 0);  c_source : in std_logic_vector (2 
         downto 0);  c_bus : in std_logic_vector (31 downto 0);  pause_any : in
         std_logic;  pause_pipeline : out std_logic);

end pipeline;

architecture SYN_logic of pipeline is

   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1X1
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB2XL
      port( B0, B1, A0N, A1N : in std_logic;  Y : out std_logic);
   end component;
   
   component XNOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OR4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3BXL
      port( AN, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component XOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component DFFRHQXL
      port( D, CK, RN : in std_logic;  Q : out std_logic);
   end component;
   
   component JKFFRXL
      port( J, K, CK, RN : in std_logic;  Q, QN : out std_logic);
   end component;
   
   signal a_busD_31_port, a_busD_30_port, a_busD_29_port, a_busD_28_port, 
      a_busD_27_port, a_busD_26_port, a_busD_25_port, a_busD_24_port, 
      a_busD_23_port, a_busD_22_port, a_busD_21_port, a_busD_20_port, 
      a_busD_19_port, a_busD_18_port, a_busD_17_port, a_busD_16_port, 
      a_busD_15_port, a_busD_14_port, a_busD_13_port, a_busD_12_port, 
      a_busD_11_port, a_busD_10_port, a_busD_9_port, a_busD_8_port, 
      a_busD_7_port, a_busD_6_port, a_busD_5_port, a_busD_4_port, a_busD_3_port
      , a_busD_2_port, a_busD_1_port, a_busD_0_port, b_busD_31_port, 
      b_busD_30_port, b_busD_29_port, b_busD_28_port, b_busD_27_port, 
      b_busD_26_port, b_busD_25_port, b_busD_24_port, b_busD_23_port, 
      b_busD_22_port, b_busD_21_port, b_busD_20_port, b_busD_19_port, 
      b_busD_18_port, b_busD_17_port, b_busD_16_port, b_busD_15_port, 
      b_busD_14_port, b_busD_13_port, b_busD_12_port, b_busD_11_port, 
      b_busD_10_port, b_busD_9_port, b_busD_8_port, b_busD_7_port, 
      b_busD_6_port, b_busD_5_port, b_busD_4_port, b_busD_3_port, b_busD_2_port
      , b_busD_1_port, b_busD_0_port, alu_funcD_3_port, alu_funcD_2_port, 
      alu_funcD_1_port, alu_funcD_0_port, shift_funcD_1_port, 
      shift_funcD_0_port, mult_funcD_3_port, mult_funcD_2_port, 
      mult_funcD_1_port, mult_funcD_0_port, reg_destD_31_port, 
      reg_destD_30_port, reg_destD_29_port, reg_destD_28_port, 
      reg_destD_27_port, reg_destD_26_port, reg_destD_25_port, 
      reg_destD_24_port, reg_destD_23_port, reg_destD_22_port, 
      reg_destD_21_port, reg_destD_20_port, reg_destD_19_port, 
      reg_destD_18_port, reg_destD_17_port, reg_destD_16_port, 
      reg_destD_15_port, reg_destD_14_port, reg_destD_13_port, 
      reg_destD_12_port, reg_destD_11_port, reg_destD_10_port, reg_destD_9_port
      , reg_destD_8_port, reg_destD_7_port, reg_destD_6_port, reg_destD_5_port,
      reg_destD_4_port, reg_destD_3_port, reg_destD_2_port, reg_destD_1_port, 
      reg_destD_0_port, rd_indexD_5_port, rd_indexD_4_port, rd_indexD_3_port, 
      rd_indexD_2_port, rd_indexD_1_port, rd_indexD_0_port, pause_pipeline_port
      , c_source_reg_2_port, c_source_reg_1_port, c_source_reg_0_port, 
      reg_dest_reg_31_port, reg_dest_reg_30_port, reg_dest_reg_29_port, 
      reg_dest_reg_28_port, reg_dest_reg_27_port, reg_dest_reg_26_port, 
      reg_dest_reg_25_port, reg_dest_reg_24_port, reg_dest_reg_23_port, 
      reg_dest_reg_22_port, reg_dest_reg_21_port, reg_dest_reg_20_port, 
      reg_dest_reg_19_port, reg_dest_reg_18_port, reg_dest_reg_17_port, 
      reg_dest_reg_16_port, reg_dest_reg_15_port, reg_dest_reg_14_port, 
      reg_dest_reg_13_port, reg_dest_reg_12_port, reg_dest_reg_11_port, 
      reg_dest_reg_10_port, reg_dest_reg_9_port, reg_dest_reg_8_port, 
      reg_dest_reg_7_port, reg_dest_reg_6_port, reg_dest_reg_5_port, 
      reg_dest_reg_4_port, reg_dest_reg_3_port, reg_dest_reg_2_port, 
      reg_dest_reg_1_port, reg_dest_reg_0_port, n2, n3, n6, n7, n8, n9, n10, 
      n11, n12, n13, n14, n15, n16, n17, n18, n19, n20, n21, n22, n23, n24, n25
      , n26, n27, n28, n29, n30, n31, n32, n33, n34, n35, n36, n37, n39, n40, 
      n41, n42, n43, n44, n45, n46, n47, n49, n50, n51, n52, n53, n54, n57, n59
      , n60, n61, n62, n63, n64, n65, n66, n67, n68, n69, n70, n71, n72, n73, 
      n74, n75, n76, n77, n78, n79, n80, n81, n82, n83, n84, n85, n86, n87, n88
      , n89, n90, n91, n92, n93, n94, n95, n96, n97, n98, n99, n100, n101, n102
      , n103, n104, n105, n106, n107, n108, n109, n110, n111, n112, n113, n114,
      n115, n116, n117, n118, n119, n120, n121, n122, n123, n124, n125, n126, 
      n127, n128, n129, n132, n133, n134, n135, n136, n137, n138, n139, n140, 
      n141, n142, n143, n144, n145, n146, n147, n148, n149, n150, n151, n152, 
      n153, n154, n155, n156, n157, n158, n159, n160, n161, n162, n163, n164, 
      n165, n166, n167, n168, n169, n170, n171, n172, n173, n174, n175, n176, 
      n177, n178, n179, n180, n181, n182, n183, n184, n185, n186, n187, n188, 
      n190, n191, n192, n193, n194, n195, n196, n197, n198, n199, n200, n201, 
      n202, n203, n204, n205, n206, n207, n208, n209, n210, n211, n212, n213, 
      n214, n215, n216, n217, n218, n219, n220, n221, n222, n223, n224, n225, 
      n226, n227, n228, n229, n230, n231, n232, n233, n234, n235, n236, n237, 
      n238, n239, n240, n241, n242, n243, n244, n245, n246, n247, n248, n249, 
      n250, n251, n252, n253, n254, n255, n256, n257, n258, n259, n260, n261, 
      n262, n263, n264, n265, n266, n267, n268, n269, n270, n271, n272, n273, 
      n274, n275, n276, n277, n278, n279, n280, n281, n282, n283, n284, n285, 
      n286, n287, n288, n289, n290, n291, n292, n293, n294, n295, n296, n297, 
      n298, n299, n300, n301, n302, n303, n304, n307, n1, n4, n5, n38, n48, n55
      , n56, n58, n130, n131, n189, n305, n306, n308, n309, n310, n_1001 : 
      std_logic;

begin
   a_busD <= ( a_busD_31_port, a_busD_30_port, a_busD_29_port, a_busD_28_port, 
      a_busD_27_port, a_busD_26_port, a_busD_25_port, a_busD_24_port, 
      a_busD_23_port, a_busD_22_port, a_busD_21_port, a_busD_20_port, 
      a_busD_19_port, a_busD_18_port, a_busD_17_port, a_busD_16_port, 
      a_busD_15_port, a_busD_14_port, a_busD_13_port, a_busD_12_port, 
      a_busD_11_port, a_busD_10_port, a_busD_9_port, a_busD_8_port, 
      a_busD_7_port, a_busD_6_port, a_busD_5_port, a_busD_4_port, a_busD_3_port
      , a_busD_2_port, a_busD_1_port, a_busD_0_port );
   b_busD <= ( b_busD_31_port, b_busD_30_port, b_busD_29_port, b_busD_28_port, 
      b_busD_27_port, b_busD_26_port, b_busD_25_port, b_busD_24_port, 
      b_busD_23_port, b_busD_22_port, b_busD_21_port, b_busD_20_port, 
      b_busD_19_port, b_busD_18_port, b_busD_17_port, b_busD_16_port, 
      b_busD_15_port, b_busD_14_port, b_busD_13_port, b_busD_12_port, 
      b_busD_11_port, b_busD_10_port, b_busD_9_port, b_busD_8_port, 
      b_busD_7_port, b_busD_6_port, b_busD_5_port, b_busD_4_port, b_busD_3_port
      , b_busD_2_port, b_busD_1_port, b_busD_0_port );
   alu_funcD <= ( alu_funcD_3_port, alu_funcD_2_port, alu_funcD_1_port, 
      alu_funcD_0_port );
   shift_funcD <= ( shift_funcD_1_port, shift_funcD_0_port );
   mult_funcD <= ( mult_funcD_3_port, mult_funcD_2_port, mult_funcD_1_port, 
      mult_funcD_0_port );
   reg_destD <= ( reg_destD_31_port, reg_destD_30_port, reg_destD_29_port, 
      reg_destD_28_port, reg_destD_27_port, reg_destD_26_port, 
      reg_destD_25_port, reg_destD_24_port, reg_destD_23_port, 
      reg_destD_22_port, reg_destD_21_port, reg_destD_20_port, 
      reg_destD_19_port, reg_destD_18_port, reg_destD_17_port, 
      reg_destD_16_port, reg_destD_15_port, reg_destD_14_port, 
      reg_destD_13_port, reg_destD_12_port, reg_destD_11_port, 
      reg_destD_10_port, reg_destD_9_port, reg_destD_8_port, reg_destD_7_port, 
      reg_destD_6_port, reg_destD_5_port, reg_destD_4_port, reg_destD_3_port, 
      reg_destD_2_port, reg_destD_1_port, reg_destD_0_port );
   rd_indexD <= ( rd_indexD_5_port, rd_indexD_4_port, rd_indexD_3_port, 
      rd_indexD_2_port, rd_indexD_1_port, rd_indexD_0_port );
   pause_pipeline <= pause_pipeline_port;
   
   pause_enable_reg_reg : JKFFRXL port map( J => n310, K => n307, CK => clk, RN
                           => n309, Q => n_1001, QN => n45);
   reg_dest_reg_regx31x : DFFRHQXL port map( D => n294, CK => clk, RN => n309, 
                           Q => reg_dest_reg_31_port);
   reg_dest_reg_regx30x : DFFRHQXL port map( D => n293, CK => clk, RN => n308, 
                           Q => reg_dest_reg_30_port);
   reg_dest_reg_regx29x : DFFRHQXL port map( D => n292, CK => clk, RN => n309, 
                           Q => reg_dest_reg_29_port);
   reg_dest_reg_regx28x : DFFRHQXL port map( D => n291, CK => clk, RN => n308, 
                           Q => reg_dest_reg_28_port);
   reg_dest_reg_regx27x : DFFRHQXL port map( D => n290, CK => clk, RN => n308, 
                           Q => reg_dest_reg_27_port);
   reg_dest_reg_regx26x : DFFRHQXL port map( D => n289, CK => clk, RN => n308, 
                           Q => reg_dest_reg_26_port);
   reg_dest_reg_regx25x : DFFRHQXL port map( D => n288, CK => clk, RN => n308, 
                           Q => reg_dest_reg_25_port);
   reg_dest_reg_regx24x : DFFRHQXL port map( D => n287, CK => clk, RN => n308, 
                           Q => reg_dest_reg_24_port);
   reg_dest_reg_regx23x : DFFRHQXL port map( D => n286, CK => clk, RN => n308, 
                           Q => reg_dest_reg_23_port);
   reg_dest_reg_regx22x : DFFRHQXL port map( D => n285, CK => clk, RN => n308, 
                           Q => reg_dest_reg_22_port);
   reg_dest_reg_regx21x : DFFRHQXL port map( D => n284, CK => clk, RN => n308, 
                           Q => reg_dest_reg_21_port);
   reg_dest_reg_regx20x : DFFRHQXL port map( D => n283, CK => clk, RN => n308, 
                           Q => reg_dest_reg_20_port);
   reg_dest_reg_regx19x : DFFRHQXL port map( D => n282, CK => clk, RN => n308, 
                           Q => reg_dest_reg_19_port);
   reg_dest_reg_regx18x : DFFRHQXL port map( D => n281, CK => clk, RN => n308, 
                           Q => reg_dest_reg_18_port);
   reg_dest_reg_regx17x : DFFRHQXL port map( D => n280, CK => clk, RN => n308, 
                           Q => reg_dest_reg_17_port);
   reg_dest_reg_regx16x : DFFRHQXL port map( D => n279, CK => clk, RN => n308, 
                           Q => reg_dest_reg_16_port);
   reg_dest_reg_regx15x : DFFRHQXL port map( D => n278, CK => clk, RN => n309, 
                           Q => reg_dest_reg_15_port);
   reg_dest_reg_regx14x : DFFRHQXL port map( D => n277, CK => clk, RN => n308, 
                           Q => reg_dest_reg_14_port);
   reg_dest_reg_regx13x : DFFRHQXL port map( D => n276, CK => clk, RN => n308, 
                           Q => reg_dest_reg_13_port);
   reg_dest_reg_regx12x : DFFRHQXL port map( D => n275, CK => clk, RN => n309, 
                           Q => reg_dest_reg_12_port);
   reg_dest_reg_regx11x : DFFRHQXL port map( D => n274, CK => clk, RN => n308, 
                           Q => reg_dest_reg_11_port);
   reg_dest_reg_regx10x : DFFRHQXL port map( D => n273, CK => clk, RN => n308, 
                           Q => reg_dest_reg_10_port);
   reg_dest_reg_regx9x : DFFRHQXL port map( D => n272, CK => clk, RN => n309, Q
                           => reg_dest_reg_9_port);
   reg_dest_reg_regx8x : DFFRHQXL port map( D => n271, CK => clk, RN => n309, Q
                           => reg_dest_reg_8_port);
   reg_dest_reg_regx7x : DFFRHQXL port map( D => n270, CK => clk, RN => n309, Q
                           => reg_dest_reg_7_port);
   reg_dest_reg_regx6x : DFFRHQXL port map( D => n269, CK => clk, RN => n308, Q
                           => reg_dest_reg_6_port);
   reg_dest_reg_regx5x : DFFRHQXL port map( D => n268, CK => clk, RN => n309, Q
                           => reg_dest_reg_5_port);
   reg_dest_reg_regx4x : DFFRHQXL port map( D => n267, CK => clk, RN => n308, Q
                           => reg_dest_reg_4_port);
   reg_dest_reg_regx3x : DFFRHQXL port map( D => n266, CK => clk, RN => n309, Q
                           => reg_dest_reg_3_port);
   reg_dest_reg_regx2x : DFFRHQXL port map( D => n265, CK => clk, RN => n308, Q
                           => reg_dest_reg_2_port);
   reg_dest_reg_regx1x : DFFRHQXL port map( D => n264, CK => clk, RN => n308, Q
                           => reg_dest_reg_1_port);
   reg_dest_reg_regx0x : DFFRHQXL port map( D => n263, CK => clk, RN => n309, Q
                           => reg_dest_reg_0_port);
   c_source_reg_regx0x : DFFRHQXL port map( D => n260, CK => clk, RN => n308, Q
                           => c_source_reg_0_port);
   c_source_reg_regx1x : DFFRHQXL port map( D => n261, CK => clk, RN => n309, Q
                           => c_source_reg_1_port);
   c_source_reg_regx2x : DFFRHQXL port map( D => n262, CK => clk, RN => n308, Q
                           => c_source_reg_2_port);
   rd_index_reg_regx5x : DFFRHQXL port map( D => n259, CK => clk, RN => n309, Q
                           => rd_indexD_5_port);
   rd_index_reg_regx3x : DFFRHQXL port map( D => n257, CK => clk, RN => n309, Q
                           => rd_indexD_3_port);
   rd_index_reg_regx0x : DFFRHQXL port map( D => n254, CK => clk, RN => n308, Q
                           => rd_indexD_0_port);
   rd_index_reg_regx1x : DFFRHQXL port map( D => n255, CK => clk, RN => n309, Q
                           => rd_indexD_1_port);
   rd_index_reg_regx2x : DFFRHQXL port map( D => n256, CK => clk, RN => n308, Q
                           => rd_indexD_2_port);
   rd_index_reg_regx4x : DFFRHQXL port map( D => n258, CK => clk, RN => n308, Q
                           => rd_indexD_4_port);
   shift_funcD_regx0x : DFFRHQXL port map( D => n299, CK => clk, RN => n308, Q 
                           => shift_funcD_0_port);
   shift_funcD_regx1x : DFFRHQXL port map( D => n300, CK => clk, RN => n308, Q 
                           => shift_funcD_1_port);
   mult_funcD_regx1x : DFFRHQXL port map( D => n296, CK => clk, RN => n309, Q 
                           => mult_funcD_1_port);
   mult_funcD_regx2x : DFFRHQXL port map( D => n297, CK => clk, RN => n308, Q 
                           => mult_funcD_2_port);
   mult_funcD_regx3x : DFFRHQXL port map( D => n298, CK => clk, RN => n309, Q 
                           => mult_funcD_3_port);
   mult_funcD_regx0x : DFFRHQXL port map( D => n295, CK => clk, RN => n308, Q 
                           => mult_funcD_0_port);
   a_busD_regx29x : DFFRHQXL port map( D => n193, CK => clk, RN => n308, Q => 
                           a_busD_29_port);
   b_busD_regx30x : DFFRHQXL port map( D => n252, CK => clk, RN => n309, Q => 
                           b_busD_30_port);
   a_busD_regx30x : DFFRHQXL port map( D => n194, CK => clk, RN => n309, Q => 
                           a_busD_30_port);
   a_busD_regx28x : DFFRHQXL port map( D => n192, CK => clk, RN => n308, Q => 
                           a_busD_28_port);
   a_busD_regx31x : DFFRHQXL port map( D => n195, CK => clk, RN => n309, Q => 
                           a_busD_31_port);
   b_busD_regx31x : DFFRHQXL port map( D => n253, CK => clk, RN => n308, Q => 
                           b_busD_31_port);
   a_busD_regx25x : DFFRHQXL port map( D => n221, CK => clk, RN => n309, Q => 
                           a_busD_25_port);
   a_busD_regx27x : DFFRHQXL port map( D => n191, CK => clk, RN => n309, Q => 
                           a_busD_27_port);
   a_busD_regx26x : DFFRHQXL port map( D => n190, CK => clk, RN => n308, Q => 
                           a_busD_26_port);
   b_busD_regx28x : DFFRHQXL port map( D => n250, CK => clk, RN => n309, Q => 
                           b_busD_28_port);
   b_busD_regx29x : DFFRHQXL port map( D => n251, CK => clk, RN => n308, Q => 
                           b_busD_29_port);
   b_busD_regx27x : DFFRHQXL port map( D => n249, CK => clk, RN => n309, Q => 
                           b_busD_27_port);
   b_busD_regx24x : DFFRHQXL port map( D => n246, CK => clk, RN => n309, Q => 
                           b_busD_24_port);
   a_busD_regx24x : DFFRHQXL port map( D => n220, CK => clk, RN => n309, Q => 
                           a_busD_24_port);
   a_busD_regx23x : DFFRHQXL port map( D => n219, CK => clk, RN => n309, Q => 
                           a_busD_23_port);
   b_busD_regx25x : DFFRHQXL port map( D => n247, CK => clk, RN => n309, Q => 
                           b_busD_25_port);
   b_busD_regx26x : DFFRHQXL port map( D => n248, CK => clk, RN => n309, Q => 
                           b_busD_26_port);
   b_busD_regx23x : DFFRHQXL port map( D => n245, CK => clk, RN => n308, Q => 
                           b_busD_23_port);
   a_busD_regx22x : DFFRHQXL port map( D => n218, CK => clk, RN => n309, Q => 
                           a_busD_22_port);
   a_busD_regx21x : DFFRHQXL port map( D => n217, CK => clk, RN => n309, Q => 
                           a_busD_21_port);
   b_busD_regx22x : DFFRHQXL port map( D => n244, CK => clk, RN => n308, Q => 
                           b_busD_22_port);
   a_busD_regx20x : DFFRHQXL port map( D => n216, CK => clk, RN => n309, Q => 
                           a_busD_20_port);
   b_busD_regx21x : DFFRHQXL port map( D => n243, CK => clk, RN => n308, Q => 
                           b_busD_21_port);
   a_busD_regx17x : DFFRHQXL port map( D => n213, CK => clk, RN => n309, Q => 
                           a_busD_17_port);
   a_busD_regx18x : DFFRHQXL port map( D => n214, CK => clk, RN => n309, Q => 
                           a_busD_18_port);
   a_busD_regx19x : DFFRHQXL port map( D => n215, CK => clk, RN => n309, Q => 
                           a_busD_19_port);
   b_busD_regx19x : DFFRHQXL port map( D => n241, CK => clk, RN => n308, Q => 
                           b_busD_19_port);
   b_busD_regx20x : DFFRHQXL port map( D => n242, CK => clk, RN => n308, Q => 
                           b_busD_20_port);
   a_busD_regx16x : DFFRHQXL port map( D => n212, CK => clk, RN => n309, Q => 
                           a_busD_16_port);
   a_busD_regx14x : DFFRHQXL port map( D => n210, CK => clk, RN => n309, Q => 
                           a_busD_14_port);
   a_busD_regx15x : DFFRHQXL port map( D => n211, CK => clk, RN => n308, Q => 
                           a_busD_15_port);
   b_busD_regx18x : DFFRHQXL port map( D => n240, CK => clk, RN => n309, Q => 
                           b_busD_18_port);
   b_busD_regx17x : DFFRHQXL port map( D => n239, CK => clk, RN => n308, Q => 
                           b_busD_17_port);
   b_busD_regx16x : DFFRHQXL port map( D => n238, CK => clk, RN => n309, Q => 
                           b_busD_16_port);
   a_busD_regx13x : DFFRHQXL port map( D => n209, CK => clk, RN => n308, Q => 
                           a_busD_13_port);
   a_busD_regx12x : DFFRHQXL port map( D => n208, CK => clk, RN => n309, Q => 
                           a_busD_12_port);
   b_busD_regx15x : DFFRHQXL port map( D => n237, CK => clk, RN => n308, Q => 
                           b_busD_15_port);
   b_busD_regx14x : DFFRHQXL port map( D => n236, CK => clk, RN => n309, Q => 
                           b_busD_14_port);
   b_busD_regx13x : DFFRHQXL port map( D => n235, CK => clk, RN => n308, Q => 
                           b_busD_13_port);
   a_busD_regx9x : DFFRHQXL port map( D => n205, CK => clk, RN => n308, Q => 
                           a_busD_9_port);
   a_busD_regx10x : DFFRHQXL port map( D => n206, CK => clk, RN => n309, Q => 
                           a_busD_10_port);
   a_busD_regx11x : DFFRHQXL port map( D => n207, CK => clk, RN => n308, Q => 
                           a_busD_11_port);
   b_busD_regx12x : DFFRHQXL port map( D => n234, CK => clk, RN => n309, Q => 
                           b_busD_12_port);
   b_busD_regx11x : DFFRHQXL port map( D => n233, CK => clk, RN => n308, Q => 
                           b_busD_11_port);
   b_busD_regx10x : DFFRHQXL port map( D => n232, CK => clk, RN => n309, Q => 
                           b_busD_10_port);
   a_busD_regx8x : DFFRHQXL port map( D => n204, CK => clk, RN => n309, Q => 
                           a_busD_8_port);
   a_busD_regx6x : DFFRHQXL port map( D => n202, CK => clk, RN => n309, Q => 
                           a_busD_6_port);
   a_busD_regx7x : DFFRHQXL port map( D => n203, CK => clk, RN => n308, Q => 
                           a_busD_7_port);
   b_busD_regx8x : DFFRHQXL port map( D => n230, CK => clk, RN => n308, Q => 
                           b_busD_8_port);
   b_busD_regx9x : DFFRHQXL port map( D => n231, CK => clk, RN => n309, Q => 
                           b_busD_9_port);
   a_busD_regx5x : DFFRHQXL port map( D => n201, CK => clk, RN => n309, Q => 
                           a_busD_5_port);
   b_busD_regx5x : DFFRHQXL port map( D => n227, CK => clk, RN => n308, Q => 
                           b_busD_5_port);
   b_busD_regx6x : DFFRHQXL port map( D => n228, CK => clk, RN => n309, Q => 
                           b_busD_6_port);
   b_busD_regx7x : DFFRHQXL port map( D => n229, CK => clk, RN => n308, Q => 
                           b_busD_7_port);
   a_busD_regx4x : DFFRHQXL port map( D => n200, CK => clk, RN => n308, Q => 
                           a_busD_4_port);
   b_busD_regx4x : DFFRHQXL port map( D => n226, CK => clk, RN => n309, Q => 
                           b_busD_4_port);
   b_busD_regx2x : DFFRHQXL port map( D => n224, CK => clk, RN => n309, Q => 
                           b_busD_2_port);
   b_busD_regx3x : DFFRHQXL port map( D => n225, CK => clk, RN => n309, Q => 
                           b_busD_3_port);
   a_busD_regx1x : DFFRHQXL port map( D => n197, CK => clk, RN => n308, Q => 
                           a_busD_1_port);
   a_busD_regx2x : DFFRHQXL port map( D => n198, CK => clk, RN => n308, Q => 
                           a_busD_2_port);
   a_busD_regx3x : DFFRHQXL port map( D => n199, CK => clk, RN => n308, Q => 
                           a_busD_3_port);
   b_busD_regx0x : DFFRHQXL port map( D => n222, CK => clk, RN => n309, Q => 
                           b_busD_0_port);
   b_busD_regx1x : DFFRHQXL port map( D => n223, CK => clk, RN => n309, Q => 
                           b_busD_1_port);
   a_busD_regx0x : DFFRHQXL port map( D => n196, CK => clk, RN => n309, Q => 
                           a_busD_0_port);
   alu_funcD_regx2x : DFFRHQXL port map( D => n303, CK => clk, RN => n309, Q =>
                           alu_funcD_2_port);
   alu_funcD_regx3x : DFFRHQXL port map( D => n304, CK => clk, RN => n309, Q =>
                           alu_funcD_3_port);
   alu_funcD_regx1x : DFFRHQXL port map( D => n302, CK => clk, RN => n308, Q =>
                           alu_funcD_1_port);
   alu_funcD_regx0x : DFFRHQXL port map( D => n301, CK => clk, RN => n309, Q =>
                           alu_funcD_0_port);
   U3 : AND2XL port map( A => n37, B => n130, Y => n1);
   U4 : AND2XL port map( A => n121, B => n130, Y => n4);
   U5 : NOR3XL port map( A => rt_index(2), B => rt_index(4), C => rt_index(3), 
                           Y => n132);
   U6 : INVXL port map( A => rt_index(1), Y => n305);
   U7 : INVXL port map( A => rt_index(0), Y => n189);
   U8 : NOR3XL port map( A => rs_index(2), B => rs_index(4), C => rs_index(3), 
                           Y => n50);
   U9 : INVXL port map( A => rs_index(1), Y => n49);
   U10 : INVXL port map( A => rs_index(0), Y => n306);
   U11 : XNOR2XL port map( A => n189, B => rd_indexD_0_port, Y => n135);
   U12 : XOR2XL port map( A => rt_index(2), B => rd_indexD_2_port, Y => n134);
   U13 : XNOR2XL port map( A => n305, B => rd_indexD_1_port, Y => n133);
   U14 : AOI211XL port map( A0 => rt_index(5), A1 => n43, B0 => n127, C0 => n45
                           , Y => n126);
   U15 : NOR2XL port map( A => b_source(1), B => b_source(0), Y => n129);
   U16 : AOI31XL port map( A0 => n189, A1 => n305, A2 => n132, B0 => 
                           rd_indexD_5_port, Y => n128);
   U17 : XNOR2XL port map( A => n49, B => rd_indexD_1_port, Y => n51);
   U18 : XNOR2XL port map( A => n306, B => rd_indexD_0_port, Y => n53);
   U19 : XOR2XL port map( A => rs_index(2), B => rd_indexD_2_port, Y => n52);
   U20 : AOI211XL port map( A0 => rs_index(5), A1 => n43, B0 => n44, C0 => n45,
                           Y => n42);
   U21 : NOR2XL port map( A => a_source(1), B => a_source(0), Y => n47);
   U22 : AOI31XL port map( A0 => n306, A1 => n49, A2 => n50, B0 => 
                           rd_indexD_5_port, Y => n46);
   U23 : NAND4XL port map( A => n123, B => n124, C => n125, D => n126, Y => 
                           n121);
   U24 : XNOR2XL port map( A => rd_indexD_3_port, B => rt_index(3), Y => n124);
   U25 : XNOR2XL port map( A => rd_indexD_4_port, B => rt_index(4), Y => n123);
   U26 : NOR3XL port map( A => n133, B => n134, C => n135, Y => n125);
   U27 : NAND4XL port map( A => n39, B => n40, C => n41, D => n42, Y => n37);
   U28 : XNOR2XL port map( A => rd_indexD_3_port, B => rs_index(3), Y => n40);
   U29 : XNOR2XL port map( A => rd_indexD_4_port, B => rs_index(4), Y => n39);
   U30 : NOR3XL port map( A => n51, B => n52, C => n53, Y => n41);
   U31 : NOR2XL port map( A => n310, B => pause_pipeline_port, Y => n2);
   U32 : NOR3BXL port map( AN => c_source_reg_0_port, B => c_source_reg_1_port,
                           C => c_source_reg_2_port, Y => n59);
   U33 : OR2XL port map( A => n121, B => n58, Y => n38);
   U34 : OR2XL port map( A => n37, B => n58, Y => n5);
   U35 : INVXL port map( A => n59, Y => n48);
   U36 : NOR2BXL port map( AN => n307, B => n45, Y => pause_pipeline_port);
   U37 : INVXL port map( A => n38, Y => n55);
   U38 : INVXL port map( A => n5, Y => n56);
   U39 : INVXL port map( A => n58, Y => n131);
   U40 : INVXL port map( A => n130, Y => n58);
   U41 : INVXL port map( A => rd_indexD_5_port, Y => n43);
   U42 : INVXL port map( A => n2, Y => n130);
   U43 : INVXL port map( A => n182, Y => n301);
   U44 : AOI22XL port map( A0 => alu_funcD_0_port, A1 => n2, B0 => alu_func(0),
                           B1 => n130, Y => n182);
   U45 : INVXL port map( A => n183, Y => n302);
   U46 : AOI22XL port map( A0 => alu_funcD_1_port, A1 => n2, B0 => alu_func(1),
                           B1 => n130, Y => n183);
   U47 : INVXL port map( A => n185, Y => n304);
   U48 : AOI22XL port map( A0 => alu_funcD_3_port, A1 => n2, B0 => alu_func(3),
                           B1 => n130, Y => n185);
   U49 : INVXL port map( A => n184, Y => n303);
   U50 : AOI22XL port map( A0 => alu_funcD_2_port, A1 => n58, B0 => alu_func(2)
                           , B1 => n130, Y => n184);
   U51 : AOI22XL port map( A0 => n56, A1 => reg_destD_0_port, B0 => a_bus(0), 
                           B1 => n1, Y => n11);
   U52 : AOI22XL port map( A0 => n55, A1 => reg_destD_1_port, B0 => b_bus(1), 
                           B1 => n4, Y => n60);
   U53 : AOI22XL port map( A0 => n55, A1 => reg_destD_0_port, B0 => b_bus(0), 
                           B1 => n4, Y => n54);
   U54 : AOI22XL port map( A0 => n56, A1 => reg_destD_3_port, B0 => a_bus(3), 
                           B1 => n1, Y => n14);
   U55 : AOI22XL port map( A0 => n56, A1 => reg_destD_2_port, B0 => a_bus(2), 
                           B1 => n1, Y => n13);
   U56 : AOI22XL port map( A0 => n56, A1 => reg_destD_1_port, B0 => a_bus(1), 
                           B1 => n1, Y => n12);
   U57 : AOI22XL port map( A0 => n55, A1 => reg_destD_3_port, B0 => b_bus(3), 
                           B1 => n4, Y => n64);
   U58 : AOI22XL port map( A0 => n55, A1 => reg_destD_2_port, B0 => b_bus(2), 
                           B1 => n4, Y => n62);
   U59 : AOI22XL port map( A0 => n55, A1 => reg_destD_4_port, B0 => b_bus(4), 
                           B1 => n4, Y => n66);
   U60 : AOI22XL port map( A0 => n56, A1 => reg_destD_4_port, B0 => a_bus(4), 
                           B1 => n1, Y => n15);
   U61 : AOI22XL port map( A0 => n55, A1 => reg_destD_7_port, B0 => b_bus(7), 
                           B1 => n4, Y => n72);
   U62 : AOI22XL port map( A0 => n55, A1 => reg_destD_6_port, B0 => b_bus(6), 
                           B1 => n4, Y => n70);
   U63 : AOI22XL port map( A0 => n55, A1 => reg_destD_5_port, B0 => b_bus(5), 
                           B1 => n4, Y => n68);
   U64 : AOI22XL port map( A0 => n56, A1 => reg_destD_5_port, B0 => a_bus(5), 
                           B1 => n1, Y => n16);
   U65 : AOI22XL port map( A0 => n55, A1 => reg_destD_9_port, B0 => b_bus(9), 
                           B1 => n4, Y => n76);
   U66 : AOI22XL port map( A0 => n55, A1 => reg_destD_8_port, B0 => b_bus(8), 
                           B1 => n4, Y => n74);
   U67 : AOI22XL port map( A0 => n56, A1 => reg_destD_7_port, B0 => a_bus(7), 
                           B1 => n1, Y => n18);
   U68 : AOI22XL port map( A0 => n56, A1 => reg_destD_6_port, B0 => a_bus(6), 
                           B1 => n1, Y => n17);
   U69 : AOI22XL port map( A0 => n56, A1 => reg_destD_8_port, B0 => a_bus(8), 
                           B1 => n1, Y => n19);
   U70 : AOI22XL port map( A0 => n55, A1 => reg_destD_10_port, B0 => b_bus(10),
                           B1 => n4, Y => n78);
   U71 : AOI22XL port map( A0 => n55, A1 => reg_destD_11_port, B0 => b_bus(11),
                           B1 => n4, Y => n80);
   U72 : AOI22XL port map( A0 => n55, A1 => reg_destD_12_port, B0 => b_bus(12),
                           B1 => n4, Y => n82);
   U73 : AOI22XL port map( A0 => n56, A1 => reg_destD_11_port, B0 => a_bus(11),
                           B1 => n1, Y => n22);
   U74 : AOI22XL port map( A0 => n56, A1 => reg_destD_10_port, B0 => a_bus(10),
                           B1 => n1, Y => n21);
   U75 : AOI22XL port map( A0 => n56, A1 => reg_destD_9_port, B0 => a_bus(9), 
                           B1 => n1, Y => n20);
   U76 : AOI22XL port map( A0 => n55, A1 => reg_destD_13_port, B0 => b_bus(13),
                           B1 => n4, Y => n84);
   U77 : AOI22XL port map( A0 => n55, A1 => reg_destD_14_port, B0 => b_bus(14),
                           B1 => n4, Y => n86);
   U78 : AOI22XL port map( A0 => n55, A1 => reg_destD_15_port, B0 => b_bus(15),
                           B1 => n4, Y => n88);
   U79 : AOI22XL port map( A0 => n56, A1 => reg_destD_12_port, B0 => a_bus(12),
                           B1 => n1, Y => n23);
   U80 : AOI22XL port map( A0 => n56, A1 => reg_destD_13_port, B0 => a_bus(13),
                           B1 => n1, Y => n24);
   U81 : AOI22XL port map( A0 => n55, A1 => reg_destD_16_port, B0 => b_bus(16),
                           B1 => n4, Y => n90);
   U82 : AOI22XL port map( A0 => n55, A1 => reg_destD_17_port, B0 => b_bus(17),
                           B1 => n4, Y => n92);
   U83 : AOI22XL port map( A0 => n55, A1 => reg_destD_18_port, B0 => b_bus(18),
                           B1 => n4, Y => n94);
   U84 : AOI22XL port map( A0 => n56, A1 => reg_destD_15_port, B0 => a_bus(15),
                           B1 => n1, Y => n26);
   U85 : AOI22XL port map( A0 => n56, A1 => reg_destD_14_port, B0 => a_bus(14),
                           B1 => n1, Y => n25);
   U86 : AOI22XL port map( A0 => n56, A1 => reg_destD_16_port, B0 => a_bus(16),
                           B1 => n1, Y => n27);
   U87 : AOI22XL port map( A0 => n55, A1 => reg_destD_20_port, B0 => b_bus(20),
                           B1 => n4, Y => n98);
   U88 : AOI22XL port map( A0 => n55, A1 => reg_destD_19_port, B0 => b_bus(19),
                           B1 => n4, Y => n96);
   U89 : AOI22XL port map( A0 => n56, A1 => reg_destD_19_port, B0 => a_bus(19),
                           B1 => n1, Y => n30);
   U90 : AOI22XL port map( A0 => n56, A1 => reg_destD_18_port, B0 => a_bus(18),
                           B1 => n1, Y => n29);
   U91 : AOI22XL port map( A0 => n56, A1 => reg_destD_17_port, B0 => a_bus(17),
                           B1 => n1, Y => n28);
   U92 : AOI22XL port map( A0 => n55, A1 => reg_destD_21_port, B0 => b_bus(21),
                           B1 => n4, Y => n100);
   U93 : AOI22XL port map( A0 => n56, A1 => reg_destD_20_port, B0 => a_bus(20),
                           B1 => n1, Y => n31);
   U94 : AOI22XL port map( A0 => n55, A1 => reg_destD_22_port, B0 => b_bus(22),
                           B1 => n4, Y => n102);
   U95 : AOI22XL port map( A0 => n56, A1 => reg_destD_21_port, B0 => a_bus(21),
                           B1 => n1, Y => n32);
   U96 : AOI22XL port map( A0 => n56, A1 => reg_destD_22_port, B0 => a_bus(22),
                           B1 => n1, Y => n33);
   U97 : AOI22XL port map( A0 => n55, A1 => reg_destD_23_port, B0 => b_bus(23),
                           B1 => n4, Y => n104);
   U98 : AOI22XL port map( A0 => n55, A1 => reg_destD_26_port, B0 => b_bus(26),
                           B1 => n4, Y => n110);
   U99 : AOI22XL port map( A0 => n55, A1 => reg_destD_25_port, B0 => b_bus(25),
                           B1 => n4, Y => n108);
   U100 : AOI22XL port map( A0 => n56, A1 => reg_destD_23_port, B0 => a_bus(23)
                           , B1 => n1, Y => n34);
   U101 : AOI22XL port map( A0 => n56, A1 => reg_destD_24_port, B0 => a_bus(24)
                           , B1 => n1, Y => n35);
   U102 : AOI22XL port map( A0 => n55, A1 => reg_destD_24_port, B0 => b_bus(24)
                           , B1 => n4, Y => n106);
   U103 : AOI22XL port map( A0 => n55, A1 => reg_destD_27_port, B0 => b_bus(27)
                           , B1 => n4, Y => n112);
   U104 : AOI22XL port map( A0 => n55, A1 => reg_destD_29_port, B0 => b_bus(29)
                           , B1 => n4, Y => n116);
   U105 : AOI22XL port map( A0 => n55, A1 => reg_destD_28_port, B0 => b_bus(28)
                           , B1 => n4, Y => n114);
   U106 : AOI22XL port map( A0 => n56, A1 => reg_destD_26_port, B0 => a_bus(26)
                           , B1 => n1, Y => n3);
   U107 : AOI22XL port map( A0 => n56, A1 => reg_destD_27_port, B0 => a_bus(27)
                           , B1 => n1, Y => n6);
   U108 : INVXL port map( A => reset, Y => n309);
   U109 : AOI22XL port map( A0 => n56, A1 => reg_destD_25_port, B0 => a_bus(25)
                           , B1 => n1, Y => n36);
   U110 : AOI22XL port map( A0 => n55, A1 => reg_destD_31_port, B0 => b_bus(31)
                           , B1 => n4, Y => n120);
   U111 : AOI22XL port map( A0 => n56, A1 => reg_destD_31_port, B0 => a_bus(31)
                           , B1 => n1, Y => n10);
   U112 : AOI22XL port map( A0 => n56, A1 => reg_destD_28_port, B0 => a_bus(28)
                           , B1 => n1, Y => n7);
   U113 : AOI22XL port map( A0 => n56, A1 => reg_destD_30_port, B0 => a_bus(30)
                           , B1 => n1, Y => n9);
   U114 : AOI22XL port map( A0 => n55, A1 => reg_destD_30_port, B0 => b_bus(30)
                           , B1 => n4, Y => n118);
   U115 : AOI22XL port map( A0 => n56, A1 => reg_destD_29_port, B0 => a_bus(29)
                           , B1 => n1, Y => n8);
   U116 : INVXL port map( A => n176, Y => n295);
   U117 : AOI22XL port map( A0 => n131, A1 => mult_func(0), B0 => 
                           mult_funcD_0_port, B1 => n58, Y => n176);
   U118 : INVXL port map( A => n179, Y => n298);
   U119 : AOI22XL port map( A0 => n131, A1 => mult_func(3), B0 => 
                           mult_funcD_3_port, B1 => n58, Y => n179);
   U120 : INVXL port map( A => n178, Y => n297);
   U121 : AOI22XL port map( A0 => n131, A1 => mult_func(2), B0 => 
                           mult_funcD_2_port, B1 => n58, Y => n178);
   U122 : INVXL port map( A => n177, Y => n296);
   U123 : AOI22XL port map( A0 => n131, A1 => mult_func(1), B0 => 
                           mult_funcD_1_port, B1 => n58, Y => n177);
   U124 : INVXL port map( A => n181, Y => n300);
   U125 : AOI22XL port map( A0 => shift_funcD_1_port, A1 => n2, B0 => 
                           shift_func(1), B1 => n130, Y => n181);
   U126 : INVXL port map( A => n180, Y => n299);
   U127 : AOI22XL port map( A0 => shift_funcD_0_port, A1 => n58, B0 => 
                           shift_func(0), B1 => n130, Y => n180);
   U128 : OR4XL port map( A => mem_source(1), B => mem_source(0), C => n186, D 
                           => n187, Y => n307);
   U129 : NOR3XL port map( A => n188, B => mult_func(3), C => mult_func(2), Y 
                           => n186);
   U130 : XNOR2XL port map( A => mult_func(0), B => mult_func(1), Y => n188);
   U131 : INVXL port map( A => pause_any, Y => n310);
   U132 : INVXL port map( A => n140, Y => n258);
   U133 : AOI22XL port map( A0 => rd_indexD_4_port, A1 => n2, B0 => rd_index(4)
                           , B1 => n130, Y => n140);
   U134 : INVXL port map( A => n138, Y => n256);
   U135 : AOI22XL port map( A0 => rd_indexD_2_port, A1 => n2, B0 => rd_index(2)
                           , B1 => n130, Y => n138);
   U136 : INVXL port map( A => n137, Y => n255);
   U137 : AOI22XL port map( A0 => rd_indexD_1_port, A1 => n2, B0 => rd_index(1)
                           , B1 => n130, Y => n137);
   U138 : INVXL port map( A => n136, Y => n254);
   U139 : AOI22XL port map( A0 => rd_indexD_0_port, A1 => n2, B0 => rd_index(0)
                           , B1 => n130, Y => n136);
   U140 : INVXL port map( A => n139, Y => n257);
   U141 : AOI22XL port map( A0 => rd_indexD_3_port, A1 => n2, B0 => rd_index(3)
                           , B1 => n130, Y => n139);
   U142 : OAI2BB2XL port map( B0 => n131, B1 => n43, A0N => rd_index(5), A1N =>
                           n130, Y => n259);
   U143 : INVXL port map( A => n143, Y => n262);
   U144 : AOI22XL port map( A0 => n58, A1 => c_source_reg_2_port, B0 => 
                           c_source(2), B1 => n130, Y => n143);
   U145 : INVXL port map( A => n142, Y => n261);
   U146 : AOI22XL port map( A0 => n58, A1 => c_source_reg_1_port, B0 => 
                           c_source(1), B1 => n130, Y => n142);
   U147 : INVXL port map( A => n141, Y => n260);
   U148 : AOI22XL port map( A0 => c_source_reg_0_port, A1 => n2, B0 => 
                           c_source(0), B1 => n130, Y => n141);
   U149 : INVXL port map( A => n144, Y => n263);
   U150 : AOI22XL port map( A0 => reg_dest_reg_0_port, A1 => n2, B0 => 
                           reg_dest(0), B1 => n130, Y => n144);
   U151 : INVXL port map( A => n145, Y => n264);
   U152 : AOI22XL port map( A0 => reg_dest_reg_1_port, A1 => n2, B0 => 
                           reg_dest(1), B1 => n130, Y => n145);
   U153 : INVXL port map( A => n146, Y => n265);
   U154 : AOI22XL port map( A0 => reg_dest_reg_2_port, A1 => n2, B0 => 
                           reg_dest(2), B1 => n130, Y => n146);
   U155 : INVXL port map( A => n147, Y => n266);
   U156 : AOI22XL port map( A0 => reg_dest_reg_3_port, A1 => n2, B0 => 
                           reg_dest(3), B1 => n130, Y => n147);
   U157 : INVXL port map( A => n148, Y => n267);
   U158 : AOI22XL port map( A0 => reg_dest_reg_4_port, A1 => n58, B0 => 
                           reg_dest(4), B1 => n130, Y => n148);
   U159 : INVXL port map( A => n149, Y => n268);
   U160 : AOI22XL port map( A0 => reg_dest_reg_5_port, A1 => n58, B0 => 
                           reg_dest(5), B1 => n130, Y => n149);
   U161 : INVXL port map( A => n150, Y => n269);
   U162 : AOI22XL port map( A0 => reg_dest_reg_6_port, A1 => n2, B0 => 
                           reg_dest(6), B1 => n130, Y => n150);
   U163 : INVXL port map( A => n151, Y => n270);
   U164 : AOI22XL port map( A0 => reg_dest_reg_7_port, A1 => n2, B0 => 
                           reg_dest(7), B1 => n131, Y => n151);
   U165 : INVXL port map( A => n152, Y => n271);
   U166 : AOI22XL port map( A0 => reg_dest_reg_8_port, A1 => n2, B0 => 
                           reg_dest(8), B1 => n130, Y => n152);
   U167 : INVXL port map( A => n153, Y => n272);
   U168 : AOI22XL port map( A0 => reg_dest_reg_9_port, A1 => n2, B0 => 
                           reg_dest(9), B1 => n130, Y => n153);
   U169 : INVXL port map( A => n154, Y => n273);
   U170 : AOI22XL port map( A0 => reg_dest_reg_10_port, A1 => n2, B0 => 
                           reg_dest(10), B1 => n130, Y => n154);
   U171 : INVXL port map( A => n155, Y => n274);
   U172 : AOI22XL port map( A0 => reg_dest_reg_11_port, A1 => n2, B0 => 
                           reg_dest(11), B1 => n130, Y => n155);
   U173 : INVXL port map( A => n156, Y => n275);
   U174 : AOI22XL port map( A0 => reg_dest_reg_12_port, A1 => n58, B0 => 
                           reg_dest(12), B1 => n130, Y => n156);
   U175 : INVXL port map( A => n157, Y => n276);
   U176 : AOI22XL port map( A0 => reg_dest_reg_13_port, A1 => n2, B0 => 
                           reg_dest(13), B1 => n130, Y => n157);
   U177 : INVXL port map( A => n158, Y => n277);
   U178 : AOI22XL port map( A0 => reg_dest_reg_14_port, A1 => n2, B0 => 
                           reg_dest(14), B1 => n130, Y => n158);
   U179 : INVXL port map( A => n159, Y => n278);
   U180 : AOI22XL port map( A0 => reg_dest_reg_15_port, A1 => n2, B0 => 
                           reg_dest(15), B1 => n130, Y => n159);
   U181 : INVXL port map( A => n160, Y => n279);
   U182 : AOI22XL port map( A0 => reg_dest_reg_16_port, A1 => n2, B0 => 
                           reg_dest(16), B1 => n130, Y => n160);
   U183 : INVXL port map( A => n161, Y => n280);
   U184 : AOI22XL port map( A0 => reg_dest_reg_17_port, A1 => n2, B0 => 
                           reg_dest(17), B1 => n130, Y => n161);
   U185 : INVXL port map( A => n162, Y => n281);
   U186 : AOI22XL port map( A0 => reg_dest_reg_18_port, A1 => n2, B0 => 
                           reg_dest(18), B1 => n130, Y => n162);
   U187 : INVXL port map( A => n163, Y => n282);
   U188 : AOI22XL port map( A0 => reg_dest_reg_19_port, A1 => n2, B0 => 
                           reg_dest(19), B1 => n130, Y => n163);
   U189 : INVXL port map( A => n164, Y => n283);
   U190 : AOI22XL port map( A0 => reg_dest_reg_20_port, A1 => n2, B0 => 
                           reg_dest(20), B1 => n130, Y => n164);
   U191 : INVXL port map( A => n165, Y => n284);
   U192 : AOI22XL port map( A0 => reg_dest_reg_21_port, A1 => n2, B0 => 
                           reg_dest(21), B1 => n130, Y => n165);
   U193 : INVXL port map( A => n166, Y => n285);
   U194 : AOI22XL port map( A0 => reg_dest_reg_22_port, A1 => n2, B0 => 
                           reg_dest(22), B1 => n130, Y => n166);
   U195 : INVXL port map( A => n167, Y => n286);
   U196 : AOI22XL port map( A0 => reg_dest_reg_23_port, A1 => n2, B0 => 
                           reg_dest(23), B1 => n130, Y => n167);
   U197 : INVXL port map( A => n168, Y => n287);
   U198 : AOI22XL port map( A0 => reg_dest_reg_24_port, A1 => n58, B0 => 
                           reg_dest(24), B1 => n130, Y => n168);
   U199 : INVXL port map( A => n169, Y => n288);
   U200 : AOI22XL port map( A0 => reg_dest_reg_25_port, A1 => n2, B0 => 
                           reg_dest(25), B1 => n130, Y => n169);
   U201 : INVXL port map( A => n170, Y => n289);
   U202 : AOI22XL port map( A0 => reg_dest_reg_26_port, A1 => n58, B0 => 
                           reg_dest(26), B1 => n130, Y => n170);
   U203 : INVXL port map( A => n171, Y => n290);
   U204 : AOI22XL port map( A0 => reg_dest_reg_27_port, A1 => n2, B0 => 
                           reg_dest(27), B1 => n130, Y => n171);
   U205 : INVXL port map( A => reset, Y => n308);
   U206 : INVXL port map( A => n172, Y => n291);
   U207 : AOI22XL port map( A0 => reg_dest_reg_28_port, A1 => n2, B0 => 
                           reg_dest(28), B1 => n130, Y => n172);
   U208 : INVXL port map( A => n173, Y => n292);
   U209 : AOI22XL port map( A0 => reg_dest_reg_29_port, A1 => n2, B0 => 
                           reg_dest(29), B1 => n130, Y => n173);
   U210 : INVXL port map( A => n174, Y => n293);
   U211 : AOI22XL port map( A0 => reg_dest_reg_30_port, A1 => n2, B0 => 
                           reg_dest(30), B1 => n130, Y => n174);
   U212 : INVXL port map( A => n175, Y => n294);
   U213 : AOI22XL port map( A0 => reg_dest_reg_31_port, A1 => n2, B0 => 
                           reg_dest(31), B1 => n130, Y => n175);
   U214 : INVXL port map( A => n122, Y => reg_destD_31_port);
   U215 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_31_port, B0 => 
                           c_bus(31), B1 => n59, Y => n122);
   U216 : INVXL port map( A => n119, Y => reg_destD_30_port);
   U217 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_30_port, B0 => 
                           c_bus(30), B1 => n59, Y => n119);
   U218 : INVXL port map( A => n117, Y => reg_destD_29_port);
   U219 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_29_port, B0 => 
                           c_bus(29), B1 => n59, Y => n117);
   U220 : INVXL port map( A => n115, Y => reg_destD_28_port);
   U221 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_28_port, B0 => 
                           c_bus(28), B1 => n59, Y => n115);
   U222 : INVXL port map( A => n113, Y => reg_destD_27_port);
   U223 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_27_port, B0 => 
                           c_bus(27), B1 => n59, Y => n113);
   U224 : INVXL port map( A => n111, Y => reg_destD_26_port);
   U225 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_26_port, B0 => 
                           c_bus(26), B1 => n59, Y => n111);
   U226 : INVXL port map( A => n109, Y => reg_destD_25_port);
   U227 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_25_port, B0 => 
                           c_bus(25), B1 => n59, Y => n109);
   U228 : INVXL port map( A => n107, Y => reg_destD_24_port);
   U229 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_24_port, B0 => 
                           c_bus(24), B1 => n59, Y => n107);
   U230 : INVXL port map( A => n105, Y => reg_destD_23_port);
   U231 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_23_port, B0 => 
                           c_bus(23), B1 => n59, Y => n105);
   U232 : INVXL port map( A => n103, Y => reg_destD_22_port);
   U233 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_22_port, B0 => 
                           c_bus(22), B1 => n59, Y => n103);
   U234 : INVXL port map( A => n101, Y => reg_destD_21_port);
   U235 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_21_port, B0 => 
                           c_bus(21), B1 => n59, Y => n101);
   U236 : INVXL port map( A => n99, Y => reg_destD_20_port);
   U237 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_20_port, B0 => 
                           c_bus(20), B1 => n59, Y => n99);
   U238 : INVXL port map( A => n97, Y => reg_destD_19_port);
   U239 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_19_port, B0 => 
                           c_bus(19), B1 => n59, Y => n97);
   U240 : INVXL port map( A => n95, Y => reg_destD_18_port);
   U241 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_18_port, B0 => 
                           c_bus(18), B1 => n59, Y => n95);
   U242 : INVXL port map( A => n93, Y => reg_destD_17_port);
   U243 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_17_port, B0 => 
                           c_bus(17), B1 => n59, Y => n93);
   U244 : INVXL port map( A => n91, Y => reg_destD_16_port);
   U245 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_16_port, B0 => 
                           c_bus(16), B1 => n59, Y => n91);
   U246 : INVXL port map( A => n89, Y => reg_destD_15_port);
   U247 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_15_port, B0 => 
                           c_bus(15), B1 => n59, Y => n89);
   U248 : INVXL port map( A => n87, Y => reg_destD_14_port);
   U249 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_14_port, B0 => 
                           c_bus(14), B1 => n59, Y => n87);
   U250 : INVXL port map( A => n85, Y => reg_destD_13_port);
   U251 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_13_port, B0 => 
                           c_bus(13), B1 => n59, Y => n85);
   U252 : INVXL port map( A => n83, Y => reg_destD_12_port);
   U253 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_12_port, B0 => 
                           c_bus(12), B1 => n59, Y => n83);
   U254 : INVXL port map( A => n81, Y => reg_destD_11_port);
   U255 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_11_port, B0 => 
                           c_bus(11), B1 => n59, Y => n81);
   U256 : INVXL port map( A => n79, Y => reg_destD_10_port);
   U257 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_10_port, B0 => 
                           c_bus(10), B1 => n59, Y => n79);
   U258 : INVXL port map( A => n77, Y => reg_destD_9_port);
   U259 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_9_port, B0 => 
                           c_bus(9), B1 => n59, Y => n77);
   U260 : INVXL port map( A => n75, Y => reg_destD_8_port);
   U261 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_8_port, B0 => 
                           c_bus(8), B1 => n59, Y => n75);
   U262 : INVXL port map( A => n73, Y => reg_destD_7_port);
   U263 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_7_port, B0 => 
                           c_bus(7), B1 => n59, Y => n73);
   U264 : INVXL port map( A => n71, Y => reg_destD_6_port);
   U265 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_6_port, B0 => 
                           c_bus(6), B1 => n59, Y => n71);
   U266 : INVXL port map( A => n69, Y => reg_destD_5_port);
   U267 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_5_port, B0 => 
                           c_bus(5), B1 => n59, Y => n69);
   U268 : INVXL port map( A => n67, Y => reg_destD_4_port);
   U269 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_4_port, B0 => 
                           c_bus(4), B1 => n59, Y => n67);
   U270 : INVXL port map( A => n65, Y => reg_destD_3_port);
   U271 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_3_port, B0 => 
                           c_bus(3), B1 => n59, Y => n65);
   U272 : INVXL port map( A => n63, Y => reg_destD_2_port);
   U273 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_2_port, B0 => 
                           c_bus(2), B1 => n59, Y => n63);
   U274 : INVXL port map( A => n61, Y => reg_destD_1_port);
   U275 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_1_port, B0 => 
                           c_bus(1), B1 => n59, Y => n61);
   U276 : INVXL port map( A => n57, Y => reg_destD_0_port);
   U277 : AOI22XL port map( A0 => n48, A1 => reg_dest_reg_0_port, B0 => 
                           c_bus(0), B1 => n59, Y => n57);
   U278 : OR3XL port map( A => pc_source(1), B => mem_source(3), C => 
                           mem_source(2), Y => n187);
   U279 : OAI2BB1X1 port map( A0N => a_busD_0_port, A1N => n2, B0 => n11, Y => 
                           n196);
   U280 : OAI2BB1X1 port map( A0N => b_busD_0_port, A1N => n58, B0 => n54, Y =>
                           n222);
   U281 : OAI2BB1X1 port map( A0N => a_busD_31_port, A1N => n2, B0 => n10, Y =>
                           n195);
   U282 : OAI2BB1X1 port map( A0N => b_busD_31_port, A1N => n58, B0 => n120, Y 
                           => n253);
   U283 : OAI2BB1X1 port map( A0N => b_busD_29_port, A1N => n58, B0 => n116, Y 
                           => n251);
   U284 : OAI2BB1X1 port map( A0N => b_busD_30_port, A1N => n58, B0 => n118, Y 
                           => n252);
   U285 : OAI2BB1X1 port map( A0N => a_busD_29_port, A1N => n2, B0 => n8, Y => 
                           n193);
   U286 : OAI2BB1X1 port map( A0N => a_busD_30_port, A1N => n2, B0 => n9, Y => 
                           n194);
   U287 : OAI2BB1X1 port map( A0N => b_busD_26_port, A1N => n58, B0 => n110, Y 
                           => n248);
   U288 : OAI2BB1X1 port map( A0N => b_busD_28_port, A1N => n58, B0 => n114, Y 
                           => n250);
   U289 : OAI2BB1X1 port map( A0N => a_busD_26_port, A1N => n58, B0 => n3, Y =>
                           n190);
   U290 : OAI2BB1X1 port map( A0N => a_busD_28_port, A1N => n58, B0 => n7, Y =>
                           n192);
   U291 : OAI2BB1X1 port map( A0N => b_busD_27_port, A1N => n58, B0 => n112, Y 
                           => n249);
   U292 : OAI2BB1X1 port map( A0N => a_busD_27_port, A1N => n58, B0 => n6, Y =>
                           n191);
   U293 : OAI2BB1X1 port map( A0N => a_busD_24_port, A1N => n58, B0 => n35, Y 
                           => n220);
   U294 : OAI2BB1X1 port map( A0N => b_busD_25_port, A1N => n58, B0 => n108, Y 
                           => n247);
   U295 : OAI2BB1X1 port map( A0N => b_busD_23_port, A1N => n58, B0 => n104, Y 
                           => n245);
   U296 : OAI2BB1X1 port map( A0N => b_busD_24_port, A1N => n58, B0 => n106, Y 
                           => n246);
   U297 : OAI2BB1X1 port map( A0N => a_busD_23_port, A1N => n58, B0 => n34, Y 
                           => n219);
   U298 : OAI2BB1X1 port map( A0N => a_busD_25_port, A1N => n58, B0 => n36, Y 
                           => n221);
   U299 : OAI2BB1X1 port map( A0N => a_busD_22_port, A1N => n58, B0 => n33, Y 
                           => n218);
   U300 : OAI2BB1X1 port map( A0N => b_busD_21_port, A1N => n58, B0 => n100, Y 
                           => n243);
   U301 : OAI2BB1X1 port map( A0N => b_busD_22_port, A1N => n58, B0 => n102, Y 
                           => n244);
   U302 : OAI2BB1X1 port map( A0N => b_busD_20_port, A1N => n58, B0 => n98, Y 
                           => n242);
   U303 : OAI2BB1X1 port map( A0N => a_busD_20_port, A1N => n58, B0 => n31, Y 
                           => n216);
   U304 : OAI2BB1X1 port map( A0N => a_busD_21_port, A1N => n58, B0 => n32, Y 
                           => n217);
   U305 : OAI2BB1X1 port map( A0N => b_busD_19_port, A1N => n58, B0 => n96, Y 
                           => n241);
   U306 : OAI2BB1X1 port map( A0N => b_busD_18_port, A1N => n58, B0 => n94, Y 
                           => n240);
   U307 : OAI2BB1X1 port map( A0N => a_busD_19_port, A1N => n2, B0 => n30, Y =>
                           n215);
   U308 : OAI2BB1X1 port map( A0N => a_busD_18_port, A1N => n2, B0 => n29, Y =>
                           n214);
   U309 : OAI2BB1X1 port map( A0N => b_busD_15_port, A1N => n58, B0 => n88, Y 
                           => n237);
   U310 : OAI2BB1X1 port map( A0N => b_busD_16_port, A1N => n58, B0 => n90, Y 
                           => n238);
   U311 : OAI2BB1X1 port map( A0N => b_busD_17_port, A1N => n58, B0 => n92, Y 
                           => n239);
   U312 : OAI2BB1X1 port map( A0N => a_busD_15_port, A1N => n58, B0 => n26, Y 
                           => n211);
   U313 : OAI2BB1X1 port map( A0N => a_busD_16_port, A1N => n2, B0 => n27, Y =>
                           n212);
   U314 : OAI2BB1X1 port map( A0N => a_busD_17_port, A1N => n2, B0 => n28, Y =>
                           n213);
   U315 : OAI2BB1X1 port map( A0N => b_busD_12_port, A1N => n58, B0 => n82, Y 
                           => n234);
   U316 : OAI2BB1X1 port map( A0N => b_busD_13_port, A1N => n58, B0 => n84, Y 
                           => n235);
   U317 : OAI2BB1X1 port map( A0N => b_busD_14_port, A1N => n58, B0 => n86, Y 
                           => n236);
   U318 : OAI2BB1X1 port map( A0N => a_busD_12_port, A1N => n2, B0 => n23, Y =>
                           n208);
   U319 : OAI2BB1X1 port map( A0N => a_busD_13_port, A1N => n2, B0 => n24, Y =>
                           n209);
   U320 : OAI2BB1X1 port map( A0N => a_busD_14_port, A1N => n2, B0 => n25, Y =>
                           n210);
   U321 : OAI2BB1X1 port map( A0N => a_busD_4_port, A1N => n58, B0 => n15, Y =>
                           n200);
   U322 : OAI2BB1X1 port map( A0N => a_busD_3_port, A1N => n58, B0 => n14, Y =>
                           n199);
   U323 : OAI2BB1X1 port map( A0N => a_busD_2_port, A1N => n58, B0 => n13, Y =>
                           n198);
   U324 : OAI2BB1X1 port map( A0N => a_busD_1_port, A1N => n2, B0 => n12, Y => 
                           n197);
   U325 : OAI2BB1X1 port map( A0N => b_busD_9_port, A1N => n58, B0 => n76, Y =>
                           n231);
   U326 : OAI2BB1X1 port map( A0N => b_busD_3_port, A1N => n58, B0 => n64, Y =>
                           n225);
   U327 : OAI2BB1X1 port map( A0N => b_busD_7_port, A1N => n58, B0 => n72, Y =>
                           n229);
   U328 : OAI2BB1X1 port map( A0N => b_busD_6_port, A1N => n58, B0 => n70, Y =>
                           n228);
   U329 : OAI2BB1X1 port map( A0N => b_busD_8_port, A1N => n58, B0 => n74, Y =>
                           n230);
   U330 : OAI2BB1X1 port map( A0N => b_busD_10_port, A1N => n58, B0 => n78, Y 
                           => n232);
   U331 : OAI2BB1X1 port map( A0N => b_busD_11_port, A1N => n58, B0 => n80, Y 
                           => n233);
   U332 : OAI2BB1X1 port map( A0N => a_busD_11_port, A1N => n2, B0 => n22, Y =>
                           n207);
   U333 : OAI2BB1X1 port map( A0N => a_busD_5_port, A1N => n58, B0 => n16, Y =>
                           n201);
   U334 : OAI2BB1X1 port map( A0N => a_busD_6_port, A1N => n2, B0 => n17, Y => 
                           n202);
   U335 : OAI2BB1X1 port map( A0N => a_busD_7_port, A1N => n2, B0 => n18, Y => 
                           n203);
   U336 : OAI2BB1X1 port map( A0N => a_busD_8_port, A1N => n58, B0 => n19, Y =>
                           n204);
   U337 : OAI2BB1X1 port map( A0N => a_busD_9_port, A1N => n2, B0 => n20, Y => 
                           n205);
   U338 : OAI2BB1X1 port map( A0N => a_busD_10_port, A1N => n2, B0 => n21, Y =>
                           n206);
   U339 : OAI2BB1X1 port map( A0N => b_busD_1_port, A1N => n58, B0 => n60, Y =>
                           n223);
   U340 : OAI2BB1X1 port map( A0N => b_busD_2_port, A1N => n58, B0 => n62, Y =>
                           n224);
   U341 : OAI2BB1X1 port map( A0N => b_busD_4_port, A1N => n58, B0 => n66, Y =>
                           n226);
   U342 : OAI2BB1X1 port map( A0N => b_busD_5_port, A1N => n58, B0 => n68, Y =>
                           n227);
   U343 : OAI21XL port map( A0 => rt_index(5), A1 => n128, B0 => n129, Y => 
                           n127);
   U344 : OAI21XL port map( A0 => rs_index(5), A1 => n46, B0 => n47, Y => n44);

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity mult_mult_typeDEFAULT is

   port( clk, reset_in : in std_logic;  a, b : in std_logic_vector (31 downto 
         0);  mult_func : in std_logic_vector (3 downto 0);  c_mult : out 
         std_logic_vector (31 downto 0);  pause_out : out std_logic);

end mult_mult_typeDEFAULT;

architecture SYN_logic of mult_mult_typeDEFAULT is

   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI222XL
      port( A0, A1, B0, B1, C0, C1 : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI32XL
      port( A0, A1, A2, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component XNOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI33XL
      port( A0, A1, A2, B0, B1, B2 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI32XL
      port( A0, A1, A2, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI33XL
      port( A0, A1, A2, B0, B1, B2 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI222XL
      port( A0, A1, B0, B1, C0, C1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI2BB2XL
      port( B0, B1, A0N, A1N : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component XOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AND4X1
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OR4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3BXL
      port( AN, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR4BXL
      port( AN, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component DFFRHQXL
      port( D, CK, RN : in std_logic;  Q : out std_logic);
   end component;
   
   component mult_mult_typeDEFAULT_DW01_dec_0
      port( A : in std_logic_vector (5 downto 0);  SUM : out std_logic_vector 
            (5 downto 0));
   end component;
   
   signal negate_reg, lower_reg_31_port, lower_reg_30_port, lower_reg_29_port, 
      lower_reg_28_port, lower_reg_27_port, lower_reg_26_port, 
      lower_reg_25_port, lower_reg_24_port, lower_reg_23_port, 
      lower_reg_22_port, lower_reg_21_port, lower_reg_20_port, 
      lower_reg_19_port, lower_reg_18_port, lower_reg_17_port, 
      lower_reg_16_port, lower_reg_15_port, lower_reg_14_port, 
      lower_reg_13_port, lower_reg_12_port, lower_reg_11_port, 
      lower_reg_10_port, lower_reg_9_port, lower_reg_8_port, lower_reg_7_port, 
      lower_reg_6_port, lower_reg_5_port, lower_reg_4_port, lower_reg_3_port, 
      lower_reg_2_port, lower_reg_1_port, lower_reg_0_port, upper_reg_31_port, 
      upper_reg_30_port, upper_reg_29_port, upper_reg_28_port, 
      upper_reg_27_port, upper_reg_26_port, upper_reg_25_port, 
      upper_reg_24_port, upper_reg_23_port, upper_reg_22_port, 
      upper_reg_21_port, upper_reg_20_port, upper_reg_19_port, 
      upper_reg_18_port, upper_reg_17_port, upper_reg_16_port, 
      upper_reg_15_port, upper_reg_14_port, upper_reg_13_port, 
      upper_reg_12_port, upper_reg_11_port, upper_reg_10_port, upper_reg_9_port
      , upper_reg_8_port, upper_reg_7_port, upper_reg_6_port, upper_reg_5_port,
      upper_reg_4_port, upper_reg_3_port, upper_reg_2_port, upper_reg_1_port, 
      upper_reg_0_port, count_reg_5_port, count_reg_4_port, count_reg_3_port, 
      count_reg_2_port, count_reg_1_port, count_reg_0_port, aa_reg_31_port, 
      aa_reg_30_port, aa_reg_29_port, aa_reg_28_port, aa_reg_27_port, 
      aa_reg_26_port, aa_reg_25_port, aa_reg_24_port, aa_reg_23_port, 
      aa_reg_22_port, aa_reg_21_port, aa_reg_20_port, aa_reg_19_port, 
      aa_reg_18_port, aa_reg_17_port, aa_reg_16_port, aa_reg_15_port, 
      aa_reg_14_port, aa_reg_13_port, aa_reg_12_port, aa_reg_11_port, 
      aa_reg_10_port, aa_reg_9_port, aa_reg_8_port, aa_reg_7_port, 
      aa_reg_6_port, aa_reg_5_port, aa_reg_4_port, aa_reg_3_port, aa_reg_2_port
      , aa_reg_1_port, aa_reg_0_port, mode_reg, sign_reg, sign2_reg, 
      bb_reg_31_port, bb_reg_30_port, bb_reg_29_port, bb_reg_28_port, 
      bb_reg_27_port, bb_reg_26_port, bb_reg_25_port, bb_reg_24_port, 
      bb_reg_23_port, bb_reg_22_port, bb_reg_21_port, bb_reg_20_port, 
      bb_reg_19_port, bb_reg_18_port, bb_reg_17_port, bb_reg_16_port, 
      bb_reg_15_port, bb_reg_14_port, bb_reg_13_port, bb_reg_12_port, 
      bb_reg_11_port, bb_reg_10_port, bb_reg_9_port, bb_reg_8_port, 
      bb_reg_7_port, bb_reg_6_port, bb_reg_5_port, bb_reg_4_port, bb_reg_3_port
      , bb_reg_2_port, bb_reg_1_port, bb_reg_0_port, N815, N816, N817, N818, 
      N819, N820, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12, n14, n15, n16, n17
      , n18, n19, n20, n21, n23, n24, n25, n26, n27, n28, n29, n30, n32, n33, 
      n34, n35, n36, n37, n38, n39, n40, n41, n42, n43, n44, n45, n46, n47, n48
      , n49, n50, n51, n52, n53, n54, n55, n56, n57, n58, n59, n60, n61, n62, 
      n63, n64, n65, n66, n67, n68, n69, n70, n71, n72, n73, n74, n75, n76, n77
      , n78, n79, n80, n81, n82, n83, n84, n85, n86, n87, n88, n89, n90, n91, 
      n92, n93, n94, n95, n96, n97, n98, n99, n100, n101, n102, n103, n104, 
      n105, n107, n108, n109, n110, n111, n112, n113, n114, n115, n116, n117, 
      n118, n119, n120, n121, n122, n123, n124, n125, n126, n127, n128, n129, 
      n130, n131, n132, n133, n134, n135, n136, n137, n138, n139, n140, n141, 
      n142, n143, n144, n145, n146, n147, n148, n149, n150, n151, n152, n153, 
      n154, n155, n156, n157, n158, n159, n160, n161, n162, n163, n164, n165, 
      n166, n167, n168, n169, n170, n171, n172, n173, n174, n175, n176, n177, 
      n178, n179, n180, n181, n182, n183, n184, n185, n186, n187, n188, n189, 
      n190, n191, n192, n193, n194, n195, n196, n197, n198, n199, n200, n201, 
      n202, n203, n204, n205, n206, n207, n208, n209, n210, n211, n212, n213, 
      n214, n215, n216, n217, n218, n219, n220, n221, n222, n223, n224, n225, 
      n226, n227, n228, n229, n230, n231, n232, n233, n234, n235, n236, n237, 
      n238, n239, n240, n241, n242, n243, n244, n245, n246, n247, n248, n249, 
      n250, n251, n252, n253, n254, n255, n256, n257, n258, n259, n260, n261, 
      n262, n263, n264, n265, n266, n267, n268, n269, n270, n271, n272, n273, 
      n274, n275, n276, n277, n278, n279, n280, n281, n282, n283, n284, n285, 
      n286, n287, n288, n289, n290, n291, n292, n293, n294, n295, n296, n297, 
      n298, n299, n300, n301, n302, n303, n304, n305, n306, n307, n308, n309, 
      n310, n311, n312, n313, n314, n315, n316, n317, n318, n319, n320, n321, 
      n322, n323, n324, n325, n326, n327, n328, n329, n330, n331, n332, n333, 
      n334, n335, n336, n338, n339, n340, n341, n343, n344, n345, n346, n347, 
      n348, n349, n351, n352, n353, n354, n355, n356, n357, n358, n359, n360, 
      n361, n362, n363, n364, n365, n366, n367, n368, n369, n370, n371, n372, 
      n373, n374, n375, n376, n377, n378, n379, n380, n381, n382, n383, n384, 
      n385, n386, n387, n388, n389, n390, n391, n392, n393, n394, n395, n396, 
      n397, n398, n399, n400, n401, n402, n403, n404, n405, n406, n407, n408, 
      n409, n410, n411, n412, n413, n414, n415, n416, n417, n418, n419, n420, 
      n421, n422, n423, n424, n425, n426, n427, n428, n429, n430, n431, n432, 
      n433, n434, n435, n436, n437, n438, n439, n440, n441, n442, n443, n444, 
      n445, n446, n447, n448, n449, n450, n451, n452, n453, n454, n455, n456, 
      n457, n458, n459, n460, n461, n462, n463, n464, n465, n466, n467, n469, 
      n470, n471, n472, n473, n474, n475, n476, n477, n478, n479, n480, n481, 
      n482, n483, n484, n485, n486, n487, n488, n489, n490, n491, n492, n493, 
      n494, n495, n496, n497, n498, n499, n500, n501, n502, n503, n504, n505, 
      n506, n507, n508, n509, n510, n511, n512, n513, n514, n515, n516, n517, 
      n518, n519, n520, n521, n522, n523, n524, n525, n526, n527, n528, n529, 
      n530, n531, n532, n533, n534, n535, n536, n537, n538, n539, n540, n541, 
      n542, n543, n544, n545, n547, n548, n549, n550, n551, n552, n554, n556, 
      n557, n558, n559, n560, n561, n562, n563, n564, n565, n566, n567, n568, 
      n569, n570, n571, n572, n573, n574, n575, n576, n577, n578, n579, n580, 
      n581, n582, n583, n584, n585, n586, n587, n588, n589, n590, n591, n592, 
      n593, n594, n595, n596, n597, n598, n599, n600, n601, n602, n603, n604, 
      n605, n606, n607, n608, n609, n610, n611, n612, n613, n614, n615, n616, 
      n617, n618, n619, n620, n621, n622, n623, n624, n625, n626, n627, n628, 
      n629, n630, n631, n632, n633, n634, n635, n636, n637, n638, n639, n640, 
      n641, n642, n643, n644, n645, n646, n647, n648, n649, n650, n651, n652, 
      n653, n654, n655, n656, n658, n659, n660, n661, n662, n663, n664, n665, 
      n666, n667, n668, n669, n670, n671, n672, n673, n674, n675, n676, n677, 
      n678, n679, n680, n681, n682, n683, n684, n685, n686, n687, n688, n689, 
      n690, n691, n692, n693, n694, n695, n696, n697, n698, n699, n700, n701, 
      n702, n703, n704, n705, n706, n707, n708, n709, n710, n711, n712, n713, 
      n714, n715, n716, n717, n718, n719, n720, n721, n722, n723, n724, n725, 
      n726, n727, n728, n729, n730, n731, n732, n733, n734, n735, n736, n737, 
      n738, n739, n740, n741, n742, n743, n744, n745, n746, n747, n748, n749, 
      n750, n751, n752, n753, n754, n755, n756, n757, n758, n759, n760, n761, 
      n762, n763, n764, n765, n766, n767, n768, n769, n770, n771, n772, n773, 
      n774, n775, n776, n777, n778, n779, n780, n781, n782, n783, n785, n786, 
      n787, n788, n789, n790, n791, n792, n793, n794, n795, n796, n797, n798, 
      n799, n800, n801, n803, n804, n805, n806, n807, n808, n809, n810, n811, 
      n812, n813, n814, n8150, n8160, n8170, n8180, n8190, n8200, n821, n822, 
      n823, n824, n825, n826, n827, n829, n830, n831, n832, n833, n834, n835, 
      n836, n837, n838, n839, n840, n841, n842, n843, n844, n845, n846, n847, 
      n848, n849, n850, n851, n852, n853, n854, n855, n856, n857, n858, n859, 
      n860, n861, n862, n863, n864, n865, n866, n867, n868, n869, n870, n871, 
      n872, n873, n874, n875, n876, n877, n878, n879, n880, n881, n882, n883, 
      n884, n885, n886, n887, n888, n889, n890, n891, n892, n893, n894, n895, 
      n896, n897, n898, n899, n900, n901, n902, n903, n904, n905, n906, n907, 
      n908, n909, n910, n911, n912, n913, n914, n915, n916, n917, n918, n919, 
      n920, n921, n922, n923, n924, n925, n926, n927, n928, n929, n930, n931, 
      n932, n933, n934, n935, n936, n937, n938, n939, n940, n941, n942, n943, 
      n944, n945, n946, n947, n948, n949, n950, n951, n952, n953, n954, n955, 
      n956, n957, n958, n959, n960, n961, n962, n963, n964, n965, n966, n967, 
      n968, n969, n970, n971, n972, n973, n974, n975, n976, n977, n978, n979, 
      n980, n981, n982, n983, n984, n985, n986, n987, n988, n989, n990, n991, 
      n992, n993, n994, n995, n996, n997, n998, n999, n1000, n1001, n1002, 
      n1003, n1004, n1005, n1006, n1007, n1008, n1009, n1010, n1011, n1012, 
      n1013, n1014, n1015, n1016, n1017, n1018, n1019, n1020, n1021, n1022, 
      n1023, n1024, n1025, n1026, n1027, n1028, n1029, n1030, n1031, n1033, n1,
      n2, n13, n22, n31, n106, n337, n342, n350, n468, n546, n553, n555, n657, 
      n784, n802, n828, n1032, n1034, n1035, n1036, n1037, n1038 : std_logic;

begin
   
   sub_199 : mult_mult_typeDEFAULT_DW01_dec_0 port map( A(5) => 
                           count_reg_5_port, A(4) => count_reg_4_port, A(3) => 
                           count_reg_3_port, A(2) => count_reg_2_port, A(1) => 
                           count_reg_1_port, A(0) => count_reg_0_port, SUM(5) 
                           => N820, SUM(4) => N819, SUM(3) => N818, SUM(2) => 
                           N817, SUM(1) => N816, SUM(0) => N815);
   sign2_reg_reg : DFFRHQXL port map( D => n959, CK => clk, RN => n1038, Q => 
                           sign2_reg);
   sign_reg_reg : DFFRHQXL port map( D => n960, CK => clk, RN => n1038, Q => 
                           sign_reg);
   lower_reg_regx31x : DFFRHQXL port map( D => n926, CK => clk, RN => n1037, Q 
                           => lower_reg_31_port);
   bb_reg_regx0x : DFFRHQXL port map( D => n961, CK => clk, RN => n1038, Q => 
                           bb_reg_0_port);
   bb_reg_regx29x : DFFRHQXL port map( D => n1022, CK => clk, RN => n1036, Q =>
                           bb_reg_29_port);
   bb_reg_regx28x : DFFRHQXL port map( D => n1021, CK => clk, RN => n1038, Q =>
                           bb_reg_28_port);
   bb_reg_regx27x : DFFRHQXL port map( D => n1020, CK => clk, RN => n1037, Q =>
                           bb_reg_27_port);
   bb_reg_regx26x : DFFRHQXL port map( D => n1019, CK => clk, RN => n1036, Q =>
                           bb_reg_26_port);
   bb_reg_regx25x : DFFRHQXL port map( D => n1018, CK => clk, RN => n1038, Q =>
                           bb_reg_25_port);
   bb_reg_regx24x : DFFRHQXL port map( D => n1017, CK => clk, RN => n1037, Q =>
                           bb_reg_24_port);
   bb_reg_regx23x : DFFRHQXL port map( D => n1016, CK => clk, RN => n1036, Q =>
                           bb_reg_23_port);
   bb_reg_regx22x : DFFRHQXL port map( D => n1015, CK => clk, RN => n1038, Q =>
                           bb_reg_22_port);
   bb_reg_regx21x : DFFRHQXL port map( D => n1014, CK => clk, RN => n1037, Q =>
                           bb_reg_21_port);
   bb_reg_regx20x : DFFRHQXL port map( D => n1013, CK => clk, RN => n1036, Q =>
                           bb_reg_20_port);
   bb_reg_regx19x : DFFRHQXL port map( D => n1012, CK => clk, RN => n1038, Q =>
                           bb_reg_19_port);
   bb_reg_regx18x : DFFRHQXL port map( D => n1011, CK => clk, RN => n1038, Q =>
                           bb_reg_18_port);
   bb_reg_regx17x : DFFRHQXL port map( D => n1010, CK => clk, RN => n1037, Q =>
                           bb_reg_17_port);
   bb_reg_regx16x : DFFRHQXL port map( D => n1009, CK => clk, RN => n1036, Q =>
                           bb_reg_16_port);
   bb_reg_regx15x : DFFRHQXL port map( D => n1008, CK => clk, RN => n1036, Q =>
                           bb_reg_15_port);
   bb_reg_regx14x : DFFRHQXL port map( D => n1007, CK => clk, RN => n1036, Q =>
                           bb_reg_14_port);
   bb_reg_regx13x : DFFRHQXL port map( D => n1006, CK => clk, RN => n1038, Q =>
                           bb_reg_13_port);
   bb_reg_regx12x : DFFRHQXL port map( D => n1005, CK => clk, RN => n1036, Q =>
                           bb_reg_12_port);
   bb_reg_regx11x : DFFRHQXL port map( D => n1004, CK => clk, RN => n1038, Q =>
                           bb_reg_11_port);
   bb_reg_regx10x : DFFRHQXL port map( D => n1003, CK => clk, RN => n1038, Q =>
                           bb_reg_10_port);
   bb_reg_regx2x : DFFRHQXL port map( D => n995, CK => clk, RN => n1037, Q => 
                           bb_reg_2_port);
   bb_reg_regx1x : DFFRHQXL port map( D => n994, CK => clk, RN => n1038, Q => 
                           bb_reg_1_port);
   lower_reg_regx22x : DFFRHQXL port map( D => n935, CK => clk, RN => n1036, Q 
                           => lower_reg_22_port);
   lower_reg_regx24x : DFFRHQXL port map( D => n933, CK => clk, RN => n1037, Q 
                           => lower_reg_24_port);
   lower_reg_regx26x : DFFRHQXL port map( D => n931, CK => clk, RN => n1037, Q 
                           => lower_reg_26_port);
   lower_reg_regx28x : DFFRHQXL port map( D => n929, CK => clk, RN => n1037, Q 
                           => lower_reg_28_port);
   lower_reg_regx30x : DFFRHQXL port map( D => n927, CK => clk, RN => n1037, Q 
                           => lower_reg_30_port);
   lower_reg_regx21x : DFFRHQXL port map( D => n936, CK => clk, RN => n1038, Q 
                           => lower_reg_21_port);
   lower_reg_regx23x : DFFRHQXL port map( D => n934, CK => clk, RN => n1036, Q 
                           => lower_reg_23_port);
   lower_reg_regx25x : DFFRHQXL port map( D => n932, CK => clk, RN => n1037, Q 
                           => lower_reg_25_port);
   lower_reg_regx27x : DFFRHQXL port map( D => n930, CK => clk, RN => n1037, Q 
                           => lower_reg_27_port);
   lower_reg_regx29x : DFFRHQXL port map( D => n928, CK => clk, RN => n1037, Q 
                           => lower_reg_29_port);
   bb_reg_regx31x : DFFRHQXL port map( D => n1024, CK => clk, RN => n1037, Q =>
                           bb_reg_31_port);
   bb_reg_regx30x : DFFRHQXL port map( D => n1023, CK => clk, RN => n1036, Q =>
                           bb_reg_30_port);
   bb_reg_regx7x : DFFRHQXL port map( D => n1000, CK => clk, RN => n1037, Q => 
                           bb_reg_7_port);
   bb_reg_regx6x : DFFRHQXL port map( D => n999, CK => clk, RN => n1037, Q => 
                           bb_reg_6_port);
   bb_reg_regx5x : DFFRHQXL port map( D => n998, CK => clk, RN => n1037, Q => 
                           bb_reg_5_port);
   bb_reg_regx4x : DFFRHQXL port map( D => n997, CK => clk, RN => n1037, Q => 
                           bb_reg_4_port);
   bb_reg_regx3x : DFFRHQXL port map( D => n996, CK => clk, RN => n1036, Q => 
                           bb_reg_3_port);
   count_reg_regx5x : DFFRHQXL port map( D => n1025, CK => clk, RN => n1038, Q 
                           => count_reg_5_port);
   negate_reg_reg : DFFRHQXL port map( D => n1031, CK => clk, RN => n1037, Q =>
                           negate_reg);
   bb_reg_regx8x : DFFRHQXL port map( D => n1001, CK => clk, RN => n1036, Q => 
                           bb_reg_8_port);
   count_reg_regx3x : DFFRHQXL port map( D => n1026, CK => clk, RN => n1038, Q 
                           => count_reg_3_port);
   upper_reg_regx30x : DFFRHQXL port map( D => n924, CK => clk, RN => n1037, Q 
                           => upper_reg_30_port);
   upper_reg_regx29x : DFFRHQXL port map( D => n923, CK => clk, RN => n1037, Q 
                           => upper_reg_29_port);
   bb_reg_regx9x : DFFRHQXL port map( D => n1002, CK => clk, RN => n1037, Q => 
                           bb_reg_9_port);
   count_reg_regx1x : DFFRHQXL port map( D => n1028, CK => clk, RN => n1037, Q 
                           => count_reg_1_port);
   count_reg_regx2x : DFFRHQXL port map( D => n1027, CK => clk, RN => n1037, Q 
                           => count_reg_2_port);
   count_reg_regx4x : DFFRHQXL port map( D => n1029, CK => clk, RN => n1038, Q 
                           => count_reg_4_port);
   upper_reg_regx31x : DFFRHQXL port map( D => n925, CK => clk, RN => n1037, Q 
                           => upper_reg_31_port);
   count_reg_regx0x : DFFRHQXL port map( D => n1030, CK => clk, RN => n1036, Q 
                           => count_reg_0_port);
   aa_reg_regx31x : DFFRHQXL port map( D => n993, CK => clk, RN => n1036, Q => 
                           aa_reg_31_port);
   lower_reg_regx14x : DFFRHQXL port map( D => n943, CK => clk, RN => n1036, Q 
                           => lower_reg_14_port);
   lower_reg_regx16x : DFFRHQXL port map( D => n941, CK => clk, RN => n1036, Q 
                           => lower_reg_16_port);
   lower_reg_regx18x : DFFRHQXL port map( D => n939, CK => clk, RN => n1038, Q 
                           => lower_reg_18_port);
   lower_reg_regx20x : DFFRHQXL port map( D => n937, CK => clk, RN => n1037, Q 
                           => lower_reg_20_port);
   lower_reg_regx15x : DFFRHQXL port map( D => n942, CK => clk, RN => n1036, Q 
                           => lower_reg_15_port);
   lower_reg_regx17x : DFFRHQXL port map( D => n940, CK => clk, RN => n1036, Q 
                           => lower_reg_17_port);
   lower_reg_regx19x : DFFRHQXL port map( D => n938, CK => clk, RN => n1036, Q 
                           => lower_reg_19_port);
   upper_reg_regx28x : DFFRHQXL port map( D => n922, CK => clk, RN => n1037, Q 
                           => upper_reg_28_port);
   upper_reg_regx27x : DFFRHQXL port map( D => n921, CK => clk, RN => n1037, Q 
                           => upper_reg_27_port);
   lower_reg_regx12x : DFFRHQXL port map( D => n945, CK => clk, RN => n1038, Q 
                           => lower_reg_12_port);
   lower_reg_regx9x : DFFRHQXL port map( D => n948, CK => clk, RN => n1038, Q 
                           => lower_reg_9_port);
   lower_reg_regx10x : DFFRHQXL port map( D => n947, CK => clk, RN => n1038, Q 
                           => lower_reg_10_port);
   lower_reg_regx11x : DFFRHQXL port map( D => n946, CK => clk, RN => n1037, Q 
                           => lower_reg_11_port);
   lower_reg_regx13x : DFFRHQXL port map( D => n944, CK => clk, RN => n1036, Q 
                           => lower_reg_13_port);
   aa_reg_regx28x : DFFRHQXL port map( D => n990, CK => clk, RN => n1037, Q => 
                           aa_reg_28_port);
   aa_reg_regx29x : DFFRHQXL port map( D => n991, CK => clk, RN => n1038, Q => 
                           aa_reg_29_port);
   aa_reg_regx30x : DFFRHQXL port map( D => n992, CK => clk, RN => n1037, Q => 
                           aa_reg_30_port);
   upper_reg_regx26x : DFFRHQXL port map( D => n920, CK => clk, RN => n1036, Q 
                           => upper_reg_26_port);
   upper_reg_regx25x : DFFRHQXL port map( D => n919, CK => clk, RN => n1036, Q 
                           => upper_reg_25_port);
   upper_reg_regx24x : DFFRHQXL port map( D => n918, CK => clk, RN => n1036, Q 
                           => upper_reg_24_port);
   lower_reg_regx4x : DFFRHQXL port map( D => n953, CK => clk, RN => n1038, Q 
                           => lower_reg_4_port);
   lower_reg_regx5x : DFFRHQXL port map( D => n952, CK => clk, RN => n1038, Q 
                           => lower_reg_5_port);
   lower_reg_regx6x : DFFRHQXL port map( D => n951, CK => clk, RN => n1038, Q 
                           => lower_reg_6_port);
   lower_reg_regx7x : DFFRHQXL port map( D => n950, CK => clk, RN => n1038, Q 
                           => lower_reg_7_port);
   lower_reg_regx8x : DFFRHQXL port map( D => n949, CK => clk, RN => n1038, Q 
                           => lower_reg_8_port);
   aa_reg_regx25x : DFFRHQXL port map( D => n987, CK => clk, RN => n1036, Q => 
                           aa_reg_25_port);
   aa_reg_regx26x : DFFRHQXL port map( D => n988, CK => clk, RN => n1038, Q => 
                           aa_reg_26_port);
   aa_reg_regx27x : DFFRHQXL port map( D => n989, CK => clk, RN => n1038, Q => 
                           aa_reg_27_port);
   upper_reg_regx23x : DFFRHQXL port map( D => n917, CK => clk, RN => n1036, Q 
                           => upper_reg_23_port);
   upper_reg_regx22x : DFFRHQXL port map( D => n916, CK => clk, RN => n1036, Q 
                           => upper_reg_22_port);
   lower_reg_regx1x : DFFRHQXL port map( D => n956, CK => clk, RN => n1038, Q 
                           => lower_reg_1_port);
   lower_reg_regx2x : DFFRHQXL port map( D => n955, CK => clk, RN => n1038, Q 
                           => lower_reg_2_port);
   lower_reg_regx3x : DFFRHQXL port map( D => n954, CK => clk, RN => n1038, Q 
                           => lower_reg_3_port);
   lower_reg_regx0x : DFFRHQXL port map( D => n957, CK => clk, RN => n1038, Q 
                           => lower_reg_0_port);
   aa_reg_regx24x : DFFRHQXL port map( D => n986, CK => clk, RN => n1038, Q => 
                           aa_reg_24_port);
   aa_reg_regx23x : DFFRHQXL port map( D => n985, CK => clk, RN => n1038, Q => 
                           aa_reg_23_port);
   upper_reg_regx21x : DFFRHQXL port map( D => n915, CK => clk, RN => n1036, Q 
                           => upper_reg_21_port);
   upper_reg_regx20x : DFFRHQXL port map( D => n914, CK => clk, RN => n1036, Q 
                           => upper_reg_20_port);
   upper_reg_regx19x : DFFRHQXL port map( D => n913, CK => clk, RN => n1036, Q 
                           => upper_reg_19_port);
   aa_reg_regx20x : DFFRHQXL port map( D => n982, CK => clk, RN => n1037, Q => 
                           aa_reg_20_port);
   aa_reg_regx21x : DFFRHQXL port map( D => n983, CK => clk, RN => n1037, Q => 
                           aa_reg_21_port);
   aa_reg_regx22x : DFFRHQXL port map( D => n984, CK => clk, RN => n1036, Q => 
                           aa_reg_22_port);
   upper_reg_regx18x : DFFRHQXL port map( D => n912, CK => clk, RN => n1036, Q 
                           => upper_reg_18_port);
   upper_reg_regx17x : DFFRHQXL port map( D => n911, CK => clk, RN => n1036, Q 
                           => upper_reg_17_port);
   upper_reg_regx16x : DFFRHQXL port map( D => n910, CK => clk, RN => n1036, Q 
                           => upper_reg_16_port);
   aa_reg_regx17x : DFFRHQXL port map( D => n979, CK => clk, RN => n1036, Q => 
                           aa_reg_17_port);
   aa_reg_regx18x : DFFRHQXL port map( D => n980, CK => clk, RN => n1036, Q => 
                           aa_reg_18_port);
   aa_reg_regx19x : DFFRHQXL port map( D => n981, CK => clk, RN => n1038, Q => 
                           aa_reg_19_port);
   upper_reg_regx15x : DFFRHQXL port map( D => n909, CK => clk, RN => n1036, Q 
                           => upper_reg_15_port);
   upper_reg_regx14x : DFFRHQXL port map( D => n908, CK => clk, RN => n1036, Q 
                           => upper_reg_14_port);
   aa_reg_regx15x : DFFRHQXL port map( D => n977, CK => clk, RN => n1038, Q => 
                           aa_reg_15_port);
   aa_reg_regx16x : DFFRHQXL port map( D => n978, CK => clk, RN => n1037, Q => 
                           aa_reg_16_port);
   upper_reg_regx13x : DFFRHQXL port map( D => n907, CK => clk, RN => n1038, Q 
                           => upper_reg_13_port);
   upper_reg_regx12x : DFFRHQXL port map( D => n906, CK => clk, RN => n1037, Q 
                           => upper_reg_12_port);
   upper_reg_regx11x : DFFRHQXL port map( D => n905, CK => clk, RN => n1038, Q 
                           => upper_reg_11_port);
   aa_reg_regx13x : DFFRHQXL port map( D => n975, CK => clk, RN => n1037, Q => 
                           aa_reg_13_port);
   aa_reg_regx14x : DFFRHQXL port map( D => n976, CK => clk, RN => n1038, Q => 
                           aa_reg_14_port);
   aa_reg_regx12x : DFFRHQXL port map( D => n974, CK => clk, RN => n1036, Q => 
                           aa_reg_12_port);
   upper_reg_regx9x : DFFRHQXL port map( D => n903, CK => clk, RN => n1036, Q 
                           => upper_reg_9_port);
   upper_reg_regx10x : DFFRHQXL port map( D => n904, CK => clk, RN => n1037, Q 
                           => upper_reg_10_port);
   upper_reg_regx8x : DFFRHQXL port map( D => n902, CK => clk, RN => n1036, Q 
                           => upper_reg_8_port);
   aa_reg_regx10x : DFFRHQXL port map( D => n972, CK => clk, RN => n1038, Q => 
                           aa_reg_10_port);
   aa_reg_regx11x : DFFRHQXL port map( D => n973, CK => clk, RN => n1038, Q => 
                           aa_reg_11_port);
   aa_reg_regx9x : DFFRHQXL port map( D => n971, CK => clk, RN => n1037, Q => 
                           aa_reg_9_port);
   upper_reg_regx7x : DFFRHQXL port map( D => n901, CK => clk, RN => n1038, Q 
                           => upper_reg_7_port);
   upper_reg_regx6x : DFFRHQXL port map( D => n900, CK => clk, RN => n1038, Q 
                           => upper_reg_6_port);
   upper_reg_regx5x : DFFRHQXL port map( D => n899, CK => clk, RN => n1036, Q 
                           => upper_reg_5_port);
   aa_reg_regx7x : DFFRHQXL port map( D => n969, CK => clk, RN => n1037, Q => 
                           aa_reg_7_port);
   aa_reg_regx8x : DFFRHQXL port map( D => n970, CK => clk, RN => n1037, Q => 
                           aa_reg_8_port);
   upper_reg_regx4x : DFFRHQXL port map( D => n898, CK => clk, RN => n1038, Q 
                           => upper_reg_4_port);
   upper_reg_regx3x : DFFRHQXL port map( D => n897, CK => clk, RN => n1037, Q 
                           => upper_reg_3_port);
   aa_reg_regx6x : DFFRHQXL port map( D => n968, CK => clk, RN => n1036, Q => 
                           aa_reg_6_port);
   aa_reg_regx4x : DFFRHQXL port map( D => n966, CK => clk, RN => n1037, Q => 
                           aa_reg_4_port);
   aa_reg_regx5x : DFFRHQXL port map( D => n967, CK => clk, RN => n1036, Q => 
                           aa_reg_5_port);
   upper_reg_regx2x : DFFRHQXL port map( D => n896, CK => clk, RN => n1037, Q 
                           => upper_reg_2_port);
   upper_reg_regx1x : DFFRHQXL port map( D => n895, CK => clk, RN => n1037, Q 
                           => upper_reg_1_port);
   aa_reg_regx1x : DFFRHQXL port map( D => n963, CK => clk, RN => n1037, Q => 
                           aa_reg_1_port);
   aa_reg_regx3x : DFFRHQXL port map( D => n965, CK => clk, RN => n1037, Q => 
                           aa_reg_3_port);
   aa_reg_regx2x : DFFRHQXL port map( D => n964, CK => clk, RN => n1036, Q => 
                           aa_reg_2_port);
   upper_reg_regx0x : DFFRHQXL port map( D => n958, CK => clk, RN => n1038, Q 
                           => upper_reg_0_port);
   aa_reg_regx0x : DFFRHQXL port map( D => n962, CK => clk, RN => n1036, Q => 
                           aa_reg_0_port);
   mode_reg_reg : DFFRHQXL port map( D => n1033, CK => clk, RN => n1038, Q => 
                           mode_reg);
   U3 : NOR2XL port map( A => n542, B => n557, Y => n1);
   U4 : NAND3XL port map( A => n319, B => n318, C => n548, Y => n2);
   U5 : NAND2XL port map( A => n413, B => n802, Y => n13);
   U6 : NAND3XL port map( A => n420, B => n548, C => b(31), Y => n22);
   U7 : NOR2XL port map( A => n793, B => n3, Y => n800);
   U8 : OR4XL port map( A => aa_reg_0_port, B => aa_reg_10_port, C => 
                           aa_reg_11_port, D => aa_reg_12_port, Y => n531);
   U9 : OR4XL port map( A => aa_reg_24_port, B => aa_reg_25_port, C => 
                           aa_reg_26_port, D => aa_reg_27_port, Y => n527);
   U10 : OR4XL port map( A => aa_reg_28_port, B => aa_reg_29_port, C => 
                           aa_reg_2_port, D => aa_reg_30_port, Y => n526);
   U11 : OR4XL port map( A => aa_reg_31_port, B => aa_reg_3_port, C => 
                           aa_reg_4_port, D => aa_reg_5_port, Y => n525);
   U12 : OR4XL port map( A => aa_reg_6_port, B => aa_reg_7_port, C => 
                           aa_reg_8_port, D => aa_reg_9_port, Y => n524);
   U13 : NOR4XL port map( A => n528, B => n529, C => n530, D => n531, Y => n460
                           );
   U14 : OR4XL port map( A => aa_reg_20_port, B => aa_reg_21_port, C => 
                           aa_reg_22_port, D => aa_reg_23_port, Y => n528);
   U15 : OR4XL port map( A => aa_reg_17_port, B => aa_reg_18_port, C => 
                           aa_reg_19_port, D => aa_reg_1_port, Y => n529);
   U16 : OR4XL port map( A => aa_reg_13_port, B => aa_reg_14_port, C => 
                           aa_reg_15_port, D => aa_reg_16_port, Y => n530);
   U17 : NOR2XL port map( A => n344, B => n1032, Y => n19);
   U18 : OAI2BB1XL port map( A0N => n321, A1N => n466, B0 => upper_reg_30_port,
                           Y => n467);
   U19 : INVXL port map( A => n322, Y => n466);
   U20 : NAND2XL port map( A => n835, B => n352, Y => n829);
   U21 : NOR2XL port map( A => n838, B => lower_reg_27_port, Y => n835);
   U22 : NAND4XL port map( A => mult_func(0), B => n652, C => n319, D => n318, 
                           Y => n3);
   U23 : OR2XL port map( A => n811, B => lower_reg_6_port, Y => n808);
   U24 : OR2XL port map( A => n808, B => lower_reg_7_port, Y => n805);
   U25 : OR2XL port map( A => n831, B => lower_reg_2_port, Y => n8200);
   U26 : OR2XL port map( A => n805, B => lower_reg_8_port, Y => n799);
   U27 : OR2XL port map( A => n8170, B => lower_reg_4_port, Y => n814);
   U28 : OR2XL port map( A => n8200, B => lower_reg_3_port, Y => n8170);
   U29 : NAND2XL port map( A => n406, B => n408, Y => n831);
   U30 : OR2XL port map( A => n799, B => lower_reg_9_port, Y => n891);
   U31 : OR2XL port map( A => n814, B => lower_reg_5_port, Y => n811);
   U32 : NAND2XL port map( A => n879, B => n380, Y => n876);
   U33 : OR2XL port map( A => n891, B => lower_reg_10_port, Y => n888);
   U34 : NOR2XL port map( A => n876, B => lower_reg_15_port, Y => n873);
   U35 : NOR2XL port map( A => n888, B => lower_reg_11_port, Y => n885);
   U36 : NAND2XL port map( A => n873, B => n376, Y => n870);
   U37 : NAND2XL port map( A => n885, B => n384, Y => n882);
   U38 : NOR2XL port map( A => n882, B => lower_reg_13_port, Y => n879);
   U39 : NAND2XL port map( A => n853, B => n364, Y => n850);
   U40 : NAND2XL port map( A => n867, B => n372, Y => n862);
   U41 : NOR2XL port map( A => n850, B => lower_reg_23_port, Y => n847);
   U42 : NOR2XL port map( A => n862, B => lower_reg_19_port, Y => n859);
   U43 : NAND2XL port map( A => n859, B => n368, Y => n856);
   U44 : NOR2XL port map( A => n856, B => lower_reg_21_port, Y => n853);
   U45 : NOR2XL port map( A => n870, B => lower_reg_17_port, Y => n867);
   U46 : NAND2XL port map( A => n841, B => n356, Y => n838);
   U47 : NOR2XL port map( A => n829, B => lower_reg_29_port, Y => n825);
   U48 : NAND2XL port map( A => n847, B => n360, Y => n844);
   U49 : INVXL port map( A => n31, Y => n803);
   U50 : INVXL port map( A => n800, Y => n468);
   U51 : NOR2XL port map( A => n844, B => lower_reg_25_port, Y => n841);
   U52 : NOR2XL port map( A => n3, B => negate_reg, Y => n31);
   U53 : NOR2XL port map( A => a(1), B => a(0), Y => n564);
   U54 : AOI22XL port map( A0 => aa_reg_0_port, A1 => upper_reg_0_port, B0 => 
                           n657, B1 => n424, Y => n422);
   U55 : XOR2XL port map( A => aa_reg_1_port, B => mode_reg, Y => n423);
   U56 : OAI2BB1XL port map( A0N => n423, A1N => n422, B0 => upper_reg_1_port, 
                           Y => n523);
   U57 : XNOR2XL port map( A => aa_reg_2_port, B => n657, Y => n32);
   U58 : NOR3XL port map( A => a(3), B => a(4), C => n567, Y => n573);
   U59 : NAND2XL port map( A => n564, B => n562, Y => n567);
   U60 : NAND2XL port map( A => n573, B => n571, Y => n576);
   U61 : OAI2BB1XL port map( A0N => n32, A1N => n521, B0 => upper_reg_2_port, Y
                           => n522);
   U62 : INVXL port map( A => n33, Y => n521);
   U63 : XNOR2XL port map( A => aa_reg_3_port, B => n657, Y => n42);
   U64 : OAI2BB1XL port map( A0N => n42, A1N => n519, B0 => upper_reg_3_port, Y
                           => n520);
   U65 : INVXL port map( A => n43, Y => n519);
   U66 : XNOR2XL port map( A => aa_reg_4_port, B => n657, Y => n52);
   U67 : NOR2XL port map( A => n576, B => a(6), Y => n579);
   U68 : OAI2BB1XL port map( A0N => n52, A1N => n517, B0 => upper_reg_4_port, Y
                           => n518);
   U69 : INVXL port map( A => n53, Y => n517);
   U70 : XNOR2XL port map( A => aa_reg_5_port, B => n657, Y => n62);
   U71 : OAI2BB1XL port map( A0N => n62, A1N => n515, B0 => upper_reg_5_port, Y
                           => n516);
   U72 : INVXL port map( A => n63, Y => n515);
   U73 : XNOR2XL port map( A => aa_reg_6_port, B => n657, Y => n72);
   U74 : OAI2BB1XL port map( A0N => n72, A1N => n513, B0 => upper_reg_6_port, Y
                           => n514);
   U75 : INVXL port map( A => n73, Y => n513);
   U76 : XNOR2XL port map( A => aa_reg_7_port, B => n657, Y => n82);
   U77 : NAND3XL port map( A => n577, B => n580, C => n579, Y => n585);
   U78 : NOR2XL port map( A => n585, B => a(9), Y => n588);
   U79 : OAI2BB1XL port map( A0N => n82, A1N => n511, B0 => upper_reg_7_port, Y
                           => n512);
   U80 : INVXL port map( A => n83, Y => n511);
   U81 : XNOR2XL port map( A => aa_reg_8_port, B => n657, Y => n92);
   U82 : XOR2XL port map( A => aa_reg_9_port, B => n657, Y => n103);
   U83 : OAI2BB1XL port map( A0N => n92, A1N => n509, B0 => upper_reg_8_port, Y
                           => n510);
   U84 : INVXL port map( A => n93, Y => n509);
   U85 : AOI21XL port map( A0 => n104, A1 => n103, B0 => n508, Y => n116);
   U86 : AOI2BB1XL port map( A0N => n103, A1N => n104, B0 => n91, Y => n508);
   U87 : XNOR2XL port map( A => aa_reg_10_port, B => n657, Y => n115);
   U88 : NAND2XL port map( A => n588, B => n586, Y => n591);
   U89 : NAND2XL port map( A => n597, B => n595, Y => n600);
   U90 : NOR3XL port map( A => a(11), B => a(12), C => n591, Y => n597);
   U91 : OAI2BB1XL port map( A0N => n115, A1N => n116, B0 => upper_reg_10_port,
                           Y => n507);
   U92 : XNOR2XL port map( A => aa_reg_11_port, B => n657, Y => n125);
   U93 : OAI2BB1XL port map( A0N => n125, A1N => n505, B0 => upper_reg_11_port,
                           Y => n506);
   U94 : INVXL port map( A => n126, Y => n505);
   U95 : XNOR2XL port map( A => aa_reg_12_port, B => n657, Y => n135);
   U96 : OAI2BB1XL port map( A0N => n135, A1N => n503, B0 => upper_reg_12_port,
                           Y => n504);
   U97 : INVXL port map( A => n136, Y => n503);
   U98 : XNOR2XL port map( A => aa_reg_13_port, B => n657, Y => n145);
   U99 : NOR2XL port map( A => n600, B => a(14), Y => n603);
   U100 : OAI2BB1XL port map( A0N => n145, A1N => n501, B0 => upper_reg_13_port
                           , Y => n502);
   U101 : INVXL port map( A => n146, Y => n501);
   U102 : XNOR2XL port map( A => aa_reg_14_port, B => n657, Y => n155);
   U103 : OAI2BB1XL port map( A0N => n155, A1N => n499, B0 => upper_reg_14_port
                           , Y => n500);
   U104 : INVXL port map( A => n156, Y => n499);
   U105 : XNOR2XL port map( A => aa_reg_15_port, B => n657, Y => n165);
   U106 : NOR2XL port map( A => n609, B => a(17), Y => n612);
   U107 : NAND3XL port map( A => n601, B => n604, C => n603, Y => n609);
   U108 : OAI2BB1XL port map( A0N => n165, A1N => n497, B0 => upper_reg_15_port
                           , Y => n498);
   U109 : INVXL port map( A => n166, Y => n497);
   U110 : XNOR2XL port map( A => aa_reg_16_port, B => n657, Y => n175);
   U111 : OAI2BB1XL port map( A0N => n175, A1N => n495, B0 => upper_reg_16_port
                           , Y => n496);
   U112 : INVXL port map( A => n176, Y => n495);
   U113 : XNOR2XL port map( A => aa_reg_17_port, B => n657, Y => n185);
   U114 : OAI2BB1XL port map( A0N => n185, A1N => n493, B0 => upper_reg_17_port
                           , Y => n494);
   U115 : INVXL port map( A => n186, Y => n493);
   U116 : XNOR2XL port map( A => aa_reg_18_port, B => n657, Y => n195);
   U117 : NAND2XL port map( A => n621, B => n619, Y => n624);
   U118 : NOR3XL port map( A => a(19), B => a(20), C => n615, Y => n621);
   U119 : NAND2XL port map( A => n612, B => n610, Y => n615);
   U120 : OAI2BB1XL port map( A0N => n195, A1N => n491, B0 => upper_reg_18_port
                           , Y => n492);
   U121 : INVXL port map( A => n196, Y => n491);
   U122 : XNOR2XL port map( A => aa_reg_19_port, B => n657, Y => n205);
   U123 : OAI2BB1XL port map( A0N => n205, A1N => n489, B0 => upper_reg_19_port
                           , Y => n490);
   U124 : INVXL port map( A => n206, Y => n489);
   U125 : XNOR2XL port map( A => aa_reg_20_port, B => n657, Y => n215);
   U126 : OAI2BB1XL port map( A0N => n215, A1N => n487, B0 => upper_reg_20_port
                           , Y => n488);
   U127 : INVXL port map( A => n216, Y => n487);
   U128 : XNOR2XL port map( A => aa_reg_21_port, B => n657, Y => n225);
   U129 : NOR2XL port map( A => n624, B => a(22), Y => n627);
   U130 : NAND4XL port map( A => n440, B => n441, C => n442, D => n443, Y => 
                           n434);
   U131 : NOR2XL port map( A => bb_reg_9_port, B => bb_reg_8_port, Y => n442);
   U132 : NAND4XL port map( A => n436, B => n437, C => n438, D => n439, Y => 
                           n435);
   U133 : NAND4XL port map( A => n444, B => n445, C => n446, D => n447, Y => 
                           n433);
   U134 : NAND4XL port map( A => n448, B => n449, C => n450, D => n451, Y => 
                           n432);
   U135 : NAND4XL port map( A => n452, B => n453, C => n454, D => n455, Y => 
                           n430);
   U136 : AND4X1 port map( A => n456, B => n457, C => n458, D => n459, Y => 
                           n455);
   U137 : AOI21XL port map( A0 => n460, A1 => n461, B0 => n332, Y => n454);
   U138 : NOR4XL port map( A => n524, B => n525, C => n526, D => n527, Y => 
                           n461);
   U139 : OAI2BB1XL port map( A0N => n225, A1N => n485, B0 => upper_reg_21_port
                           , Y => n486);
   U140 : INVXL port map( A => n226, Y => n485);
   U141 : XNOR2XL port map( A => aa_reg_22_port, B => n657, Y => n235);
   U142 : OAI2BB1XL port map( A0N => n235, A1N => n483, B0 => upper_reg_22_port
                           , Y => n484);
   U143 : INVXL port map( A => n236, Y => n483);
   U144 : XNOR2XL port map( A => aa_reg_23_port, B => n657, Y => n245);
   U145 : NAND2XL port map( A => n636, B => n634, Y => n639);
   U146 : NOR2XL port map( A => n633, B => a(25), Y => n636);
   U147 : NAND3XL port map( A => n625, B => n628, C => n627, Y => n633);
   U148 : OAI2BB1XL port map( A0N => n245, A1N => n481, B0 => upper_reg_23_port
                           , Y => n482);
   U149 : INVXL port map( A => n246, Y => n481);
   U150 : XNOR2XL port map( A => aa_reg_24_port, B => n657, Y => n255);
   U151 : OAI2BB1XL port map( A0N => n255, A1N => n479, B0 => upper_reg_24_port
                           , Y => n480);
   U152 : INVXL port map( A => n256, Y => n479);
   U153 : XNOR2XL port map( A => aa_reg_25_port, B => n657, Y => n265);
   U154 : OAI2BB1XL port map( A0N => n265, A1N => n477, B0 => upper_reg_25_port
                           , Y => n478);
   U155 : INVXL port map( A => n266, Y => n477);
   U156 : XNOR2XL port map( A => aa_reg_26_port, B => n657, Y => n275);
   U157 : NAND2XL port map( A => n642, B => n640, Y => n645);
   U158 : NOR2XL port map( A => n639, B => a(27), Y => n642);
   U159 : OAI2BB1XL port map( A0N => n275, A1N => n475, B0 => upper_reg_26_port
                           , Y => n476);
   U160 : INVXL port map( A => n276, Y => n475);
   U161 : XNOR2XL port map( A => aa_reg_27_port, B => n657, Y => n285);
   U162 : OAI2BB1XL port map( A0N => n285, A1N => n473, B0 => upper_reg_27_port
                           , Y => n474);
   U163 : INVXL port map( A => n286, Y => n473);
   U164 : XNOR2XL port map( A => aa_reg_28_port, B => n657, Y => n295);
   U165 : AOI21XL port map( A0 => n427, A1 => n21, B0 => n654, Y => n557);
   U166 : XNOR2XL port map( A => mode_reg, B => n462, Y => n332);
   U167 : AOI22XL port map( A0 => n463, A1 => n320, B0 => n334, B1 => n464, Y 
                           => n462);
   U168 : INVXL port map( A => n333, Y => n464);
   U169 : NAND2XL port map( A => n333, B => n465, Y => n463);
   U170 : INVXL port map( A => n4, Y => n427);
   U171 : NAND3XL port map( A => n101, B => n421, C => n413, Y => n15);
   U172 : OAI2BB1XL port map( A0N => n295, A1N => n471, B0 => upper_reg_28_port
                           , Y => n472);
   U173 : INVXL port map( A => n296, Y => n471);
   U174 : XNOR2XL port map( A => aa_reg_29_port, B => n657, Y => n305);
   U175 : INVXL port map( A => n19, Y => n1034);
   U176 : AOI21XL port map( A0 => n652, A1 => mult_func(0), B0 => n548, Y => 
                           n315);
   U177 : XOR2XL port map( A => aa_reg_31_port, B => n657, Y => n333);
   U178 : INVXL port map( A => n465, Y => n334);
   U179 : OAI2BB1XL port map( A0N => n305, A1N => n469, B0 => upper_reg_29_port
                           , Y => n470);
   U180 : INVXL port map( A => n306, Y => n469);
   U181 : XNOR2XL port map( A => aa_reg_30_port, B => n657, Y => n321);
   U182 : INVXL port map( A => mult_func(3), Y => n318);
   U183 : NOR2XL port map( A => n542, B => n4, Y => n414);
   U184 : NAND2XL port map( A => n544, B => n553, Y => n551);
   U185 : NAND3XL port map( A => b(31), B => n316, C => n781, Y => n659);
   U186 : AOI21XL port map( A0 => n652, A1 => n318, B0 => mult_func(0), Y => 
                           n781);
   U187 : INVXL port map( A => mult_func(0), Y => n797);
   U188 : NAND2XL port map( A => n797, B => n652, Y => n796);
   U189 : AOI22XL port map( A0 => mult_func(2), A1 => mult_func(3), B0 => n319,
                           B1 => mult_func(1), Y => n316);
   U190 : NOR2XL port map( A => n414, B => n784, Y => n345);
   U191 : INVXL port map( A => n345, Y => n802);
   U192 : NOR2XL port map( A => n657, B => n542, Y => n413);
   U193 : INVXL port map( A => mult_func(1), Y => n652);
   U194 : INVXL port map( A => mult_func(2), Y => n319);
   U195 : NOR2XL port map( A => n645, B => a(29), Y => n648);
   U196 : NOR3XL port map( A => n797, B => mult_func(3), C => n652, Y => n791);
   U197 : NOR4BXL port map( AN => n790, B => count_reg_1_port, C => 
                           count_reg_2_port, D => count_reg_0_port, Y => n4);
   U198 : NOR3XL port map( A => count_reg_3_port, B => count_reg_5_port, C => 
                           count_reg_4_port, Y => n790);
   U199 : AOI32XL port map( A0 => n823, A1 => n338, A2 => n800, B0 => 
                           lower_reg_31_port, B1 => n824, Y => n822);
   U200 : NAND2XL port map( A => n825, B => n351, Y => n823);
   U201 : AOI32XL port map( A0 => n829, A1 => n348, A2 => n800, B0 => 
                           lower_reg_29_port, B1 => n834, Y => n833);
   U202 : AOI21XL port map( A0 => n835, A1 => n800, B0 => n31, Y => n836);
   U203 : AOI32XL port map( A0 => lower_reg_0_port, A1 => n406, A2 => n800, B0 
                           => lower_reg_1_port, B1 => n864, Y => n863);
   U204 : AOI21XL port map( A0 => n2, A1 => n3, B0 => n4, Y => pause_out);
   U205 : AOI32XL port map( A0 => n808, A1 => n394, A2 => n800, B0 => 
                           lower_reg_7_port, B1 => n809, Y => n807);
   U206 : AOI32XL port map( A0 => n805, A1 => n392, A2 => n800, B0 => 
                           lower_reg_8_port, B1 => n806, Y => n804);
   U207 : AOI32XL port map( A0 => n8200, A1 => n402, A2 => n800, B0 => 
                           lower_reg_3_port, B1 => n821, Y => n8190);
   U208 : AOI32XL port map( A0 => n799, A1 => n390, A2 => n800, B0 => 
                           lower_reg_9_port, B1 => n801, Y => n798);
   U209 : AOI32XL port map( A0 => n814, A1 => n398, A2 => n800, B0 => 
                           lower_reg_5_port, B1 => n8150, Y => n813);
   U210 : AOI32XL port map( A0 => n8170, A1 => n400, A2 => n800, B0 => 
                           lower_reg_4_port, B1 => n8180, Y => n8160);
   U211 : AOI32XL port map( A0 => n831, A1 => n404, A2 => n800, B0 => 
                           lower_reg_2_port, B1 => n832, Y => n830);
   U212 : AOI32XL port map( A0 => n891, A1 => n388, A2 => n800, B0 => 
                           lower_reg_10_port, B1 => n893, Y => n892);
   U213 : AOI32XL port map( A0 => n811, A1 => n396, A2 => n800, B0 => 
                           lower_reg_6_port, B1 => n812, Y => n810);
   U214 : AOI32XL port map( A0 => n876, A1 => n378, A2 => n800, B0 => 
                           lower_reg_15_port, B1 => n878, Y => n877);
   U215 : AOI32XL port map( A0 => n888, A1 => n386, A2 => n800, B0 => 
                           lower_reg_11_port, B1 => n890, Y => n889);
   U216 : AOI21XL port map( A0 => n873, A1 => n800, B0 => n31, Y => n874);
   U217 : AOI21XL port map( A0 => n885, A1 => n800, B0 => n31, Y => n886);
   U218 : AOI32XL port map( A0 => n870, A1 => n374, A2 => n800, B0 => 
                           lower_reg_17_port, B1 => n872, Y => n871);
   U219 : AOI32XL port map( A0 => n882, A1 => n382, A2 => n800, B0 => 
                           lower_reg_13_port, B1 => n884, Y => n883);
   U220 : AOI21XL port map( A0 => n879, A1 => n800, B0 => n31, Y => n880);
   U221 : AOI32XL port map( A0 => n850, A1 => n362, A2 => n800, B0 => 
                           lower_reg_23_port, B1 => n852, Y => n851);
   U222 : AOI32XL port map( A0 => n862, A1 => n370, A2 => n800, B0 => 
                           lower_reg_19_port, B1 => n866, Y => n865);
   U223 : AOI21XL port map( A0 => n847, A1 => n800, B0 => n31, Y => n848);
   U224 : AOI21XL port map( A0 => n859, A1 => n800, B0 => n31, Y => n860);
   U225 : AOI32XL port map( A0 => n856, A1 => n366, A2 => n800, B0 => 
                           lower_reg_21_port, B1 => n858, Y => n857);
   U226 : AOI21XL port map( A0 => n853, A1 => n800, B0 => n31, Y => n854);
   U227 : AOI21XL port map( A0 => n867, A1 => n800, B0 => n31, Y => n868);
   U228 : AOI32XL port map( A0 => n838, A1 => n354, A2 => n800, B0 => 
                           lower_reg_27_port, B1 => n840, Y => n839);
   U229 : NAND3BXL port map( AN => n825, B => n351, C => n800, Y => n827);
   U230 : AOI21XL port map( A0 => n825, A1 => n800, B0 => n31, Y => n826);
   U231 : AOI32XL port map( A0 => n844, A1 => n358, A2 => n800, B0 => 
                           lower_reg_25_port, B1 => n846, Y => n845);
   U232 : AOI21XL port map( A0 => n841, A1 => n800, B0 => n31, Y => n842);
   U233 : INVXL port map( A => mode_reg, Y => n657);
   U234 : INVXL port map( A => n556, Y => n550);
   U235 : NOR2XL port map( A => n329, B => n315, Y => n556);
   U236 : OAI32XL port map( A0 => n1034, A1 => upper_reg_1_port, A2 => n20, B0 
                           => n419, B1 => n410, Y => n418);
   U237 : AOI21XL port map( A0 => n315, A1 => n420, B0 => n335, Y => n419);
   U238 : INVXL port map( A => aa_reg_0_port, Y => n424);
   U239 : INVXL port map( A => a(2), Y => n562);
   U240 : XOR2XL port map( A => a(2), B => n564, Y => n28);
   U241 : INVXL port map( A => a(3), Y => n565);
   U242 : XOR2XL port map( A => n567, B => n565, Y => n39);
   U243 : INVXL port map( A => a(1), Y => n559);
   U244 : XNOR2XL port map( A => a(0), B => n559, Y => n12);
   U245 : INVXL port map( A => n20, Y => n16);
   U246 : XNOR2XL port map( A => n422, B => n423, Y => n20);
   U247 : INVXL port map( A => upper_reg_1_port, Y => n6);
   U248 : INVXL port map( A => n28, Y => n27);
   U249 : INVXL port map( A => upper_reg_2_port, Y => n18);
   U250 : XNOR2XL port map( A => n32, B => n33, Y => n14);
   U251 : INVXL port map( A => a(5), Y => n571);
   U252 : XOR2XL port map( A => a(5), B => n573, Y => n59);
   U253 : XOR2XL port map( A => a(4), B => n570, Y => n49);
   U254 : NOR2XL port map( A => a(3), B => n567, Y => n570);
   U255 : INVXL port map( A => a(6), Y => n574);
   U256 : XOR2XL port map( A => n576, B => n574, Y => n69);
   U257 : INVXL port map( A => n39, Y => n38);
   U258 : INVXL port map( A => upper_reg_3_port, Y => n30);
   U259 : XNOR2XL port map( A => n42, B => n43, Y => n29);
   U260 : INVXL port map( A => n49, Y => n48);
   U261 : INVXL port map( A => upper_reg_4_port, Y => n41);
   U262 : XNOR2XL port map( A => n52, B => n53, Y => n40);
   U263 : INVXL port map( A => a(8), Y => n580);
   U264 : XOR2XL port map( A => n582, B => n580, Y => n89);
   U265 : NAND2XL port map( A => n579, B => n577, Y => n582);
   U266 : INVXL port map( A => a(7), Y => n577);
   U267 : XOR2XL port map( A => a(7), B => n579, Y => n79);
   U268 : INVXL port map( A => n59, Y => n58);
   U269 : INVXL port map( A => upper_reg_5_port, Y => n51);
   U270 : XNOR2XL port map( A => n62, B => n63, Y => n50);
   U271 : INVXL port map( A => n69, Y => n68);
   U272 : INVXL port map( A => upper_reg_6_port, Y => n61);
   U273 : XNOR2XL port map( A => n72, B => n73, Y => n60);
   U274 : INVXL port map( A => n79, Y => n78);
   U275 : INVXL port map( A => upper_reg_7_port, Y => n71);
   U276 : XNOR2XL port map( A => n82, B => n83, Y => n70);
   U277 : INVXL port map( A => a(9), Y => n583);
   U278 : XOR2XL port map( A => n585, B => n583, Y => n100);
   U279 : INVXL port map( A => a(11), Y => n589);
   U280 : XOR2XL port map( A => n591, B => n589, Y => n122);
   U281 : INVXL port map( A => a(10), Y => n586);
   U282 : XOR2XL port map( A => a(10), B => n588, Y => n112);
   U283 : INVXL port map( A => n89, Y => n88);
   U284 : INVXL port map( A => upper_reg_8_port, Y => n81);
   U285 : XNOR2XL port map( A => n92, B => n93, Y => n80);
   U286 : INVXL port map( A => n112, Y => n111);
   U287 : NOR2XL port map( A => n542, B => mode_reg, Y => n21);
   U288 : INVXL port map( A => upper_reg_9_port, Y => n91);
   U289 : INVXL port map( A => n105, Y => n102);
   U290 : XOR2XL port map( A => n103, B => n104, Y => n90);
   U291 : INVXL port map( A => upper_reg_10_port, Y => n95);
   U292 : XNOR2XL port map( A => n115, B => n116, Y => n105);
   U293 : INVXL port map( A => n100, Y => n99);
   U294 : XOR2XL port map( A => a(12), B => n594, Y => n132);
   U295 : NOR2XL port map( A => a(11), B => n591, Y => n594);
   U296 : INVXL port map( A => a(14), Y => n598);
   U297 : XOR2XL port map( A => n600, B => n598, Y => n152);
   U298 : INVXL port map( A => a(13), Y => n595);
   U299 : XOR2XL port map( A => a(13), B => n597, Y => n142);
   U300 : INVXL port map( A => n122, Y => n121);
   U301 : INVXL port map( A => upper_reg_11_port, Y => n114);
   U302 : XNOR2XL port map( A => n125, B => n126, Y => n113);
   U303 : INVXL port map( A => n132, Y => n131);
   U304 : INVXL port map( A => upper_reg_12_port, Y => n124);
   U305 : XNOR2XL port map( A => n135, B => n136, Y => n123);
   U306 : INVXL port map( A => n142, Y => n141);
   U307 : INVXL port map( A => upper_reg_13_port, Y => n134);
   U308 : XNOR2XL port map( A => n145, B => n146, Y => n133);
   U309 : INVXL port map( A => a(16), Y => n604);
   U310 : XOR2XL port map( A => n606, B => n604, Y => n172);
   U311 : NAND2XL port map( A => n603, B => n601, Y => n606);
   U312 : INVXL port map( A => a(15), Y => n601);
   U313 : XOR2XL port map( A => a(15), B => n603, Y => n162);
   U314 : INVXL port map( A => n152, Y => n151);
   U315 : INVXL port map( A => upper_reg_14_port, Y => n144);
   U316 : INVXL port map( A => n101, Y => n1032);
   U317 : XNOR2XL port map( A => n155, B => n156, Y => n143);
   U318 : INVXL port map( A => n162, Y => n161);
   U319 : INVXL port map( A => upper_reg_15_port, Y => n154);
   U320 : XNOR2XL port map( A => n165, B => n166, Y => n153);
   U321 : INVXL port map( A => a(19), Y => n613);
   U322 : XOR2XL port map( A => n615, B => n613, Y => n202);
   U323 : INVXL port map( A => a(18), Y => n610);
   U324 : XOR2XL port map( A => a(18), B => n612, Y => n192);
   U325 : INVXL port map( A => a(17), Y => n607);
   U326 : XOR2XL port map( A => n609, B => n607, Y => n182);
   U327 : INVXL port map( A => n172, Y => n171);
   U328 : INVXL port map( A => upper_reg_16_port, Y => n164);
   U329 : XNOR2XL port map( A => n175, B => n176, Y => n163);
   U330 : INVXL port map( A => n182, Y => n181);
   U331 : INVXL port map( A => upper_reg_17_port, Y => n174);
   U332 : XNOR2XL port map( A => n185, B => n186, Y => n173);
   U333 : INVXL port map( A => n192, Y => n191);
   U334 : INVXL port map( A => upper_reg_18_port, Y => n184);
   U335 : XNOR2XL port map( A => n195, B => n196, Y => n183);
   U336 : INVXL port map( A => a(22), Y => n622);
   U337 : XOR2XL port map( A => n624, B => n622, Y => n232);
   U338 : INVXL port map( A => a(21), Y => n619);
   U339 : XOR2XL port map( A => a(21), B => n621, Y => n222);
   U340 : XOR2XL port map( A => a(20), B => n618, Y => n212);
   U341 : NOR2XL port map( A => a(19), B => n615, Y => n618);
   U342 : INVXL port map( A => n202, Y => n201);
   U343 : INVXL port map( A => upper_reg_19_port, Y => n194);
   U344 : XNOR2XL port map( A => n205, B => n206, Y => n193);
   U345 : INVXL port map( A => n212, Y => n211);
   U346 : INVXL port map( A => upper_reg_20_port, Y => n204);
   U347 : XNOR2XL port map( A => n215, B => n216, Y => n203);
   U348 : INVXL port map( A => n222, Y => n221);
   U349 : INVXL port map( A => upper_reg_21_port, Y => n214);
   U350 : XNOR2XL port map( A => n225, B => n226, Y => n213);
   U351 : INVXL port map( A => a(23), Y => n625);
   U352 : XOR2XL port map( A => a(23), B => n627, Y => n242);
   U353 : INVXL port map( A => a(24), Y => n628);
   U354 : XOR2XL port map( A => n630, B => n628, Y => n252);
   U355 : NAND2XL port map( A => n627, B => n625, Y => n630);
   U356 : NOR4XL port map( A => n428, B => n429, C => n430, D => n431, Y => 
                           n412);
   U357 : NAND4XL port map( A => n532, B => n533, C => n534, D => n535, Y => 
                           n429);
   U358 : NAND4XL port map( A => n536, B => n537, C => n538, D => n539, Y => 
                           n428);
   U359 : OR4XL port map( A => n432, B => n433, C => n434, D => n435, Y => n431
                           );
   U360 : INVXL port map( A => a(0), Y => n410);
   U361 : INVXL port map( A => lower_reg_1_port, Y => n406);
   U362 : INVXL port map( A => lower_reg_0_port, Y => n408);
   U363 : INVXL port map( A => lower_reg_2_port, Y => n404);
   U364 : INVXL port map( A => n232, Y => n231);
   U365 : INVXL port map( A => upper_reg_22_port, Y => n224);
   U366 : XNOR2XL port map( A => n235, B => n236, Y => n223);
   U367 : INVXL port map( A => n242, Y => n241);
   U368 : INVXL port map( A => upper_reg_23_port, Y => n234);
   U369 : XNOR2XL port map( A => n245, B => n246, Y => n233);
   U370 : INVXL port map( A => a(27), Y => n637);
   U371 : XOR2XL port map( A => n639, B => n637, Y => n282);
   U372 : INVXL port map( A => a(26), Y => n634);
   U373 : XOR2XL port map( A => a(26), B => n636, Y => n272);
   U374 : INVXL port map( A => a(25), Y => n631);
   U375 : XOR2XL port map( A => n633, B => n631, Y => n262);
   U376 : INVXL port map( A => lower_reg_7_port, Y => n394);
   U377 : INVXL port map( A => lower_reg_4_port, Y => n400);
   U378 : INVXL port map( A => lower_reg_6_port, Y => n396);
   U379 : INVXL port map( A => lower_reg_3_port, Y => n402);
   U380 : INVXL port map( A => lower_reg_5_port, Y => n398);
   U381 : INVXL port map( A => n252, Y => n251);
   U382 : INVXL port map( A => upper_reg_24_port, Y => n244);
   U383 : XNOR2XL port map( A => n255, B => n256, Y => n243);
   U384 : INVXL port map( A => n262, Y => n261);
   U385 : INVXL port map( A => upper_reg_25_port, Y => n254);
   U386 : XNOR2XL port map( A => n265, B => n266, Y => n253);
   U387 : INVXL port map( A => n272, Y => n271);
   U388 : INVXL port map( A => upper_reg_26_port, Y => n264);
   U389 : XNOR2XL port map( A => n275, B => n276, Y => n263);
   U390 : XOR2XL port map( A => a(30), B => n648, Y => n313);
   U391 : INVXL port map( A => a(30), Y => n646);
   U392 : INVXL port map( A => a(29), Y => n643);
   U393 : XOR2XL port map( A => n645, B => n643, Y => n302);
   U394 : INVXL port map( A => a(28), Y => n640);
   U395 : XOR2XL port map( A => a(28), B => n642, Y => n292);
   U396 : INVXL port map( A => lower_reg_12_port, Y => n384);
   U397 : INVXL port map( A => lower_reg_9_port, Y => n390);
   U398 : INVXL port map( A => lower_reg_10_port, Y => n388);
   U399 : INVXL port map( A => lower_reg_8_port, Y => n392);
   U400 : INVXL port map( A => lower_reg_11_port, Y => n386);
   U401 : INVXL port map( A => n282, Y => n281);
   U402 : INVXL port map( A => upper_reg_27_port, Y => n274);
   U403 : XNOR2XL port map( A => n285, B => n286, Y => n273);
   U404 : AND4X1 port map( A => n315, B => n316, C => n317, D => n101, Y => 
                           n342);
   U405 : INVXL port map( A => n292, Y => n291);
   U406 : NOR2XL port map( A => n828, B => n1032, Y => n17);
   U407 : INVXL port map( A => upper_reg_28_port, Y => n284);
   U408 : XNOR2XL port map( A => n295, B => n296, Y => n283);
   U409 : INVXL port map( A => lower_reg_18_port, Y => n372);
   U410 : INVXL port map( A => lower_reg_16_port, Y => n376);
   U411 : INVXL port map( A => lower_reg_14_port, Y => n380);
   U412 : INVXL port map( A => lower_reg_19_port, Y => n370);
   U413 : INVXL port map( A => lower_reg_17_port, Y => n374);
   U414 : INVXL port map( A => lower_reg_15_port, Y => n378);
   U415 : INVXL port map( A => lower_reg_13_port, Y => n382);
   U416 : XNOR2XL port map( A => sign_reg, B => n332, Y => n331);
   U417 : INVXL port map( A => n420, Y => n329);
   U418 : INVXL port map( A => n315, Y => n328);
   U419 : NAND3XL port map( A => n425, B => n329, C => n426, Y => n101);
   U420 : AOI22XL port map( A0 => n412, A1 => n414, B0 => n413, B1 => n427, Y 
                           => n426);
   U421 : INVXL port map( A => n15, Y => n1035);
   U422 : NAND2XL port map( A => a(31), B => n335, Y => n312);
   U423 : OR2XL port map( A => n688, B => b(8), Y => n692);
   U424 : AND4X1 port map( A => n315, B => n316, C => n317, D => n101, Y => 
                           n350);
   U425 : INVXL port map( A => n302, Y => n301);
   U426 : NOR2XL port map( A => n828, B => n1032, Y => n106);
   U427 : INVXL port map( A => upper_reg_29_port, Y => n294);
   U428 : XNOR2XL port map( A => n305, B => n306, Y => n293);
   U429 : AND4X1 port map( A => n315, B => n316, C => n317, D => n101, Y => n9)
                           ;
   U430 : INVXL port map( A => n313, Y => n311);
   U431 : INVXL port map( A => n312, Y => n11);
   U432 : NOR2XL port map( A => n828, B => n1032, Y => n337);
   U433 : XNOR2XL port map( A => n333, B => n334, Y => n314);
   U434 : INVXL port map( A => upper_reg_31_port, Y => n320);
   U435 : INVXL port map( A => upper_reg_30_port, Y => n304);
   U436 : XNOR2XL port map( A => n321, B => n322, Y => n303);
   U437 : INVXL port map( A => n21, Y => n828);
   U438 : OR2XL port map( A => n683, B => b(7), Y => n688);
   U439 : INVXL port map( A => bb_reg_9_port, Y => n686);
   U440 : INVXL port map( A => negate_reg, Y => n793);
   U441 : INVXL port map( A => n336, Y => n784);
   U442 : NOR3XL port map( A => n796, B => mult_func(2), C => n318, Y => n335);
   U443 : NOR2XL port map( A => n414, B => n654, Y => n782);
   U444 : INVXL port map( A => n425, Y => n654);
   U445 : INVXL port map( A => n551, Y => n555);
   U446 : OR2XL port map( A => n662, B => b(2), Y => n666);
   U447 : INVXL port map( A => bb_reg_4_port, Y => n439);
   U448 : OR2XL port map( A => n666, B => b(3), Y => n670);
   U449 : INVXL port map( A => bb_reg_5_port, Y => n440);
   U450 : OR2XL port map( A => n670, B => b(4), Y => n674);
   U451 : INVXL port map( A => bb_reg_6_port, Y => n441);
   U452 : OR2XL port map( A => n674, B => b(5), Y => n678);
   U453 : INVXL port map( A => bb_reg_7_port, Y => n443);
   U454 : OR2XL port map( A => n678, B => b(6), Y => n683);
   U455 : INVXL port map( A => bb_reg_8_port, Y => n681);
   U456 : INVXL port map( A => bb_reg_31_port, Y => n437);
   U457 : NOR2XL port map( A => n772, B => b(29), Y => n779);
   U458 : INVXL port map( A => b(30), Y => n777);
   U459 : INVXL port map( A => lower_reg_30_port, Y => n351);
   U460 : INVXL port map( A => lower_reg_28_port, Y => n352);
   U461 : INVXL port map( A => lower_reg_26_port, Y => n356);
   U462 : INVXL port map( A => lower_reg_24_port, Y => n360);
   U463 : INVXL port map( A => lower_reg_20_port, Y => n368);
   U464 : INVXL port map( A => lower_reg_22_port, Y => n364);
   U465 : INVXL port map( A => lower_reg_29_port, Y => n348);
   U466 : INVXL port map( A => lower_reg_27_port, Y => n354);
   U467 : INVXL port map( A => lower_reg_25_port, Y => n358);
   U468 : INVXL port map( A => lower_reg_23_port, Y => n362);
   U469 : INVXL port map( A => lower_reg_21_port, Y => n366);
   U470 : INVXL port map( A => n340, Y => n347);
   U471 : INVXL port map( A => b(1), Y => n656);
   U472 : INVXL port map( A => bb_reg_3_port, Y => n438);
   U473 : INVXL port map( A => bb_reg_2_port, Y => n451);
   U474 : NAND2XL port map( A => n656, B => n552, Y => n662);
   U475 : INVXL port map( A => bb_reg_10_port, Y => n452);
   U476 : OR2XL port map( A => n692, B => b(9), Y => n696);
   U477 : INVXL port map( A => bb_reg_11_port, Y => n453);
   U478 : OR2XL port map( A => n696, B => b(10), Y => n700);
   U479 : INVXL port map( A => bb_reg_12_port, Y => n459);
   U480 : OR2XL port map( A => n700, B => b(11), Y => n704);
   U481 : INVXL port map( A => bb_reg_13_port, Y => n458);
   U482 : OR2XL port map( A => n704, B => b(12), Y => n708);
   U483 : INVXL port map( A => bb_reg_14_port, Y => n457);
   U484 : OR2XL port map( A => n708, B => b(13), Y => n712);
   U485 : INVXL port map( A => bb_reg_15_port, Y => n456);
   U486 : OR2XL port map( A => n712, B => b(14), Y => n716);
   U487 : INVXL port map( A => bb_reg_16_port, Y => n532);
   U488 : OR2XL port map( A => n716, B => b(15), Y => n720);
   U489 : INVXL port map( A => bb_reg_17_port, Y => n533);
   U490 : OR2XL port map( A => n720, B => b(16), Y => n724);
   U491 : INVXL port map( A => bb_reg_18_port, Y => n534);
   U492 : OR2XL port map( A => n724, B => b(17), Y => n728);
   U493 : INVXL port map( A => bb_reg_19_port, Y => n535);
   U494 : OR2XL port map( A => n728, B => b(18), Y => n732);
   U495 : INVXL port map( A => bb_reg_20_port, Y => n537);
   U496 : OR2XL port map( A => n732, B => b(19), Y => n736);
   U497 : INVXL port map( A => bb_reg_21_port, Y => n538);
   U498 : OR2XL port map( A => n736, B => b(20), Y => n740);
   U499 : INVXL port map( A => bb_reg_22_port, Y => n539);
   U500 : OR2XL port map( A => n740, B => b(21), Y => n744);
   U501 : INVXL port map( A => bb_reg_23_port, Y => n444);
   U502 : OR2XL port map( A => n744, B => b(22), Y => n748);
   U503 : INVXL port map( A => bb_reg_24_port, Y => n445);
   U504 : OR2XL port map( A => n748, B => b(23), Y => n752);
   U505 : INVXL port map( A => bb_reg_25_port, Y => n446);
   U506 : OR2XL port map( A => n752, B => b(24), Y => n756);
   U507 : INVXL port map( A => bb_reg_26_port, Y => n447);
   U508 : OR2XL port map( A => n756, B => b(25), Y => n760);
   U509 : INVXL port map( A => bb_reg_27_port, Y => n448);
   U510 : OR2XL port map( A => n760, B => b(26), Y => n764);
   U511 : INVXL port map( A => bb_reg_28_port, Y => n449);
   U512 : OR2XL port map( A => n764, B => b(27), Y => n768);
   U513 : INVXL port map( A => bb_reg_30_port, Y => n436);
   U514 : INVXL port map( A => bb_reg_29_port, Y => n450);
   U515 : OR2XL port map( A => n768, B => b(28), Y => n772);
   U516 : INVXL port map( A => n659, Y => n546);
   U517 : INVXL port map( A => bb_reg_1_port, Y => n536);
   U518 : INVXL port map( A => bb_reg_0_port, Y => n421);
   U519 : INVXL port map( A => n782, Y => n553);
   U520 : INVXL port map( A => b(0), Y => n552);
   U521 : NAND2XL port map( A => n791, B => n319, Y => n336);
   U522 : INVXL port map( A => lower_reg_31_port, Y => n338);
   U523 : INVXL port map( A => a(31), Y => n330);
   U524 : NAND2XL port map( A => bb_reg_0_port, B => n413, Y => n344);
   U525 : INVXL port map( A => upper_reg_0_port, Y => n343);
   U526 : NOR2XL port map( A => n828, B => n345, Y => n340);
   U527 : NOR2XL port map( A => n652, B => mult_func(0), Y => n548);
   U528 : NOR2XL port map( A => n319, B => mult_func(3), Y => n420);
   U529 : INVXL port map( A => b(31), Y => n549);
   U530 : XOR2XL port map( A => n653, B => n330, Y => n323);
   U531 : NAND2XL port map( A => n648, B => n646, Y => n653);
   U532 : NOR3XL port map( A => n335, B => n420, C => n791, Y => n544);
   U533 : INVXL port map( A => n544, Y => n542);
   U534 : INVXL port map( A => sign_reg, Y => n540);
   U535 : INVXL port map( A => sign2_reg, Y => n324);
   U536 : INVXL port map( A => n554, Y => n962);
   U537 : AOI222XL port map( A0 => aa_reg_1_port, A1 => n1, B0 => a(0), B1 => 
                           n556, C0 => aa_reg_0_port, C1 => n557, Y => n554);
   U538 : AOI21XL port map( A0 => n21, A1 => n424, B0 => n1032, Y => n415);
   U539 : AOI21XL port map( A0 => n20, A1 => n19, B0 => n1035, Y => n416);
   U540 : AOI31XL port map( A0 => aa_reg_0_port, A1 => n343, A2 => n17, B0 => 
                           n418, Y => n417);
   U541 : AOI22XL port map( A0 => n1, A1 => aa_reg_3_port, B0 => n557, B1 => 
                           aa_reg_2_port, Y => n563);
   U542 : AOI22XL port map( A0 => n1, A1 => aa_reg_4_port, B0 => n557, B1 => 
                           aa_reg_3_port, Y => n566);
   U543 : INVXL port map( A => n12, Y => n558);
   U544 : AOI22XL port map( A0 => n1, A1 => aa_reg_2_port, B0 => n557, B1 => 
                           aa_reg_1_port, Y => n561);
   U545 : OAI211XL port map( A0 => n5, A1 => n6, B0 => n7, C0 => n8, Y => n895)
                           ;
   U546 : AOI21XL port map( A0 => n20, A1 => n21, B0 => n1032, Y => n5);
   U547 : AOI33XL port map( A0 => n16, A1 => n6, A2 => n106, B0 => n14, B1 => 
                           n18, B2 => n19, Y => n7);
   U548 : AOI222XL port map( A0 => n342, A1 => a(1), B0 => upper_reg_2_port, B1
                           => n10, C0 => n11, C1 => n12, Y => n8);
   U549 : OAI211XL port map( A0 => n23, A1 => n18, B0 => n24, C0 => n25, Y => 
                           n896);
   U550 : AOI2BB1XL port map( A0N => n14, A1N => n828, B0 => n1032, Y => n23);
   U551 : AOI33XL port map( A0 => n29, A1 => n30, A2 => n19, B0 => n14, B1 => 
                           n18, B2 => n337, Y => n24);
   U552 : AOI222XL port map( A0 => a(2), A1 => n350, B0 => upper_reg_3_port, B1
                           => n26, C0 => n27, C1 => n11, Y => n25);
   U553 : AOI22XL port map( A0 => n1, A1 => aa_reg_6_port, B0 => n557, B1 => 
                           aa_reg_5_port, Y => n572);
   U554 : INVXL port map( A => a(4), Y => n568);
   U555 : AOI22XL port map( A0 => n1, A1 => aa_reg_5_port, B0 => n557, B1 => 
                           aa_reg_4_port, Y => n569);
   U556 : AOI22XL port map( A0 => n1, A1 => aa_reg_7_port, B0 => n557, B1 => 
                           aa_reg_6_port, Y => n575);
   U557 : OAI211XL port map( A0 => n34, A1 => n30, B0 => n35, C0 => n36, Y => 
                           n897);
   U558 : AOI2BB1XL port map( A0N => n29, A1N => n828, B0 => n1032, Y => n34);
   U559 : AOI33XL port map( A0 => n40, A1 => n41, A2 => n19, B0 => n29, B1 => 
                           n30, B2 => n337, Y => n35);
   U560 : AOI222XL port map( A0 => a(3), A1 => n9, B0 => upper_reg_4_port, B1 
                           => n37, C0 => n38, C1 => n11, Y => n36);
   U561 : OAI211XL port map( A0 => n44, A1 => n41, B0 => n45, C0 => n46, Y => 
                           n898);
   U562 : AOI2BB1XL port map( A0N => n40, A1N => n828, B0 => n1032, Y => n44);
   U563 : AOI33XL port map( A0 => n50, A1 => n51, A2 => n19, B0 => n40, B1 => 
                           n41, B2 => n17, Y => n45);
   U564 : AOI222XL port map( A0 => a(4), A1 => n342, B0 => upper_reg_5_port, B1
                           => n47, C0 => n48, C1 => n11, Y => n46);
   U565 : AOI22XL port map( A0 => n1, A1 => aa_reg_9_port, B0 => n557, B1 => 
                           aa_reg_8_port, Y => n581);
   U566 : AOI22XL port map( A0 => n1, A1 => aa_reg_8_port, B0 => n557, B1 => 
                           aa_reg_7_port, Y => n578);
   U567 : OAI211XL port map( A0 => n54, A1 => n51, B0 => n55, C0 => n56, Y => 
                           n899);
   U568 : AOI2BB1XL port map( A0N => n50, A1N => n828, B0 => n1032, Y => n54);
   U569 : AOI33XL port map( A0 => n60, A1 => n61, A2 => n19, B0 => n50, B1 => 
                           n51, B2 => n106, Y => n55);
   U570 : AOI222XL port map( A0 => a(5), A1 => n350, B0 => upper_reg_6_port, B1
                           => n57, C0 => n58, C1 => n11, Y => n56);
   U571 : OAI211XL port map( A0 => n64, A1 => n61, B0 => n65, C0 => n66, Y => 
                           n900);
   U572 : AOI2BB1XL port map( A0N => n60, A1N => n828, B0 => n1032, Y => n64);
   U573 : AOI33XL port map( A0 => n70, A1 => n71, A2 => n19, B0 => n60, B1 => 
                           n61, B2 => n337, Y => n65);
   U574 : AOI222XL port map( A0 => a(6), A1 => n9, B0 => upper_reg_7_port, B1 
                           => n67, C0 => n68, C1 => n11, Y => n66);
   U575 : OAI211XL port map( A0 => n74, A1 => n71, B0 => n75, C0 => n76, Y => 
                           n901);
   U576 : AOI2BB1XL port map( A0N => n70, A1N => n828, B0 => n1032, Y => n74);
   U577 : AOI33XL port map( A0 => n80, A1 => n81, A2 => n19, B0 => n70, B1 => 
                           n71, B2 => n17, Y => n75);
   U578 : AOI222XL port map( A0 => a(7), A1 => n342, B0 => upper_reg_8_port, B1
                           => n77, C0 => n78, C1 => n11, Y => n76);
   U579 : AOI22XL port map( A0 => n1, A1 => aa_reg_10_port, B0 => n557, B1 => 
                           aa_reg_9_port, Y => n584);
   U580 : AOI22XL port map( A0 => n1, A1 => aa_reg_12_port, B0 => n557, B1 => 
                           aa_reg_11_port, Y => n590);
   U581 : AOI22XL port map( A0 => n1, A1 => aa_reg_11_port, B0 => n557, B1 => 
                           aa_reg_10_port, Y => n587);
   U582 : OAI211XL port map( A0 => n84, A1 => n81, B0 => n85, C0 => n86, Y => 
                           n902);
   U583 : AOI2BB1XL port map( A0N => n80, A1N => n828, B0 => n1032, Y => n84);
   U584 : AOI33XL port map( A0 => n90, A1 => n91, A2 => n19, B0 => n80, B1 => 
                           n81, B2 => n106, Y => n85);
   U585 : AOI222XL port map( A0 => a(8), A1 => n350, B0 => upper_reg_9_port, B1
                           => n87, C0 => n88, C1 => n11, Y => n86);
   U586 : OAI211XL port map( A0 => n107, A1 => n95, B0 => n108, C0 => n109, Y 
                           => n904);
   U587 : AOI21XL port map( A0 => n105, A1 => n21, B0 => n1032, Y => n107);
   U588 : AOI33XL port map( A0 => n113, A1 => n114, A2 => n19, B0 => n102, B1 
                           => n95, B2 => n17, Y => n108);
   U589 : AOI222XL port map( A0 => a(10), A1 => n342, B0 => upper_reg_11_port, 
                           B1 => n110, C0 => n111, C1 => n11, Y => n109);
   U590 : OAI211XL port map( A0 => n94, A1 => n95, B0 => n96, C0 => n97, Y => 
                           n903);
   U591 : AOI222XL port map( A0 => a(9), A1 => n9, B0 => upper_reg_9_port, B1 
                           => n98, C0 => n99, C1 => n11, Y => n97);
   U592 : AOI21XL port map( A0 => n105, A1 => n19, B0 => n1035, Y => n94);
   U593 : AOI33XL port map( A0 => n102, A1 => n95, A2 => n19, B0 => n90, B1 => 
                           n91, B2 => n337, Y => n96);
   U594 : INVXL port map( A => a(12), Y => n592);
   U595 : AOI22XL port map( A0 => n1, A1 => aa_reg_13_port, B0 => n557, B1 => 
                           aa_reg_12_port, Y => n593);
   U596 : AOI22XL port map( A0 => n1, A1 => aa_reg_15_port, B0 => n557, B1 => 
                           aa_reg_14_port, Y => n599);
   U597 : AOI22XL port map( A0 => n1, A1 => aa_reg_14_port, B0 => n557, B1 => 
                           aa_reg_13_port, Y => n596);
   U598 : OAI211XL port map( A0 => n117, A1 => n114, B0 => n118, C0 => n119, Y 
                           => n905);
   U599 : AOI2BB1XL port map( A0N => n113, A1N => n828, B0 => n1032, Y => n117)
                           ;
   U600 : AOI33XL port map( A0 => n123, A1 => n124, A2 => n19, B0 => n113, B1 
                           => n114, B2 => n106, Y => n118);
   U601 : AOI222XL port map( A0 => a(11), A1 => n350, B0 => upper_reg_12_port, 
                           B1 => n120, C0 => n121, C1 => n11, Y => n119);
   U602 : OAI211XL port map( A0 => n127, A1 => n124, B0 => n128, C0 => n129, Y 
                           => n906);
   U603 : AOI2BB1XL port map( A0N => n123, A1N => n828, B0 => n1032, Y => n127)
                           ;
   U604 : AOI33XL port map( A0 => n133, A1 => n134, A2 => n19, B0 => n123, B1 
                           => n124, B2 => n337, Y => n128);
   U605 : AOI222XL port map( A0 => a(12), A1 => n9, B0 => upper_reg_13_port, B1
                           => n130, C0 => n131, C1 => n11, Y => n129);
   U606 : OAI211XL port map( A0 => n137, A1 => n134, B0 => n138, C0 => n139, Y 
                           => n907);
   U607 : AOI2BB1XL port map( A0N => n133, A1N => n828, B0 => n1032, Y => n137)
                           ;
   U608 : AOI33XL port map( A0 => n143, A1 => n144, A2 => n19, B0 => n133, B1 
                           => n134, B2 => n17, Y => n138);
   U609 : AOI222XL port map( A0 => a(13), A1 => n342, B0 => upper_reg_14_port, 
                           B1 => n140, C0 => n141, C1 => n11, Y => n139);
   U610 : AOI22XL port map( A0 => n1, A1 => aa_reg_17_port, B0 => n557, B1 => 
                           aa_reg_16_port, Y => n605);
   U611 : AOI22XL port map( A0 => n1, A1 => aa_reg_16_port, B0 => n557, B1 => 
                           aa_reg_15_port, Y => n602);
   U612 : OAI211XL port map( A0 => n147, A1 => n144, B0 => n148, C0 => n149, Y 
                           => n908);
   U613 : AOI2BB1XL port map( A0N => n143, A1N => n828, B0 => n1032, Y => n147)
                           ;
   U614 : AOI33XL port map( A0 => n153, A1 => n154, A2 => n19, B0 => n143, B1 
                           => n144, B2 => n106, Y => n148);
   U615 : AOI222XL port map( A0 => a(14), A1 => n350, B0 => upper_reg_15_port, 
                           B1 => n150, C0 => n151, C1 => n11, Y => n149);
   U616 : OAI211XL port map( A0 => n157, A1 => n154, B0 => n158, C0 => n159, Y 
                           => n909);
   U617 : AOI2BB1XL port map( A0N => n153, A1N => n828, B0 => n1032, Y => n157)
                           ;
   U618 : AOI33XL port map( A0 => n163, A1 => n164, A2 => n19, B0 => n153, B1 
                           => n154, B2 => n337, Y => n158);
   U619 : AOI222XL port map( A0 => a(15), A1 => n9, B0 => upper_reg_16_port, B1
                           => n160, C0 => n161, C1 => n11, Y => n159);
   U620 : AOI22XL port map( A0 => n1, A1 => aa_reg_20_port, B0 => n557, B1 => 
                           aa_reg_19_port, Y => n614);
   U621 : AOI22XL port map( A0 => n1, A1 => aa_reg_19_port, B0 => n557, B1 => 
                           aa_reg_18_port, Y => n611);
   U622 : AOI22XL port map( A0 => n1, A1 => aa_reg_18_port, B0 => n557, B1 => 
                           aa_reg_17_port, Y => n608);
   U623 : OAI211XL port map( A0 => n167, A1 => n164, B0 => n168, C0 => n169, Y 
                           => n910);
   U624 : AOI2BB1XL port map( A0N => n163, A1N => n828, B0 => n1032, Y => n167)
                           ;
   U625 : AOI33XL port map( A0 => n173, A1 => n174, A2 => n19, B0 => n163, B1 
                           => n164, B2 => n17, Y => n168);
   U626 : AOI222XL port map( A0 => a(16), A1 => n342, B0 => upper_reg_17_port, 
                           B1 => n170, C0 => n171, C1 => n11, Y => n169);
   U627 : OAI211XL port map( A0 => n177, A1 => n174, B0 => n178, C0 => n179, Y 
                           => n911);
   U628 : AOI2BB1XL port map( A0N => n173, A1N => n828, B0 => n1032, Y => n177)
                           ;
   U629 : AOI33XL port map( A0 => n183, A1 => n184, A2 => n19, B0 => n173, B1 
                           => n174, B2 => n106, Y => n178);
   U630 : AOI222XL port map( A0 => a(17), A1 => n350, B0 => upper_reg_18_port, 
                           B1 => n180, C0 => n181, C1 => n11, Y => n179);
   U631 : OAI211XL port map( A0 => n187, A1 => n184, B0 => n188, C0 => n189, Y 
                           => n912);
   U632 : AOI2BB1XL port map( A0N => n183, A1N => n828, B0 => n1032, Y => n187)
                           ;
   U633 : AOI33XL port map( A0 => n193, A1 => n194, A2 => n19, B0 => n183, B1 
                           => n184, B2 => n337, Y => n188);
   U634 : AOI222XL port map( A0 => a(18), A1 => n9, B0 => upper_reg_19_port, B1
                           => n190, C0 => n191, C1 => n11, Y => n189);
   U635 : AOI22XL port map( A0 => n1, A1 => aa_reg_23_port, B0 => n557, B1 => 
                           aa_reg_22_port, Y => n623);
   U636 : AOI22XL port map( A0 => n1, A1 => aa_reg_22_port, B0 => n557, B1 => 
                           aa_reg_21_port, Y => n620);
   U637 : INVXL port map( A => a(20), Y => n616);
   U638 : AOI22XL port map( A0 => n1, A1 => aa_reg_21_port, B0 => n557, B1 => 
                           aa_reg_20_port, Y => n617);
   U639 : OAI211XL port map( A0 => n197, A1 => n194, B0 => n198, C0 => n199, Y 
                           => n913);
   U640 : AOI2BB1XL port map( A0N => n193, A1N => n828, B0 => n1032, Y => n197)
                           ;
   U641 : AOI33XL port map( A0 => n203, A1 => n204, A2 => n19, B0 => n193, B1 
                           => n194, B2 => n17, Y => n198);
   U642 : AOI222XL port map( A0 => a(19), A1 => n342, B0 => upper_reg_20_port, 
                           B1 => n200, C0 => n201, C1 => n11, Y => n199);
   U643 : OAI211XL port map( A0 => n207, A1 => n204, B0 => n208, C0 => n209, Y 
                           => n914);
   U644 : AOI2BB1XL port map( A0N => n203, A1N => n828, B0 => n1032, Y => n207)
                           ;
   U645 : AOI33XL port map( A0 => n213, A1 => n214, A2 => n19, B0 => n203, B1 
                           => n204, B2 => n106, Y => n208);
   U646 : AOI222XL port map( A0 => a(20), A1 => n350, B0 => upper_reg_21_port, 
                           B1 => n210, C0 => n211, C1 => n11, Y => n209);
   U647 : OAI211XL port map( A0 => n217, A1 => n214, B0 => n218, C0 => n219, Y 
                           => n915);
   U648 : AOI2BB1XL port map( A0N => n213, A1N => n828, B0 => n1032, Y => n217)
                           ;
   U649 : AOI33XL port map( A0 => n223, A1 => n224, A2 => n19, B0 => n213, B1 
                           => n214, B2 => n337, Y => n218);
   U650 : AOI222XL port map( A0 => a(21), A1 => n9, B0 => upper_reg_22_port, B1
                           => n220, C0 => n221, C1 => n11, Y => n219);
   U651 : AOI22XL port map( A0 => n1, A1 => aa_reg_24_port, B0 => n557, B1 => 
                           aa_reg_23_port, Y => n626);
   U652 : AOI22XL port map( A0 => n1, A1 => aa_reg_25_port, B0 => n557, B1 => 
                           aa_reg_24_port, Y => n629);
   U653 : AOI2BB2XL port map( B0 => n340, B1 => n412, A0N => n406, A1N => n13, 
                           Y => n411);
   U654 : AOI22XL port map( A0 => n784, A1 => a(3), B0 => lower_reg_3_port, B1 
                           => n345, Y => n405);
   U655 : AOI22XL port map( A0 => n784, A1 => a(2), B0 => lower_reg_2_port, B1 
                           => n345, Y => n407);
   U656 : AOI22XL port map( A0 => n784, A1 => a(1), B0 => lower_reg_1_port, B1 
                           => n345, Y => n409);
   U657 : OAI211XL port map( A0 => n227, A1 => n224, B0 => n228, C0 => n229, Y 
                           => n916);
   U658 : AOI2BB1XL port map( A0N => n223, A1N => n828, B0 => n1032, Y => n227)
                           ;
   U659 : AOI33XL port map( A0 => n233, A1 => n234, A2 => n19, B0 => n223, B1 
                           => n224, B2 => n17, Y => n228);
   U660 : AOI222XL port map( A0 => a(22), A1 => n342, B0 => upper_reg_23_port, 
                           B1 => n230, C0 => n231, C1 => n11, Y => n229);
   U661 : OAI211XL port map( A0 => n237, A1 => n234, B0 => n238, C0 => n239, Y 
                           => n917);
   U662 : AOI2BB1XL port map( A0N => n233, A1N => n828, B0 => n1032, Y => n237)
                           ;
   U663 : AOI33XL port map( A0 => n243, A1 => n244, A2 => n19, B0 => n233, B1 
                           => n234, B2 => n106, Y => n238);
   U664 : AOI222XL port map( A0 => a(23), A1 => n350, B0 => upper_reg_24_port, 
                           B1 => n240, C0 => n241, C1 => n11, Y => n239);
   U665 : AOI22XL port map( A0 => n1, A1 => aa_reg_28_port, B0 => n557, B1 => 
                           aa_reg_27_port, Y => n638);
   U666 : AOI22XL port map( A0 => n1, A1 => aa_reg_27_port, B0 => n557, B1 => 
                           aa_reg_26_port, Y => n635);
   U667 : AOI22XL port map( A0 => n1, A1 => aa_reg_26_port, B0 => n557, B1 => 
                           aa_reg_25_port, Y => n632);
   U668 : AOI22XL port map( A0 => n784, A1 => a(8), B0 => lower_reg_8_port, B1 
                           => n345, Y => n395);
   U669 : AOI22XL port map( A0 => n784, A1 => a(7), B0 => lower_reg_7_port, B1 
                           => n345, Y => n397);
   U670 : AOI22XL port map( A0 => n784, A1 => a(6), B0 => lower_reg_6_port, B1 
                           => n345, Y => n399);
   U671 : AOI22XL port map( A0 => n784, A1 => a(5), B0 => lower_reg_5_port, B1 
                           => n345, Y => n401);
   U672 : AOI22XL port map( A0 => n784, A1 => a(4), B0 => lower_reg_4_port, B1 
                           => n345, Y => n403);
   U673 : OAI211XL port map( A0 => n247, A1 => n244, B0 => n248, C0 => n249, Y 
                           => n918);
   U674 : AOI2BB1XL port map( A0N => n243, A1N => n828, B0 => n1032, Y => n247)
                           ;
   U675 : AOI33XL port map( A0 => n253, A1 => n254, A2 => n19, B0 => n243, B1 
                           => n244, B2 => n337, Y => n248);
   U676 : AOI222XL port map( A0 => a(24), A1 => n9, B0 => upper_reg_25_port, B1
                           => n250, C0 => n251, C1 => n11, Y => n249);
   U677 : OAI211XL port map( A0 => n257, A1 => n254, B0 => n258, C0 => n259, Y 
                           => n919);
   U678 : AOI2BB1XL port map( A0N => n253, A1N => n828, B0 => n1032, Y => n257)
                           ;
   U679 : AOI33XL port map( A0 => n263, A1 => n264, A2 => n19, B0 => n253, B1 
                           => n254, B2 => n17, Y => n258);
   U680 : AOI222XL port map( A0 => a(25), A1 => n342, B0 => upper_reg_26_port, 
                           B1 => n260, C0 => n261, C1 => n11, Y => n259);
   U681 : INVXL port map( A => reset_in, Y => n1036);
   U682 : OAI211XL port map( A0 => n267, A1 => n264, B0 => n268, C0 => n269, Y 
                           => n920);
   U683 : AOI2BB1XL port map( A0N => n263, A1N => n828, B0 => n1032, Y => n267)
                           ;
   U684 : AOI33XL port map( A0 => n273, A1 => n274, A2 => n19, B0 => n263, B1 
                           => n264, B2 => n106, Y => n268);
   U685 : AOI222XL port map( A0 => a(26), A1 => n350, B0 => upper_reg_27_port, 
                           B1 => n270, C0 => n271, C1 => n11, Y => n269);
   U686 : AOI22XL port map( A0 => n1, A1 => aa_reg_31_port, B0 => n557, B1 => 
                           aa_reg_30_port, Y => n647);
   U687 : AOI22XL port map( A0 => n1, A1 => aa_reg_30_port, B0 => n557, B1 => 
                           aa_reg_29_port, Y => n644);
   U688 : AOI22XL port map( A0 => n1, A1 => aa_reg_29_port, B0 => n557, B1 => 
                           aa_reg_28_port, Y => n641);
   U689 : AOI22XL port map( A0 => n784, A1 => a(13), B0 => lower_reg_13_port, 
                           B1 => n345, Y => n385);
   U690 : AOI22XL port map( A0 => n784, A1 => a(11), B0 => lower_reg_11_port, 
                           B1 => n345, Y => n389);
   U691 : AOI22XL port map( A0 => n784, A1 => a(10), B0 => lower_reg_10_port, 
                           B1 => n345, Y => n391);
   U692 : AOI22XL port map( A0 => n784, A1 => a(9), B0 => lower_reg_9_port, B1 
                           => n345, Y => n393);
   U693 : AOI22XL port map( A0 => n784, A1 => a(12), B0 => lower_reg_12_port, 
                           B1 => n345, Y => n387);
   U694 : OAI211XL port map( A0 => n277, A1 => n274, B0 => n278, C0 => n279, Y 
                           => n921);
   U695 : AOI2BB1XL port map( A0N => n273, A1N => n828, B0 => n1032, Y => n277)
                           ;
   U696 : AOI33XL port map( A0 => n283, A1 => n284, A2 => n19, B0 => n273, B1 
                           => n274, B2 => n337, Y => n278);
   U697 : AOI222XL port map( A0 => a(27), A1 => n9, B0 => upper_reg_28_port, B1
                           => n280, C0 => n281, C1 => n11, Y => n279);
   U698 : OAI211XL port map( A0 => n287, A1 => n284, B0 => n288, C0 => n289, Y 
                           => n922);
   U699 : AOI2BB1XL port map( A0N => n283, A1N => n828, B0 => n1032, Y => n287)
                           ;
   U700 : AOI33XL port map( A0 => n293, A1 => n294, A2 => n19, B0 => n283, B1 
                           => n284, B2 => n17, Y => n288);
   U701 : AOI222XL port map( A0 => a(28), A1 => n342, B0 => upper_reg_29_port, 
                           B1 => n290, C0 => n291, C1 => n11, Y => n289);
   U702 : AOI22XL port map( A0 => n784, A1 => a(19), B0 => lower_reg_19_port, 
                           B1 => n345, Y => n373);
   U703 : AOI22XL port map( A0 => n784, A1 => a(17), B0 => lower_reg_17_port, 
                           B1 => n345, Y => n377);
   U704 : AOI22XL port map( A0 => n784, A1 => a(15), B0 => lower_reg_15_port, 
                           B1 => n345, Y => n381);
   U705 : AOI22XL port map( A0 => n784, A1 => a(20), B0 => lower_reg_20_port, 
                           B1 => n345, Y => n371);
   U706 : AOI22XL port map( A0 => n784, A1 => a(18), B0 => lower_reg_18_port, 
                           B1 => n345, Y => n375);
   U707 : AOI22XL port map( A0 => n784, A1 => a(16), B0 => lower_reg_16_port, 
                           B1 => n345, Y => n379);
   U708 : AOI22XL port map( A0 => n784, A1 => a(14), B0 => lower_reg_14_port, 
                           B1 => n345, Y => n383);
   U709 : INVXL port map( A => n649, Y => n993);
   U710 : AOI221XL port map( A0 => bb_reg_1_port, A1 => n1, B0 => 
                           aa_reg_31_port, B1 => n557, C0 => n650, Y => n649);
   U711 : AOI31XL port map( A0 => n420, A1 => mult_func(0), A2 => mult_func(1),
                           B0 => n335, Y => n651);
   U712 : INVXL port map( A => n789, Y => n1030);
   U713 : AOI22XL port map( A0 => n782, A1 => count_reg_0_port, B0 => N815, B1 
                           => n555, Y => n789);
   U714 : AOI21XL port map( A0 => upper_reg_31_port, A1 => n326, B0 => n327, Y 
                           => n325);
   U715 : OAI32XL port map( A0 => n328, A1 => n329, A2 => n330, B0 => n1034, B1
                           => n331, Y => n327);
   U716 : INVXL port map( A => n788, Y => n1029);
   U717 : AOI22XL port map( A0 => n782, A1 => count_reg_4_port, B0 => N819, B1 
                           => n555, Y => n788);
   U718 : INVXL port map( A => n786, Y => n1027);
   U719 : AOI22XL port map( A0 => n782, A1 => count_reg_2_port, B0 => N817, B1 
                           => n555, Y => n786);
   U720 : INVXL port map( A => n787, Y => n1028);
   U721 : AOI22XL port map( A0 => n782, A1 => count_reg_1_port, B0 => N816, B1 
                           => n555, Y => n787);
   U722 : AOI32XL port map( A0 => n692, A1 => n693, A2 => n546, B0 => b(9), B1 
                           => n694, Y => n691);
   U723 : INVXL port map( A => b(9), Y => n693);
   U724 : OAI211XL port map( A0 => n297, A1 => n294, B0 => n298, C0 => n299, Y 
                           => n923);
   U725 : AOI2BB1XL port map( A0N => n293, A1N => n828, B0 => n1032, Y => n297)
                           ;
   U726 : AOI33XL port map( A0 => n303, A1 => n304, A2 => n19, B0 => n293, B1 
                           => n294, B2 => n106, Y => n298);
   U727 : AOI222XL port map( A0 => a(29), A1 => n350, B0 => upper_reg_30_port, 
                           B1 => n300, C0 => n301, C1 => n11, Y => n299);
   U728 : OAI211XL port map( A0 => n307, A1 => n304, B0 => n308, C0 => n309, Y 
                           => n924);
   U729 : AOI2BB1XL port map( A0N => n303, A1N => n828, B0 => n1032, Y => n307)
                           ;
   U730 : AOI33XL port map( A0 => n314, A1 => n320, A2 => n19, B0 => n303, B1 
                           => n304, B2 => n337, Y => n308);
   U731 : AOI222XL port map( A0 => a(30), A1 => n9, B0 => upper_reg_31_port, B1
                           => n310, C0 => n311, C1 => n11, Y => n309);
   U732 : INVXL port map( A => n785, Y => n1026);
   U733 : AOI22XL port map( A0 => n782, A1 => count_reg_3_port, B0 => N818, B1 
                           => n555, Y => n785);
   U734 : AOI32XL port map( A0 => n688, A1 => n689, A2 => n546, B0 => b(8), B1 
                           => n690, Y => n687);
   U735 : INVXL port map( A => b(8), Y => n689);
   U736 : AOI211XL port map( A0 => n792, A1 => n793, B0 => n794, C0 => n420, Y 
                           => n1031);
   U737 : AOI21XL port map( A0 => n335, A1 => n795, B0 => n792, Y => n794);
   U738 : NOR2XL port map( A => n654, B => n784, Y => n792);
   U739 : XNOR2XL port map( A => n549, B => a(31), Y => n795);
   U740 : OAI2BB1XL port map( A0N => n782, A1N => count_reg_5_port, B0 => n783,
                           Y => n1025);
   U741 : AOI21XL port map( A0 => N820, A1 => n555, B0 => n654, Y => n783);
   U742 : AOI32XL port map( A0 => n666, A1 => n667, A2 => n546, B0 => b(3), B1 
                           => n668, Y => n665);
   U743 : INVXL port map( A => b(3), Y => n667);
   U744 : AOI32XL port map( A0 => n670, A1 => n671, A2 => n546, B0 => b(4), B1 
                           => n672, Y => n669);
   U745 : INVXL port map( A => b(4), Y => n671);
   U746 : AOI32XL port map( A0 => n674, A1 => n675, A2 => n546, B0 => b(5), B1 
                           => n676, Y => n673);
   U747 : INVXL port map( A => b(5), Y => n675);
   U748 : AOI32XL port map( A0 => n678, A1 => n679, A2 => n546, B0 => b(6), B1 
                           => n680, Y => n677);
   U749 : INVXL port map( A => b(6), Y => n679);
   U750 : AOI32XL port map( A0 => n683, A1 => n684, A2 => n546, B0 => b(7), B1 
                           => n685, Y => n682);
   U751 : INVXL port map( A => b(7), Y => n684);
   U752 : AOI32XL port map( A0 => n776, A1 => n777, A2 => n546, B0 => b(30), B1
                           => n778, Y => n775);
   U753 : INVXL port map( A => n779, Y => n776);
   U754 : AOI33XL port map( A0 => n420, A1 => mult_func(0), A2 => b(31), B0 => 
                           n546, B1 => n777, B2 => n779, Y => n780);
   U755 : AOI22XL port map( A0 => n784, A1 => a(29), B0 => lower_reg_29_port, 
                           B1 => n345, Y => n353);
   U756 : AOI22XL port map( A0 => n784, A1 => a(27), B0 => lower_reg_27_port, 
                           B1 => n345, Y => n357);
   U757 : AOI22XL port map( A0 => n784, A1 => a(25), B0 => lower_reg_25_port, 
                           B1 => n345, Y => n361);
   U758 : AOI22XL port map( A0 => n784, A1 => a(23), B0 => lower_reg_23_port, 
                           B1 => n345, Y => n365);
   U759 : AOI22XL port map( A0 => n784, A1 => a(21), B0 => lower_reg_21_port, 
                           B1 => n345, Y => n369);
   U760 : AOI22XL port map( A0 => n784, A1 => a(30), B0 => n345, B1 => 
                           lower_reg_30_port, Y => n349);
   U761 : AOI22XL port map( A0 => n784, A1 => a(28), B0 => lower_reg_28_port, 
                           B1 => n345, Y => n355);
   U762 : AOI22XL port map( A0 => n784, A1 => a(26), B0 => lower_reg_26_port, 
                           B1 => n345, Y => n359);
   U763 : AOI22XL port map( A0 => n784, A1 => a(24), B0 => lower_reg_24_port, 
                           B1 => n345, Y => n363);
   U764 : AOI22XL port map( A0 => n784, A1 => a(22), B0 => lower_reg_22_port, 
                           B1 => n345, Y => n367);
   U765 : AOI32XL port map( A0 => b(0), A1 => n656, A2 => n546, B0 => b(1), B1 
                           => n658, Y => n655);
   U766 : AOI32XL port map( A0 => n662, A1 => n663, A2 => n546, B0 => b(2), B1 
                           => n664, Y => n661);
   U767 : INVXL port map( A => b(2), Y => n663);
   U768 : AOI32XL port map( A0 => n696, A1 => n697, A2 => n546, B0 => b(10), B1
                           => n698, Y => n695);
   U769 : INVXL port map( A => b(10), Y => n697);
   U770 : AOI32XL port map( A0 => n700, A1 => n701, A2 => n546, B0 => b(11), B1
                           => n702, Y => n699);
   U771 : INVXL port map( A => b(11), Y => n701);
   U772 : AOI32XL port map( A0 => n704, A1 => n705, A2 => n546, B0 => b(12), B1
                           => n706, Y => n703);
   U773 : INVXL port map( A => b(12), Y => n705);
   U774 : AOI32XL port map( A0 => n708, A1 => n709, A2 => n546, B0 => b(13), B1
                           => n710, Y => n707);
   U775 : INVXL port map( A => b(13), Y => n709);
   U776 : AOI32XL port map( A0 => n712, A1 => n713, A2 => n546, B0 => b(14), B1
                           => n714, Y => n711);
   U777 : INVXL port map( A => b(14), Y => n713);
   U778 : AOI32XL port map( A0 => n716, A1 => n717, A2 => n546, B0 => b(15), B1
                           => n718, Y => n715);
   U779 : INVXL port map( A => b(15), Y => n717);
   U780 : AOI32XL port map( A0 => n720, A1 => n721, A2 => n546, B0 => b(16), B1
                           => n722, Y => n719);
   U781 : INVXL port map( A => b(16), Y => n721);
   U782 : AOI32XL port map( A0 => n724, A1 => n725, A2 => n546, B0 => b(17), B1
                           => n726, Y => n723);
   U783 : INVXL port map( A => b(17), Y => n725);
   U784 : AOI32XL port map( A0 => n728, A1 => n729, A2 => n546, B0 => b(18), B1
                           => n730, Y => n727);
   U785 : INVXL port map( A => b(18), Y => n729);
   U786 : AOI32XL port map( A0 => n732, A1 => n733, A2 => n546, B0 => b(19), B1
                           => n734, Y => n731);
   U787 : INVXL port map( A => b(19), Y => n733);
   U788 : AOI32XL port map( A0 => n736, A1 => n737, A2 => n546, B0 => b(20), B1
                           => n738, Y => n735);
   U789 : INVXL port map( A => b(20), Y => n737);
   U790 : AOI32XL port map( A0 => n740, A1 => n741, A2 => n546, B0 => b(21), B1
                           => n742, Y => n739);
   U791 : INVXL port map( A => b(21), Y => n741);
   U792 : AOI32XL port map( A0 => n744, A1 => n745, A2 => n546, B0 => b(22), B1
                           => n746, Y => n743);
   U793 : INVXL port map( A => b(22), Y => n745);
   U794 : AOI32XL port map( A0 => n748, A1 => n749, A2 => n546, B0 => b(23), B1
                           => n750, Y => n747);
   U795 : INVXL port map( A => b(23), Y => n749);
   U796 : AOI32XL port map( A0 => n752, A1 => n753, A2 => n546, B0 => b(24), B1
                           => n754, Y => n751);
   U797 : INVXL port map( A => b(24), Y => n753);
   U798 : AOI32XL port map( A0 => n756, A1 => n757, A2 => n546, B0 => b(25), B1
                           => n758, Y => n755);
   U799 : INVXL port map( A => b(25), Y => n757);
   U800 : AOI32XL port map( A0 => n760, A1 => n761, A2 => n546, B0 => b(26), B1
                           => n762, Y => n759);
   U801 : INVXL port map( A => b(26), Y => n761);
   U802 : AOI32XL port map( A0 => n764, A1 => n765, A2 => n546, B0 => b(27), B1
                           => n766, Y => n763);
   U803 : INVXL port map( A => b(27), Y => n765);
   U804 : AOI32XL port map( A0 => n768, A1 => n769, A2 => n546, B0 => b(28), B1
                           => n770, Y => n767);
   U805 : INVXL port map( A => b(28), Y => n769);
   U806 : AOI32XL port map( A0 => n772, A1 => n773, A2 => n546, B0 => b(29), B1
                           => n774, Y => n771);
   U807 : INVXL port map( A => b(29), Y => n773);
   U808 : INVXL port map( A => reset_in, Y => n1037);
   U809 : AOI21XL port map( A0 => lower_reg_30_port, A1 => n340, B0 => n341, Y 
                           => n339);
   U810 : OAI33XL port map( A0 => n13, A1 => bb_reg_0_port, A2 => n343, B0 => 
                           n344, B1 => n345, B2 => n346, Y => n341);
   U811 : XNOR2XL port map( A => aa_reg_0_port, B => upper_reg_0_port, Y => 
                           n346);
   U812 : NAND4XL port map( A => a(31), B => n420, C => n548, D => n549, Y => 
                           n547);
   U813 : INVXL port map( A => reset_in, Y => n1038);
   U814 : OAI32XL port map( A0 => n540, A1 => n541, A2 => n542, B0 => n543, B1 
                           => n324, Y => n959);
   U815 : NOR2XL port map( A => n541, B => n544, Y => n543);
   U816 : INVXL port map( A => n545, Y => n541);
   U817 : OAI21XL port map( A0 => n1034, A1 => n90, B0 => n15, Y => n87);
   U818 : OAI21XL port map( A0 => n1034, A1 => n314, B0 => n15, Y => n310);
   U819 : OAI21XL port map( A0 => n1034, A1 => n14, B0 => n15, Y => n10);
   U820 : OAI21XL port map( A0 => n1034, A1 => n29, B0 => n15, Y => n26);
   U821 : OAI21XL port map( A0 => n1034, A1 => n40, B0 => n15, Y => n37);
   U822 : OAI21XL port map( A0 => n1034, A1 => n50, B0 => n15, Y => n47);
   U823 : OAI21XL port map( A0 => n1034, A1 => n60, B0 => n15, Y => n57);
   U824 : OAI21XL port map( A0 => n1034, A1 => n70, B0 => n15, Y => n67);
   U825 : OAI21XL port map( A0 => n1034, A1 => n80, B0 => n15, Y => n77);
   U826 : OAI21XL port map( A0 => n1034, A1 => n113, B0 => n15, Y => n110);
   U827 : OAI21XL port map( A0 => n1034, A1 => n123, B0 => n15, Y => n120);
   U828 : OAI21XL port map( A0 => n1034, A1 => n133, B0 => n15, Y => n130);
   U829 : OAI21XL port map( A0 => n1034, A1 => n143, B0 => n15, Y => n140);
   U830 : OAI21XL port map( A0 => n1034, A1 => n153, B0 => n15, Y => n150);
   U831 : OAI21XL port map( A0 => n1034, A1 => n163, B0 => n15, Y => n160);
   U832 : OAI21XL port map( A0 => n1034, A1 => n173, B0 => n15, Y => n170);
   U833 : OAI21XL port map( A0 => n1034, A1 => n183, B0 => n15, Y => n180);
   U834 : OAI21XL port map( A0 => n1034, A1 => n193, B0 => n15, Y => n190);
   U835 : OAI21XL port map( A0 => n1034, A1 => n203, B0 => n15, Y => n200);
   U836 : OAI21XL port map( A0 => n1034, A1 => n213, B0 => n15, Y => n210);
   U837 : OAI21XL port map( A0 => n1034, A1 => n223, B0 => n15, Y => n220);
   U838 : OAI21XL port map( A0 => n1034, A1 => n233, B0 => n15, Y => n230);
   U839 : OAI21XL port map( A0 => n1034, A1 => n243, B0 => n15, Y => n240);
   U840 : OAI21XL port map( A0 => n1034, A1 => n253, B0 => n15, Y => n250);
   U841 : OAI21XL port map( A0 => n1034, A1 => n263, B0 => n15, Y => n260);
   U842 : OAI21XL port map( A0 => n1034, A1 => n273, B0 => n15, Y => n270);
   U843 : OAI21XL port map( A0 => n1034, A1 => n283, B0 => n15, Y => n280);
   U844 : OAI21XL port map( A0 => n1034, A1 => n293, B0 => n15, Y => n290);
   U845 : OAI21XL port map( A0 => n1034, A1 => n303, B0 => n15, Y => n300);
   U846 : OAI21XL port map( A0 => n828, A1 => n90, B0 => n101, Y => n98);
   U847 : OAI21XL port map( A0 => n654, A1 => n657, B0 => n550, Y => n1033);
   U848 : OAI21XL port map( A0 => n343, A1 => n2, B0 => n894, Y => c_mult(0));
   U849 : OAI21XL port map( A0 => n320, A1 => n2, B0 => n822, Y => c_mult(31));
   U850 : OAI221XL port map( A0 => n410, A1 => n336, B0 => n802, B1 => n408, C0
                           => n411, Y => n957);
   U851 : OAI222XL port map( A0 => n551, A1 => n536, B0 => n425, B1 => n552, C0
                           => n421, C1 => n553, Y => n961);
   U852 : OAI21XL port map( A0 => n4, A1 => n344, B0 => n550, Y => n545);
   U853 : OAI21XL port map( A0 => n652, A1 => n549, B0 => n556, Y => n560);
   U854 : OAI21XL port map( A0 => n509, A1 => n92, B0 => n510, Y => n104);
   U855 : OAI21XL port map( A0 => n116, A1 => n115, B0 => n507, Y => n126);
   U856 : OAI21XL port map( A0 => n521, A1 => n32, B0 => n522, Y => n43);
   U857 : OAI21XL port map( A0 => n519, A1 => n42, B0 => n520, Y => n53);
   U858 : OAI21XL port map( A0 => n517, A1 => n52, B0 => n518, Y => n63);
   U859 : OAI21XL port map( A0 => n515, A1 => n62, B0 => n516, Y => n73);
   U860 : OAI21XL port map( A0 => n513, A1 => n72, B0 => n514, Y => n83);
   U861 : OAI21XL port map( A0 => n511, A1 => n82, B0 => n512, Y => n93);
   U862 : OAI21XL port map( A0 => n505, A1 => n125, B0 => n506, Y => n136);
   U863 : OAI21XL port map( A0 => n503, A1 => n135, B0 => n504, Y => n146);
   U864 : OAI21XL port map( A0 => n501, A1 => n145, B0 => n502, Y => n156);
   U865 : OAI21XL port map( A0 => n499, A1 => n155, B0 => n500, Y => n166);
   U866 : OAI21XL port map( A0 => n497, A1 => n165, B0 => n498, Y => n176);
   U867 : OAI21XL port map( A0 => n495, A1 => n175, B0 => n496, Y => n186);
   U868 : OAI21XL port map( A0 => n493, A1 => n185, B0 => n494, Y => n196);
   U869 : OAI21XL port map( A0 => n491, A1 => n195, B0 => n492, Y => n206);
   U870 : OAI21XL port map( A0 => n489, A1 => n205, B0 => n490, Y => n216);
   U871 : OAI21XL port map( A0 => n487, A1 => n215, B0 => n488, Y => n226);
   U872 : OAI21XL port map( A0 => n485, A1 => n225, B0 => n486, Y => n236);
   U873 : OAI21XL port map( A0 => n483, A1 => n235, B0 => n484, Y => n246);
   U874 : OAI21XL port map( A0 => n481, A1 => n245, B0 => n482, Y => n256);
   U875 : OAI21XL port map( A0 => n479, A1 => n255, B0 => n480, Y => n266);
   U876 : OAI21XL port map( A0 => n477, A1 => n265, B0 => n478, Y => n276);
   U877 : OAI21XL port map( A0 => n475, A1 => n275, B0 => n476, Y => n286);
   U878 : OAI21XL port map( A0 => n473, A1 => n285, B0 => n474, Y => n296);
   U879 : OAI21XL port map( A0 => n471, A1 => n295, B0 => n472, Y => n306);
   U880 : OAI21XL port map( A0 => n469, A1 => n305, B0 => n470, Y => n322);
   U881 : OAI21XL port map( A0 => n466, A1 => n321, B0 => n467, Y => n465);
   U882 : OAI21XL port map( A0 => n422, A1 => n423, B0 => n523, Y => n33);
   U883 : OAI221XL port map( A0 => n415, A1 => n343, B0 => n416, B1 => n6, C0 
                           => n417, Y => n958);
   U884 : OAI21XL port map( A0 => n294, A1 => n2, B0 => n833, Y => c_mult(29));
   U885 : OAI221XL port map( A0 => n312, A1 => n323, B0 => n15, B1 => n324, C0 
                           => n325, Y => n925);
   U886 : OAI21XL port map( A0 => n828, A1 => n314, B0 => n101, Y => n326);
   U887 : OAI221XL port map( A0 => n836, A1 => n352, B0 => n284, B1 => n2, C0 
                           => n837, Y => c_mult(28));
   U888 : OAI221XL port map( A0 => n826, A1 => n351, B0 => n304, B1 => n2, C0 
                           => n827, Y => c_mult(30));
   U889 : OAI21XL port map( A0 => n254, A1 => n2, B0 => n845, Y => c_mult(25));
   U890 : OAI221XL port map( A0 => n842, A1 => n356, B0 => n264, B1 => n2, C0 
                           => n843, Y => c_mult(26));
   U891 : OAI21XL port map( A0 => n274, A1 => n2, B0 => n839, Y => c_mult(27));
   U892 : OAI21XL port map( A0 => n234, A1 => n2, B0 => n851, Y => c_mult(23));
   U893 : OAI221XL port map( A0 => n848, A1 => n360, B0 => n244, B1 => n2, C0 
                           => n849, Y => c_mult(24));
   U894 : OAI21XL port map( A0 => n194, A1 => n2, B0 => n865, Y => c_mult(19));
   U895 : OAI221XL port map( A0 => n860, A1 => n368, B0 => n204, B1 => n2, C0 
                           => n861, Y => c_mult(20));
   U896 : OAI21XL port map( A0 => n214, A1 => n2, B0 => n857, Y => c_mult(21));
   U897 : OAI221XL port map( A0 => n854, A1 => n364, B0 => n224, B1 => n2, C0 
                           => n855, Y => c_mult(22));
   U898 : OAI21XL port map( A0 => n174, A1 => n2, B0 => n871, Y => c_mult(17));
   U899 : OAI221XL port map( A0 => n868, A1 => n372, B0 => n184, B1 => n2, C0 
                           => n869, Y => c_mult(18));
   U900 : OAI221XL port map( A0 => n880, A1 => n380, B0 => n144, B1 => n2, C0 
                           => n881, Y => c_mult(14));
   U901 : OAI21XL port map( A0 => n154, A1 => n2, B0 => n877, Y => c_mult(15));
   U902 : OAI221XL port map( A0 => n874, A1 => n376, B0 => n164, B1 => n2, C0 
                           => n875, Y => c_mult(16));
   U903 : OAI21XL port map( A0 => n114, A1 => n2, B0 => n889, Y => c_mult(11));
   U904 : OAI221XL port map( A0 => n886, A1 => n384, B0 => n124, B1 => n2, C0 
                           => n887, Y => c_mult(12));
   U905 : OAI21XL port map( A0 => n134, A1 => n2, B0 => n883, Y => c_mult(13));
   U906 : OAI21XL port map( A0 => n95, A1 => n2, B0 => n892, Y => c_mult(10));
   U907 : OAI221XL port map( A0 => n551, A1 => n437, B0 => n553, B1 => n436, C0
                           => n775, Y => n1023);
   U908 : OAI21XL port map( A0 => n659, A1 => n776, B0 => n660, Y => n778);
   U909 : OAI221XL port map( A0 => n551, A1 => n450, B0 => n553, B1 => n449, C0
                           => n767, Y => n1021);
   U910 : OAI21XL port map( A0 => n659, A1 => n768, B0 => n660, Y => n770);
   U911 : OAI221XL port map( A0 => n551, A1 => n436, B0 => n553, B1 => n450, C0
                           => n771, Y => n1022);
   U912 : OAI21XL port map( A0 => n659, A1 => n772, B0 => n660, Y => n774);
   U913 : OAI21XL port map( A0 => n91, A1 => n2, B0 => n798, Y => c_mult(9));
   U914 : OAI21XL port map( A0 => n553, A1 => n437, B0 => n780, Y => n1024);
   U915 : OAI21XL port map( A0 => n468, A1 => n829, B0 => n803, Y => n834);
   U916 : OAI21XL port map( A0 => n468, A1 => n823, B0 => n803, Y => n824);
   U917 : OAI21XL port map( A0 => n6, A1 => n2, B0 => n863, Y => c_mult(1));
   U918 : OAI21XL port map( A0 => n81, A1 => n2, B0 => n804, Y => c_mult(8));
   U919 : OAI221XL port map( A0 => n551, A1 => n445, B0 => n553, B1 => n444, C0
                           => n747, Y => n1016);
   U920 : OAI21XL port map( A0 => n659, A1 => n748, B0 => n660, Y => n750);
   U921 : OAI221XL port map( A0 => n551, A1 => n446, B0 => n553, B1 => n445, C0
                           => n751, Y => n1017);
   U922 : OAI21XL port map( A0 => n659, A1 => n752, B0 => n660, Y => n754);
   U923 : OAI221XL port map( A0 => n551, A1 => n447, B0 => n553, B1 => n446, C0
                           => n755, Y => n1018);
   U924 : OAI21XL port map( A0 => n659, A1 => n756, B0 => n660, Y => n758);
   U925 : OAI221XL port map( A0 => n551, A1 => n448, B0 => n553, B1 => n447, C0
                           => n759, Y => n1019);
   U926 : OAI21XL port map( A0 => n659, A1 => n760, B0 => n660, Y => n762);
   U927 : OAI221XL port map( A0 => n551, A1 => n449, B0 => n553, B1 => n448, C0
                           => n763, Y => n1020);
   U928 : OAI21XL port map( A0 => n659, A1 => n764, B0 => n660, Y => n766);
   U929 : OAI21XL port map( A0 => n61, A1 => n2, B0 => n810, Y => c_mult(6));
   U930 : OAI21XL port map( A0 => n71, A1 => n2, B0 => n807, Y => c_mult(7));
   U931 : OAI21XL port map( A0 => n468, A1 => n850, B0 => n803, Y => n852);
   U932 : OAI21XL port map( A0 => n468, A1 => n844, B0 => n803, Y => n846);
   U933 : OAI21XL port map( A0 => n468, A1 => n838, B0 => n803, Y => n840);
   U934 : OR3XL port map( A => n847, B => lower_reg_24_port, C => n468, Y => 
                           n849);
   U935 : OR3XL port map( A => n841, B => lower_reg_26_port, C => n468, Y => 
                           n843);
   U936 : OR3XL port map( A => n835, B => lower_reg_28_port, C => n468, Y => 
                           n837);
   U937 : OAI21XL port map( A0 => n30, A1 => n2, B0 => n8190, Y => c_mult(3));
   U938 : OAI21XL port map( A0 => n51, A1 => n2, B0 => n813, Y => c_mult(5));
   U939 : OAI221XL port map( A0 => n323, A1 => n22, B0 => n540, B1 => n545, C0 
                           => n547, Y => n960);
   U940 : OAI221XL port map( A0 => n643, A1 => n560, B0 => n302, B1 => n22, C0 
                           => n644, Y => n991);
   U941 : OAI221XL port map( A0 => n313, A1 => n22, B0 => n646, B1 => n560, C0 
                           => n647, Y => n992);
   U942 : OAI221XL port map( A0 => n551, A1 => n537, B0 => n553, B1 => n535, C0
                           => n731, Y => n1012);
   U943 : OAI21XL port map( A0 => n659, A1 => n732, B0 => n660, Y => n734);
   U944 : OAI221XL port map( A0 => n551, A1 => n538, B0 => n553, B1 => n537, C0
                           => n735, Y => n1013);
   U945 : OAI21XL port map( A0 => n659, A1 => n736, B0 => n660, Y => n738);
   U946 : OAI221XL port map( A0 => n551, A1 => n539, B0 => n553, B1 => n538, C0
                           => n739, Y => n1014);
   U947 : OAI21XL port map( A0 => n659, A1 => n740, B0 => n660, Y => n742);
   U948 : OAI221XL port map( A0 => n551, A1 => n444, B0 => n553, B1 => n539, C0
                           => n743, Y => n1015);
   U949 : OAI21XL port map( A0 => n659, A1 => n744, B0 => n660, Y => n746);
   U950 : OAI21XL port map( A0 => n41, A1 => n2, B0 => n8160, Y => c_mult(4));
   U951 : OAI222XL port map( A0 => n22, A1 => n323, B0 => n552, B1 => n651, C0 
                           => n560, C1 => n330, Y => n650);
   U952 : OAI21XL port map( A0 => n468, A1 => n876, B0 => n803, Y => n878);
   U953 : OAI21XL port map( A0 => n468, A1 => n870, B0 => n803, Y => n872);
   U954 : OAI21XL port map( A0 => n468, A1 => n862, B0 => n803, Y => n866);
   U955 : OAI21XL port map( A0 => n468, A1 => n856, B0 => n803, Y => n858);
   U956 : OAI21XL port map( A0 => lower_reg_0_port, A1 => n468, B0 => n803, Y 
                           => n864);
   U957 : OR3XL port map( A => n873, B => lower_reg_16_port, C => n468, Y => 
                           n875);
   U958 : OR3XL port map( A => n867, B => lower_reg_18_port, C => n468, Y => 
                           n869);
   U959 : OR3XL port map( A => n859, B => lower_reg_20_port, C => n468, Y => 
                           n861);
   U960 : OR3XL port map( A => n853, B => lower_reg_22_port, C => n468, Y => 
                           n855);
   U961 : OAI221XL port map( A0 => mult_func(3), A1 => n796, B0 => mult_func(2)
                           , B1 => n797, C0 => n316, Y => n425);
   U962 : OAI221XL port map( A0 => n252, A1 => n22, B0 => n628, B1 => n560, C0 
                           => n629, Y => n986);
   U963 : OAI221XL port map( A0 => n242, A1 => n22, B0 => n625, B1 => n560, C0 
                           => n626, Y => n985);
   U964 : OAI221XL port map( A0 => n222, A1 => n22, B0 => n619, B1 => n560, C0 
                           => n620, Y => n983);
   U965 : OAI221XL port map( A0 => n232, A1 => n22, B0 => n622, B1 => n560, C0 
                           => n623, Y => n984);
   U966 : OAI221XL port map( A0 => n262, A1 => n22, B0 => n631, B1 => n560, C0 
                           => n632, Y => n987);
   U967 : OAI221XL port map( A0 => n272, A1 => n22, B0 => n634, B1 => n560, C0 
                           => n635, Y => n988);
   U968 : OAI221XL port map( A0 => n282, A1 => n22, B0 => n637, B1 => n560, C0 
                           => n638, Y => n989);
   U969 : OAI221XL port map( A0 => n292, A1 => n22, B0 => n640, B1 => n560, C0 
                           => n641, Y => n990);
   U970 : OAI221XL port map( A0 => n551, A1 => n456, B0 => n553, B1 => n457, C0
                           => n711, Y => n1007);
   U971 : OAI21XL port map( A0 => n659, A1 => n712, B0 => n660, Y => n714);
   U972 : OAI221XL port map( A0 => n551, A1 => n532, B0 => n553, B1 => n456, C0
                           => n715, Y => n1008);
   U973 : OAI21XL port map( A0 => n659, A1 => n716, B0 => n660, Y => n718);
   U974 : OAI221XL port map( A0 => n551, A1 => n533, B0 => n553, B1 => n532, C0
                           => n719, Y => n1009);
   U975 : OAI21XL port map( A0 => n659, A1 => n720, B0 => n660, Y => n722);
   U976 : OAI221XL port map( A0 => n551, A1 => n534, B0 => n553, B1 => n533, C0
                           => n723, Y => n1010);
   U977 : OAI21XL port map( A0 => n659, A1 => n724, B0 => n660, Y => n726);
   U978 : OAI221XL port map( A0 => n551, A1 => n535, B0 => n553, B1 => n534, C0
                           => n727, Y => n1011);
   U979 : OAI21XL port map( A0 => n659, A1 => n728, B0 => n660, Y => n730);
   U980 : OAI21XL port map( A0 => n800, A1 => n31, B0 => lower_reg_0_port, Y =>
                           n894);
   U981 : OAI21XL port map( A0 => n18, A1 => n2, B0 => n830, Y => c_mult(2));
   U982 : OAI21XL port map( A0 => n468, A1 => n891, B0 => n803, Y => n893);
   U983 : OAI21XL port map( A0 => n468, A1 => n888, B0 => n803, Y => n890);
   U984 : OAI21XL port map( A0 => n468, A1 => n882, B0 => n803, Y => n884);
   U985 : OR3XL port map( A => n885, B => lower_reg_12_port, C => n468, Y => 
                           n887);
   U986 : OR3XL port map( A => n879, B => lower_reg_14_port, C => n468, Y => 
                           n881);
   U987 : OAI221XL port map( A0 => n13, A1 => n338, B0 => n347, B1 => n348, C0 
                           => n349, Y => n927);
   U988 : OAI221XL port map( A0 => n13, A1 => n351, B0 => n347, B1 => n352, C0 
                           => n353, Y => n928);
   U989 : OAI221XL port map( A0 => n13, A1 => n348, B0 => n347, B1 => n354, C0 
                           => n355, Y => n929);
   U990 : OAI221XL port map( A0 => n13, A1 => n352, B0 => n347, B1 => n356, C0 
                           => n357, Y => n930);
   U991 : OAI221XL port map( A0 => n13, A1 => n354, B0 => n347, B1 => n358, C0 
                           => n359, Y => n931);
   U992 : OAI221XL port map( A0 => n13, A1 => n356, B0 => n347, B1 => n360, C0 
                           => n361, Y => n932);
   U993 : OAI221XL port map( A0 => n13, A1 => n358, B0 => n347, B1 => n362, C0 
                           => n363, Y => n933);
   U994 : OAI221XL port map( A0 => n13, A1 => n360, B0 => n347, B1 => n364, C0 
                           => n365, Y => n934);
   U995 : OAI221XL port map( A0 => n13, A1 => n362, B0 => n347, B1 => n366, C0 
                           => n367, Y => n935);
   U996 : OAI221XL port map( A0 => n13, A1 => n364, B0 => n347, B1 => n368, C0 
                           => n369, Y => n936);
   U997 : OAI221XL port map( A0 => n13, A1 => n366, B0 => n347, B1 => n370, C0 
                           => n371, Y => n937);
   U998 : OAI221XL port map( A0 => n13, A1 => n368, B0 => n347, B1 => n372, C0 
                           => n373, Y => n938);
   U999 : OAI221XL port map( A0 => n13, A1 => n370, B0 => n347, B1 => n374, C0 
                           => n375, Y => n939);
   U1000 : OAI221XL port map( A0 => n13, A1 => n372, B0 => n347, B1 => n376, C0
                           => n377, Y => n940);
   U1001 : OAI221XL port map( A0 => n13, A1 => n374, B0 => n347, B1 => n378, C0
                           => n379, Y => n941);
   U1002 : OAI221XL port map( A0 => n13, A1 => n376, B0 => n347, B1 => n380, C0
                           => n381, Y => n942);
   U1003 : OAI221XL port map( A0 => n13, A1 => n378, B0 => n347, B1 => n382, C0
                           => n383, Y => n943);
   U1004 : OAI221XL port map( A0 => n13, A1 => n380, B0 => n347, B1 => n384, C0
                           => n385, Y => n944);
   U1005 : OAI221XL port map( A0 => n13, A1 => n382, B0 => n347, B1 => n386, C0
                           => n387, Y => n945);
   U1006 : OAI221XL port map( A0 => n13, A1 => n384, B0 => n347, B1 => n388, C0
                           => n389, Y => n946);
   U1007 : OAI221XL port map( A0 => n13, A1 => n386, B0 => n347, B1 => n390, C0
                           => n391, Y => n947);
   U1008 : OAI221XL port map( A0 => n13, A1 => n388, B0 => n347, B1 => n392, C0
                           => n393, Y => n948);
   U1009 : OAI221XL port map( A0 => n13, A1 => n390, B0 => n347, B1 => n394, C0
                           => n395, Y => n949);
   U1010 : OAI221XL port map( A0 => n13, A1 => n392, B0 => n347, B1 => n396, C0
                           => n397, Y => n950);
   U1011 : OAI221XL port map( A0 => n13, A1 => n394, B0 => n347, B1 => n398, C0
                           => n399, Y => n951);
   U1012 : OAI221XL port map( A0 => n13, A1 => n396, B0 => n347, B1 => n400, C0
                           => n401, Y => n952);
   U1013 : OAI221XL port map( A0 => n13, A1 => n398, B0 => n347, B1 => n402, C0
                           => n403, Y => n953);
   U1014 : OAI221XL port map( A0 => n13, A1 => n400, B0 => n347, B1 => n404, C0
                           => n405, Y => n954);
   U1015 : OAI221XL port map( A0 => n13, A1 => n402, B0 => n347, B1 => n406, C0
                           => n407, Y => n955);
   U1016 : OAI221XL port map( A0 => n13, A1 => n404, B0 => n347, B1 => n408, C0
                           => n409, Y => n956);
   U1017 : OAI221XL port map( A0 => n39, A1 => n22, B0 => n565, B1 => n560, C0 
                           => n566, Y => n965);
   U1018 : OAI221XL port map( A0 => n558, A1 => n22, B0 => n559, B1 => n560, C0
                           => n561, Y => n963);
   U1019 : OAI221XL port map( A0 => n28, A1 => n22, B0 => n562, B1 => n560, C0 
                           => n563, Y => n964);
   U1020 : OAI221XL port map( A0 => n89, A1 => n22, B0 => n580, B1 => n560, C0 
                           => n581, Y => n970);
   U1021 : OAI221XL port map( A0 => n172, A1 => n22, B0 => n604, B1 => n560, C0
                           => n605, Y => n978);
   U1022 : OAI221XL port map( A0 => n122, A1 => n22, B0 => n589, B1 => n560, C0
                           => n590, Y => n973);
   U1023 : OAI221XL port map( A0 => n202, A1 => n22, B0 => n613, B1 => n560, C0
                           => n614, Y => n981);
   U1024 : OAI221XL port map( A0 => n79, A1 => n22, B0 => n577, B1 => n560, C0 
                           => n578, Y => n969);
   U1025 : OAI221XL port map( A0 => n162, A1 => n22, B0 => n601, B1 => n560, C0
                           => n602, Y => n977);
   U1026 : OAI221XL port map( A0 => n49, A1 => n22, B0 => n568, B1 => n560, C0 
                           => n569, Y => n966);
   U1027 : OAI221XL port map( A0 => n59, A1 => n22, B0 => n571, B1 => n560, C0 
                           => n572, Y => n967);
   U1028 : OAI221XL port map( A0 => n69, A1 => n22, B0 => n574, B1 => n560, C0 
                           => n575, Y => n968);
   U1029 : OAI221XL port map( A0 => n100, A1 => n22, B0 => n583, B1 => n560, C0
                           => n584, Y => n971);
   U1030 : OAI221XL port map( A0 => n112, A1 => n22, B0 => n586, B1 => n560, C0
                           => n587, Y => n972);
   U1031 : OAI221XL port map( A0 => n132, A1 => n22, B0 => n592, B1 => n560, C0
                           => n593, Y => n974);
   U1032 : OAI221XL port map( A0 => n142, A1 => n22, B0 => n595, B1 => n560, C0
                           => n596, Y => n975);
   U1033 : OAI221XL port map( A0 => n152, A1 => n22, B0 => n598, B1 => n560, C0
                           => n599, Y => n976);
   U1034 : OAI221XL port map( A0 => n182, A1 => n22, B0 => n607, B1 => n560, C0
                           => n608, Y => n979);
   U1035 : OAI221XL port map( A0 => n192, A1 => n22, B0 => n610, B1 => n560, C0
                           => n611, Y => n980);
   U1036 : OAI221XL port map( A0 => n212, A1 => n22, B0 => n616, B1 => n560, C0
                           => n617, Y => n982);
   U1037 : OAI221XL port map( A0 => n551, A1 => n451, B0 => n536, B1 => n553, 
                           C0 => n655, Y => n994);
   U1038 : OAI21XL port map( A0 => b(0), A1 => n659, B0 => n660, Y => n658);
   U1039 : OAI221XL port map( A0 => n551, A1 => n438, B0 => n553, B1 => n451, 
                           C0 => n661, Y => n995);
   U1040 : OAI21XL port map( A0 => n659, A1 => n662, B0 => n660, Y => n664);
   U1041 : OAI221XL port map( A0 => n551, A1 => n439, B0 => n553, B1 => n438, 
                           C0 => n665, Y => n996);
   U1042 : OAI21XL port map( A0 => n659, A1 => n666, B0 => n660, Y => n668);
   U1043 : OAI221XL port map( A0 => n551, A1 => n440, B0 => n553, B1 => n439, 
                           C0 => n669, Y => n997);
   U1044 : OAI21XL port map( A0 => n659, A1 => n670, B0 => n660, Y => n672);
   U1045 : OAI221XL port map( A0 => n551, A1 => n441, B0 => n553, B1 => n440, 
                           C0 => n673, Y => n998);
   U1046 : OAI21XL port map( A0 => n659, A1 => n674, B0 => n660, Y => n676);
   U1047 : OAI221XL port map( A0 => n551, A1 => n443, B0 => n553, B1 => n441, 
                           C0 => n677, Y => n999);
   U1048 : OAI21XL port map( A0 => n659, A1 => n678, B0 => n660, Y => n680);
   U1049 : OAI221XL port map( A0 => n551, A1 => n681, B0 => n553, B1 => n443, 
                           C0 => n682, Y => n1000);
   U1050 : OAI21XL port map( A0 => n659, A1 => n683, B0 => n660, Y => n685);
   U1051 : OAI221XL port map( A0 => n551, A1 => n686, B0 => n553, B1 => n681, 
                           C0 => n687, Y => n1001);
   U1052 : OAI21XL port map( A0 => n659, A1 => n688, B0 => n660, Y => n690);
   U1053 : OAI221XL port map( A0 => n551, A1 => n452, B0 => n553, B1 => n686, 
                           C0 => n691, Y => n1002);
   U1054 : OAI21XL port map( A0 => n659, A1 => n692, B0 => n660, Y => n694);
   U1055 : OAI221XL port map( A0 => n551, A1 => n453, B0 => n553, B1 => n452, 
                           C0 => n695, Y => n1003);
   U1056 : OAI21XL port map( A0 => n659, A1 => n696, B0 => n660, Y => n698);
   U1057 : OAI221XL port map( A0 => n551, A1 => n459, B0 => n553, B1 => n453, 
                           C0 => n699, Y => n1004);
   U1058 : OAI21XL port map( A0 => n659, A1 => n700, B0 => n660, Y => n702);
   U1059 : OAI221XL port map( A0 => n551, A1 => n458, B0 => n553, B1 => n459, 
                           C0 => n703, Y => n1005);
   U1060 : OAI21XL port map( A0 => n659, A1 => n704, B0 => n660, Y => n706);
   U1061 : OAI221XL port map( A0 => n551, A1 => n457, B0 => n553, B1 => n458, 
                           C0 => n707, Y => n1006);
   U1062 : OAI21XL port map( A0 => n659, A1 => n708, B0 => n660, Y => n710);
   U1063 : OAI221XL port map( A0 => n330, A1 => n336, B0 => n802, B1 => n338, 
                           C0 => n339, Y => n926);
   U1064 : OAI21XL port map( A0 => n831, A1 => n468, B0 => n803, Y => n832);
   U1065 : OAI21XL port map( A0 => n8200, A1 => n468, B0 => n803, Y => n821);
   U1066 : OAI21XL port map( A0 => n8170, A1 => n468, B0 => n803, Y => n8180);
   U1067 : OAI21XL port map( A0 => n814, A1 => n468, B0 => n803, Y => n8150);
   U1068 : OAI21XL port map( A0 => n811, A1 => n468, B0 => n803, Y => n812);
   U1069 : OAI21XL port map( A0 => n808, A1 => n468, B0 => n803, Y => n809);
   U1070 : OAI21XL port map( A0 => n805, A1 => n468, B0 => n803, Y => n806);
   U1071 : OAI21XL port map( A0 => n468, A1 => n799, B0 => n803, Y => n801);
   U1072 : OAI21XL port map( A0 => mult_func(0), A1 => n549, B0 => n654, Y => 
                           n660);
   U1073 : OAI21XL port map( A0 => a(31), A1 => n318, B0 => n319, Y => n317);

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity shifter_shifter_typeDEFAULT is

   port( value : in std_logic_vector (31 downto 0);  shift_amount : in 
         std_logic_vector (4 downto 0);  shift_func : in std_logic_vector (1 
         downto 0);  c_shift : out std_logic_vector (31 downto 0));

end shifter_shifter_typeDEFAULT;

architecture SYN_logic of shifter_shifter_typeDEFAULT is

   component OAI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI222XL
      port( A0, A1, B0, B1, C0, C1 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB2XL
      port( B0, B1, A0N, A1N : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI222XL
      port( A0, A1, B0, B1, C0, C1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI32XL
      port( A0, A1, A2, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   signal n94, n95, n96, n97, n98, n99, n100, n101, n102, n103, n105, n107, 
      n108, n109, n110, n111, n112, n113, n114, n115, n116, n117, n118, n119, 
      n120, n121, n122, n123, n124, n125, n126, n127, n128, n129, n130, n131, 
      n132, n133, n134, n135, n136, n137, n138, n139, n140, n141, n142, n143, 
      n144, n145, n146, n147, n148, n149, n150, n151, n152, n153, n154, n155, 
      n156, n157, n158, n161, n162, n163, n164, n165, n166, n167, n168, n169, 
      n170, n171, n172, n173, n174, n175, n176, n177, n178, n179, n180, n181, 
      n182, n183, n184, n185, n186, n187, n188, n189, n190, n191, n192, n193, 
      n194, n195, n196, n197, n198, n199, n200, n201, n202, n203, n204, n205, 
      n206, n207, n208, n209, n210, n211, n212, n213, n214, n215, n216, n217, 
      n218, n219, n220, n221, n222, n223, n224, n225, n226, n227, n228, n229, 
      n230, n231, n232, n233, n234, n235, n236, n237, n238, n239, n240, n241, 
      n242, n243, n245, n246, n249, n250, n251, n252, n253, n254, n255, n256, 
      n257, n258, n259, n260, n261, n262, n263, n264, n265, n266, n267, n268, 
      n270, n272, n273, n274, n275, n276, n277, n280, n283, n284, n285, n286, 
      n287, n288, n289, n290, n291, n292, n293, n294, n295, n296, n297, n298, 
      n299, n300, n301, n302, n303, n304, n305, n306, n307, n308, n309, n310, 
      n311, n312, n313, n314, n315, n316, n317, n318, n319, n321, n322, n323, 
      n324, n325, n326, n327, n328, n329, n330, n331, n332, n333, n334, n335, 
      n336, n337, n338, n339, n340, n341, n342, n343, n344, n345, n346, n347, 
      n348, n349, n350, n351, n352, n353, n354, n355, n356, n357, n358, n359, 
      n360, n361, n362, n363, n364, n365, n366, n367, n368, n369, n370, n371, 
      n372, n373, n374, n375, n376, n377, n378, n379, n380, n381, n382, n383, 
      n384, n385, n386, n387, n388, n389, n390, n391, n392, n393, n394, n395, 
      n396, n397, n398, n399, n400, n401, n402, n403, n404, n405, n406, n407, 
      n408, n409, n410, n411, n412, n413, n414, n415, n416, n417, n418, n419, 
      n420, n421, n422, n423, n424, n425, n426, n427, n428, n429, n430, n431, 
      n432, n433, n434, n435, n436, n437, n438, n439, n440, n441, n442, n443, 
      n444, n445, n446, n447, n448, n449, n450, n451, n1, n2, n3, n4, n5, n6, 
      n7, n8, n9, n10, n11, n12, n13, n14, n15, n16, n17, n18 : std_logic;

begin
   
   U2 : INVXL port map( A => value(31), Y => n394);
   U3 : NAND2XL port map( A => shift_amount(1), B => n451, Y => n156);
   U4 : NAND2XL port map( A => shift_amount(1), B => shift_amount(0), Y => n154
                           );
   U5 : INVXL port map( A => shift_amount(0), Y => n451);
   U6 : INVXL port map( A => shift_amount(1), Y => n431);
   U7 : INVXL port map( A => value(30), Y => n420);
   U8 : AOI22XL port map( A0 => value(29), A1 => n13, B0 => value(28), B1 => 
                           n11, Y => n448);
   U9 : AOI22XL port map( A0 => value(30), A1 => n14, B0 => value(29), B1 => 
                           n12, Y => n395);
   U10 : INVXL port map( A => value(20), Y => n396);
   U11 : INVXL port map( A => n349, Y => n295);
   U12 : INVXL port map( A => n408, Y => n286);
   U13 : INVXL port map( A => n409, Y => n287);
   U14 : INVXL port map( A => shift_amount(2), Y => n411);
   U15 : INVXL port map( A => value(9), Y => n166);
   U16 : INVXL port map( A => n296, Y => n322);
   U17 : NAND2XL port map( A => n329, B => n163, Y => n296);
   U18 : INVXL port map( A => n358, Y => n308);
   U19 : INVXL port map( A => n18, Y => n17);
   U20 : INVXL port map( A => value(7), Y => n337);
   U21 : INVXL port map( A => value(8), Y => n206);
   U22 : INVXL port map( A => n299, Y => n347);
   U23 : INVXL port map( A => n298, Y => n377);
   U24 : INVXL port map( A => value(13), Y => n372);
   U25 : INVXL port map( A => value(17), Y => n326);
   U26 : INVXL port map( A => n156, Y => n16);
   U27 : INVXL port map( A => n154, Y => n18);
   U28 : NAND2XL port map( A => n329, B => n1, Y => n378);
   U29 : NAND2XL port map( A => value(31), B => shift_func(0), Y => n393);
   U30 : INVXL port map( A => n367, Y => n268);
   U31 : INVXL port map( A => n366, Y => n270);
   U32 : INVXL port map( A => value(10), Y => n165);
   U33 : INVXL port map( A => value(14), Y => n362);
   U34 : INVXL port map( A => value(18), Y => n317);
   U35 : INVXL port map( A => value(19), Y => n318);
   U36 : INVXL port map( A => value(23), Y => n254);
   U37 : INVXL port map( A => value(24), Y => n241);
   U38 : NAND2XL port map( A => n411, B => n143, Y => n1);
   U39 : INVXL port map( A => n311, Y => n356);
   U40 : INVXL port map( A => n310, Y => n384);
   U41 : NAND2XL port map( A => n329, B => shift_amount(3), Y => n272);
   U42 : INVXL port map( A => value(11), Y => n340);
   U43 : INVXL port map( A => value(12), Y => n339);
   U44 : INVXL port map( A => n5, Y => n14);
   U45 : INVXL port map( A => value(15), Y => n354);
   U46 : INVXL port map( A => value(16), Y => n345);
   U47 : INVXL port map( A => value(22), Y => n262);
   U48 : INVXL port map( A => value(21), Y => n305);
   U49 : AOI222XL port map( A0 => n180, A1 => n162, B0 => n181, B1 => n163, C0 
                           => n182, C1 => n164, Y => n179);
   U50 : AOI22XL port map( A0 => value(30), A1 => n14, B0 => value(31), B1 => 
                           n11, Y => n178);
   U51 : INVXL port map( A => value(29), Y => n177);
   U52 : AOI222XL port map( A0 => n216, A1 => n162, B0 => n217, B1 => n163, C0 
                           => n218, C1 => n164, Y => n215);
   U53 : AOI22XL port map( A0 => value(28), A1 => n14, B0 => value(29), B1 => 
                           n12, Y => n214);
   U54 : AOI222XL port map( A0 => n229, A1 => n162, B0 => n230, B1 => n163, C0 
                           => n231, C1 => n164, Y => n228);
   U55 : INVXL port map( A => value(25), Y => n226);
   U56 : AOI22XL port map( A0 => value(27), A1 => n13, B0 => value(28), B1 => 
                           n12, Y => n227);
   U57 : INVXL port map( A => value(26), Y => n213);
   U58 : INVXL port map( A => n4, Y => n12);
   U59 : AOI222XL port map( A0 => n103, A1 => n162, B0 => n105, B1 => n163, C0 
                           => n135, C1 => n164, Y => n336);
   U60 : INVXL port map( A => value(3), Y => n334);
   U61 : INVXL port map( A => n114, Y => n441);
   U62 : INVXL port map( A => n113, Y => n440);
   U63 : NAND2XL port map( A => shift_amount(3), B => shift_amount(2), Y => 
                           n374);
   U64 : NAND2XL port map( A => shift_amount(2), B => n143, Y => n2);
   U65 : NAND2XL port map( A => shift_amount(3), B => n411, Y => n3);
   U66 : AOI22XL port map( A0 => value(8), A1 => n14, B0 => value(7), B1 => n12
                           , Y => n167);
   U67 : INVXL port map( A => n4, Y => n11);
   U68 : INVXL port map( A => n5, Y => n13);
   U69 : AOI222XL port map( A0 => n122, A1 => n162, B0 => n119, B1 => n163, C0 
                           => n121, C1 => n164, Y => n161);
   U70 : INVXL port map( A => value(6), Y => n155);
   U71 : AOI22XL port map( A0 => value(6), A1 => n14, B0 => value(5), B1 => n12
                           , Y => n338);
   U72 : AOI22XL port map( A0 => value(10), A1 => n14, B0 => value(9), B1 => 
                           n12, Y => n341);
   U73 : AOI22XL port map( A0 => value(9), A1 => n13, B0 => value(8), B1 => n11
                           , Y => n444);
   U74 : AOI222XL port map( A0 => n130, A1 => n162, B0 => n127, B1 => n163, C0 
                           => n129, C1 => n164, Y => n205);
   U75 : INVXL port map( A => value(4), Y => n203);
   U76 : INVXL port map( A => value(5), Y => n157);
   U77 : NAND2XL port map( A => n451, B => n431, Y => n4);
   U78 : NAND2XL port map( A => shift_amount(0), B => n431, Y => n5);
   U79 : INVXL port map( A => value(0), Y => n402);
   U80 : INVXL port map( A => value(2), Y => n404);
   U81 : INVXL port map( A => value(1), Y => n401);
   U82 : AOI22XL port map( A0 => value(7), A1 => n14, B0 => value(6), B1 => n12
                           , Y => n207);
   U83 : AOI22XL port map( A0 => value(11), A1 => n13, B0 => value(10), B1 => 
                           n11, Y => n428);
   U84 : AOI22XL port map( A0 => value(28), A1 => n13, B0 => value(27), B1 => 
                           n11, Y => n421);
   U85 : AOI22XL port map( A0 => value(12), A1 => n13, B0 => value(11), B1 => 
                           n12, Y => n417);
   U86 : AOI22XL port map( A0 => value(20), A1 => n13, B0 => value(19), B1 => 
                           n11, Y => n419);
   U87 : AOI22XL port map( A0 => value(26), A1 => n18, B0 => value(25), B1 => 
                           n16, Y => n418);
   U88 : AOI22XL port map( A0 => value(16), A1 => n13, B0 => value(15), B1 => 
                           n11, Y => n422);
   U89 : INVXL port map( A => n112, Y => n445);
   U90 : AOI22XL port map( A0 => n163, A1 => n409, B0 => n266, B1 => n111, Y =>
                           n446);
   U91 : AOI22XL port map( A0 => value(3), A1 => n13, B0 => value(4), B1 => n11
                           , Y => n412);
   U92 : AOI22XL port map( A0 => value(21), A1 => n14, B0 => value(20), B1 => 
                           n12, Y => n449);
   U93 : AOI22XL port map( A0 => value(27), A1 => n18, B0 => value(26), B1 => 
                           n16, Y => n450);
   U94 : AOI22XL port map( A0 => value(13), A1 => n13, B0 => value(12), B1 => 
                           n11, Y => n443);
   U95 : AOI22XL port map( A0 => value(17), A1 => n13, B0 => value(16), B1 => 
                           n11, Y => n447);
   U96 : INVXL port map( A => n101, Y => n364);
   U97 : AOI22XL port map( A0 => n163, A1 => n366, B0 => n266, B1 => n99, Y => 
                           n365);
   U98 : AOI22XL port map( A0 => value(4), A1 => n13, B0 => value(5), B1 => n11
                           , Y => n405);
   U99 : AOI22XL port map( A0 => value(22), A1 => n14, B0 => value(21), B1 => 
                           n12, Y => n392);
   U100 : AOI22XL port map( A0 => value(26), A1 => n14, B0 => value(25), B1 => 
                           n11, Y => n391);
   U101 : AOI22XL port map( A0 => value(14), A1 => n14, B0 => value(13), B1 => 
                           n12, Y => n390);
   U102 : AOI22XL port map( A0 => value(18), A1 => n14, B0 => value(17), B1 => 
                           n12, Y => n397);
   U103 : INVXL port map( A => n2, Y => n164);
   U104 : INVXL port map( A => n3, Y => n162);
   U105 : INVXL port map( A => n374, Y => n163);
   U106 : INVXL port map( A => n1, Y => n266);
   U107 : AOI22XL port map( A0 => n13, A1 => value(23), B0 => n12, B1 => 
                           value(22), Y => n429);
   U108 : AOI22XL port map( A0 => value(27), A1 => n13, B0 => value(26), B1 => 
                           n11, Y => n432);
   U109 : INVXL port map( A => n138, Y => n102);
   U110 : AOI22XL port map( A0 => value(15), A1 => n13, B0 => value(14), B1 => 
                           n11, Y => n433);
   U111 : NOR2XL port map( A => n3, B => n169, Y => n98);
   U112 : NOR2XL port map( A => n169, B => n374, Y => n100);
   U113 : NOR2XL port map( A => n171, B => shift_amount(4), Y => n144);
   U114 : AOI22XL port map( A0 => value(19), A1 => n13, B0 => value(18), B1 => 
                           n11, Y => n430);
   U115 : NOR2XL port map( A => n2, B => n169, Y => n6);
   U116 : OAI211XL port map( A0 => n295, A1 => n1, B0 => n296, C0 => n297, Y =>
                           n123);
   U117 : AOI22XL port map( A0 => n162, A1 => n298, B0 => n164, B1 => n299, Y 
                           => n297);
   U118 : AOI22XL port map( A0 => n163, A1 => n298, B0 => n266, B1 => n120, Y 
                           => n348);
   U119 : AOI22XL port map( A0 => value(6), A1 => n13, B0 => value(7), B1 => 
                           n11, Y => n423);
   U120 : AOI22XL port map( A0 => value(10), A1 => n13, B0 => value(11), B1 => 
                           n11, Y => n424);
   U121 : AOI22XL port map( A0 => value(21), A1 => n18, B0 => value(22), B1 => 
                           n16, Y => n280);
   U122 : AOI22XL port map( A0 => value(15), A1 => n14, B0 => value(16), B1 => 
                           n11, Y => n373);
   U123 : AOI22XL port map( A0 => value(19), A1 => n13, B0 => value(20), B1 => 
                           n12, Y => n327);
   U124 : AOI21XL port map( A0 => n266, A1 => n112, B0 => n322, Y => n328);
   U125 : INVXL port map( A => shift_amount(3), Y => n143);
   U126 : INVXL port map( A => n410, Y => n142);
   U127 : AOI22XL port map( A0 => n411, A1 => n285, B0 => shift_amount(2), B1 
                           => n284, Y => n410);
   U128 : AOI22XL port map( A0 => value(11), A1 => n13, B0 => value(12), B1 => 
                           n11, Y => n413);
   U129 : AOI22XL port map( A0 => value(7), A1 => n14, B0 => value(8), B1 => 
                           n11, Y => n414);
   U130 : AOI21XL port map( A0 => n266, A1 => n101, B0 => n322, Y => n321);
   U131 : AOI22XL port map( A0 => value(8), A1 => n13, B0 => value(9), B1 => 
                           n11, Y => n403);
   U132 : OAI211XL port map( A0 => n308, A1 => n1, B0 => n296, C0 => n309, Y =>
                           n131);
   U133 : AOI22XL port map( A0 => n162, A1 => n310, B0 => n164, B1 => n311, Y 
                           => n309);
   U134 : AOI22XL port map( A0 => n163, A1 => n310, B0 => n266, B1 => n128, Y 
                           => n357);
   U135 : NOR2BXL port map( AN => n239, B => n1, Y => n350);
   U136 : AOI22XL port map( A0 => value(5), A1 => n13, B0 => value(6), B1 => 
                           n11, Y => n434);
   U137 : AOI22XL port map( A0 => value(9), A1 => n13, B0 => value(10), B1 => 
                           n11, Y => n435);
   U138 : AOI22XL port map( A0 => value(26), A1 => n14, B0 => value(27), B1 => 
                           n12, Y => n242);
   U139 : AOI22XL port map( A0 => value(14), A1 => n14, B0 => value(15), B1 => 
                           n11, Y => n381);
   U140 : AOI22XL port map( A0 => value(18), A1 => n14, B0 => value(19), B1 => 
                           n12, Y => n346);
   U141 : AOI22XL port map( A0 => value(20), A1 => n18, B0 => value(21), B1 => 
                           n16, Y => n293);
   U142 : AOI222XL port map( A0 => n193, A1 => n162, B0 => n194, B1 => n163, C0
                           => n195, C1 => n164, Y => n192);
   U143 : INVXL port map( A => value(27), Y => n190);
   U144 : AOI22XL port map( A0 => value(29), A1 => n14, B0 => value(30), B1 => 
                           n11, Y => n191);
   U145 : INVXL port map( A => value(28), Y => n176);
   U146 : INVXL port map( A => n16, Y => n15);
   U147 : NOR2XL port map( A => n1, B => shift_amount(4), Y => n152);
   U148 : INVXL port map( A => shift_amount(4), Y => n151);
   U149 : INVXL port map( A => n393, Y => n329);
   U150 : NOR2XL port map( A => n151, B => n146, Y => n7);
   U151 : AOI22XL port map( A0 => n13, A1 => value(24), B0 => value(25), B1 => 
                           n12, Y => n263);
   U152 : AOI22XL port map( A0 => value(12), A1 => n13, B0 => value(13), B1 => 
                           n11, Y => n400);
   U153 : AOI22XL port map( A0 => value(16), A1 => n14, B0 => value(17), B1 => 
                           n12, Y => n363);
   U154 : AOI22XL port map( A0 => value(20), A1 => n14, B0 => value(21), B1 => 
                           n12, Y => n319);
   U155 : AOI22XL port map( A0 => value(25), A1 => n13, B0 => value(26), B1 => 
                           n12, Y => n255);
   U156 : NOR2XL port map( A => n171, B => n151, Y => n239);
   U157 : INVXL port map( A => n169, Y => n237);
   U158 : AOI22XL port map( A0 => value(13), A1 => n14, B0 => value(14), B1 => 
                           n11, Y => n387);
   U159 : INVXL port map( A => n172, Y => n246);
   U160 : NOR2XL port map( A => n95, B => n374, Y => n8);
   U161 : AOI22XL port map( A0 => value(17), A1 => n14, B0 => value(18), B1 => 
                           n12, Y => n355);
   U162 : AOI22XL port map( A0 => value(19), A1 => n18, B0 => value(20), B1 => 
                           n16, Y => n306);
   U163 : NOR2XL port map( A => n95, B => n2, Y => n9);
   U164 : NOR2XL port map( A => n95, B => n3, Y => n10);
   U165 : AOI222XL port map( A0 => shift_amount(4), A1 => n173, B0 => n174, B1 
                           => n151, C0 => n152, C1 => n175, Y => n170);
   U166 : INVXL port map( A => n179, Y => n174);
   U167 : INVXL port map( A => n183, Y => n173);
   U168 : INVXL port map( A => n184, Y => n168);
   U169 : AOI222XL port map( A0 => shift_amount(4), A1 => n210, B0 => n211, B1 
                           => n151, C0 => n152, C1 => n212, Y => n209);
   U170 : INVXL port map( A => n215, Y => n211);
   U171 : INVXL port map( A => n219, Y => n210);
   U172 : INVXL port map( A => n220, Y => n208);
   U173 : AOI222XL port map( A0 => shift_amount(4), A1 => n223, B0 => n224, B1 
                           => n151, C0 => n152, C1 => n225, Y => n222);
   U174 : INVXL port map( A => n228, Y => n224);
   U175 : INVXL port map( A => n232, Y => n223);
   U176 : INVXL port map( A => n233, Y => n221);
   U177 : AOI222XL port map( A0 => shift_amount(4), A1 => n331, B0 => n332, B1 
                           => n151, C0 => n152, C1 => n333, Y => n330);
   U178 : INVXL port map( A => n336, Y => n332);
   U179 : AOI22XL port map( A0 => value(2), A1 => n14, B0 => value(1), B1 => 
                           n12, Y => n335);
   U180 : OAI22XL port map( A0 => n4, A1 => n401, B0 => n5, B1 => n402, Y => 
                           n264);
   U181 : AOI222XL port map( A0 => shift_amount(4), A1 => n371, B0 => n437, B1 
                           => n151, C0 => n152, C1 => n438, Y => n436);
   U182 : AOI21XL port map( A0 => value(3), A1 => n18, B0 => n284, Y => n439);
   U183 : NOR2XL port map( A => n402, B => n4, Y => n284);
   U184 : AOI222XL port map( A0 => n98, A1 => n119, B0 => n100, B1 => n120, C0 
                           => n102, C1 => n121, Y => n118);
   U185 : AOI22XL port map( A0 => n147, A1 => n164, B0 => n294, B1 => n266, Y 
                           => n116);
   U186 : AOI22XL port map( A0 => n6, A1 => n122, B0 => n7, B1 => n123, Y => 
                           n117);
   U187 : AOI222XL port map( A0 => n98, A1 => n111, B0 => n100, B1 => n112, C0 
                           => n102, C1 => n113, Y => n110);
   U188 : AOI222XL port map( A0 => n283, A1 => n266, B0 => n162, B1 => n284, C0
                           => n285, C1 => n164, Y => n108);
   U189 : AOI22XL port map( A0 => n6, A1 => n114, B0 => n7, B1 => n115, Y => 
                           n109);
   U190 : AOI22XL port map( A0 => value(2), A1 => n13, B0 => value(3), B1 => 
                           n11, Y => n425);
   U191 : AOI222XL port map( A0 => shift_amount(4), A1 => n149, B0 => n150, B1 
                           => n151, C0 => n152, C1 => n153, Y => n145);
   U192 : INVXL port map( A => n161, Y => n150);
   U193 : AOI22XL port map( A0 => value(4), A1 => n13, B0 => value(3), B1 => 
                           n11, Y => n158);
   U194 : AOI222XL port map( A0 => n98, A1 => n99, B0 => n100, B1 => n101, C0 
                           => n102, C1 => n103, Y => n97);
   U195 : AOI222XL port map( A0 => n264, A1 => n162, B0 => n265, B1 => n266, C0
                           => n267, C1 => n164, Y => n94);
   U196 : AOI22XL port map( A0 => n6, A1 => n105, B0 => n7, B1 => n107, Y => 
                           n96);
   U197 : AOI222XL port map( A0 => n98, A1 => n105, B0 => n100, B1 => n99, C0 
                           => n102, C1 => n135, Y => n134);
   U198 : AOI22XL port map( A0 => n264, A1 => n164, B0 => n267, B1 => n266, Y 
                           => n132);
   U199 : AOI22XL port map( A0 => n6, A1 => n103, B0 => n7, B1 => n136, Y => 
                           n133);
   U200 : AOI221XL port map( A0 => n18, A1 => value(7), B0 => n16, B1 => 
                           value(6), C0 => n442, Y => n137);
   U201 : OAI22XL port map( A0 => n157, A1 => n5, B0 => n203, B1 => n4, Y => 
                           n442);
   U202 : NAND2XL port map( A => n266, B => n237, Y => n138);
   U203 : AOI222XL port map( A0 => n7, A1 => n141, B0 => n98, B1 => n114, C0 =>
                           n6, C1 => n113, Y => n140);
   U204 : AOI32XL port map( A0 => n142, A1 => n143, A2 => n144, B0 => n100, B1 
                           => n111, Y => n139);
   U205 : NOR2BXL port map( AN => n152, B => n171, Y => n148);
   U206 : AOI222XL port map( A0 => shift_amount(4), A1 => n200, B0 => n201, B1 
                           => n151, C0 => n152, C1 => n202, Y => n198);
   U207 : INVXL port map( A => n205, Y => n201);
   U208 : AOI22XL port map( A0 => value(3), A1 => n13, B0 => value(2), B1 => 
                           n12, Y => n204);
   U209 : INVXL port map( A => shift_func(1), Y => n146);
   U210 : AOI222XL port map( A0 => n98, A1 => n128, B0 => n100, B1 => n358, C0 
                           => n102, C1 => n130, Y => n427);
   U211 : AOI222XL port map( A0 => n199, A1 => n162, B0 => n312, B1 => n266, C0
                           => n307, C1 => n164, Y => n256);
   U212 : AOI22XL port map( A0 => n6, A1 => n127, B0 => n7, B1 => n252, Y => 
                           n426);
   U213 : AOI222XL port map( A0 => n98, A1 => n127, B0 => n100, B1 => n128, C0 
                           => n102, C1 => n129, Y => n126);
   U214 : AOI22XL port map( A0 => n199, A1 => n164, B0 => n307, B1 => n266, Y 
                           => n124);
   U215 : AOI22XL port map( A0 => n6, A1 => n130, B0 => n7, B1 => n131, Y => 
                           n125);
   U216 : AOI221XL port map( A0 => n294, A1 => n162, B0 => n300, B1 => n164, C0
                           => n379, Y => n183);
   U217 : INVXL port map( A => n380, Y => n379);
   U218 : AOI22XL port map( A0 => n163, A1 => n147, B0 => n266, B1 => n245, Y 
                           => n380);
   U219 : AOI222XL port map( A0 => n98, A1 => n349, B0 => n100, B1 => n299, C0 
                           => n102, C1 => n119, Y => n376);
   U220 : AOI22XL port map( A0 => n6, A1 => n120, B0 => n7, B1 => n184, Y => 
                           n375);
   U221 : AOI222XL port map( A0 => n98, A1 => n120, B0 => n100, B1 => n349, C0 
                           => n102, C1 => n122, Y => n416);
   U222 : AOI222XL port map( A0 => n147, A1 => n162, B0 => n300, B1 => n266, C0
                           => n294, C1 => n164, Y => n243);
   U223 : AOI22XL port map( A0 => n6, A1 => n119, B0 => n7, B1 => n238, Y => 
                           n415);
   U224 : AOI222XL port map( A0 => n237, A1 => n371, B0 => n9, B1 => n288, C0 
                           => n148, C1 => n230, Y => n370);
   U225 : AOI21XL port map( A0 => n350, A1 => n284, B0 => n246, Y => n369);
   U226 : AOI22XL port map( A0 => n8, A1 => n285, B0 => n10, B1 => n283, Y => 
                           n368);
   U227 : AOI222XL port map( A0 => n283, A1 => n164, B0 => n288, B1 => n266, C0
                           => n142, C1 => shift_amount(3), Y => n232);
   U228 : AOI222XL port map( A0 => n98, A1 => n112, B0 => n100, B1 => n408, C0 
                           => n102, C1 => n114, Y => n407);
   U229 : AOI22XL port map( A0 => n6, A1 => n111, B0 => n7, B1 => n233, Y => 
                           n406);
   U230 : AOI222XL port map( A0 => n237, A1 => n331, B0 => n9, B1 => n273, C0 
                           => n148, C1 => n217, Y => n361);
   U231 : AOI21XL port map( A0 => n350, A1 => n264, B0 => n246, Y => n360);
   U232 : AOI22XL port map( A0 => n8, A1 => n267, B0 => n10, B1 => n265, Y => 
                           n359);
   U233 : AOI221XL port map( A0 => n267, A1 => n162, B0 => n265, B1 => n164, C0
                           => n398, Y => n219);
   U234 : INVXL port map( A => n399, Y => n398);
   U235 : AOI22XL port map( A0 => n163, A1 => n264, B0 => n266, B1 => n273, Y 
                           => n399);
   U236 : AOI222XL port map( A0 => n98, A1 => n101, B0 => n100, B1 => n367, C0 
                           => n102, C1 => n105, Y => n389);
   U237 : AOI22XL port map( A0 => n6, A1 => n99, B0 => n7, B1 => n220, Y => 
                           n388);
   U238 : AOI221XL port map( A0 => n307, A1 => n162, B0 => n312, B1 => n164, C0
                           => n385, Y => n196);
   U239 : INVXL port map( A => n386, Y => n385);
   U240 : AOI22XL port map( A0 => n163, A1 => n199, B0 => n266, B1 => n257, Y 
                           => n386);
   U241 : AOI222XL port map( A0 => n98, A1 => n358, B0 => n100, B1 => n311, C0 
                           => n102, C1 => n127, Y => n383);
   U242 : INVXL port map( A => n144, Y => n95);
   U243 : AOI22XL port map( A0 => n6, A1 => n128, B0 => n7, B1 => n197, Y => 
                           n382);
   U244 : AOI222XL port map( A0 => n237, A1 => n123, B0 => n239, B1 => n292, C0
                           => n148, C1 => n180, Y => n291);
   U245 : INVXL port map( A => n116, Y => n292);
   U246 : AOI21XL port map( A0 => n8, A1 => n300, B0 => n246, Y => n290);
   U247 : AOI22XL port map( A0 => n10, A1 => n245, B0 => n9, B1 => n181, Y => 
                           n289);
   U248 : AOI222XL port map( A0 => n237, A1 => n149, B0 => n9, B1 => n245, C0 
                           => n148, C1 => n181, Y => n344);
   U249 : AOI21XL port map( A0 => n350, A1 => n147, B0 => n246, Y => n343);
   U250 : AOI22XL port map( A0 => n8, A1 => n294, B0 => n10, B1 => n300, Y => 
                           n342);
   U251 : AOI222XL port map( A0 => n237, A1 => n115, B0 => n239, B1 => n277, C0
                           => n148, C1 => n231, Y => n276);
   U252 : INVXL port map( A => n108, Y => n277);
   U253 : AOI21XL port map( A0 => n8, A1 => n288, B0 => n246, Y => n275);
   U254 : AOI22XL port map( A0 => n10, A1 => n230, B0 => n9, B1 => n229, Y => 
                           n274);
   U255 : AOI222XL port map( A0 => n237, A1 => n141, B0 => n9, B1 => n230, C0 
                           => n148, C1 => n229, Y => n325);
   U256 : AOI31XL port map( A0 => n142, A1 => n143, A2 => n239, B0 => n246, Y 
                           => n324);
   U257 : AOI22XL port map( A0 => n8, A1 => n283, B0 => n10, B1 => n288, Y => 
                           n323);
   U258 : AOI222XL port map( A0 => n237, A1 => n136, B0 => n239, B1 => n316, C0
                           => n148, C1 => n216, Y => n315);
   U259 : INVXL port map( A => n132, Y => n316);
   U260 : AOI21XL port map( A0 => n8, A1 => n265, B0 => n246, Y => n314);
   U261 : AOI22XL port map( A0 => n10, A1 => n273, B0 => n9, B1 => n217, Y => 
                           n313);
   U262 : AOI222XL port map( A0 => n237, A1 => n131, B0 => n239, B1 => n304, C0
                           => n148, C1 => n193, Y => n303);
   U263 : INVXL port map( A => n124, Y => n304);
   U264 : AOI21XL port map( A0 => n8, A1 => n312, B0 => n246, Y => n302);
   U265 : AOI22XL port map( A0 => n10, A1 => n257, B0 => n9, B1 => n194, Y => 
                           n301);
   U266 : AOI222XL port map( A0 => n237, A1 => n200, B0 => n9, B1 => n257, C0 
                           => n148, C1 => n194, Y => n353);
   U267 : AOI21XL port map( A0 => n350, A1 => n199, B0 => n246, Y => n352);
   U268 : AOI22XL port map( A0 => n8, A1 => n307, B0 => n10, B1 => n312, Y => 
                           n351);
   U269 : AOI222XL port map( A0 => n237, A1 => n238, B0 => n239, B1 => n240, C0
                           => n148, C1 => n182, Y => n236);
   U270 : INVXL port map( A => n243, Y => n240);
   U271 : AOI21XL port map( A0 => n8, A1 => n245, B0 => n246, Y => n235);
   U272 : AOI22XL port map( A0 => n10, A1 => n181, B0 => n9, B1 => n180, Y => 
                           n234);
   U273 : AOI222XL port map( A0 => shift_amount(4), A1 => n187, B0 => n188, B1 
                           => n151, C0 => n152, C1 => n189, Y => n186);
   U274 : INVXL port map( A => n192, Y => n188);
   U275 : INVXL port map( A => n196, Y => n187);
   U276 : INVXL port map( A => n197, Y => n185);
   U277 : NAND2XL port map( A => shift_func(1), B => n151, Y => n169);
   U278 : NAND2XL port map( A => shift_func(0), B => n146, Y => n171);
   U279 : NAND2XL port map( A => n7, B => n329, Y => n172);
   U280 : AOI222XL port map( A0 => n237, A1 => n107, B0 => n239, B1 => n261, C0
                           => n148, C1 => n218, Y => n260);
   U281 : INVXL port map( A => n94, Y => n261);
   U282 : AOI21XL port map( A0 => n8, A1 => n273, B0 => n246, Y => n259);
   U283 : AOI22XL port map( A0 => n10, A1 => n217, B0 => n9, B1 => n216, Y => 
                           n258);
   U284 : AOI222XL port map( A0 => n237, A1 => n252, B0 => n239, B1 => n253, C0
                           => n148, C1 => n195, Y => n251);
   U285 : INVXL port map( A => n256, Y => n253);
   U286 : AOI21XL port map( A0 => n8, A1 => n257, B0 => n246, Y => n250);
   U287 : AOI22XL port map( A0 => n10, A1 => n194, B0 => n9, B1 => n193, Y => 
                           n249);
   U288 : OAI2BB2XL port map( B0 => n330, B1 => n146, A0N => n264, A1N => n148,
                           Y => c_shift(1));
   U289 : OAI2BB2XL port map( B0 => n436, B1 => n146, A0N => n284, A1N => n148,
                           Y => c_shift(0));
   U290 : OAI211XL port map( A0 => n116, A1 => n95, B0 => n117, C0 => n118, Y 
                           => c_shift(7));
   U291 : OAI211XL port map( A0 => n108, A1 => n95, B0 => n109, C0 => n110, Y 
                           => c_shift(8));
   U292 : OAI2BB2XL port map( B0 => n145, B1 => n146, A0N => n147, A1N => n148,
                           Y => c_shift(3));
   U293 : OAI211XL port map( A0 => n94, A1 => n95, B0 => n96, C0 => n97, Y => 
                           c_shift(9));
   U294 : OAI211XL port map( A0 => n132, A1 => n95, B0 => n133, C0 => n134, Y 
                           => c_shift(5));
   U295 : OAI211XL port map( A0 => n137, A1 => n138, B0 => n139, C0 => n140, Y 
                           => c_shift(4));
   U296 : OAI2BB2XL port map( B0 => n198, B1 => n146, A0N => n199, A1N => n148,
                           Y => c_shift(2));
   U297 : OAI211XL port map( A0 => n256, A1 => n95, B0 => n426, C0 => n427, Y 
                           => c_shift(10));
   U298 : OAI211XL port map( A0 => n124, A1 => n95, B0 => n125, C0 => n126, Y 
                           => c_shift(6));
   U299 : OAI211XL port map( A0 => n183, A1 => n95, B0 => n375, C0 => n376, Y 
                           => c_shift(15));
   U300 : OAI211XL port map( A0 => n243, A1 => n95, B0 => n415, C0 => n416, Y 
                           => c_shift(11));
   U301 : NAND3XL port map( A => n368, B => n369, C => n370, Y => c_shift(16));
   U302 : OAI211XL port map( A0 => n232, A1 => n95, B0 => n406, C0 => n407, Y 
                           => c_shift(12));
   U303 : NAND3XL port map( A => n359, B => n360, C => n361, Y => c_shift(17));
   U304 : OAI211XL port map( A0 => n219, A1 => n95, B0 => n388, C0 => n389, Y 
                           => c_shift(13));
   U305 : OAI211XL port map( A0 => n196, A1 => n95, B0 => n382, C0 => n383, Y 
                           => c_shift(14));
   U306 : NAND3XL port map( A => n289, B => n290, C => n291, Y => c_shift(23));
   U307 : NAND3XL port map( A => n342, B => n343, C => n344, Y => c_shift(19));
   U308 : NAND3XL port map( A => n274, B => n275, C => n276, Y => c_shift(24));
   U309 : NAND3XL port map( A => n323, B => n324, C => n325, Y => c_shift(20));
   U310 : NAND3XL port map( A => n313, B => n314, C => n315, Y => c_shift(21));
   U311 : NAND3XL port map( A => n301, B => n302, C => n303, Y => c_shift(22));
   U312 : NAND3XL port map( A => n351, B => n352, C => n353, Y => c_shift(18));
   U313 : NAND3XL port map( A => n234, B => n235, C => n236, Y => c_shift(27));
   U314 : NAND3XL port map( A => n258, B => n259, C => n260, Y => c_shift(25));
   U315 : NAND3XL port map( A => n249, B => n250, C => n251, Y => c_shift(26));
   U316 : OAI21XL port map( A0 => n287, A1 => n1, B0 => n378, Y => n233);
   U317 : OAI21XL port map( A0 => n270, A1 => n1, B0 => n378, Y => n220);
   U318 : OAI21XL port map( A0 => n384, A1 => n1, B0 => n378, Y => n197);
   U319 : OAI21XL port map( A0 => n377, A1 => n1, B0 => n378, Y => n184);
   U320 : OAI221XL port map( A0 => n168, A1 => n169, B0 => n170, B1 => n171, C0
                           => n172, Y => c_shift(31));
   U321 : OAI221XL port map( A0 => n286, A1 => n3, B0 => n445, B1 => n2, C0 => 
                           n446, Y => n371);
   U322 : OAI221XL port map( A0 => n268, A1 => n3, B0 => n364, B1 => n2, C0 => 
                           n365, Y => n331);
   U323 : OAI222XL port map( A0 => n5, A1 => n401, B0 => n156, B1 => n402, C0 
                           => n4, C1 => n404, Y => n199);
   U324 : OAI221XL port map( A0 => n356, A1 => n1, B0 => n384, B1 => n2, C0 => 
                           n272, Y => n252);
   U325 : OAI221XL port map( A0 => n286, A1 => n1, B0 => n287, B1 => n2, C0 => 
                           n272, Y => n115);
   U326 : OAI221XL port map( A0 => n268, A1 => n1, B0 => n270, B1 => n2, C0 => 
                           n272, Y => n107);
   U327 : OAI221XL port map( A0 => n347, A1 => n1, B0 => n377, B1 => n2, C0 => 
                           n272, Y => n238);
   U328 : OAI221XL port map( A0 => n356, A1 => n3, B0 => n308, B1 => n2, C0 => 
                           n357, Y => n200);
   U329 : OAI221XL port map( A0 => n347, A1 => n3, B0 => n295, B1 => n2, C0 => 
                           n348, Y => n149);
   U330 : OAI221XL port map( A0 => n287, A1 => n3, B0 => n286, B1 => n2, C0 => 
                           n328, Y => n141);
   U331 : OAI221XL port map( A0 => n270, A1 => n3, B0 => n268, B1 => n2, C0 => 
                           n321, Y => n136);
   U332 : OAI222XL port map( A0 => n4, A1 => n420, B0 => n5, B1 => n394, C0 => 
                           n431, C1 => n393, Y => n310);
   U333 : OAI21XL port map( A0 => n4, A1 => n394, B0 => n393, Y => n298);
   U334 : OAI221XL port map( A0 => n208, A1 => n169, B0 => n209, B1 => n171, C0
                           => n172, Y => c_shift(29));
   U335 : OAI221XL port map( A0 => n221, A1 => n169, B0 => n222, B1 => n171, C0
                           => n172, Y => c_shift(28));
   U336 : OAI221XL port map( A0 => n185, A1 => n169, B0 => n186, B1 => n171, C0
                           => n172, Y => c_shift(30));
   U337 : OAI221XL port map( A0 => n17, A1 => n354, B0 => n156, B1 => n362, C0 
                           => n443, Y => n114);
   U338 : OAI221XL port map( A0 => n17, A1 => n318, B0 => n156, B1 => n317, C0 
                           => n447, Y => n111);
   U339 : OAI221XL port map( A0 => n17, A1 => n394, B0 => n156, B1 => n420, C0 
                           => n448, Y => n409);
   U340 : OAI221XL port map( A0 => n154, A1 => n345, B0 => n156, B1 => n354, C0
                           => n390, Y => n105);
   U341 : OAI221XL port map( A0 => n17, A1 => n339, B0 => n15, B1 => n340, C0 
                           => n341, Y => n103);
   U342 : OAI221XL port map( A0 => n17, A1 => n206, B0 => n156, B1 => n337, C0 
                           => n338, Y => n135);
   U343 : OAI221XL port map( A0 => n15, A1 => n404, B0 => n5, B1 => n401, C0 =>
                           n439, Y => n438);
   U344 : OAI222XL port map( A0 => n440, A1 => n3, B0 => n441, B1 => n374, C0 
                           => n137, C1 => n2, Y => n437);
   U345 : OAI221XL port map( A0 => n17, A1 => n203, B0 => n156, B1 => n334, C0 
                           => n335, Y => n333);
   U346 : OAI221XL port map( A0 => n17, A1 => n402, B0 => n156, B1 => n401, C0 
                           => n425, Y => n147);
   U347 : OAI221XL port map( A0 => n154, A1 => n165, B0 => n156, B1 => n340, C0
                           => n400, Y => n273);
   U348 : OAI221XL port map( A0 => n154, A1 => n340, B0 => n156, B1 => n339, C0
                           => n387, Y => n257);
   U349 : OAI221XL port map( A0 => n154, A1 => n339, B0 => n156, B1 => n372, C0
                           => n381, Y => n245);
   U350 : OAI221XL port map( A0 => n154, A1 => n241, B0 => n15, B1 => n254, C0 
                           => n392, Y => n101);
   U351 : OAI221XL port map( A0 => n17, A1 => n254, B0 => n15, B1 => n262, C0 
                           => n449, Y => n112);
   U352 : OAI221XL port map( A0 => n17, A1 => n157, B0 => n156, B1 => n155, C0 
                           => n414, Y => n283);
   U353 : OAI221XL port map( A0 => n154, A1 => n396, B0 => n156, B1 => n318, C0
                           => n397, Y => n99);
   U354 : OAI221XL port map( A0 => n154, A1 => n362, B0 => n156, B1 => n354, C0
                           => n363, Y => n217);
   U355 : OAI221XL port map( A0 => n154, A1 => n354, B0 => n156, B1 => n345, C0
                           => n355, Y => n194);
   U356 : OAI221XL port map( A0 => n154, A1 => n345, B0 => n156, B1 => n326, C0
                           => n346, Y => n181);
   U357 : OAI221XL port map( A0 => n17, A1 => n334, B0 => n156, B1 => n203, C0 
                           => n434, Y => n307);
   U358 : OAI221XL port map( A0 => n17, A1 => n203, B0 => n156, B1 => n157, C0 
                           => n423, Y => n294);
   U359 : OAI221XL port map( A0 => n154, A1 => n404, B0 => n156, B1 => n334, C0
                           => n405, Y => n267);
   U360 : OAI221XL port map( A0 => n17, A1 => n206, B0 => n156, B1 => n166, C0 
                           => n424, Y => n300);
   U361 : OAI221XL port map( A0 => n17, A1 => n337, B0 => n156, B1 => n206, C0 
                           => n435, Y => n312);
   U362 : OAI221XL port map( A0 => n17, A1 => n155, B0 => n156, B1 => n337, C0 
                           => n403, Y => n265);
   U363 : OAI221XL port map( A0 => n17, A1 => n166, B0 => n156, B1 => n165, C0 
                           => n413, Y => n288);
   U364 : OAI221XL port map( A0 => n154, A1 => n393, B0 => n156, B1 => n394, C0
                           => n395, Y => n366);
   U365 : OAI221XL port map( A0 => n17, A1 => n401, B0 => n15, B1 => n404, C0 
                           => n412, Y => n285);
   U366 : OAI221XL port map( A0 => n17, A1 => n340, B0 => n156, B1 => n165, C0 
                           => n444, Y => n113);
   U367 : OAI221XL port map( A0 => n154, A1 => n317, B0 => n156, B1 => n326, C0
                           => n422, Y => n119);
   U368 : OAI221XL port map( A0 => n154, A1 => n372, B0 => n156, B1 => n362, C0
                           => n373, Y => n230);
   U369 : OAI221XL port map( A0 => n5, A1 => n226, B0 => n241, B1 => n4, C0 => 
                           n450, Y => n408);
   U370 : OAI221XL port map( A0 => n154, A1 => n176, B0 => n15, B1 => n190, C0 
                           => n391, Y => n367);
   U371 : OAI221XL port map( A0 => n17, A1 => n326, B0 => n15, B1 => n317, C0 
                           => n327, Y => n229);
   U372 : OAI221XL port map( A0 => n154, A1 => n317, B0 => n15, B1 => n318, C0 
                           => n319, Y => n216);
   U373 : OAI221XL port map( A0 => n154, A1 => n254, B0 => n241, B1 => n15, C0 
                           => n255, Y => n195);
   U374 : OAI221XL port map( A0 => n154, A1 => n165, B0 => n15, B1 => n166, C0 
                           => n167, Y => n121);
   U375 : OAI221XL port map( A0 => n154, A1 => n262, B0 => n15, B1 => n254, C0 
                           => n263, Y => n218);
   U376 : OAI221XL port map( A0 => n17, A1 => n241, B0 => n15, B1 => n226, C0 
                           => n242, Y => n182);
   U377 : OAI221XL port map( A0 => n17, A1 => n155, B0 => n156, B1 => n157, C0 
                           => n158, Y => n153);
   U378 : OAI221XL port map( A0 => n154, A1 => n190, B0 => n15, B1 => n176, C0 
                           => n191, Y => n189);
   U379 : OAI221XL port map( A0 => n154, A1 => n226, B0 => n15, B1 => n213, C0 
                           => n227, Y => n225);
   U380 : OAI221XL port map( A0 => n154, A1 => n213, B0 => n15, B1 => n190, C0 
                           => n214, Y => n212);
   U381 : OAI221XL port map( A0 => n154, A1 => n176, B0 => n15, B1 => n177, C0 
                           => n178, Y => n175);
   U382 : OAI221XL port map( A0 => n17, A1 => n177, B0 => n156, B1 => n176, C0 
                           => n432, Y => n311);
   U383 : OAI221XL port map( A0 => n154, A1 => n420, B0 => n156, B1 => n177, C0
                           => n421, Y => n299);
   U384 : OAI221XL port map( A0 => n17, A1 => n262, B0 => n156, B1 => n305, C0 
                           => n419, Y => n120);
   U385 : OAI221XL port map( A0 => n17, A1 => n305, B0 => n156, B1 => n396, C0 
                           => n430, Y => n128);
   U386 : OAI221XL port map( A0 => n241, A1 => n5, B0 => n254, B1 => n4, C0 => 
                           n418, Y => n349);
   U387 : OAI221XL port map( A0 => n17, A1 => n326, B0 => n156, B1 => n345, C0 
                           => n433, Y => n127);
   U388 : OAI221XL port map( A0 => n17, A1 => n372, B0 => n156, B1 => n339, C0 
                           => n428, Y => n130);
   U389 : OAI221XL port map( A0 => n154, A1 => n362, B0 => n156, B1 => n372, C0
                           => n417, Y => n122);
   U390 : OAI221XL port map( A0 => n5, A1 => n262, B0 => n254, B1 => n4, C0 => 
                           n293, Y => n180);
   U391 : OAI221XL port map( A0 => n5, A1 => n305, B0 => n262, B1 => n4, C0 => 
                           n306, Y => n193);
   U392 : OAI221XL port map( A0 => n254, A1 => n5, B0 => n241, B1 => n4, C0 => 
                           n280, Y => n231);
   U393 : OAI221XL port map( A0 => n154, A1 => n166, B0 => n15, B1 => n206, C0 
                           => n207, Y => n129);
   U394 : OAI221XL port map( A0 => n17, A1 => n226, B0 => n241, B1 => n15, C0 
                           => n429, Y => n358);
   U395 : OAI221XL port map( A0 => n154, A1 => n157, B0 => n15, B1 => n203, C0 
                           => n204, Y => n202);

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity alu_alu_typeDEFAULT is

   port( a_in, b_in : in std_logic_vector (31 downto 0);  alu_function : in 
         std_logic_vector (3 downto 0);  c_alu : out std_logic_vector (31 
         downto 0));

end alu_alu_typeDEFAULT;

architecture SYN_logic of alu_alu_typeDEFAULT is

   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component XNOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component XOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI32XL
      port( A0, A1, A2, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI222XL
      port( A0, A1, B0, B1, C0, C1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   signal n107, n108, n109, n110, n111, n112, n113, n114, n115, n116, n117, 
      n118, n119, n121, n122, n123, n124, n125, n126, n127, n128, n129, n130, 
      n131, n132, n133, n134, n135, n136, n137, n138, n139, n140, n141, n142, 
      n143, n144, n145, n146, n147, n148, n149, n150, n151, n152, n153, n154, 
      n155, n156, n157, n158, n159, n160, n161, n162, n163, n164, n165, n166, 
      n167, n168, n169, n170, n171, n172, n173, n174, n176, n178, n179, n180, 
      n181, n182, n183, n186, n187, n188, n189, n190, n191, n192, n193, n194, 
      n195, n196, n197, n198, n199, n200, n201, n202, n203, n204, n205, n206, 
      n207, n208, n209, n210, n211, n212, n213, n214, n215, n216, n217, n218, 
      n219, n220, n221, n222, n223, n224, n225, n226, n227, n228, n229, n230, 
      n231, n232, n233, n234, n235, n236, n237, n238, n239, n240, n241, n242, 
      n243, n244, n245, n246, n247, n248, n249, n250, n251, n252, n253, n254, 
      n255, n256, n257, n258, n259, n260, n261, n262, n263, n264, n265, n266, 
      n267, n268, n269, n270, n271, n272, n273, n274, n275, n276, n277, n278, 
      n279, n280, n281, n282, n283, n284, n285, n286, n287, n288, n289, n290, 
      n291, n292, n293, n294, n295, n296, n297, n298, n299, n300, n301, n302, 
      n303, n304, n305, n306, n307, n308, n309, n310, n311, n312, n313, n314, 
      n315, n316, n317, n318, n319, n320, n321, n322, n323, n324, n325, n326, 
      n327, n328, n329, n330, n331, n332, n333, n334, n335, n336, n337, n338, 
      n339, n340, n341, n342, n343, n344, n345, n346, n347, n348, n349, n350, 
      n351, n352, n353, n354, n355, n356, n357, n358, n359, n360, n361, n362, 
      n363, n364, n365, n366, n367, n368, n369, n370, n371, n372, n373, n374, 
      n375, n376, n377, n378, n379, n380, n381, n382, n383, n384, n385, n386, 
      n387, n388, n389, n390, n391, n392, n393, n394, n395, n396, n397, n398, 
      n399, n400, n401, n402, n403, n404, n405, n406, n407, n408, n409, n410, 
      n411, n412, n413, n414, n415, n417, n418, n419, n420, n421, n422, n423, 
      n424, n425, n426, n427, n428, n429, n430, n431, n432, n433, n434, n435, 
      n436, n437, n438, n439, n440, n441, n442, n443, n444, n445, n446, n447, 
      n448, n449, n450, n451, n452, n453, n454, n455, n456, n457, n458, n459, 
      n460, n461, n462, n463, n464, n465, n466, n467, n468, n469, n470, n471, 
      n472, n473, n474, n475, n476, n477, n478, n479, n1, n2, n3, n4, n5, n6, 
      n7, n8, n9 : std_logic;

begin
   
   U2 : INVXL port map( A => alu_function(3), Y => n479);
   U3 : AOI21XL port map( A0 => n188, A1 => n187, B0 => n176, Y => n410);
   U4 : INVXL port map( A => alu_function(1), Y => n405);
   U5 : INVXL port map( A => alu_function(2), Y => n406);
   U6 : NOR3XL port map( A => n404, B => n407, C => n405, Y => n118);
   U7 : NAND4XL port map( A => alu_function(0), B => n405, C => n406, D => n479
                           , Y => n413);
   U8 : INVXL port map( A => n413, Y => n1);
   U9 : INVXL port map( A => n119, Y => n4);
   U10 : INVXL port map( A => n413, Y => n2);
   U11 : INVXL port map( A => n113, Y => n7);
   U12 : INVXL port map( A => n414, Y => n188);
   U13 : OAI2BB1XL port map( A0N => n194, A1N => n197, B0 => a_in(30), Y => 
                           n415);
   U14 : XNOR2XL port map( A => n413, B => b_in(31), Y => n187);
   U15 : NOR3XL port map( A => n407, B => alu_function(0), C => n405, Y => n119
                           );
   U16 : XOR2XL port map( A => b_in(31), B => n176, Y => n183);
   U17 : INVXL port map( A => n111, Y => n9);
   U18 : XNOR2XL port map( A => n2, B => n209, Y => n212);
   U19 : INVXL port map( A => n419, Y => n215);
   U20 : OAI2BB1XL port map( A0N => n221, A1N => n224, B0 => a_in(28), Y => 
                           n420);
   U21 : INVXL port map( A => a_in(29), Y => n214);
   U22 : XNOR2XL port map( A => n2, B => n218, Y => n221);
   U23 : INVXL port map( A => n421, Y => n224);
   U24 : OAI2BB1XL port map( A0N => n230, A1N => n233, B0 => a_in(27), Y => 
                           n422);
   U25 : INVXL port map( A => a_in(28), Y => n223);
   U26 : XNOR2XL port map( A => n1, B => n299, Y => n302);
   U27 : INVXL port map( A => n475, Y => n305);
   U28 : OAI2BB1XL port map( A0N => n476, A1N => n2, B0 => a_in(0), Y => n477);
   U29 : INVXL port map( A => a_in(1), Y => n304);
   U30 : INVXL port map( A => alu_function(0), Y => n404);
   U31 : NAND2XL port map( A => alu_function(2), B => n479, Y => n407);
   U32 : NAND3XL port map( A => n406, B => n479, C => alu_function(1), Y => 
                           n408);
   U33 : INVXL port map( A => n118, Y => n6);
   U34 : OAI32XL port map( A0 => n409, A1 => n410, A2 => n411, B0 => n176, B1 
                           => n412, Y => n400);
   U35 : INVXL port map( A => n409, Y => n412);
   U36 : NOR2BXL port map( AN => n408, B => n183, Y => n409);
   U37 : NOR2XL port map( A => n188, B => n187, Y => n411);
   U38 : INVXL port map( A => b_in(0), Y => n403);
   U39 : NAND4XL port map( A => alu_function(3), B => n404, C => n405, D => 
                           n406, Y => n182);
   U40 : XNOR2XL port map( A => n1, B => n403, Y => n476);
   U41 : NOR3XL port map( A => n407, B => alu_function(1), C => n404, Y => n111
                           );
   U42 : XNOR2XL port map( A => n1, B => n132, Y => n135);
   U43 : INVXL port map( A => n463, Y => n138);
   U44 : OAI2BB1XL port map( A0N => n144, A1N => n147, B0 => a_in(6), Y => n464
                           );
   U45 : INVXL port map( A => a_in(7), Y => n137);
   U46 : XNOR2XL port map( A => n1, B => n123, Y => n126);
   U47 : INVXL port map( A => n461, Y => n129);
   U48 : OAI2BB1XL port map( A0N => n135, A1N => n138, B0 => a_in(7), Y => n462
                           );
   U49 : INVXL port map( A => a_in(8), Y => n128);
   U50 : INVXL port map( A => n471, Y => n174);
   U51 : OAI2BB1XL port map( A0N => n203, A1N => n206, B0 => a_in(2), Y => n472
                           );
   U52 : XNOR2XL port map( A => n1, B => n168, Y => n171);
   U53 : INVXL port map( A => a_in(3), Y => n173);
   U54 : XOR2XL port map( A => n1, B => b_in(9), Y => n114);
   U55 : INVXL port map( A => n459, Y => n117);
   U56 : OAI2BB1XL port map( A0N => n126, A1N => n129, B0 => a_in(8), Y => n460
                           );
   U57 : INVXL port map( A => a_in(9), Y => n116);
   U58 : XNOR2XL port map( A => n1, B => n150, Y => n153);
   U59 : INVXL port map( A => n467, Y => n156);
   U60 : OAI2BB1XL port map( A0N => n162, A1N => n165, B0 => a_in(4), Y => n468
                           );
   U61 : INVXL port map( A => a_in(5), Y => n155);
   U62 : INVXL port map( A => n469, Y => n165);
   U63 : OAI2BB1XL port map( A0N => n171, A1N => n174, B0 => a_in(3), Y => n470
                           );
   U64 : XNOR2XL port map( A => n1, B => n159, Y => n162);
   U65 : INVXL port map( A => a_in(4), Y => n164);
   U66 : XNOR2XL port map( A => n1, B => n200, Y => n203);
   U67 : INVXL port map( A => n473, Y => n206);
   U68 : OAI2BB1XL port map( A0N => n302, A1N => n305, B0 => a_in(1), Y => n474
                           );
   U69 : INVXL port map( A => a_in(2), Y => n205);
   U70 : XNOR2XL port map( A => n1, B => n389, Y => n392);
   U71 : INVXL port map( A => n457, Y => n395);
   U72 : OAI2BB1XL port map( A0N => n114, A1N => n117, B0 => a_in(9), Y => n458
                           );
   U73 : INVXL port map( A => a_in(10), Y => n394);
   U74 : XNOR2XL port map( A => n1, B => n141, Y => n144);
   U75 : INVXL port map( A => n465, Y => n147);
   U76 : OAI2BB1XL port map( A0N => n153, A1N => n156, B0 => a_in(5), Y => n466
                           );
   U77 : INVXL port map( A => a_in(6), Y => n146);
   U78 : XNOR2XL port map( A => n2, B => n344, Y => n347);
   U79 : INVXL port map( A => n447, Y => n350);
   U80 : OAI2BB1XL port map( A0N => n356, A1N => n359, B0 => a_in(14), Y => 
                           n448);
   U81 : INVXL port map( A => a_in(15), Y => n349);
   U82 : XNOR2XL port map( A => n1, B => n380, Y => n383);
   U83 : INVXL port map( A => n455, Y => n386);
   U84 : OAI2BB1XL port map( A0N => n392, A1N => n395, B0 => a_in(10), Y => 
                           n456);
   U85 : INVXL port map( A => a_in(11), Y => n385);
   U86 : XNOR2XL port map( A => n2, B => n335, Y => n338);
   U87 : INVXL port map( A => n445, Y => n341);
   U88 : OAI2BB1XL port map( A0N => n347, A1N => n350, B0 => a_in(15), Y => 
                           n446);
   U89 : INVXL port map( A => a_in(16), Y => n340);
   U90 : XNOR2XL port map( A => n2, B => n371, Y => n374);
   U91 : INVXL port map( A => n453, Y => n377);
   U92 : OAI2BB1XL port map( A0N => n383, A1N => n386, B0 => a_in(11), Y => 
                           n454);
   U93 : INVXL port map( A => a_in(12), Y => n376);
   U94 : XNOR2XL port map( A => n2, B => n326, Y => n329);
   U95 : INVXL port map( A => n443, Y => n332);
   U96 : OAI2BB1XL port map( A0N => n338, A1N => n341, B0 => a_in(16), Y => 
                           n444);
   U97 : INVXL port map( A => a_in(17), Y => n331);
   U98 : XNOR2XL port map( A => n2, B => n362, Y => n365);
   U99 : INVXL port map( A => n451, Y => n368);
   U100 : OAI2BB1XL port map( A0N => n374, A1N => n377, B0 => a_in(12), Y => 
                           n452);
   U101 : INVXL port map( A => a_in(13), Y => n367);
   U102 : XNOR2XL port map( A => n2, B => n353, Y => n356);
   U103 : INVXL port map( A => n449, Y => n359);
   U104 : OAI2BB1XL port map( A0N => n365, A1N => n368, B0 => a_in(13), Y => 
                           n450);
   U105 : INVXL port map( A => a_in(14), Y => n358);
   U106 : XNOR2XL port map( A => n2, B => n263, Y => n266);
   U107 : INVXL port map( A => n431, Y => n269);
   U108 : OAI2BB1XL port map( A0N => n275, A1N => n278, B0 => a_in(22), Y => 
                           n432);
   U109 : INVXL port map( A => a_in(23), Y => n268);
   U110 : XNOR2XL port map( A => n2, B => n308, Y => n311);
   U111 : INVXL port map( A => n439, Y => n314);
   U112 : OAI2BB1XL port map( A0N => n320, A1N => n323, B0 => a_in(18), Y => 
                           n440);
   U113 : INVXL port map( A => a_in(19), Y => n313);
   U114 : XNOR2XL port map( A => n2, B => n254, Y => n257);
   U115 : INVXL port map( A => n429, Y => n260);
   U116 : OAI2BB1XL port map( A0N => n266, A1N => n269, B0 => a_in(23), Y => 
                           n430);
   U117 : INVXL port map( A => a_in(24), Y => n259);
   U118 : XNOR2XL port map( A => n2, B => n290, Y => n293);
   U119 : INVXL port map( A => n437, Y => n296);
   U120 : OAI2BB1XL port map( A0N => n311, A1N => n314, B0 => a_in(19), Y => 
                           n438);
   U121 : INVXL port map( A => a_in(20), Y => n295);
   U122 : XNOR2XL port map( A => n2, B => n281, Y => n284);
   U123 : INVXL port map( A => n435, Y => n287);
   U124 : OAI2BB1XL port map( A0N => n293, A1N => n296, B0 => a_in(20), Y => 
                           n436);
   U125 : INVXL port map( A => a_in(21), Y => n286);
   U126 : XNOR2XL port map( A => n2, B => n272, Y => n275);
   U127 : INVXL port map( A => n433, Y => n278);
   U128 : OAI2BB1XL port map( A0N => n284, A1N => n287, B0 => a_in(21), Y => 
                           n434);
   U129 : INVXL port map( A => a_in(22), Y => n277);
   U130 : XNOR2XL port map( A => n2, B => n317, Y => n320);
   U131 : INVXL port map( A => n441, Y => n323);
   U132 : OAI2BB1XL port map( A0N => n329, A1N => n332, B0 => a_in(17), Y => 
                           n442);
   U133 : INVXL port map( A => a_in(18), Y => n322);
   U134 : XNOR2XL port map( A => n2, B => n227, Y => n230);
   U135 : INVXL port map( A => n423, Y => n233);
   U136 : OAI2BB1XL port map( A0N => n239, A1N => n242, B0 => a_in(26), Y => 
                           n424);
   U137 : INVXL port map( A => a_in(27), Y => n232);
   U138 : XNOR2XL port map( A => n1, B => n191, Y => n194);
   U139 : INVXL port map( A => n417, Y => n197);
   U140 : OAI2BB1XL port map( A0N => n212, A1N => n215, B0 => a_in(29), Y => 
                           n418);
   U141 : INVXL port map( A => a_in(30), Y => n196);
   U142 : XNOR2XL port map( A => n2, B => n245, Y => n248);
   U143 : INVXL port map( A => n427, Y => n251);
   U144 : OAI2BB1XL port map( A0N => n257, A1N => n260, B0 => a_in(24), Y => 
                           n428);
   U145 : INVXL port map( A => n9, Y => n8);
   U146 : INVXL port map( A => n182, Y => n3);
   U147 : INVXL port map( A => a_in(25), Y => n250);
   U148 : XNOR2XL port map( A => n2, B => n236, Y => n239);
   U149 : INVXL port map( A => n425, Y => n242);
   U150 : OAI2BB1XL port map( A0N => n248, A1N => n251, B0 => a_in(25), Y => 
                           n426);
   U151 : INVXL port map( A => n6, Y => n5);
   U152 : INVXL port map( A => a_in(26), Y => n241);
   U153 : XOR2XL port map( A => n186, B => n187, Y => n178);
   U154 : XNOR2XL port map( A => a_in(31), B => n188, Y => n186);
   U155 : INVXL port map( A => a_in(31), Y => n176);
   U156 : AOI21XL port map( A0 => b_in(31), A1 => n180, B0 => n181, Y => n179);
   U157 : OAI32XL port map( A0 => n182, A1 => b_in(31), A2 => a_in(31), B0 => 
                           n183, B1 => n6, Y => n181);
   U158 : AOI22XL port map( A0 => a_in(29), A1 => n111, B0 => n211, B1 => n113,
                           Y => n210);
   U159 : XOR2XL port map( A => n212, B => n213, Y => n211);
   U160 : XNOR2XL port map( A => n214, B => n215, Y => n213);
   U161 : INVXL port map( A => b_in(29), Y => n209);
   U162 : AOI221XL port map( A0 => n5, A1 => n214, B0 => a_in(29), B1 => n119, 
                           C0 => n8, Y => n208);
   U163 : AOI22XL port map( A0 => a_in(29), A1 => n5, B0 => n3, B1 => n214, Y 
                           => n207);
   U164 : AOI22XL port map( A0 => a_in(28), A1 => n8, B0 => n220, B1 => n113, Y
                           => n219);
   U165 : XOR2XL port map( A => n221, B => n222, Y => n220);
   U166 : XNOR2XL port map( A => n223, B => n224, Y => n222);
   U167 : INVXL port map( A => b_in(28), Y => n218);
   U168 : AOI221XL port map( A0 => n118, A1 => n223, B0 => a_in(28), B1 => n119
                           , C0 => n111, Y => n217);
   U169 : AOI22XL port map( A0 => a_in(28), A1 => n5, B0 => n3, B1 => n223, Y 
                           => n216);
   U170 : INVXL port map( A => b_in(1), Y => n299);
   U171 : AOI22XL port map( A0 => a_in(1), A1 => n8, B0 => n301, B1 => n113, Y 
                           => n300);
   U172 : XOR2XL port map( A => n302, B => n303, Y => n301);
   U173 : XNOR2XL port map( A => n304, B => n305, Y => n303);
   U174 : AOI221XL port map( A0 => n118, A1 => n304, B0 => a_in(1), B1 => n119,
                           C0 => n111, Y => n298);
   U175 : AOI22XL port map( A0 => a_in(1), A1 => n5, B0 => n3, B1 => n304, Y =>
                           n297);
   U176 : AOI222XL port map( A0 => b_in(0), A1 => n399, B0 => n400, B1 => n401,
                           C0 => n402, C1 => n403, Y => n398);
   U177 : OAI32XL port map( A0 => n407, A1 => alu_function(1), A2 => 
                           alu_function(0), B0 => n404, B1 => n408, Y => n401);
   U178 : OAI22XL port map( A0 => a_in(0), A1 => n182, B0 => n6, B1 => n396, Y 
                           => n402);
   U179 : XOR2XL port map( A => n478, B => n476, Y => n397);
   U180 : XNOR2XL port map( A => n2, B => a_in(0), Y => n478);
   U181 : INVXL port map( A => a_in(0), Y => n396);
   U182 : INVXL port map( A => b_in(7), Y => n132);
   U183 : AOI22XL port map( A0 => a_in(7), A1 => n8, B0 => n134, B1 => n113, Y 
                           => n133);
   U184 : XOR2XL port map( A => n135, B => n136, Y => n134);
   U185 : XNOR2XL port map( A => n137, B => n138, Y => n136);
   U186 : AOI221XL port map( A0 => n118, A1 => n137, B0 => a_in(7), B1 => n119,
                           C0 => n111, Y => n131);
   U187 : AOI22XL port map( A0 => a_in(7), A1 => n5, B0 => n3, B1 => n137, Y =>
                           n130);
   U188 : INVXL port map( A => b_in(8), Y => n123);
   U189 : AOI22XL port map( A0 => a_in(8), A1 => n8, B0 => n125, B1 => n113, Y 
                           => n124);
   U190 : XOR2XL port map( A => n126, B => n127, Y => n125);
   U191 : XNOR2XL port map( A => n128, B => n129, Y => n127);
   U192 : AOI221XL port map( A0 => n118, A1 => n128, B0 => a_in(8), B1 => n119,
                           C0 => n111, Y => n122);
   U193 : AOI22XL port map( A0 => a_in(8), A1 => n5, B0 => n3, B1 => n128, Y =>
                           n121);
   U194 : INVXL port map( A => b_in(3), Y => n168);
   U195 : AOI22XL port map( A0 => a_in(3), A1 => n8, B0 => n170, B1 => n113, Y 
                           => n169);
   U196 : XOR2XL port map( A => n171, B => n172, Y => n170);
   U197 : XNOR2XL port map( A => n173, B => n174, Y => n172);
   U198 : AOI221XL port map( A0 => n118, A1 => n173, B0 => a_in(3), B1 => n119,
                           C0 => n111, Y => n167);
   U199 : AOI22XL port map( A0 => a_in(3), A1 => n5, B0 => n3, B1 => n173, Y =>
                           n166);
   U200 : AOI22XL port map( A0 => n111, A1 => a_in(9), B0 => n112, B1 => n113, 
                           Y => n110);
   U201 : XOR2XL port map( A => n114, B => n115, Y => n112);
   U202 : XNOR2XL port map( A => n116, B => n117, Y => n115);
   U203 : AOI221XL port map( A0 => n118, A1 => n116, B0 => n119, B1 => a_in(9),
                           C0 => n111, Y => n108);
   U204 : AOI22XL port map( A0 => a_in(9), A1 => n5, B0 => n3, B1 => n116, Y =>
                           n107);
   U205 : INVXL port map( A => b_in(9), Y => n109);
   U206 : INVXL port map( A => b_in(5), Y => n150);
   U207 : AOI22XL port map( A0 => a_in(5), A1 => n8, B0 => n152, B1 => n113, Y 
                           => n151);
   U208 : XOR2XL port map( A => n153, B => n154, Y => n152);
   U209 : XNOR2XL port map( A => n155, B => n156, Y => n154);
   U210 : AOI221XL port map( A0 => n118, A1 => n155, B0 => a_in(5), B1 => n119,
                           C0 => n111, Y => n149);
   U211 : AOI22XL port map( A0 => a_in(5), A1 => n5, B0 => n3, B1 => n155, Y =>
                           n148);
   U212 : INVXL port map( A => b_in(4), Y => n159);
   U213 : AOI22XL port map( A0 => a_in(4), A1 => n8, B0 => n161, B1 => n113, Y 
                           => n160);
   U214 : XOR2XL port map( A => n162, B => n163, Y => n161);
   U215 : XNOR2XL port map( A => n164, B => n165, Y => n163);
   U216 : AOI221XL port map( A0 => n118, A1 => n164, B0 => a_in(4), B1 => n119,
                           C0 => n111, Y => n158);
   U217 : AOI22XL port map( A0 => a_in(4), A1 => n5, B0 => n3, B1 => n164, Y =>
                           n157);
   U218 : INVXL port map( A => b_in(2), Y => n200);
   U219 : AOI22XL port map( A0 => a_in(2), A1 => n111, B0 => n202, B1 => n113, 
                           Y => n201);
   U220 : XOR2XL port map( A => n203, B => n204, Y => n202);
   U221 : XNOR2XL port map( A => n205, B => n206, Y => n204);
   U222 : AOI221XL port map( A0 => n118, A1 => n205, B0 => a_in(2), B1 => n119,
                           C0 => n111, Y => n199);
   U223 : AOI22XL port map( A0 => a_in(2), A1 => n5, B0 => n3, B1 => n205, Y =>
                           n198);
   U224 : INVXL port map( A => b_in(10), Y => n389);
   U225 : AOI22XL port map( A0 => a_in(10), A1 => n8, B0 => n391, B1 => n113, Y
                           => n390);
   U226 : XOR2XL port map( A => n392, B => n393, Y => n391);
   U227 : XNOR2XL port map( A => n394, B => n395, Y => n393);
   U228 : AOI221XL port map( A0 => n118, A1 => n394, B0 => a_in(10), B1 => n119
                           , C0 => n111, Y => n388);
   U229 : AOI22XL port map( A0 => a_in(10), A1 => n5, B0 => n3, B1 => n394, Y 
                           => n387);
   U230 : INVXL port map( A => b_in(6), Y => n141);
   U231 : AOI22XL port map( A0 => a_in(6), A1 => n8, B0 => n143, B1 => n113, Y 
                           => n142);
   U232 : XOR2XL port map( A => n144, B => n145, Y => n143);
   U233 : XNOR2XL port map( A => n146, B => n147, Y => n145);
   U234 : AOI221XL port map( A0 => n118, A1 => n146, B0 => a_in(6), B1 => n119,
                           C0 => n111, Y => n140);
   U235 : AOI22XL port map( A0 => a_in(6), A1 => n5, B0 => n3, B1 => n146, Y =>
                           n139);
   U236 : INVXL port map( A => b_in(15), Y => n344);
   U237 : AOI22XL port map( A0 => a_in(15), A1 => n8, B0 => n346, B1 => n113, Y
                           => n345);
   U238 : XOR2XL port map( A => n347, B => n348, Y => n346);
   U239 : XNOR2XL port map( A => n349, B => n350, Y => n348);
   U240 : AOI221XL port map( A0 => n118, A1 => n349, B0 => a_in(15), B1 => n119
                           , C0 => n111, Y => n343);
   U241 : AOI22XL port map( A0 => a_in(15), A1 => n118, B0 => n3, B1 => n349, Y
                           => n342);
   U242 : INVXL port map( A => b_in(11), Y => n380);
   U243 : AOI22XL port map( A0 => a_in(11), A1 => n8, B0 => n382, B1 => n113, Y
                           => n381);
   U244 : XOR2XL port map( A => n383, B => n384, Y => n382);
   U245 : XNOR2XL port map( A => n385, B => n386, Y => n384);
   U246 : AOI221XL port map( A0 => n118, A1 => n385, B0 => a_in(11), B1 => n119
                           , C0 => n111, Y => n379);
   U247 : AOI22XL port map( A0 => a_in(11), A1 => n118, B0 => n3, B1 => n385, Y
                           => n378);
   U248 : AOI22XL port map( A0 => a_in(16), A1 => n8, B0 => n337, B1 => n113, Y
                           => n336);
   U249 : XOR2XL port map( A => n338, B => n339, Y => n337);
   U250 : XNOR2XL port map( A => n340, B => n341, Y => n339);
   U251 : INVXL port map( A => b_in(16), Y => n335);
   U252 : AOI221XL port map( A0 => n118, A1 => n340, B0 => a_in(16), B1 => n119
                           , C0 => n111, Y => n334);
   U253 : AOI22XL port map( A0 => a_in(16), A1 => n118, B0 => n3, B1 => n340, Y
                           => n333);
   U254 : INVXL port map( A => b_in(12), Y => n371);
   U255 : AOI22XL port map( A0 => a_in(12), A1 => n8, B0 => n373, B1 => n113, Y
                           => n372);
   U256 : XOR2XL port map( A => n374, B => n375, Y => n373);
   U257 : XNOR2XL port map( A => n376, B => n377, Y => n375);
   U258 : AOI221XL port map( A0 => n118, A1 => n376, B0 => a_in(12), B1 => n119
                           , C0 => n111, Y => n370);
   U259 : AOI22XL port map( A0 => a_in(12), A1 => n118, B0 => n3, B1 => n376, Y
                           => n369);
   U260 : AOI22XL port map( A0 => a_in(17), A1 => n8, B0 => n328, B1 => n113, Y
                           => n327);
   U261 : XOR2XL port map( A => n329, B => n330, Y => n328);
   U262 : XNOR2XL port map( A => n331, B => n332, Y => n330);
   U263 : INVXL port map( A => b_in(17), Y => n326);
   U264 : AOI221XL port map( A0 => n118, A1 => n331, B0 => a_in(17), B1 => n119
                           , C0 => n111, Y => n325);
   U265 : AOI22XL port map( A0 => a_in(17), A1 => n5, B0 => n3, B1 => n331, Y 
                           => n324);
   U266 : INVXL port map( A => b_in(13), Y => n362);
   U267 : AOI22XL port map( A0 => a_in(13), A1 => n8, B0 => n364, B1 => n113, Y
                           => n363);
   U268 : XOR2XL port map( A => n365, B => n366, Y => n364);
   U269 : XNOR2XL port map( A => n367, B => n368, Y => n366);
   U270 : AOI221XL port map( A0 => n118, A1 => n367, B0 => a_in(13), B1 => n119
                           , C0 => n111, Y => n361);
   U271 : AOI22XL port map( A0 => a_in(13), A1 => n118, B0 => n3, B1 => n367, Y
                           => n360);
   U272 : INVXL port map( A => b_in(14), Y => n353);
   U273 : AOI22XL port map( A0 => a_in(14), A1 => n8, B0 => n355, B1 => n113, Y
                           => n354);
   U274 : XOR2XL port map( A => n356, B => n357, Y => n355);
   U275 : XNOR2XL port map( A => n358, B => n359, Y => n357);
   U276 : AOI221XL port map( A0 => n118, A1 => n358, B0 => a_in(14), B1 => n119
                           , C0 => n111, Y => n352);
   U277 : AOI22XL port map( A0 => a_in(14), A1 => n5, B0 => n3, B1 => n358, Y 
                           => n351);
   U278 : AOI22XL port map( A0 => a_in(23), A1 => n8, B0 => n265, B1 => n113, Y
                           => n264);
   U279 : XOR2XL port map( A => n266, B => n267, Y => n265);
   U280 : XNOR2XL port map( A => n268, B => n269, Y => n267);
   U281 : INVXL port map( A => b_in(23), Y => n263);
   U282 : AOI221XL port map( A0 => n118, A1 => n268, B0 => a_in(23), B1 => n119
                           , C0 => n111, Y => n262);
   U283 : AOI22XL port map( A0 => a_in(23), A1 => n5, B0 => n3, B1 => n268, Y 
                           => n261);
   U284 : AOI22XL port map( A0 => a_in(19), A1 => n8, B0 => n310, B1 => n113, Y
                           => n309);
   U285 : XOR2XL port map( A => n311, B => n312, Y => n310);
   U286 : XNOR2XL port map( A => n313, B => n314, Y => n312);
   U287 : INVXL port map( A => b_in(19), Y => n308);
   U288 : AOI221XL port map( A0 => n118, A1 => n313, B0 => a_in(19), B1 => n119
                           , C0 => n111, Y => n307);
   U289 : AOI22XL port map( A0 => a_in(19), A1 => n5, B0 => n3, B1 => n313, Y 
                           => n306);
   U290 : AOI22XL port map( A0 => a_in(24), A1 => n111, B0 => n256, B1 => n113,
                           Y => n255);
   U291 : XOR2XL port map( A => n257, B => n258, Y => n256);
   U292 : XNOR2XL port map( A => n259, B => n260, Y => n258);
   U293 : INVXL port map( A => b_in(24), Y => n254);
   U294 : AOI221XL port map( A0 => n118, A1 => n259, B0 => a_in(24), B1 => n119
                           , C0 => n111, Y => n253);
   U295 : AOI22XL port map( A0 => a_in(24), A1 => n5, B0 => n3, B1 => n259, Y 
                           => n252);
   U296 : AOI22XL port map( A0 => a_in(20), A1 => n8, B0 => n292, B1 => n113, Y
                           => n291);
   U297 : XOR2XL port map( A => n293, B => n294, Y => n292);
   U298 : XNOR2XL port map( A => n295, B => n296, Y => n294);
   U299 : INVXL port map( A => b_in(20), Y => n290);
   U300 : AOI221XL port map( A0 => n118, A1 => n295, B0 => a_in(20), B1 => n119
                           , C0 => n111, Y => n289);
   U301 : AOI22XL port map( A0 => a_in(20), A1 => n5, B0 => n3, B1 => n295, Y 
                           => n288);
   U302 : AOI22XL port map( A0 => a_in(21), A1 => n8, B0 => n283, B1 => n113, Y
                           => n282);
   U303 : XOR2XL port map( A => n284, B => n285, Y => n283);
   U304 : XNOR2XL port map( A => n286, B => n287, Y => n285);
   U305 : INVXL port map( A => b_in(21), Y => n281);
   U306 : AOI221XL port map( A0 => n118, A1 => n286, B0 => a_in(21), B1 => n119
                           , C0 => n111, Y => n280);
   U307 : AOI22XL port map( A0 => a_in(21), A1 => n5, B0 => n3, B1 => n286, Y 
                           => n279);
   U308 : AOI22XL port map( A0 => a_in(22), A1 => n8, B0 => n274, B1 => n113, Y
                           => n273);
   U309 : XOR2XL port map( A => n275, B => n276, Y => n274);
   U310 : XNOR2XL port map( A => n277, B => n278, Y => n276);
   U311 : INVXL port map( A => b_in(22), Y => n272);
   U312 : AOI221XL port map( A0 => n118, A1 => n277, B0 => a_in(22), B1 => n119
                           , C0 => n111, Y => n271);
   U313 : AOI22XL port map( A0 => a_in(22), A1 => n5, B0 => n3, B1 => n277, Y 
                           => n270);
   U314 : AOI22XL port map( A0 => a_in(18), A1 => n8, B0 => n319, B1 => n113, Y
                           => n318);
   U315 : XOR2XL port map( A => n320, B => n321, Y => n319);
   U316 : XNOR2XL port map( A => n322, B => n323, Y => n321);
   U317 : INVXL port map( A => b_in(18), Y => n317);
   U318 : AOI221XL port map( A0 => n118, A1 => n322, B0 => a_in(18), B1 => n119
                           , C0 => n111, Y => n316);
   U319 : AOI22XL port map( A0 => a_in(18), A1 => n118, B0 => n3, B1 => n322, Y
                           => n315);
   U320 : AOI22XL port map( A0 => a_in(27), A1 => n111, B0 => n229, B1 => n113,
                           Y => n228);
   U321 : XOR2XL port map( A => n230, B => n231, Y => n229);
   U322 : XNOR2XL port map( A => n232, B => n233, Y => n231);
   U323 : INVXL port map( A => b_in(27), Y => n227);
   U324 : AOI221XL port map( A0 => n118, A1 => n232, B0 => a_in(27), B1 => n119
                           , C0 => n111, Y => n226);
   U325 : AOI22XL port map( A0 => a_in(27), A1 => n5, B0 => n3, B1 => n232, Y 
                           => n225);
   U326 : AOI22XL port map( A0 => a_in(30), A1 => n8, B0 => n193, B1 => n113, Y
                           => n192);
   U327 : XOR2XL port map( A => n194, B => n195, Y => n193);
   U328 : XNOR2XL port map( A => n196, B => n197, Y => n195);
   U329 : INVXL port map( A => b_in(30), Y => n191);
   U330 : AOI221XL port map( A0 => n118, A1 => n196, B0 => a_in(30), B1 => n119
                           , C0 => n111, Y => n190);
   U331 : AOI22XL port map( A0 => a_in(30), A1 => n5, B0 => n3, B1 => n196, Y 
                           => n189);
   U332 : AOI22XL port map( A0 => a_in(25), A1 => n8, B0 => n247, B1 => n113, Y
                           => n246);
   U333 : XOR2XL port map( A => n248, B => n249, Y => n247);
   U334 : XNOR2XL port map( A => n250, B => n251, Y => n249);
   U335 : INVXL port map( A => b_in(25), Y => n245);
   U336 : AOI221XL port map( A0 => n118, A1 => n250, B0 => a_in(25), B1 => n119
                           , C0 => n111, Y => n244);
   U337 : AOI22XL port map( A0 => a_in(25), A1 => n5, B0 => n3, B1 => n250, Y 
                           => n243);
   U338 : AOI22XL port map( A0 => a_in(26), A1 => n111, B0 => n238, B1 => n113,
                           Y => n237);
   U339 : XOR2XL port map( A => n239, B => n240, Y => n238);
   U340 : XNOR2XL port map( A => n241, B => n242, Y => n240);
   U341 : INVXL port map( A => b_in(26), Y => n236);
   U342 : AOI221XL port map( A0 => n118, A1 => n241, B0 => a_in(26), B1 => n119
                           , C0 => n111, Y => n235);
   U343 : AOI22XL port map( A0 => a_in(26), A1 => n5, B0 => n3, B1 => n241, Y 
                           => n234);
   U344 : OAI221XL port map( A0 => n9, A1 => n396, B0 => n7, B1 => n397, C0 => 
                           n398, Y => c_alu(0));
   U345 : OAI221XL port map( A0 => n9, A1 => n176, B0 => n7, B1 => n178, C0 => 
                           n179, Y => c_alu(31));
   U346 : OAI21XL port map( A0 => n1, A1 => n476, B0 => n477, Y => n475);
   U347 : OAI21XL port map( A0 => n341, A1 => n338, B0 => n444, Y => n443);
   U348 : OAI21XL port map( A0 => n197, A1 => n194, B0 => n415, Y => n414);
   U349 : OAI21XL port map( A0 => n305, A1 => n302, B0 => n474, Y => n473);
   U350 : OAI21XL port map( A0 => n206, A1 => n203, B0 => n472, Y => n471);
   U351 : OAI21XL port map( A0 => n174, A1 => n171, B0 => n470, Y => n469);
   U352 : OAI21XL port map( A0 => n165, A1 => n162, B0 => n468, Y => n467);
   U353 : OAI21XL port map( A0 => n156, A1 => n153, B0 => n466, Y => n465);
   U354 : OAI21XL port map( A0 => n147, A1 => n144, B0 => n464, Y => n463);
   U355 : OAI21XL port map( A0 => n138, A1 => n135, B0 => n462, Y => n461);
   U356 : OAI21XL port map( A0 => n129, A1 => n126, B0 => n460, Y => n459);
   U357 : OAI21XL port map( A0 => n117, A1 => n114, B0 => n458, Y => n457);
   U358 : OAI21XL port map( A0 => n395, A1 => n392, B0 => n456, Y => n455);
   U359 : OAI21XL port map( A0 => n386, A1 => n383, B0 => n454, Y => n453);
   U360 : OAI21XL port map( A0 => n377, A1 => n374, B0 => n452, Y => n451);
   U361 : OAI21XL port map( A0 => n368, A1 => n365, B0 => n450, Y => n449);
   U362 : OAI21XL port map( A0 => n359, A1 => n356, B0 => n448, Y => n447);
   U363 : OAI21XL port map( A0 => n350, A1 => n347, B0 => n446, Y => n445);
   U364 : OAI21XL port map( A0 => n332, A1 => n329, B0 => n442, Y => n441);
   U365 : OAI21XL port map( A0 => n323, A1 => n320, B0 => n440, Y => n439);
   U366 : OAI21XL port map( A0 => n314, A1 => n311, B0 => n438, Y => n437);
   U367 : OAI21XL port map( A0 => n296, A1 => n293, B0 => n436, Y => n435);
   U368 : OAI21XL port map( A0 => n287, A1 => n284, B0 => n434, Y => n433);
   U369 : OAI21XL port map( A0 => n278, A1 => n275, B0 => n432, Y => n431);
   U370 : OAI21XL port map( A0 => n269, A1 => n266, B0 => n430, Y => n429);
   U371 : OAI21XL port map( A0 => n260, A1 => n257, B0 => n428, Y => n427);
   U372 : OAI21XL port map( A0 => n251, A1 => n248, B0 => n426, Y => n425);
   U373 : OAI21XL port map( A0 => n242, A1 => n239, B0 => n424, Y => n423);
   U374 : OAI21XL port map( A0 => n233, A1 => n230, B0 => n422, Y => n421);
   U375 : OAI21XL port map( A0 => n224, A1 => n221, B0 => n420, Y => n419);
   U376 : OAI21XL port map( A0 => n215, A1 => n212, B0 => n418, Y => n417);
   U377 : OAI221XL port map( A0 => n4, A1 => n396, B0 => a_in(0), B1 => n6, C0 
                           => n9, Y => n399);
   U378 : OAI221XL port map( A0 => b_in(29), A1 => n207, B0 => n208, B1 => n209
                           , C0 => n210, Y => c_alu(29));
   U379 : OAI221XL port map( A0 => b_in(28), A1 => n216, B0 => n217, B1 => n218
                           , C0 => n219, Y => c_alu(28));
   U380 : OAI221XL port map( A0 => b_in(30), A1 => n189, B0 => n190, B1 => n191
                           , C0 => n192, Y => c_alu(30));
   U381 : OAI221XL port map( A0 => b_in(25), A1 => n243, B0 => n244, B1 => n245
                           , C0 => n246, Y => c_alu(25));
   U382 : OAI221XL port map( A0 => b_in(26), A1 => n234, B0 => n235, B1 => n236
                           , C0 => n237, Y => c_alu(26));
   U383 : OAI221XL port map( A0 => b_in(27), A1 => n225, B0 => n226, B1 => n227
                           , C0 => n228, Y => c_alu(27));
   U384 : OAI221XL port map( A0 => b_in(23), A1 => n261, B0 => n262, B1 => n263
                           , C0 => n264, Y => c_alu(23));
   U385 : OAI221XL port map( A0 => b_in(24), A1 => n252, B0 => n253, B1 => n254
                           , C0 => n255, Y => c_alu(24));
   U386 : OAI221XL port map( A0 => b_in(19), A1 => n306, B0 => n307, B1 => n308
                           , C0 => n309, Y => c_alu(19));
   U387 : OAI221XL port map( A0 => b_in(20), A1 => n288, B0 => n289, B1 => n290
                           , C0 => n291, Y => c_alu(20));
   U388 : OAI221XL port map( A0 => b_in(21), A1 => n279, B0 => n280, B1 => n281
                           , C0 => n282, Y => c_alu(21));
   U389 : OAI221XL port map( A0 => b_in(22), A1 => n270, B0 => n271, B1 => n272
                           , C0 => n273, Y => c_alu(22));
   U390 : OAI221XL port map( A0 => b_in(17), A1 => n324, B0 => n325, B1 => n326
                           , C0 => n327, Y => c_alu(17));
   U391 : OAI221XL port map( A0 => b_in(18), A1 => n315, B0 => n316, B1 => n317
                           , C0 => n318, Y => c_alu(18));
   U392 : OAI221XL port map( A0 => b_in(14), A1 => n351, B0 => n352, B1 => n353
                           , C0 => n354, Y => c_alu(14));
   U393 : OAI221XL port map( A0 => b_in(15), A1 => n342, B0 => n343, B1 => n344
                           , C0 => n345, Y => c_alu(15));
   U394 : OAI221XL port map( A0 => b_in(16), A1 => n333, B0 => n334, B1 => n335
                           , C0 => n336, Y => c_alu(16));
   U395 : OAI221XL port map( A0 => b_in(11), A1 => n378, B0 => n379, B1 => n380
                           , C0 => n381, Y => c_alu(11));
   U396 : OAI221XL port map( A0 => b_in(12), A1 => n369, B0 => n370, B1 => n371
                           , C0 => n372, Y => c_alu(12));
   U397 : OAI221XL port map( A0 => b_in(13), A1 => n360, B0 => n361, B1 => n362
                           , C0 => n363, Y => c_alu(13));
   U398 : OAI221XL port map( A0 => b_in(10), A1 => n387, B0 => n388, B1 => n389
                           , C0 => n390, Y => c_alu(10));
   U399 : OAI221XL port map( A0 => b_in(9), A1 => n107, B0 => n108, B1 => n109,
                           C0 => n110, Y => c_alu(9));
   U400 : OAI221XL port map( A0 => b_in(1), A1 => n297, B0 => n298, B1 => n299,
                           C0 => n300, Y => c_alu(1));
   U401 : OAI221XL port map( A0 => b_in(8), A1 => n121, B0 => n122, B1 => n123,
                           C0 => n124, Y => c_alu(8));
   U402 : OAI221XL port map( A0 => b_in(6), A1 => n139, B0 => n140, B1 => n141,
                           C0 => n142, Y => c_alu(6));
   U403 : OAI221XL port map( A0 => b_in(7), A1 => n130, B0 => n131, B1 => n132,
                           C0 => n133, Y => c_alu(7));
   U404 : OAI221XL port map( A0 => b_in(3), A1 => n166, B0 => n167, B1 => n168,
                           C0 => n169, Y => c_alu(3));
   U405 : OAI221XL port map( A0 => b_in(5), A1 => n148, B0 => n149, B1 => n150,
                           C0 => n151, Y => c_alu(5));
   U406 : OAI221XL port map( A0 => b_in(4), A1 => n157, B0 => n158, B1 => n159,
                           C0 => n160, Y => c_alu(4));
   U407 : OAI21XL port map( A0 => alu_function(0), A1 => n408, B0 => n413, Y =>
                           n113);
   U408 : OAI221XL port map( A0 => b_in(2), A1 => n198, B0 => n199, B1 => n200,
                           C0 => n201, Y => c_alu(2));
   U409 : OAI21XL port map( A0 => n4, A1 => n176, B0 => n9, Y => n180);

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity bus_mux is

   port( imm_in : in std_logic_vector (15 downto 0);  reg_source : in 
         std_logic_vector (31 downto 0);  a_mux : in std_logic_vector (1 downto
         0);  a_out : out std_logic_vector (31 downto 0);  reg_target : in 
         std_logic_vector (31 downto 0);  b_mux : in std_logic_vector (1 downto
         0);  b_out : out std_logic_vector (31 downto 0);  c_bus, c_memory : in
         std_logic_vector (31 downto 0);  c_pc, c_pc_plus4 : in 
         std_logic_vector (31 downto 2);  c_mux : in std_logic_vector (2 downto
         0);  reg_dest_out : out std_logic_vector (31 downto 0);  branch_func :
         in std_logic_vector (2 downto 0);  take_branch : out std_logic);

end bus_mux;

architecture SYN_logic of bus_mux is

   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI222XL
      port( A0, A1, B0, B1, C0, C1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI32XL
      port( A0, A1, A2, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component XNOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   signal n78, n79, n80, n81, n82, n83, n84, n85, n86, n87, n88, n89, n90, n91,
      n92, n93, n94, n95, n96, n97, n98, n99, n100, n101, n102, n103, n104, 
      n105, n106, n107, n108, n109, n110, n111, n112, n113, n114, n115, n116, 
      n117, n118, n119, n120, n121, n122, n123, n124, n125, n126, n127, n128, 
      n129, n130, n131, n133, n136, n137, n138, n139, n140, n141, n142, n143, 
      n144, n145, n146, n147, n149, n150, n151, n152, n153, n154, n155, n156, 
      n157, n158, n159, n160, n161, n162, n163, n164, n165, n166, n167, n168, 
      n169, n170, n171, n172, n173, n174, n175, n176, n177, n178, n179, n180, 
      n181, n182, n183, n184, n185, n186, n187, n188, n189, n190, n191, n192, 
      n193, n194, n195, n196, n197, n198, n199, n200, n201, n202, n203, n204, 
      n205, n206, n207, n208, n209, n210, n211, n212, n213, n214, n215, n216, 
      n217, n218, n219, n220, n221, n222, n223, n224, n225, n226, n227, n228, 
      n229, n230, n231, n232, n233, n234, n235, n236, n237, n238, n239, n240, 
      n242, n243, n244, n245, n246, n247, n248, n249, n250, n251, n252, n253, 
      n254, n255, n256, n257, n258, n259, n260, n261, n262, n263, n264, n265, 
      n266, n267, n268, n269, n270, n271, n272, n273, n1, n2, n3, n4, n5, n6 : 
      std_logic;

begin
   
   U3 : NOR3XL port map( A => c_mux(0), B => c_mux(2), C => n201, Y => n1);
   U4 : NOR2XL port map( A => a_mux(0), B => a_mux(1), Y => n2);
   U5 : XNOR2XL port map( A => c_mux(1), B => c_mux(2), Y => n3);
   U6 : NOR3XL port map( A => c_mux(0), B => c_mux(1), C => n214, Y => n4);
   U7 : XNOR2XL port map( A => reg_source(0), B => reg_target(0), Y => n114);
   U8 : XNOR2XL port map( A => reg_source(7), B => reg_target(7), Y => n121);
   U9 : XNOR2XL port map( A => reg_source(9), B => reg_target(9), Y => n123);
   U10 : XNOR2XL port map( A => reg_source(15), B => reg_target(15), Y => n129)
                           ;
   U11 : XNOR2XL port map( A => reg_source(19), B => reg_target(19), Y => n97);
   U12 : XNOR2XL port map( A => reg_source(23), B => reg_target(23), Y => n101)
                           ;
   U13 : XNOR2XL port map( A => reg_source(27), B => reg_target(27), Y => n105)
                           ;
   U14 : XNOR2XL port map( A => reg_target(31), B => reg_source(31), Y => n109)
                           ;
   U15 : XNOR2XL port map( A => reg_source(30), B => reg_target(30), Y => n108)
                           ;
   U16 : XNOR2XL port map( A => reg_source(29), B => reg_target(29), Y => n107)
                           ;
   U17 : XNOR2XL port map( A => reg_source(28), B => reg_target(28), Y => n106)
                           ;
   U18 : NAND4XL port map( A => n114, B => n115, C => n116, D => n117, Y => 
                           n113);
   U19 : XNOR2XL port map( A => reg_source(3), B => reg_target(3), Y => n117);
   U20 : XNOR2XL port map( A => reg_source(2), B => reg_target(2), Y => n116);
   U21 : XNOR2XL port map( A => reg_source(1), B => reg_target(1), Y => n115);
   U22 : NAND4XL port map( A => n118, B => n119, C => n120, D => n121, Y => 
                           n112);
   U23 : XNOR2XL port map( A => reg_source(4), B => reg_target(4), Y => n118);
   U24 : XNOR2XL port map( A => reg_source(5), B => reg_target(5), Y => n119);
   U25 : XNOR2XL port map( A => reg_source(6), B => reg_target(6), Y => n120);
   U26 : NAND4XL port map( A => n122, B => n123, C => n124, D => n125, Y => 
                           n111);
   U27 : XNOR2XL port map( A => reg_source(10), B => reg_target(10), Y => n124)
                           ;
   U28 : XNOR2XL port map( A => reg_source(11), B => reg_target(11), Y => n125)
                           ;
   U29 : XNOR2XL port map( A => reg_source(8), B => reg_target(8), Y => n122);
   U30 : NAND4XL port map( A => n126, B => n127, C => n128, D => n129, Y => 
                           n110);
   U31 : XNOR2XL port map( A => reg_source(12), B => reg_target(12), Y => n126)
                           ;
   U32 : XNOR2XL port map( A => reg_source(14), B => reg_target(14), Y => n128)
                           ;
   U33 : XNOR2XL port map( A => reg_source(13), B => reg_target(13), Y => n127)
                           ;
   U34 : NAND4XL port map( A => n94, B => n95, C => n96, D => n97, Y => n93);
   U35 : XNOR2XL port map( A => reg_source(16), B => reg_target(16), Y => n94);
   U36 : XNOR2XL port map( A => reg_source(17), B => reg_target(17), Y => n95);
   U37 : XNOR2XL port map( A => reg_source(18), B => reg_target(18), Y => n96);
   U38 : NAND4XL port map( A => n98, B => n99, C => n100, D => n101, Y => n92);
   U39 : XNOR2XL port map( A => reg_source(20), B => reg_target(20), Y => n98);
   U40 : XNOR2XL port map( A => reg_source(21), B => reg_target(21), Y => n99);
   U41 : XNOR2XL port map( A => reg_source(22), B => reg_target(22), Y => n100)
                           ;
   U42 : NAND4XL port map( A => n102, B => n103, C => n104, D => n105, Y => n91
                           );
   U43 : XNOR2XL port map( A => reg_source(24), B => reg_target(24), Y => n102)
                           ;
   U44 : XNOR2XL port map( A => reg_source(25), B => reg_target(25), Y => n103)
                           ;
   U45 : XNOR2XL port map( A => reg_source(26), B => reg_target(26), Y => n104)
                           ;
   U46 : INVXL port map( A => reg_source(31), Y => n81);
   U47 : INVXL port map( A => n84, Y => n86);
   U48 : NAND2XL port map( A => n88, B => n89, Y => n84);
   U49 : NOR4XL port map( A => n90, B => n91, C => n92, D => n93, Y => n89);
   U50 : NOR4XL port map( A => n110, B => n111, C => n112, D => n113, Y => n88)
                           ;
   U51 : NAND4XL port map( A => n106, B => n107, C => n108, D => n109, Y => n90
                           );
   U52 : OAI2BB1XL port map( A0N => branch_func(0), A1N => n86, B0 => n81, Y =>
                           n87);
   U53 : INVXL port map( A => branch_func(1), Y => n85);
   U54 : AOI211XL port map( A0 => n83, A1 => n84, B0 => n85, C0 => 
                           branch_func(0), Y => n82);
   U55 : INVXL port map( A => branch_func(2), Y => n83);
   U56 : INVXL port map( A => c_mux(1), Y => n201);
   U57 : AOI31XL port map( A0 => n80, A1 => n81, A2 => branch_func(2), B0 => 
                           n82, Y => n79);
   U58 : AOI32XL port map( A0 => branch_func(1), A1 => n84, A2 => 
                           branch_func(0), B0 => n87, B1 => n85, Y => n78);
   U59 : INVXL port map( A => b_mux(1), Y => n239);
   U60 : INVXL port map( A => n6, Y => n152);
   U61 : NAND3XL port map( A => c_mux(2), B => n201, C => c_mux(0), Y => n6);
   U62 : INVXL port map( A => c_mux(2), Y => n214);
   U63 : NOR2BXL port map( AN => a_mux(0), B => a_mux(1), Y => n247);
   U64 : INVXL port map( A => n238, Y => n217);
   U65 : AOI21XL port map( A0 => n239, A1 => b_mux(0), B0 => n229, Y => n238);
   U66 : NOR2XL port map( A => n239, B => b_mux(0), Y => n229);
   U67 : AND2XL port map( A => b_mux(0), B => b_mux(1), Y => n218);
   U68 : NOR2XL port map( A => b_mux(0), B => b_mux(1), Y => n219);
   U69 : AOI22XL port map( A0 => c_memory(2), A1 => n1, B0 => c_bus(2), B1 => 
                           n3, Y => n156);
   U70 : AOI22XL port map( A0 => c_memory(3), A1 => n1, B0 => c_bus(3), B1 => 
                           n3, Y => n146);
   U71 : AOI22XL port map( A0 => c_memory(4), A1 => n1, B0 => c_bus(4), B1 => 
                           n3, Y => n144);
   U72 : AOI22XL port map( A0 => c_memory(5), A1 => n1, B0 => c_bus(5), B1 => 
                           n3, Y => n142);
   U73 : AOI22XL port map( A0 => c_memory(6), A1 => n1, B0 => c_bus(6), B1 => 
                           n3, Y => n140);
   U74 : AOI22XL port map( A0 => c_memory(7), A1 => n1, B0 => c_bus(7), B1 => 
                           n3, Y => n138);
   U75 : AOI22XL port map( A0 => c_memory(8), A1 => n1, B0 => c_bus(8), B1 => 
                           n3, Y => n136);
   U76 : AOI22XL port map( A0 => c_memory(9), A1 => n1, B0 => c_bus(9), B1 => 
                           n3, Y => n130);
   U77 : AOI22XL port map( A0 => c_memory(10), A1 => n1, B0 => c_bus(10), B1 =>
                           n3, Y => n212);
   U78 : AOI22XL port map( A0 => c_memory(11), A1 => n1, B0 => c_bus(11), B1 =>
                           n3, Y => n210);
   U79 : AOI22XL port map( A0 => c_memory(12), A1 => n1, B0 => c_bus(12), B1 =>
                           n3, Y => n208);
   U80 : AOI22XL port map( A0 => c_memory(13), A1 => n1, B0 => c_bus(13), B1 =>
                           n3, Y => n206);
   U81 : AOI22XL port map( A0 => c_memory(14), A1 => n1, B0 => c_bus(14), B1 =>
                           n3, Y => n204);
   U82 : INVXL port map( A => n5, Y => n133);
   U83 : AOI22XL port map( A0 => c_memory(15), A1 => n1, B0 => c_bus(15), B1 =>
                           n3, Y => n202);
   U84 : AOI22XL port map( A0 => c_memory(16), A1 => n1, B0 => c_bus(16), B1 =>
                           n3, Y => n200);
   U85 : AOI22XL port map( A0 => imm_in(0), A1 => n152, B0 => c_pc_plus4(16), 
                           B1 => n4, Y => n199);
   U86 : AOI22XL port map( A0 => c_memory(17), A1 => n1, B0 => c_bus(17), B1 =>
                           n3, Y => n197);
   U87 : AOI22XL port map( A0 => imm_in(1), A1 => n152, B0 => c_pc_plus4(17), 
                           B1 => n4, Y => n196);
   U88 : AOI22XL port map( A0 => c_memory(18), A1 => n1, B0 => c_bus(18), B1 =>
                           n3, Y => n194);
   U89 : AOI22XL port map( A0 => imm_in(2), A1 => n152, B0 => c_pc_plus4(18), 
                           B1 => n4, Y => n193);
   U90 : AOI22XL port map( A0 => c_memory(19), A1 => n1, B0 => c_bus(19), B1 =>
                           n3, Y => n191);
   U91 : AOI22XL port map( A0 => imm_in(3), A1 => n152, B0 => c_pc_plus4(19), 
                           B1 => n4, Y => n190);
   U92 : AOI22XL port map( A0 => c_memory(20), A1 => n1, B0 => c_bus(20), B1 =>
                           n3, Y => n187);
   U93 : AOI22XL port map( A0 => imm_in(4), A1 => n152, B0 => c_pc_plus4(20), 
                           B1 => n4, Y => n186);
   U94 : AOI22XL port map( A0 => c_memory(21), A1 => n1, B0 => c_bus(21), B1 =>
                           n3, Y => n184);
   U95 : AOI22XL port map( A0 => imm_in(5), A1 => n152, B0 => c_pc_plus4(21), 
                           B1 => n4, Y => n183);
   U96 : AOI22XL port map( A0 => c_memory(22), A1 => n1, B0 => c_bus(22), B1 =>
                           n3, Y => n181);
   U97 : AOI22XL port map( A0 => imm_in(6), A1 => n152, B0 => c_pc_plus4(22), 
                           B1 => n4, Y => n180);
   U98 : AOI22XL port map( A0 => c_memory(23), A1 => n1, B0 => c_bus(23), B1 =>
                           n3, Y => n178);
   U99 : AOI22XL port map( A0 => imm_in(7), A1 => n152, B0 => c_pc_plus4(23), 
                           B1 => n4, Y => n177);
   U100 : AOI22XL port map( A0 => c_memory(24), A1 => n1, B0 => c_bus(24), B1 
                           => n3, Y => n175);
   U101 : AOI22XL port map( A0 => imm_in(8), A1 => n152, B0 => c_pc_plus4(24), 
                           B1 => n4, Y => n174);
   U102 : AOI22XL port map( A0 => c_memory(25), A1 => n1, B0 => c_bus(25), B1 
                           => n3, Y => n172);
   U103 : AOI22XL port map( A0 => imm_in(9), A1 => n152, B0 => c_pc_plus4(25), 
                           B1 => n4, Y => n171);
   U104 : AOI22XL port map( A0 => c_memory(26), A1 => n1, B0 => c_bus(26), B1 
                           => n3, Y => n169);
   U105 : AOI22XL port map( A0 => imm_in(10), A1 => n152, B0 => c_pc_plus4(26),
                           B1 => n4, Y => n168);
   U106 : AOI22XL port map( A0 => c_memory(27), A1 => n1, B0 => c_bus(27), B1 
                           => n3, Y => n166);
   U107 : AOI22XL port map( A0 => imm_in(11), A1 => n152, B0 => c_pc_plus4(27),
                           B1 => n4, Y => n165);
   U108 : AOI22XL port map( A0 => c_memory(28), A1 => n1, B0 => c_bus(28), B1 
                           => n3, Y => n163);
   U109 : AOI22XL port map( A0 => imm_in(12), A1 => n152, B0 => c_pc_plus4(28),
                           B1 => n4, Y => n162);
   U110 : AOI22XL port map( A0 => c_memory(29), A1 => n1, B0 => c_bus(29), B1 
                           => n3, Y => n160);
   U111 : AOI22XL port map( A0 => imm_in(13), A1 => n152, B0 => c_pc_plus4(29),
                           B1 => n4, Y => n159);
   U112 : AOI22XL port map( A0 => c_memory(30), A1 => n1, B0 => c_bus(30), B1 
                           => n3, Y => n155);
   U113 : AOI22XL port map( A0 => imm_in(14), A1 => n152, B0 => c_pc_plus4(30),
                           B1 => n4, Y => n154);
   U114 : AOI22XL port map( A0 => c_memory(31), A1 => n1, B0 => c_bus(31), B1 
                           => n3, Y => n151);
   U115 : AOI22XL port map( A0 => imm_in(15), A1 => n152, B0 => c_pc_plus4(31),
                           B1 => n4, Y => n150);
   U116 : NAND3XL port map( A => c_mux(0), B => n214, C => c_mux(1), Y => n5);
   U117 : INVXL port map( A => n273, Y => a_out(0));
   U118 : AOI22XL port map( A0 => reg_source(0), A1 => n2, B0 => imm_in(6), B1 
                           => n247, Y => n273);
   U119 : INVXL port map( A => n228, Y => b_out(1));
   U120 : AOI22XL port map( A0 => reg_target(1), A1 => n219, B0 => n217, B1 => 
                           imm_in(1), Y => n228);
   U121 : INVXL port map( A => n237, Y => b_out(0));
   U122 : AOI22XL port map( A0 => reg_target(0), A1 => n219, B0 => n217, B1 => 
                           imm_in(0), Y => n237);
   U123 : INVXL port map( A => n248, Y => a_out(3));
   U124 : AOI222XL port map( A0 => reg_source(3), A1 => n2, B0 => imm_in(9), B1
                           => n247, C0 => c_pc(3), C1 => a_mux(1), Y => n248);
   U125 : INVXL port map( A => n251, Y => a_out(2));
   U126 : AOI222XL port map( A0 => reg_source(2), A1 => n2, B0 => imm_in(8), B1
                           => n247, C0 => c_pc(2), C1 => a_mux(1), Y => n251);
   U127 : INVXL port map( A => n262, Y => a_out(1));
   U128 : AOI22XL port map( A0 => reg_source(1), A1 => n2, B0 => imm_in(7), B1 
                           => n247, Y => n262);
   U129 : INVXL port map( A => n225, Y => b_out(3));
   U130 : AOI222XL port map( A0 => n217, A1 => imm_in(3), B0 => imm_in(1), B1 
                           => n218, C0 => reg_target(3), C1 => n219, Y => n225)
                           ;
   U131 : INVXL port map( A => n227, Y => b_out(2));
   U132 : AOI222XL port map( A0 => n217, A1 => imm_in(2), B0 => imm_in(0), B1 
                           => n218, C0 => reg_target(2), C1 => n219, Y => n227)
                           ;
   U133 : INVXL port map( A => n224, Y => b_out(4));
   U134 : AOI222XL port map( A0 => n217, A1 => imm_in(4), B0 => imm_in(2), B1 
                           => n218, C0 => reg_target(4), C1 => n219, Y => n224)
                           ;
   U135 : INVXL port map( A => n246, Y => a_out(4));
   U136 : AOI222XL port map( A0 => reg_source(4), A1 => n2, B0 => imm_in(10), 
                           B1 => n247, C0 => c_pc(4), C1 => a_mux(1), Y => n246
                           );
   U137 : INVXL port map( A => n221, Y => b_out(7));
   U138 : AOI222XL port map( A0 => n217, A1 => imm_in(7), B0 => imm_in(5), B1 
                           => n218, C0 => reg_target(7), C1 => n219, Y => n221)
                           ;
   U139 : INVXL port map( A => n222, Y => b_out(6));
   U140 : AOI222XL port map( A0 => n217, A1 => imm_in(6), B0 => imm_in(4), B1 
                           => n218, C0 => reg_target(6), C1 => n219, Y => n222)
                           ;
   U141 : INVXL port map( A => n223, Y => b_out(5));
   U142 : AOI222XL port map( A0 => n217, A1 => imm_in(5), B0 => imm_in(3), B1 
                           => n218, C0 => reg_target(5), C1 => n219, Y => n223)
                           ;
   U143 : INVXL port map( A => n245, Y => a_out(5));
   U144 : AOI22XL port map( A0 => c_pc(5), A1 => a_mux(1), B0 => reg_source(5),
                           B1 => n2, Y => n245);
   U145 : INVXL port map( A => n216, Y => b_out(9));
   U146 : AOI222XL port map( A0 => n217, A1 => imm_in(9), B0 => imm_in(7), B1 
                           => n218, C0 => reg_target(9), C1 => n219, Y => n216)
                           ;
   U147 : INVXL port map( A => n220, Y => b_out(8));
   U148 : AOI222XL port map( A0 => n217, A1 => imm_in(8), B0 => imm_in(6), B1 
                           => n218, C0 => reg_target(8), C1 => n219, Y => n220)
                           ;
   U149 : INVXL port map( A => n243, Y => a_out(7));
   U150 : AOI22XL port map( A0 => c_pc(7), A1 => a_mux(1), B0 => reg_source(7),
                           B1 => n2, Y => n243);
   U151 : INVXL port map( A => n244, Y => a_out(6));
   U152 : AOI22XL port map( A0 => c_pc(6), A1 => a_mux(1), B0 => reg_source(6),
                           B1 => n2, Y => n244);
   U153 : INVXL port map( A => n242, Y => a_out(8));
   U154 : AOI22XL port map( A0 => c_pc(8), A1 => a_mux(1), B0 => reg_source(8),
                           B1 => n2, Y => n242);
   U155 : INVXL port map( A => n236, Y => b_out(10));
   U156 : AOI222XL port map( A0 => n217, A1 => imm_in(10), B0 => imm_in(8), B1 
                           => n218, C0 => reg_target(10), C1 => n219, Y => n236
                           );
   U157 : INVXL port map( A => n235, Y => b_out(11));
   U158 : AOI222XL port map( A0 => n217, A1 => imm_in(11), B0 => imm_in(9), B1 
                           => n218, C0 => reg_target(11), C1 => n219, Y => n235
                           );
   U159 : INVXL port map( A => n234, Y => b_out(12));
   U160 : AOI222XL port map( A0 => n217, A1 => imm_in(12), B0 => imm_in(10), B1
                           => n218, C0 => reg_target(12), C1 => n219, Y => n234
                           );
   U161 : INVXL port map( A => n271, Y => a_out(11));
   U162 : AOI22XL port map( A0 => c_pc(11), A1 => a_mux(1), B0 => 
                           reg_source(11), B1 => n2, Y => n271);
   U163 : INVXL port map( A => n272, Y => a_out(10));
   U164 : AOI22XL port map( A0 => c_pc(10), A1 => a_mux(1), B0 => 
                           reg_source(10), B1 => n2, Y => n272);
   U165 : INVXL port map( A => n240, Y => a_out(9));
   U166 : AOI22XL port map( A0 => c_pc(9), A1 => a_mux(1), B0 => reg_source(9),
                           B1 => n2, Y => n240);
   U167 : INVXL port map( A => n233, Y => b_out(13));
   U168 : AOI222XL port map( A0 => n217, A1 => imm_in(13), B0 => imm_in(11), B1
                           => n218, C0 => reg_target(13), C1 => n219, Y => n233
                           );
   U169 : INVXL port map( A => n232, Y => b_out(14));
   U170 : AOI222XL port map( A0 => n217, A1 => imm_in(14), B0 => imm_in(12), B1
                           => n218, C0 => reg_target(14), C1 => n219, Y => n232
                           );
   U171 : INVXL port map( A => n231, Y => b_out(15));
   U172 : AOI222XL port map( A0 => n217, A1 => imm_in(15), B0 => imm_in(13), B1
                           => n218, C0 => reg_target(15), C1 => n219, Y => n231
                           );
   U173 : INVXL port map( A => n270, Y => a_out(12));
   U174 : AOI22XL port map( A0 => c_pc(12), A1 => a_mux(1), B0 => 
                           reg_source(12), B1 => n2, Y => n270);
   U175 : INVXL port map( A => n269, Y => a_out(13));
   U176 : AOI22XL port map( A0 => c_pc(13), A1 => a_mux(1), B0 => 
                           reg_source(13), B1 => n2, Y => n269);
   U177 : INVXL port map( A => n230, Y => b_out(16));
   U178 : AOI222XL port map( A0 => imm_in(15), A1 => n229, B0 => imm_in(14), B1
                           => n218, C0 => reg_target(16), C1 => n219, Y => n230
                           );
   U179 : OAI2BB1XL port map( A0N => reg_target(17), A1N => n219, B0 => n226, Y
                           => b_out(17));
   U180 : OAI2BB1XL port map( A0N => reg_target(18), A1N => n219, B0 => n226, Y
                           => b_out(18));
   U181 : INVXL port map( A => n267, Y => a_out(15));
   U182 : AOI22XL port map( A0 => c_pc(15), A1 => a_mux(1), B0 => 
                           reg_source(15), B1 => n2, Y => n267);
   U183 : INVXL port map( A => n268, Y => a_out(14));
   U184 : AOI22XL port map( A0 => c_pc(14), A1 => a_mux(1), B0 => 
                           reg_source(14), B1 => n2, Y => n268);
   U185 : INVXL port map( A => n266, Y => a_out(16));
   U186 : AOI22XL port map( A0 => c_pc(16), A1 => a_mux(1), B0 => 
                           reg_source(16), B1 => n2, Y => n266);
   U187 : OAI2BB1XL port map( A0N => reg_target(20), A1N => n219, B0 => n226, Y
                           => b_out(20));
   U188 : OAI2BB1XL port map( A0N => reg_target(19), A1N => n219, B0 => n226, Y
                           => b_out(19));
   U189 : INVXL port map( A => n263, Y => a_out(19));
   U190 : AOI22XL port map( A0 => c_pc(19), A1 => a_mux(1), B0 => 
                           reg_source(19), B1 => n2, Y => n263);
   U191 : INVXL port map( A => n264, Y => a_out(18));
   U192 : AOI22XL port map( A0 => c_pc(18), A1 => a_mux(1), B0 => 
                           reg_source(18), B1 => n2, Y => n264);
   U193 : INVXL port map( A => n265, Y => a_out(17));
   U194 : AOI22XL port map( A0 => c_pc(17), A1 => a_mux(1), B0 => 
                           reg_source(17), B1 => n2, Y => n265);
   U195 : OAI2BB1XL port map( A0N => reg_target(21), A1N => n219, B0 => n226, Y
                           => b_out(21));
   U196 : INVXL port map( A => n261, Y => a_out(20));
   U197 : AOI22XL port map( A0 => c_pc(20), A1 => a_mux(1), B0 => 
                           reg_source(20), B1 => n2, Y => n261);
   U198 : OAI2BB1XL port map( A0N => reg_target(22), A1N => n219, B0 => n226, Y
                           => b_out(22));
   U199 : INVXL port map( A => n260, Y => a_out(21));
   U200 : AOI22XL port map( A0 => c_pc(21), A1 => a_mux(1), B0 => 
                           reg_source(21), B1 => n2, Y => n260);
   U201 : INVXL port map( A => n259, Y => a_out(22));
   U202 : AOI22XL port map( A0 => c_pc(22), A1 => a_mux(1), B0 => 
                           reg_source(22), B1 => n2, Y => n259);
   U203 : OAI2BB1XL port map( A0N => reg_target(23), A1N => n219, B0 => n226, Y
                           => b_out(23));
   U204 : OAI2BB1XL port map( A0N => reg_target(26), A1N => n219, B0 => n226, Y
                           => b_out(26));
   U205 : OAI2BB1XL port map( A0N => reg_target(25), A1N => n219, B0 => n226, Y
                           => b_out(25));
   U206 : INVXL port map( A => n258, Y => a_out(23));
   U207 : AOI22XL port map( A0 => c_pc(23), A1 => a_mux(1), B0 => 
                           reg_source(23), B1 => n2, Y => n258);
   U208 : INVXL port map( A => n257, Y => a_out(24));
   U209 : AOI22XL port map( A0 => c_pc(24), A1 => a_mux(1), B0 => 
                           reg_source(24), B1 => n2, Y => n257);
   U210 : OAI2BB1XL port map( A0N => reg_target(24), A1N => n219, B0 => n226, Y
                           => b_out(24));
   U211 : OAI2BB1XL port map( A0N => reg_target(27), A1N => n219, B0 => n226, Y
                           => b_out(27));
   U212 : OAI2BB1XL port map( A0N => reg_target(29), A1N => n219, B0 => n226, Y
                           => b_out(29));
   U213 : OAI2BB1XL port map( A0N => reg_target(28), A1N => n219, B0 => n226, Y
                           => b_out(28));
   U214 : INVXL port map( A => n255, Y => a_out(26));
   U215 : AOI22XL port map( A0 => c_pc(26), A1 => a_mux(1), B0 => 
                           reg_source(26), B1 => n2, Y => n255);
   U216 : INVXL port map( A => n254, Y => a_out(27));
   U217 : AOI22XL port map( A0 => c_pc(27), A1 => a_mux(1), B0 => 
                           reg_source(27), B1 => n2, Y => n254);
   U218 : INVXL port map( A => n256, Y => a_out(25));
   U219 : AOI22XL port map( A0 => c_pc(25), A1 => a_mux(1), B0 => 
                           reg_source(25), B1 => n2, Y => n256);
   U220 : OAI2BB1XL port map( A0N => reg_target(31), A1N => n219, B0 => n226, Y
                           => b_out(31));
   U221 : INVXL port map( A => n249, Y => a_out(31));
   U222 : AOI22XL port map( A0 => c_pc(31), A1 => a_mux(1), B0 => 
                           reg_source(31), B1 => n2, Y => n249);
   U223 : INVXL port map( A => n253, Y => a_out(28));
   U224 : AOI22XL port map( A0 => c_pc(28), A1 => a_mux(1), B0 => 
                           reg_source(28), B1 => n2, Y => n253);
   U225 : INVXL port map( A => n250, Y => a_out(30));
   U226 : AOI22XL port map( A0 => c_pc(30), A1 => a_mux(1), B0 => 
                           reg_source(30), B1 => n2, Y => n250);
   U227 : OAI2BB1XL port map( A0N => reg_target(30), A1N => n219, B0 => n226, Y
                           => b_out(30));
   U228 : INVXL port map( A => n252, Y => a_out(29));
   U229 : AOI22XL port map( A0 => c_pc(29), A1 => a_mux(1), B0 => 
                           reg_source(29), B1 => n2, Y => n252);
   U230 : INVXL port map( A => n215, Y => reg_dest_out(0));
   U231 : AOI22XL port map( A0 => c_memory(0), A1 => n1, B0 => c_bus(0), B1 => 
                           n3, Y => n215);
   U232 : INVXL port map( A => n188, Y => reg_dest_out(1));
   U233 : AOI22XL port map( A0 => c_memory(1), A1 => n1, B0 => c_bus(1), B1 => 
                           n3, Y => n188);
   U234 : NAND2XL port map( A => n156, B => n157, Y => reg_dest_out(2));
   U235 : AOI22XL port map( A0 => c_pc_plus4(2), A1 => n4, B0 => c_pc(2), B1 =>
                           n133, Y => n157);
   U236 : NAND2XL port map( A => n146, B => n147, Y => reg_dest_out(3));
   U237 : AOI22XL port map( A0 => c_pc_plus4(3), A1 => n4, B0 => c_pc(3), B1 =>
                           n133, Y => n147);
   U238 : NAND2XL port map( A => n144, B => n145, Y => reg_dest_out(4));
   U239 : AOI22XL port map( A0 => c_pc_plus4(4), A1 => n4, B0 => c_pc(4), B1 =>
                           n133, Y => n145);
   U240 : NAND2XL port map( A => n142, B => n143, Y => reg_dest_out(5));
   U241 : AOI22XL port map( A0 => c_pc_plus4(5), A1 => n4, B0 => c_pc(5), B1 =>
                           n133, Y => n143);
   U242 : NAND2XL port map( A => n140, B => n141, Y => reg_dest_out(6));
   U243 : AOI22XL port map( A0 => c_pc_plus4(6), A1 => n4, B0 => c_pc(6), B1 =>
                           n133, Y => n141);
   U244 : NAND2XL port map( A => n138, B => n139, Y => reg_dest_out(7));
   U245 : AOI22XL port map( A0 => c_pc_plus4(7), A1 => n4, B0 => c_pc(7), B1 =>
                           n133, Y => n139);
   U246 : NAND2XL port map( A => n136, B => n137, Y => reg_dest_out(8));
   U247 : AOI22XL port map( A0 => c_pc_plus4(8), A1 => n4, B0 => c_pc(8), B1 =>
                           n133, Y => n137);
   U248 : NAND2XL port map( A => n130, B => n131, Y => reg_dest_out(9));
   U249 : AOI22XL port map( A0 => c_pc_plus4(9), A1 => n4, B0 => c_pc(9), B1 =>
                           n133, Y => n131);
   U250 : NAND2XL port map( A => n212, B => n213, Y => reg_dest_out(10));
   U251 : AOI22XL port map( A0 => c_pc_plus4(10), A1 => n4, B0 => c_pc(10), B1 
                           => n133, Y => n213);
   U252 : NAND2XL port map( A => n210, B => n211, Y => reg_dest_out(11));
   U253 : AOI22XL port map( A0 => c_pc_plus4(11), A1 => n4, B0 => c_pc(11), B1 
                           => n133, Y => n211);
   U254 : NAND2XL port map( A => n208, B => n209, Y => reg_dest_out(12));
   U255 : AOI22XL port map( A0 => c_pc_plus4(12), A1 => n4, B0 => c_pc(12), B1 
                           => n133, Y => n209);
   U256 : NAND2XL port map( A => n206, B => n207, Y => reg_dest_out(13));
   U257 : AOI22XL port map( A0 => c_pc_plus4(13), A1 => n4, B0 => c_pc(13), B1 
                           => n133, Y => n207);
   U258 : NAND2XL port map( A => n204, B => n205, Y => reg_dest_out(14));
   U259 : AOI22XL port map( A0 => c_pc_plus4(14), A1 => n4, B0 => c_pc(14), B1 
                           => n133, Y => n205);
   U260 : NAND2XL port map( A => n202, B => n203, Y => reg_dest_out(15));
   U261 : AOI22XL port map( A0 => c_pc_plus4(15), A1 => n4, B0 => c_pc(15), B1 
                           => n133, Y => n203);
   U262 : OAI211XL port map( A0 => n5, A1 => n198, B0 => n199, C0 => n200, Y =>
                           reg_dest_out(16));
   U263 : INVXL port map( A => c_pc(16), Y => n198);
   U264 : OAI211XL port map( A0 => n5, A1 => n195, B0 => n196, C0 => n197, Y =>
                           reg_dest_out(17));
   U265 : INVXL port map( A => c_pc(17), Y => n195);
   U266 : OAI211XL port map( A0 => n5, A1 => n192, B0 => n193, C0 => n194, Y =>
                           reg_dest_out(18));
   U267 : INVXL port map( A => c_pc(18), Y => n192);
   U268 : OAI211XL port map( A0 => n5, A1 => n189, B0 => n190, C0 => n191, Y =>
                           reg_dest_out(19));
   U269 : INVXL port map( A => c_pc(19), Y => n189);
   U270 : OAI211XL port map( A0 => n5, A1 => n185, B0 => n186, C0 => n187, Y =>
                           reg_dest_out(20));
   U271 : INVXL port map( A => c_pc(20), Y => n185);
   U272 : OAI211XL port map( A0 => n5, A1 => n182, B0 => n183, C0 => n184, Y =>
                           reg_dest_out(21));
   U273 : INVXL port map( A => c_pc(21), Y => n182);
   U274 : OAI211XL port map( A0 => n5, A1 => n179, B0 => n180, C0 => n181, Y =>
                           reg_dest_out(22));
   U275 : INVXL port map( A => c_pc(22), Y => n179);
   U276 : OAI211XL port map( A0 => n5, A1 => n176, B0 => n177, C0 => n178, Y =>
                           reg_dest_out(23));
   U277 : INVXL port map( A => c_pc(23), Y => n176);
   U278 : OAI211XL port map( A0 => n5, A1 => n173, B0 => n174, C0 => n175, Y =>
                           reg_dest_out(24));
   U279 : INVXL port map( A => c_pc(24), Y => n173);
   U280 : OAI211XL port map( A0 => n5, A1 => n170, B0 => n171, C0 => n172, Y =>
                           reg_dest_out(25));
   U281 : INVXL port map( A => c_pc(25), Y => n170);
   U282 : OAI211XL port map( A0 => n5, A1 => n167, B0 => n168, C0 => n169, Y =>
                           reg_dest_out(26));
   U283 : INVXL port map( A => c_pc(26), Y => n167);
   U284 : OAI211XL port map( A0 => n5, A1 => n164, B0 => n165, C0 => n166, Y =>
                           reg_dest_out(27));
   U285 : INVXL port map( A => c_pc(27), Y => n164);
   U286 : OAI211XL port map( A0 => n5, A1 => n161, B0 => n162, C0 => n163, Y =>
                           reg_dest_out(28));
   U287 : INVXL port map( A => c_pc(28), Y => n161);
   U288 : OAI211XL port map( A0 => n5, A1 => n158, B0 => n159, C0 => n160, Y =>
                           reg_dest_out(29));
   U289 : INVXL port map( A => c_pc(29), Y => n158);
   U290 : OAI211XL port map( A0 => n5, A1 => n153, B0 => n154, C0 => n155, Y =>
                           reg_dest_out(30));
   U291 : INVXL port map( A => c_pc(30), Y => n153);
   U292 : OAI211XL port map( A0 => n5, A1 => n149, B0 => n150, C0 => n151, Y =>
                           reg_dest_out(31));
   U293 : INVXL port map( A => c_pc(31), Y => n149);
   U294 : OAI21XL port map( A0 => branch_func(2), A1 => n78, B0 => n79, Y => 
                           take_branch);
   U295 : OAI21XL port map( A0 => branch_func(1), A1 => n86, B0 => 
                           branch_func(0), Y => n80);
   U296 : OAI21XL port map( A0 => n218, A1 => n229, B0 => imm_in(15), Y => n226
                           );

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity reg_bank_entity is

   port( clk, reset_in, pause : in std_logic;  rs_index, rt_index, rd_index : 
         in std_logic_vector (5 downto 0);  reg_source_out, reg_target_out : 
         out std_logic_vector (31 downto 0);  reg_dest_new : in 
         std_logic_vector (31 downto 0);  intr_enable : out std_logic);

end reg_bank_entity;

architecture SYN_ram_block of reg_bank_entity is

   component NAND4BXL
      port( AN, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component OR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1X1
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR4BXL
      port( AN, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI33XL
      port( A0, A1, A2, B0, B1, B2 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component AND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component DFFRHQXL
      port( D, CK, RN : in std_logic;  Q : out std_logic);
   end component;
   
   component RAM16X1D
      port( DPO, SPO : out std_logic;  A0, A1, A2, A3, D, DPRA0, DPRA1, DPRA2, 
            DPRA3, WCLK, WE : in std_logic);
   end component;
   
   signal intr_enable_port, addr_read1_3_port, addr_read1_2_port, 
      addr_read1_1_port, addr_write_3_port, addr_write_2_port, 
      addr_write_1_port, weA, weB, data_out1A_31_port, data_out1A_30_port, 
      data_out1A_29_port, data_out1A_28_port, data_out1A_27_port, 
      data_out1A_26_port, data_out1A_25_port, data_out1A_24_port, 
      data_out1A_23_port, data_out1A_22_port, data_out1A_21_port, 
      data_out1A_20_port, data_out1A_19_port, data_out1A_18_port, 
      data_out1A_17_port, data_out1A_16_port, data_out1A_15_port, 
      data_out1A_14_port, data_out1A_13_port, data_out1A_12_port, 
      data_out1A_11_port, data_out1A_10_port, data_out1A_9_port, 
      data_out1A_8_port, data_out1A_7_port, data_out1A_6_port, 
      data_out1A_5_port, data_out1A_4_port, data_out1A_3_port, 
      data_out1A_2_port, data_out1A_1_port, data_out1A_0_port, 
      data_out1B_31_port, data_out1B_30_port, data_out1B_29_port, 
      data_out1B_28_port, data_out1B_27_port, data_out1B_26_port, 
      data_out1B_25_port, data_out1B_24_port, data_out1B_23_port, 
      data_out1B_22_port, data_out1B_21_port, data_out1B_20_port, 
      data_out1B_19_port, data_out1B_18_port, data_out1B_17_port, 
      data_out1B_16_port, data_out1B_15_port, data_out1B_14_port, 
      data_out1B_13_port, data_out1B_12_port, data_out1B_11_port, 
      data_out1B_10_port, data_out1B_9_port, data_out1B_8_port, 
      data_out1B_7_port, data_out1B_6_port, data_out1B_5_port, 
      data_out1B_4_port, data_out1B_3_port, data_out1B_2_port, 
      data_out1B_1_port, data_out1B_0_port, data_out2A_31_port, 
      data_out2A_30_port, data_out2A_29_port, data_out2A_28_port, 
      data_out2A_27_port, data_out2A_26_port, data_out2A_25_port, 
      data_out2A_24_port, data_out2A_23_port, data_out2A_22_port, 
      data_out2A_21_port, data_out2A_20_port, data_out2A_19_port, 
      data_out2A_18_port, data_out2A_17_port, data_out2A_16_port, 
      data_out2A_15_port, data_out2A_14_port, data_out2A_13_port, 
      data_out2A_12_port, data_out2A_11_port, data_out2A_10_port, 
      data_out2A_9_port, data_out2A_8_port, data_out2A_7_port, 
      data_out2A_6_port, data_out2A_5_port, data_out2A_4_port, 
      data_out2A_3_port, data_out2A_2_port, data_out2A_1_port, 
      data_out2A_0_port, data_out2B_31_port, data_out2B_30_port, 
      data_out2B_29_port, data_out2B_28_port, data_out2B_27_port, 
      data_out2B_26_port, data_out2B_25_port, data_out2B_24_port, 
      data_out2B_23_port, data_out2B_22_port, data_out2B_21_port, 
      data_out2B_20_port, data_out2B_19_port, data_out2B_18_port, 
      data_out2B_17_port, data_out2B_16_port, data_out2B_15_port, 
      data_out2B_14_port, data_out2B_13_port, data_out2B_12_port, 
      data_out2B_11_port, data_out2B_10_port, data_out2B_9_port, 
      data_out2B_8_port, data_out2B_7_port, data_out2B_6_port, 
      data_out2B_5_port, data_out2B_4_port, data_out2B_3_port, 
      data_out2B_2_port, data_out2B_1_port, data_out2B_0_port, n2, n3, n4, n5, 
      n6, n7, n8, n9, n10, n12, n13, n14, n15, n16, n17, n18, n19, n20, n21, 
      n22, n23, n24, n25, n26, n27, n28, n29, n30, n31, n32, n33, n34, n35, n36
      , n37, n38, n39, n40, n41, n42, n43, n44, n46, n48, n49, n50, n51, n52, 
      n53, n54, n55, n56, n57, n58, n59, n60, n61, n62, n63, n64, n65, n66, n67
      , n68, n69, n70, n71, n72, n73, n74, n75, n76, n77, n78, n79, n80, n81, 
      n82, n83, n84, n85, n86, n87, n88, n89, n90, n91, n92, n93, n94, n95, n96
      , n98, net999, net998, net997, net996, net995, net994, net993, net992, 
      net991, net990, net989, net988, net987, net986, net985, net984, net983, 
      net982, net981, net980, net979, net978, net977, net976, net975, net974, 
      net973, net972, net971, net970, net969, net968, net967, net966, net965, 
      net964, net963, net962, net961, net960, net959, net958, net957, net956, 
      net955, net954, net953, net952, net951, net950, net949, net948, net947, 
      net946, net945, net944, net943, net942, net941, net940, net939, net938, 
      net937, net936, net935, net934, net933, net932, net931, net930, net929, 
      net928, net927, net926, net925, net924, net923, net922, net921, net920, 
      net919, net918, net917, net916, net915, net914, net913, net912, net911, 
      net910, net909, net908, net907, net906, net905, net904, net903, net902, 
      net901, net900, net899, net898, net897, net896, net895, net894, net893, 
      net892, net891, net890, net889, net888, net887, net886, net885, net884, 
      net883, net882, net881, net880, net879, net878, net877, net876, net1003, 
      net1002, net1001, net1000, n1, n11, n45, n47, n97 : std_logic;

begin
   intr_enable <= intr_enable_port;
   
   reg_bit1a_0 : RAM16X1D port map( DPO => data_out1A_0_port, SPO => net1003, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(0), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_0 : RAM16X1D port map( DPO => data_out1B_0_port, SPO => net1002, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(0), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_0 : RAM16X1D port map( DPO => data_out2A_0_port, SPO => net1001, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(0), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_0 : RAM16X1D port map( DPO => data_out2B_0_port, SPO => net1000, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(0), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_1 : RAM16X1D port map( DPO => data_out1A_1_port, SPO => net999, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(1), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_1 : RAM16X1D port map( DPO => data_out1B_1_port, SPO => net998, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(1), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_1 : RAM16X1D port map( DPO => data_out2A_1_port, SPO => net997, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(1), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_1 : RAM16X1D port map( DPO => data_out2B_1_port, SPO => net996, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(1), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_2 : RAM16X1D port map( DPO => data_out1A_2_port, SPO => net995, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(2), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_2 : RAM16X1D port map( DPO => data_out1B_2_port, SPO => net994, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(2), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_2 : RAM16X1D port map( DPO => data_out2A_2_port, SPO => net993, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(2), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_2 : RAM16X1D port map( DPO => data_out2B_2_port, SPO => net992, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(2), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_3 : RAM16X1D port map( DPO => data_out1A_3_port, SPO => net991, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(3), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_3 : RAM16X1D port map( DPO => data_out1B_3_port, SPO => net990, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(3), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_3 : RAM16X1D port map( DPO => data_out2A_3_port, SPO => net989, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(3), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_3 : RAM16X1D port map( DPO => data_out2B_3_port, SPO => net988, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(3), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_4 : RAM16X1D port map( DPO => data_out1A_4_port, SPO => net987, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(4), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_4 : RAM16X1D port map( DPO => data_out1B_4_port, SPO => net986, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(4), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_4 : RAM16X1D port map( DPO => data_out2A_4_port, SPO => net985, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(4), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_4 : RAM16X1D port map( DPO => data_out2B_4_port, SPO => net984, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(4), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_5 : RAM16X1D port map( DPO => data_out1A_5_port, SPO => net983, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(5), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_5 : RAM16X1D port map( DPO => data_out1B_5_port, SPO => net982, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(5), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_5 : RAM16X1D port map( DPO => data_out2A_5_port, SPO => net981, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(5), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_5 : RAM16X1D port map( DPO => data_out2B_5_port, SPO => net980, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(5), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_6 : RAM16X1D port map( DPO => data_out1A_6_port, SPO => net979, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(6), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_6 : RAM16X1D port map( DPO => data_out1B_6_port, SPO => net978, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(6), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_6 : RAM16X1D port map( DPO => data_out2A_6_port, SPO => net977, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(6), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_6 : RAM16X1D port map( DPO => data_out2B_6_port, SPO => net976, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(6), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_7 : RAM16X1D port map( DPO => data_out1A_7_port, SPO => net975, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(7), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_7 : RAM16X1D port map( DPO => data_out1B_7_port, SPO => net974, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(7), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_7 : RAM16X1D port map( DPO => data_out2A_7_port, SPO => net973, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(7), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_7 : RAM16X1D port map( DPO => data_out2B_7_port, SPO => net972, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(7), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_8 : RAM16X1D port map( DPO => data_out1A_8_port, SPO => net971, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(8), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_8 : RAM16X1D port map( DPO => data_out1B_8_port, SPO => net970, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(8), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_8 : RAM16X1D port map( DPO => data_out2A_8_port, SPO => net969, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(8), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_8 : RAM16X1D port map( DPO => data_out2B_8_port, SPO => net968, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(8), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_9 : RAM16X1D port map( DPO => data_out1A_9_port, SPO => net967, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(9), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_9 : RAM16X1D port map( DPO => data_out1B_9_port, SPO => net966, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(9), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_9 : RAM16X1D port map( DPO => data_out2A_9_port, SPO => net965, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(9), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_9 : RAM16X1D port map( DPO => data_out2B_9_port, SPO => net964, A0
                           => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(9), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_10 : RAM16X1D port map( DPO => data_out1A_10_port, SPO => net963, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(10), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_10 : RAM16X1D port map( DPO => data_out1B_10_port, SPO => net962, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(10), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_10 : RAM16X1D port map( DPO => data_out2A_10_port, SPO => net961, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(10), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_10 : RAM16X1D port map( DPO => data_out2B_10_port, SPO => net960, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(10), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_11 : RAM16X1D port map( DPO => data_out1A_11_port, SPO => net959, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(11), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_11 : RAM16X1D port map( DPO => data_out1B_11_port, SPO => net958, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(11), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_11 : RAM16X1D port map( DPO => data_out2A_11_port, SPO => net957, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(11), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_11 : RAM16X1D port map( DPO => data_out2B_11_port, SPO => net956, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(11), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_12 : RAM16X1D port map( DPO => data_out1A_12_port, SPO => net955, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(12), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_12 : RAM16X1D port map( DPO => data_out1B_12_port, SPO => net954, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(12), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_12 : RAM16X1D port map( DPO => data_out2A_12_port, SPO => net953, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(12), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_12 : RAM16X1D port map( DPO => data_out2B_12_port, SPO => net952, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(12), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_13 : RAM16X1D port map( DPO => data_out1A_13_port, SPO => net951, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(13), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_13 : RAM16X1D port map( DPO => data_out1B_13_port, SPO => net950, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(13), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_13 : RAM16X1D port map( DPO => data_out2A_13_port, SPO => net949, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(13), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_13 : RAM16X1D port map( DPO => data_out2B_13_port, SPO => net948, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(13), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_14 : RAM16X1D port map( DPO => data_out1A_14_port, SPO => net947, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(14), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_14 : RAM16X1D port map( DPO => data_out1B_14_port, SPO => net946, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(14), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_14 : RAM16X1D port map( DPO => data_out2A_14_port, SPO => net945, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(14), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_14 : RAM16X1D port map( DPO => data_out2B_14_port, SPO => net944, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(14), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_15 : RAM16X1D port map( DPO => data_out1A_15_port, SPO => net943, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(15), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_15 : RAM16X1D port map( DPO => data_out1B_15_port, SPO => net942, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(15), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_15 : RAM16X1D port map( DPO => data_out2A_15_port, SPO => net941, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(15), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_15 : RAM16X1D port map( DPO => data_out2B_15_port, SPO => net940, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(15), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_16 : RAM16X1D port map( DPO => data_out1A_16_port, SPO => net939, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(16), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_16 : RAM16X1D port map( DPO => data_out1B_16_port, SPO => net938, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(16), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_16 : RAM16X1D port map( DPO => data_out2A_16_port, SPO => net937, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(16), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_16 : RAM16X1D port map( DPO => data_out2B_16_port, SPO => net936, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(16), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_17 : RAM16X1D port map( DPO => data_out1A_17_port, SPO => net935, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(17), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_17 : RAM16X1D port map( DPO => data_out1B_17_port, SPO => net934, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(17), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_17 : RAM16X1D port map( DPO => data_out2A_17_port, SPO => net933, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(17), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_17 : RAM16X1D port map( DPO => data_out2B_17_port, SPO => net932, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(17), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_18 : RAM16X1D port map( DPO => data_out1A_18_port, SPO => net931, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(18), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_18 : RAM16X1D port map( DPO => data_out1B_18_port, SPO => net930, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(18), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_18 : RAM16X1D port map( DPO => data_out2A_18_port, SPO => net929, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(18), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_18 : RAM16X1D port map( DPO => data_out2B_18_port, SPO => net928, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(18), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_19 : RAM16X1D port map( DPO => data_out1A_19_port, SPO => net927, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(19), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_19 : RAM16X1D port map( DPO => data_out1B_19_port, SPO => net926, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(19), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_19 : RAM16X1D port map( DPO => data_out2A_19_port, SPO => net925, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(19), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_19 : RAM16X1D port map( DPO => data_out2B_19_port, SPO => net924, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(19), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_20 : RAM16X1D port map( DPO => data_out1A_20_port, SPO => net923, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(20), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_20 : RAM16X1D port map( DPO => data_out1B_20_port, SPO => net922, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(20), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_20 : RAM16X1D port map( DPO => data_out2A_20_port, SPO => net921, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(20), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_20 : RAM16X1D port map( DPO => data_out2B_20_port, SPO => net920, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(20), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_21 : RAM16X1D port map( DPO => data_out1A_21_port, SPO => net919, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(21), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_21 : RAM16X1D port map( DPO => data_out1B_21_port, SPO => net918, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(21), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_21 : RAM16X1D port map( DPO => data_out2A_21_port, SPO => net917, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(21), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_21 : RAM16X1D port map( DPO => data_out2B_21_port, SPO => net916, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(21), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_22 : RAM16X1D port map( DPO => data_out1A_22_port, SPO => net915, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(22), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_22 : RAM16X1D port map( DPO => data_out1B_22_port, SPO => net914, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(22), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_22 : RAM16X1D port map( DPO => data_out2A_22_port, SPO => net913, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(22), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_22 : RAM16X1D port map( DPO => data_out2B_22_port, SPO => net912, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(22), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_23 : RAM16X1D port map( DPO => data_out1A_23_port, SPO => net911, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(23), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_23 : RAM16X1D port map( DPO => data_out1B_23_port, SPO => net910, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(23), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_23 : RAM16X1D port map( DPO => data_out2A_23_port, SPO => net909, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(23), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_23 : RAM16X1D port map( DPO => data_out2B_23_port, SPO => net908, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(23), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_24 : RAM16X1D port map( DPO => data_out1A_24_port, SPO => net907, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(24), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_24 : RAM16X1D port map( DPO => data_out1B_24_port, SPO => net906, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(24), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_24 : RAM16X1D port map( DPO => data_out2A_24_port, SPO => net905, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(24), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_24 : RAM16X1D port map( DPO => data_out2B_24_port, SPO => net904, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(24), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_25 : RAM16X1D port map( DPO => data_out1A_25_port, SPO => net903, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(25), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_25 : RAM16X1D port map( DPO => data_out1B_25_port, SPO => net902, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(25), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_25 : RAM16X1D port map( DPO => data_out2A_25_port, SPO => net901, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(25), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_25 : RAM16X1D port map( DPO => data_out2B_25_port, SPO => net900, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(25), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_26 : RAM16X1D port map( DPO => data_out1A_26_port, SPO => net899, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(26), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_26 : RAM16X1D port map( DPO => data_out1B_26_port, SPO => net898, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(26), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_26 : RAM16X1D port map( DPO => data_out2A_26_port, SPO => net897, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(26), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_26 : RAM16X1D port map( DPO => data_out2B_26_port, SPO => net896, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(26), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_27 : RAM16X1D port map( DPO => data_out1A_27_port, SPO => net895, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(27), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_27 : RAM16X1D port map( DPO => data_out1B_27_port, SPO => net894, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(27), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_27 : RAM16X1D port map( DPO => data_out2A_27_port, SPO => net893, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(27), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_27 : RAM16X1D port map( DPO => data_out2B_27_port, SPO => net892, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(27), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_28 : RAM16X1D port map( DPO => data_out1A_28_port, SPO => net891, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(28), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_28 : RAM16X1D port map( DPO => data_out1B_28_port, SPO => net890, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(28), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_28 : RAM16X1D port map( DPO => data_out2A_28_port, SPO => net889, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(28), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_28 : RAM16X1D port map( DPO => data_out2B_28_port, SPO => net888, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(28), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_29 : RAM16X1D port map( DPO => data_out1A_29_port, SPO => net887, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(29), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_29 : RAM16X1D port map( DPO => data_out1B_29_port, SPO => net886, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(29), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_29 : RAM16X1D port map( DPO => data_out2A_29_port, SPO => net885, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(29), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_29 : RAM16X1D port map( DPO => data_out2B_29_port, SPO => net884, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(29), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_30 : RAM16X1D port map( DPO => data_out1A_30_port, SPO => net883, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(30), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_30 : RAM16X1D port map( DPO => data_out1B_30_port, SPO => net882, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(30), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_30 : RAM16X1D port map( DPO => data_out2A_30_port, SPO => net881, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(30), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_30 : RAM16X1D port map( DPO => data_out2B_30_port, SPO => net880, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(30), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   reg_bit1a_31 : RAM16X1D port map( DPO => data_out1A_31_port, SPO => net879, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(31), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weA);
   reg_bit1b_31 : RAM16X1D port map( DPO => data_out1B_31_port, SPO => net878, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(31), DPRA0 => rs_index(0), DPRA1 => 
                           addr_read1_1_port, DPRA2 => addr_read1_2_port, DPRA3
                           => addr_read1_3_port, WCLK => clk, WE => weB);
   reg_bit2a_31 : RAM16X1D port map( DPO => data_out2A_31_port, SPO => net877, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(31), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weA);
   reg_bit2b_31 : RAM16X1D port map( DPO => data_out2B_31_port, SPO => net876, 
                           A0 => rd_index(0), A1 => addr_write_1_port, A2 => 
                           addr_write_2_port, A3 => addr_write_3_port, D => 
                           reg_dest_new(31), DPRA0 => rt_index(0), DPRA1 => 
                           rt_index(1), DPRA2 => rt_index(2), DPRA3 => 
                           rt_index(3), WCLK => clk, WE => weB);
   intr_enable_reg_reg : DFFRHQXL port map( D => n98, CK => clk, RN => n97, Q 
                           => intr_enable_port);
   U3 : AND3XL port map( A => n59, B => n81, C => n85, Y => n84);
   U4 : NAND4XL port map( A => n86, B => n87, C => n88, D => n89, Y => n85);
   U5 : NOR3XL port map( A => rs_index(1), B => rs_index(4), C => rs_index(0), 
                           Y => n86);
   U6 : NAND4XL port map( A => rs_index(4), B => rs_index(1), C => rs_index(0),
                           D => n90, Y => n59);
   U7 : OAI31XL port map( A0 => n44, A1 => rt_index(1), A2 => rt_index(0), B0 
                           => n47, Y => n43);
   U8 : NOR2BXL port map( AN => n84, B => rs_index(4), Y => n48);
   U9 : NAND2XL port map( A => n84, B => rs_index(4), Y => n1);
   U10 : INVXL port map( A => rs_index(5), Y => n89);
   U11 : NAND2XL port map( A => n90, B => n86, Y => n81);
   U12 : INVXL port map( A => n59, Y => n53);
   U13 : INVXL port map( A => n43, Y => n45);
   U14 : INVXL port map( A => rt_index(4), Y => n47);
   U15 : INVXL port map( A => n1, Y => n11);
   U16 : INVXL port map( A => rd_index(5), Y => n7);
   U17 : OAI33XL port map( A0 => n7, A1 => n8, A2 => n9, B0 => rd_index(2), B1 
                           => rd_index(5), B2 => rd_index(3), Y => n5);
   U18 : NOR3XL port map( A => n88, B => n87, C => n89, Y => n90);
   U19 : AOI22XL port map( A0 => data_out1B_0_port, A1 => n11, B0 => 
                           data_out1A_0_port, B1 => n48, Y => n83);
   U20 : INVXL port map( A => n31, Y => reg_target_out(1));
   U21 : AOI22XL port map( A0 => data_out2A_1_port, A1 => n45, B0 => 
                           data_out2B_1_port, B1 => rt_index(4), Y => n31);
   U22 : INVXL port map( A => n42, Y => reg_target_out(0));
   U23 : AOI22XL port map( A0 => data_out2A_0_port, A1 => n45, B0 => 
                           data_out2B_0_port, B1 => rt_index(4), Y => n42);
   U24 : AOI21XL port map( A0 => data_out1B_3_port, A1 => n11, B0 => n53, Y => 
                           n55);
   U25 : AOI21XL port map( A0 => data_out1B_2_port, A1 => n11, B0 => n53, Y => 
                           n58);
   U26 : INVXL port map( A => n70, Y => reg_source_out(1));
   U27 : AOI22XL port map( A0 => data_out1B_1_port, A1 => n11, B0 => 
                           data_out1A_1_port, B1 => n48, Y => n70);
   U28 : INVXL port map( A => n17, Y => reg_target_out(3));
   U29 : AOI22XL port map( A0 => data_out2A_3_port, A1 => n45, B0 => 
                           data_out2B_3_port, B1 => rt_index(4), Y => n17);
   U30 : INVXL port map( A => n20, Y => reg_target_out(2));
   U31 : AOI22XL port map( A0 => data_out2A_2_port, A1 => n45, B0 => 
                           data_out2B_2_port, B1 => rt_index(4), Y => n20);
   U32 : INVXL port map( A => n16, Y => reg_target_out(4));
   U33 : AOI22XL port map( A0 => data_out2A_4_port, A1 => n45, B0 => 
                           data_out2B_4_port, B1 => rt_index(4), Y => n16);
   U34 : AOI21XL port map( A0 => data_out1B_4_port, A1 => n11, B0 => n53, Y => 
                           n54);
   U35 : INVXL port map( A => n13, Y => reg_target_out(7));
   U36 : AOI22XL port map( A0 => data_out2A_7_port, A1 => n45, B0 => 
                           data_out2B_7_port, B1 => rt_index(4), Y => n13);
   U37 : INVXL port map( A => n14, Y => reg_target_out(6));
   U38 : AOI22XL port map( A0 => data_out2A_6_port, A1 => n45, B0 => 
                           data_out2B_6_port, B1 => rt_index(4), Y => n14);
   U39 : INVXL port map( A => n15, Y => reg_target_out(5));
   U40 : AOI22XL port map( A0 => data_out2A_5_port, A1 => n45, B0 => 
                           data_out2B_5_port, B1 => rt_index(4), Y => n15);
   U41 : AOI21XL port map( A0 => data_out1B_5_port, A1 => n11, B0 => n53, Y => 
                           n52);
   U42 : INVXL port map( A => n10, Y => reg_target_out(9));
   U43 : AOI22XL port map( A0 => data_out2A_9_port, A1 => n45, B0 => 
                           rt_index(4), B1 => data_out2B_9_port, Y => n10);
   U44 : INVXL port map( A => n12, Y => reg_target_out(8));
   U45 : AOI22XL port map( A0 => data_out2A_8_port, A1 => n45, B0 => 
                           data_out2B_8_port, B1 => rt_index(4), Y => n12);
   U46 : INVXL port map( A => n50, Y => reg_source_out(7));
   U47 : AOI22XL port map( A0 => data_out1B_7_port, A1 => n11, B0 => 
                           data_out1A_7_port, B1 => n48, Y => n50);
   U48 : INVXL port map( A => n51, Y => reg_source_out(6));
   U49 : AOI22XL port map( A0 => data_out1B_6_port, A1 => n11, B0 => 
                           data_out1A_6_port, B1 => n48, Y => n51);
   U50 : INVXL port map( A => n49, Y => reg_source_out(8));
   U51 : AOI22XL port map( A0 => data_out1B_8_port, A1 => n11, B0 => 
                           data_out1A_8_port, B1 => n48, Y => n49);
   U52 : INVXL port map( A => n41, Y => reg_target_out(10));
   U53 : AOI22XL port map( A0 => data_out2A_10_port, A1 => n45, B0 => 
                           data_out2B_10_port, B1 => rt_index(4), Y => n41);
   U54 : INVXL port map( A => n40, Y => reg_target_out(11));
   U55 : AOI22XL port map( A0 => data_out2A_11_port, A1 => n45, B0 => 
                           data_out2B_11_port, B1 => rt_index(4), Y => n40);
   U56 : INVXL port map( A => n39, Y => reg_target_out(12));
   U57 : AOI22XL port map( A0 => data_out2A_12_port, A1 => n45, B0 => 
                           data_out2B_12_port, B1 => rt_index(4), Y => n39);
   U58 : INVXL port map( A => n79, Y => reg_source_out(11));
   U59 : AOI22XL port map( A0 => data_out1B_11_port, A1 => n11, B0 => 
                           data_out1A_11_port, B1 => n48, Y => n79);
   U60 : INVXL port map( A => n80, Y => reg_source_out(10));
   U61 : AOI22XL port map( A0 => data_out1B_10_port, A1 => n11, B0 => 
                           data_out1A_10_port, B1 => n48, Y => n80);
   U62 : INVXL port map( A => n46, Y => reg_source_out(9));
   U63 : AOI22XL port map( A0 => data_out1B_9_port, A1 => n11, B0 => 
                           data_out1A_9_port, B1 => n48, Y => n46);
   U64 : INVXL port map( A => n38, Y => reg_target_out(13));
   U65 : AOI22XL port map( A0 => data_out2A_13_port, A1 => n45, B0 => 
                           data_out2B_13_port, B1 => rt_index(4), Y => n38);
   U66 : INVXL port map( A => n37, Y => reg_target_out(14));
   U67 : AOI22XL port map( A0 => data_out2A_14_port, A1 => n45, B0 => 
                           data_out2B_14_port, B1 => rt_index(4), Y => n37);
   U68 : INVXL port map( A => n36, Y => reg_target_out(15));
   U69 : AOI22XL port map( A0 => data_out2A_15_port, A1 => n45, B0 => 
                           data_out2B_15_port, B1 => rt_index(4), Y => n36);
   U70 : INVXL port map( A => n78, Y => reg_source_out(12));
   U71 : AOI22XL port map( A0 => data_out1B_12_port, A1 => n11, B0 => 
                           data_out1A_12_port, B1 => n48, Y => n78);
   U72 : INVXL port map( A => n77, Y => reg_source_out(13));
   U73 : AOI22XL port map( A0 => data_out1B_13_port, A1 => n11, B0 => 
                           data_out1A_13_port, B1 => n48, Y => n77);
   U74 : INVXL port map( A => n35, Y => reg_target_out(16));
   U75 : AOI22XL port map( A0 => data_out2A_16_port, A1 => n45, B0 => 
                           data_out2B_16_port, B1 => rt_index(4), Y => n35);
   U76 : INVXL port map( A => n34, Y => reg_target_out(17));
   U77 : AOI22XL port map( A0 => data_out2A_17_port, A1 => n45, B0 => 
                           data_out2B_17_port, B1 => rt_index(4), Y => n34);
   U78 : INVXL port map( A => n33, Y => reg_target_out(18));
   U79 : AOI22XL port map( A0 => data_out2A_18_port, A1 => n45, B0 => 
                           data_out2B_18_port, B1 => rt_index(4), Y => n33);
   U80 : INVXL port map( A => n75, Y => reg_source_out(15));
   U81 : AOI22XL port map( A0 => data_out1B_15_port, A1 => n11, B0 => 
                           data_out1A_15_port, B1 => n48, Y => n75);
   U82 : INVXL port map( A => n76, Y => reg_source_out(14));
   U83 : AOI22XL port map( A0 => data_out1B_14_port, A1 => n11, B0 => 
                           data_out1A_14_port, B1 => n48, Y => n76);
   U84 : INVXL port map( A => n74, Y => reg_source_out(16));
   U85 : AOI22XL port map( A0 => data_out1B_16_port, A1 => n11, B0 => 
                           data_out1A_16_port, B1 => n48, Y => n74);
   U86 : INVXL port map( A => n30, Y => reg_target_out(20));
   U87 : AOI22XL port map( A0 => data_out2A_20_port, A1 => n45, B0 => 
                           data_out2B_20_port, B1 => rt_index(4), Y => n30);
   U88 : INVXL port map( A => n32, Y => reg_target_out(19));
   U89 : AOI22XL port map( A0 => data_out2A_19_port, A1 => n45, B0 => 
                           data_out2B_19_port, B1 => rt_index(4), Y => n32);
   U90 : INVXL port map( A => n71, Y => reg_source_out(19));
   U91 : AOI22XL port map( A0 => data_out1B_19_port, A1 => n11, B0 => 
                           data_out1A_19_port, B1 => n48, Y => n71);
   U92 : INVXL port map( A => n72, Y => reg_source_out(18));
   U93 : AOI22XL port map( A0 => data_out1B_18_port, A1 => n11, B0 => 
                           data_out1A_18_port, B1 => n48, Y => n72);
   U94 : INVXL port map( A => n73, Y => reg_source_out(17));
   U95 : AOI22XL port map( A0 => data_out1B_17_port, A1 => n11, B0 => 
                           data_out1A_17_port, B1 => n48, Y => n73);
   U96 : INVXL port map( A => n29, Y => reg_target_out(21));
   U97 : AOI22XL port map( A0 => data_out2A_21_port, A1 => n45, B0 => 
                           data_out2B_21_port, B1 => rt_index(4), Y => n29);
   U98 : INVXL port map( A => n69, Y => reg_source_out(20));
   U99 : AOI22XL port map( A0 => data_out1B_20_port, A1 => n11, B0 => 
                           data_out1A_20_port, B1 => n48, Y => n69);
   U100 : INVXL port map( A => n28, Y => reg_target_out(22));
   U101 : AOI22XL port map( A0 => data_out2A_22_port, A1 => n45, B0 => 
                           data_out2B_22_port, B1 => rt_index(4), Y => n28);
   U102 : INVXL port map( A => n68, Y => reg_source_out(21));
   U103 : AOI22XL port map( A0 => data_out1B_21_port, A1 => n11, B0 => 
                           data_out1A_21_port, B1 => n48, Y => n68);
   U104 : INVXL port map( A => n67, Y => reg_source_out(22));
   U105 : AOI22XL port map( A0 => data_out1B_22_port, A1 => n11, B0 => 
                           data_out1A_22_port, B1 => n48, Y => n67);
   U106 : INVXL port map( A => n27, Y => reg_target_out(23));
   U107 : AOI22XL port map( A0 => data_out2A_23_port, A1 => n45, B0 => 
                           data_out2B_23_port, B1 => rt_index(4), Y => n27);
   U108 : INVXL port map( A => n24, Y => reg_target_out(26));
   U109 : AOI22XL port map( A0 => data_out2A_26_port, A1 => n45, B0 => 
                           data_out2B_26_port, B1 => rt_index(4), Y => n24);
   U110 : INVXL port map( A => n25, Y => reg_target_out(25));
   U111 : AOI22XL port map( A0 => data_out2A_25_port, A1 => n45, B0 => 
                           data_out2B_25_port, B1 => rt_index(4), Y => n25);
   U112 : INVXL port map( A => n66, Y => reg_source_out(23));
   U113 : AOI22XL port map( A0 => data_out1B_23_port, A1 => n11, B0 => 
                           data_out1A_23_port, B1 => n48, Y => n66);
   U114 : INVXL port map( A => n65, Y => reg_source_out(24));
   U115 : AOI22XL port map( A0 => data_out1B_24_port, A1 => n11, B0 => 
                           data_out1A_24_port, B1 => n48, Y => n65);
   U116 : INVXL port map( A => n26, Y => reg_target_out(24));
   U117 : AOI22XL port map( A0 => data_out2A_24_port, A1 => n45, B0 => 
                           data_out2B_24_port, B1 => rt_index(4), Y => n26);
   U118 : INVXL port map( A => n23, Y => reg_target_out(27));
   U119 : AOI22XL port map( A0 => data_out2A_27_port, A1 => n45, B0 => 
                           data_out2B_27_port, B1 => rt_index(4), Y => n23);
   U120 : INVXL port map( A => n21, Y => reg_target_out(29));
   U121 : AOI22XL port map( A0 => data_out2A_29_port, A1 => n45, B0 => 
                           data_out2B_29_port, B1 => rt_index(4), Y => n21);
   U122 : INVXL port map( A => n22, Y => reg_target_out(28));
   U123 : AOI22XL port map( A0 => data_out2A_28_port, A1 => n45, B0 => 
                           data_out2B_28_port, B1 => rt_index(4), Y => n22);
   U124 : INVXL port map( A => n63, Y => reg_source_out(26));
   U125 : AOI22XL port map( A0 => data_out1B_26_port, A1 => n11, B0 => 
                           data_out1A_26_port, B1 => n48, Y => n63);
   U126 : INVXL port map( A => n62, Y => reg_source_out(27));
   U127 : AOI22XL port map( A0 => data_out1B_27_port, A1 => n11, B0 => 
                           data_out1A_27_port, B1 => n48, Y => n62);
   U128 : INVXL port map( A => n64, Y => reg_source_out(25));
   U129 : AOI22XL port map( A0 => data_out1B_25_port, A1 => n11, B0 => 
                           data_out1A_25_port, B1 => n48, Y => n64);
   U130 : INVXL port map( A => n18, Y => reg_target_out(31));
   U131 : AOI22XL port map( A0 => data_out2A_31_port, A1 => n45, B0 => 
                           data_out2B_31_port, B1 => rt_index(4), Y => n18);
   U132 : INVXL port map( A => n56, Y => reg_source_out(31));
   U133 : AOI22XL port map( A0 => data_out1B_31_port, A1 => n11, B0 => 
                           data_out1A_31_port, B1 => n48, Y => n56);
   U134 : INVXL port map( A => n61, Y => reg_source_out(28));
   U135 : AOI22XL port map( A0 => data_out1B_28_port, A1 => n11, B0 => 
                           data_out1A_28_port, B1 => n48, Y => n61);
   U136 : INVXL port map( A => n57, Y => reg_source_out(30));
   U137 : AOI22XL port map( A0 => data_out1B_30_port, A1 => n11, B0 => 
                           data_out1A_30_port, B1 => n48, Y => n57);
   U138 : INVXL port map( A => n19, Y => reg_target_out(30));
   U139 : AOI22XL port map( A0 => data_out2A_30_port, A1 => n45, B0 => 
                           data_out2B_30_port, B1 => rt_index(4), Y => n19);
   U140 : INVXL port map( A => n60, Y => reg_source_out(29));
   U141 : AOI22XL port map( A0 => data_out1B_29_port, A1 => n11, B0 => 
                           data_out1A_29_port, B1 => n48, Y => n60);
   U142 : INVXL port map( A => intr_enable_port, Y => n82);
   U143 : NOR4BXL port map( AN => n94, B => n7, C => n9, D => n8, Y => n91);
   U144 : NOR2XL port map( A => rd_index(4), B => rd_index(0), Y => n94);
   U145 : INVXL port map( A => rd_index(4), Y => n3);
   U146 : NAND2BXL port map( AN => pause, B => n4, Y => n2);
   U147 : INVXL port map( A => rs_index(3), Y => n88);
   U148 : INVXL port map( A => rs_index(2), Y => n87);
   U149 : INVXL port map( A => rs_index(1), Y => n96);
   U150 : NOR4BXL port map( AN => n90, B => n96, C => rs_index(0), D => 
                           rs_index(4), Y => n95);
   U151 : INVXL port map( A => rd_index(3), Y => n9);
   U152 : INVXL port map( A => rd_index(2), Y => n8);
   U153 : INVXL port map( A => rd_index(1), Y => n6);
   U154 : NOR2BXL port map( AN => n91, B => n6, Y => n93);
   U155 : INVXL port map( A => reset_in, Y => n97);
   U156 : NAND3XL port map( A => n91, B => n6, C => reg_dest_new(0), Y => n92);
   U157 : NOR2XL port map( A => n2, B => n3, Y => weB);
   U158 : NOR2XL port map( A => rd_index(4), B => n2, Y => weA);
   U159 : NOR2XL port map( A => n95, B => n88, Y => addr_read1_3_port);
   U160 : NOR2XL port map( A => n95, B => n87, Y => addr_read1_2_port);
   U161 : NOR2XL port map( A => n95, B => n96, Y => addr_read1_1_port);
   U162 : NOR2XL port map( A => n93, B => n9, Y => addr_write_3_port);
   U163 : NOR2XL port map( A => n93, B => n8, Y => addr_write_2_port);
   U164 : NOR2XL port map( A => n93, B => n6, Y => addr_write_1_port);
   U165 : OAI21XL port map( A0 => n91, A1 => n82, B0 => n92, Y => n98);
   U166 : OAI21XL port map( A0 => n81, A1 => n82, B0 => n83, Y => 
                           reg_source_out(0));
   U167 : OAI2BB1X1 port map( A0N => data_out1A_5_port, A1N => n48, B0 => n52, 
                           Y => reg_source_out(5));
   U168 : OAI2BB1X1 port map( A0N => data_out1A_2_port, A1N => n48, B0 => n58, 
                           Y => reg_source_out(2));
   U169 : OAI2BB1X1 port map( A0N => data_out1A_3_port, A1N => n48, B0 => n55, 
                           Y => reg_source_out(3));
   U170 : OAI2BB1X1 port map( A0N => data_out1A_4_port, A1N => n48, B0 => n54, 
                           Y => reg_source_out(4));
   U171 : OR3XL port map( A => rt_index(5), B => rt_index(3), C => rt_index(2),
                           Y => n44);
   U172 : NAND4BXL port map( AN => rd_index(0), B => n5, C => n6, D => n3, Y =>
                           n4);

end SYN_ram_block;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity control is

   port( opcode : in std_logic_vector (31 downto 0);  intr_signal : in 
         std_logic;  rs_index, rt_index, rd_index : out std_logic_vector (5 
         downto 0);  imm_out : out std_logic_vector (15 downto 0);  alu_func : 
         out std_logic_vector (3 downto 0);  shift_func : out std_logic_vector 
         (1 downto 0);  mult_func : out std_logic_vector (3 downto 0);  
         branch_func : out std_logic_vector (2 downto 0);  a_source_out, 
         b_source_out : out std_logic_vector (1 downto 0);  c_source_out : out 
         std_logic_vector (2 downto 0);  pc_source_out : out std_logic_vector 
         (1 downto 0);  mem_source_out : out std_logic_vector (3 downto 0);  
         exception_out : out std_logic);

end control;

architecture SYN_logic of control is

   component OAI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4BXL
      port( AN, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR4BXL
      port( AN, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3BXL
      port( AN, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component AND4X1
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI32XL
      port( A0, A1, A2, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component AND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI211XL
      port( A0, A1, B0, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   signal c_source_out_2_port, c_source_out_1_port, c_source_out_0_port, 
      pc_source_out_1_port, pc_source_out_0_port, mem_source_out_3_port, 
      mem_source_out_2_port, mem_source_out_1_port, mem_source_out_0_port, n5, 
      n42, n43, n44, n45, n46, n47, n48, n49, n50, n51, n52, n53, n54, n55, n56
      , n57, n58, n59, n60, n61, n62, n63, n64, n65, n66, n67, n68, n69, n70, 
      n71, n72, n73, n74, n75, n76, n77, n78, n79, n80, n81, n82, n83, n84, n85
      , n86, n87, n88, n89, n90, n91, n92, n93, n94, n95, n96, n97, n98, n99, 
      n100, n101, n102, n103, n104, n105, n106, n107, n108, n109, n110, n111, 
      n112, n113, n114, n115, n116, n117, n118, n119, n120, n121, n122, n123, 
      n124, n125, n126, n127, n128, n129, n130, n131, n132, n133, n134, n135, 
      n136, n137, n138, n139, n140, n141, n142, n143, n144, n145, n146, n147, 
      n148, n149, n150, n151, n152, n153, n154, n155, n156, n157, n158, n159, 
      n160, n161, n162, n163, n165, n166, n167, n168, n169, n170, n171, n172, 
      n173, n174, n175, n176, n177, n178, n179, n180, n181, n182, n183, n184, 
      n185, n186, n187, n188, n189, n190, n191, n192, n193, n194, n195, n196, 
      n197, n198, n199, n200, n1, n2, exception_out_port : std_logic;

begin
   imm_out <= ( opcode(15), opcode(14), opcode(13), opcode(12), opcode(11), 
      opcode(10), opcode(9), opcode(8), opcode(7), opcode(6), opcode(5), 
      opcode(4), opcode(3), opcode(2), opcode(1), opcode(0) );
   c_source_out <= ( c_source_out_2_port, c_source_out_1_port, 
      c_source_out_0_port );
   pc_source_out <= ( pc_source_out_1_port, pc_source_out_0_port );
   mem_source_out <= ( mem_source_out_3_port, mem_source_out_2_port, 
      mem_source_out_1_port, mem_source_out_0_port );
   exception_out <= exception_out_port;
   
   rt_index(5) <= '0';
   U3 : INVXL port map( A => n161, Y => n134);
   U4 : OAI211XL port map( A0 => n119, A1 => n122, B0 => n88, C0 => n56, Y => 
                           n161);
   U5 : NOR2XL port map( A => opcode(31), B => opcode(30), Y => n197);
   U6 : NAND2XL port map( A => n57, B => n55, Y => rs_index(5));
   U7 : OAI211XL port map( A0 => n52, A1 => n122, B0 => n91, C0 => n134, Y => 
                           branch_func(2));
   U8 : INVXL port map( A => n90, Y => n136);
   U9 : AOI21XL port map( A0 => n134, A1 => n160, B0 => exception_out_port, Y 
                           => branch_func(0));
   U10 : INVXL port map( A => opcode(27), Y => n194);
   U11 : NAND2XL port map( A => n186, B => n195, Y => n138);
   U12 : NOR2BXL port map( AN => n196, B => n145, Y => n178);
   U13 : NOR2BXL port map( AN => n197, B => n182, Y => n195);
   U14 : INVXL port map( A => opcode(31), Y => n143);
   U15 : INVXL port map( A => opcode(28), Y => n145);
   U16 : NOR2BXL port map( AN => n197, B => opcode(26), Y => n196);
   U17 : INVXL port map( A => n166, Y => n179);
   U18 : NAND2XL port map( A => n179, B => n98, Y => n131);
   U19 : NOR2BXL port map( AN => opcode(20), B => n48, Y => rt_index(4));
   U20 : NAND2XL port map( A => n177, B => n178, Y => n150);
   U21 : OR2XL port map( A => n92, B => n2, Y => n1);
   U22 : INVXL port map( A => n1, Y => a_source_out(1));
   U23 : NAND2BXL port map( AN => n112, B => n114, Y => n141);
   U24 : NAND2XL port map( A => n187, B => n145, Y => n192);
   U25 : NAND2XL port map( A => opcode(29), B => n194, Y => n193);
   U26 : NOR2XL port map( A => n143, B => opcode(30), Y => n187);
   U27 : INVXL port map( A => opcode(26), Y => n182);
   U28 : NOR2XL port map( A => n192, B => opcode(26), Y => n188);
   U29 : NOR2BXL port map( AN => n144, B => n145, Y => n186);
   U30 : INVXL port map( A => n192, Y => n183);
   U31 : NOR2XL port map( A => opcode(29), B => opcode(27), Y => n144);
   U32 : NOR3XL port map( A => n107, B => opcode(2), C => n98, Y => n101);
   U33 : INVXL port map( A => n43, Y => n102);
   U34 : AND4X1 port map( A => n137, B => n128, C => n138, D => n139, Y => n86)
                           ;
   U35 : AOI211XL port map( A0 => opcode(30), A1 => n140, B0 => n141, C0 => 
                           n142, Y => n139);
   U36 : NOR3XL port map( A => n143, B => n144, C => n145, Y => n142);
   U37 : NAND2XL port map( A => n178, B => n157, Y => n89);
   U38 : NAND2XL port map( A => n151, B => n152, Y => n117);
   U39 : AND2XL port map( A => n101, B => n46, Y => n120);
   U40 : NOR3XL port map( A => opcode(18), B => opcode(19), C => opcode(17), Y 
                           => n119);
   U41 : OR2XL port map( A => n146, B => n147, Y => n124);
   U42 : NAND4XL port map( A => n153, B => n154, C => n155, D => n156, Y => 
                           n146);
   U43 : NAND4XL port map( A => n117, B => n148, C => n149, D => n150, Y => 
                           n147);
   U44 : INVXL port map( A => n131, Y => n130);
   U45 : AOI21XL port map( A0 => opcode(4), A1 => opcode(2), B0 => opcode(5), Y
                           => n129);
   U46 : AOI21XL port map( A0 => opcode(3), A1 => n95, B0 => opcode(2), Y => 
                           n132);
   U47 : INVXL port map( A => opcode(23), Y => n64);
   U48 : NOR2XL port map( A => n54, B => n67, Y => n63);
   U49 : NOR2XL port map( A => n194, B => opcode(29), Y => n157);
   U50 : NOR2BXL port map( AN => n195, B => opcode(28), Y => n159);
   U51 : NAND2XL port map( A => n186, B => n196, Y => n137);
   U52 : NAND2XL port map( A => n152, B => n157, Y => n88);
   U53 : NAND2XL port map( A => n159, B => n144, Y => n122);
   U54 : AND2XL port map( A => n138, B => n89, Y => n160);
   U55 : AOI21XL port map( A0 => n46, A1 => n93, B0 => n120, Y => n135);
   U56 : AND3XL port map( A => n172, B => n190, C => n191, Y => n125);
   U57 : AOI31XL port map( A0 => n133, A1 => opcode(2), A2 => n108, B0 => n176,
                           Y => n191);
   U58 : NAND3XL port map( A => opcode(1), B => n98, C => n179, Y => n190);
   U59 : INVXL port map( A => opcode(2), Y => n105);
   U60 : NAND2XL port map( A => n177, B => n158, Y => n156);
   U61 : NAND2XL port map( A => n177, B => n159, Y => n155);
   U62 : NAND2XL port map( A => n151, B => n178, Y => n148);
   U63 : NAND2XL port map( A => n151, B => n159, Y => n153);
   U64 : NAND2XL port map( A => n144, B => n158, Y => n128);
   U65 : AND3XL port map( A => n101, B => opcode(1), C => n133, Y => n176);
   U66 : NOR2XL port map( A => n106, B => opcode(4), Y => n133);
   U67 : INVXL port map( A => n193, Y => n177);
   U68 : NOR2BXL port map( AN => n195, B => n145, Y => n152);
   U69 : NAND4XL port map( A => n144, B => n182, C => n145, D => n143, Y => 
                           n140);
   U70 : AND2XL port map( A => opcode(29), B => opcode(27), Y => n151);
   U71 : NOR2BXL port map( AN => n196, B => opcode(28), Y => n158);
   U72 : NAND3XL port map( A => opcode(0), B => n95, C => n179, Y => n172);
   U73 : OAI211XL port map( A0 => n131, A1 => n128, B0 => n148, C0 => n150, Y 
                           => n171);
   U74 : NOR2XL port map( A => n198, B => n43, Y => a_source_out(0));
   U75 : AOI32XL port map( A0 => n45, A1 => n105, A2 => n46, B0 => n105, B1 => 
                           n200, Y => n198);
   U76 : INVXL port map( A => n42, Y => n200);
   U77 : AOI31XL port map( A0 => n84, A1 => n154, A2 => n162, B0 => n5, Y => 
                           b_source_out(1));
   U78 : AND2XL port map( A => n150, B => n165, Y => n163);
   U79 : INVXL port map( A => opcode(4), Y => n104);
   U80 : INVXL port map( A => opcode(5), Y => n106);
   U81 : INVXL port map( A => n128, Y => n121);
   U82 : INVXL port map( A => n141, Y => n116);
   U83 : NOR2XL port map( A => n193, B => n192, Y => n112);
   U84 : NAND2XL port map( A => n187, B => n186, Y => n111);
   U85 : AOI22XL port map( A0 => n177, A1 => n188, B0 => n183, B1 => n151, Y =>
                           n114);
   U86 : AOI21XL port map( A0 => n183, A1 => n157, B0 => n184, Y => n113);
   U87 : INVXL port map( A => n185, Y => n184);
   U88 : AOI32XL port map( A0 => n186, A1 => n182, A2 => n187, B0 => n188, B1 
                           => n144, Y => n185);
   U89 : NAND2XL port map( A => n183, B => n144, Y => n115);
   U90 : NAND2BXL port map( AN => intr_signal, B => n199, Y => 
                           exception_out_port);
   U91 : NOR3XL port map( A => n107, B => opcode(1), C => n98, Y => n108);
   U92 : NOR3XL port map( A => n95, B => opcode(3), C => n98, Y => n99);
   U93 : NAND3XL port map( A => n101, B => n102, C => n103, Y => n96);
   U94 : NOR3XL port map( A => n95, B => opcode(5), C => n104, Y => n103);
   U95 : NOR2XL port map( A => n107, B => opcode(0), Y => n93);
   U96 : INVXL port map( A => opcode(1), Y => n95);
   U97 : NAND4XL port map( A => n102, B => opcode(4), C => n105, D => n106, Y 
                           => n94);
   U98 : INVXL port map( A => n122, Y => n67);
   U99 : AND4X1 port map( A => n86, B => n87, C => n88, D => n89, Y => n70);
   U100 : NOR2BXL port map( AN => n84, B => n124, Y => n68);
   U101 : NAND2XL port map( A => n90, B => n64, Y => n57);
   U102 : NAND4XL port map( A => opcode(3), B => n121, C => n46, D => opcode(2)
                           , Y => n199);
   U103 : NAND3XL port map( A => n83, B => n84, C => n85, Y => n81);
   U104 : AND3XL port map( A => n117, B => n69, C => n118, Y => n85);
   U105 : AOI32XL port map( A0 => n67, A1 => opcode(20), A2 => n119, B0 => n120
                           , B1 => n121, Y => n118);
   U106 : AND3XL port map( A => n111, B => n115, C => n113, Y => n84);
   U107 : NOR3XL port map( A => n123, B => n90, C => n124, Y => n83);
   U108 : AOI31XL port map( A0 => n125, A1 => n126, A2 => n127, B0 => n128, Y 
                           => n123);
   U109 : AOI211XL port map( A0 => n129, A1 => n45, B0 => n47, C0 => n130, Y =>
                           n127);
   U110 : NAND3XL port map( A => n68, B => n69, C => n70, Y => n54);
   U111 : OAI2BB1XL port map( A0N => opcode(24), A1N => n58, B0 => n61, Y => 
                           rs_index(3));
   U112 : AOI21XL port map( A0 => opcode(14), A1 => n60, B0 => n5, Y => n61);
   U113 : OAI2BB1XL port map( A0N => opcode(22), A1N => n58, B0 => n65, Y => 
                           rs_index(1));
   U114 : AOI21XL port map( A0 => opcode(12), A1 => n60, B0 => 
                           exception_out_port, Y => n65);
   U115 : OAI2BB1XL port map( A0N => opcode(25), A1N => n58, B0 => n59, Y => 
                           rs_index(4));
   U116 : AOI21XL port map( A0 => opcode(15), A1 => n60, B0 => n2, Y => n59);
   U117 : INVXL port map( A => n57, Y => n60);
   U118 : INVXL port map( A => n63, Y => n58);
   U119 : NAND2XL port map( A => n157, B => n158, Y => n87);
   U120 : NAND2XL port map( A => n157, B => n159, Y => n69);
   U121 : AOI2BB1XL port map( A0N => n135, A1N => n128, B0 => 
                           exception_out_port, Y => n91);
   U122 : AND4X1 port map( A => n160, B => n122, C => n137, D => n88, Y => n92)
                           ;
   U123 : AOI31XL port map( A0 => n125, A1 => n189, A2 => n135, B0 => n128, Y 
                           => n181);
   U124 : NAND4XL port map( A => n133, B => n95, C => n105, D => n107, Y => 
                           n189);
   U125 : AND4X1 port map( A => n92, B => n116, C => n155, D => n156, Y => n162
                           );
   U126 : AND3XL port map( A => n149, B => n153, C => n148, Y => n165);
   U127 : INVXL port map( A => opcode(3), Y => n107);
   U128 : INVXL port map( A => n153, Y => n175);
   U129 : NOR4BXL port map( AN => n133, B => n95, C => n128, D => opcode(2), Y 
                           => n169);
   U130 : NAND2BXL port map( AN => intr_signal, B => n199, Y => n5);
   U131 : AOI21XL port map( A0 => n176, A1 => n121, B0 => n171, Y => n173);
   U132 : NAND3XL port map( A => opcode(2), B => n107, C => n133, Y => n166);
   U133 : INVXL port map( A => opcode(0), Y => n98);
   U134 : NAND2XL port map( A => n177, B => n152, Y => n149);
   U135 : NOR2BXL port map( AN => opcode(30), B => n140, Y => n90);
   U136 : NAND2XL port map( A => n151, B => n158, Y => n154);
   U137 : AOI221XL port map( A0 => n169, A1 => n93, B0 => n170, B1 => n121, C0 
                           => n171, Y => n168);
   U138 : INVXL port map( A => n172, Y => n170);
   U139 : NAND4XL port map( A => opcode(1), B => n107, C => n104, D => n106, Y 
                           => n42);
   U140 : NOR3XL port map( A => opcode(4), B => opcode(5), C => opcode(1), Y =>
                           n46);
   U141 : NOR2XL port map( A => opcode(3), B => opcode(0), Y => n45);
   U142 : NOR2XL port map( A => n98, B => n42, Y => n47);
   U143 : NAND2XL port map( A => n121, B => n55, Y => n43);
   U144 : NOR2XL port map( A => n2, B => n116, Y => mem_source_out_0_port);
   U145 : NOR2BXL port map( AN => n111, B => n112, Y => n109);
   U146 : INVXL port map( A => mem_source_out_1_port, Y => n110);
   U147 : AOI21XL port map( A0 => n113, A1 => n114, B0 => n5, Y => 
                           mem_source_out_2_port);
   U148 : NOR2XL port map( A => n115, B => exception_out_port, Y => 
                           mem_source_out_1_port);
   U149 : AOI21XL port map( A0 => opcode(1), A1 => n107, B0 => n108, Y => n100)
                           ;
   U150 : AOI21XL port map( A0 => n98, A1 => n95, B0 => n99, Y => n97);
   U151 : OAI31XL port map( A0 => n94, A1 => opcode(1), A2 => n45, B0 => n96, Y
                           => mult_func(2));
   U152 : NOR3BXL port map( AN => n93, B => n94, C => n95, Y => mult_func(3));
   U153 : INVXL port map( A => opcode(13), Y => n62);
   U154 : NOR2BXL port map( AN => n69, B => n67, Y => n72);
   U155 : NAND2XL port map( A => n70, B => n56, Y => n75);
   U156 : NAND2XL port map( A => n68, B => n57, Y => n74);
   U157 : NAND2BXL port map( AN => intr_signal, B => n199, Y => n2);
   U158 : INVXL port map( A => n81, Y => n73);
   U159 : AOI2BB1XL port map( A0N => n73, A1N => n72, B0 => n5, Y => n79);
   U160 : NAND2XL port map( A => n75, B => n81, Y => n78);
   U161 : INVXL port map( A => n74, Y => n76);
   U162 : NAND2XL port map( A => opcode(23), B => n90, Y => n56);
   U163 : INVXL port map( A => n2, Y => n55);
   U164 : NOR2XL port map( A => n5, B => n85, Y => c_source_out_2_port);
   U165 : NAND2XL port map( A => n84, B => n55, Y => c_source_out_1_port);
   U166 : NAND2XL port map( A => n83, B => n55, Y => c_source_out_0_port);
   U167 : INVXL port map( A => opcode(19), Y => n49);
   U168 : INVXL port map( A => opcode(18), Y => n50);
   U169 : INVXL port map( A => opcode(17), Y => n51);
   U170 : INVXL port map( A => opcode(16), Y => n52);
   U171 : INVXL port map( A => n56, Y => n53);
   U172 : OAI2BB1XL port map( A0N => opcode(21), A1N => n58, B0 => n66, Y => 
                           rs_index(0));
   U173 : AOI21XL port map( A0 => opcode(11), A1 => n60, B0 => n2, Y => n66);
   U174 : NAND3XL port map( A => n69, B => n55, C => n87, Y => 
                           pc_source_out_0_port);
   U175 : NAND3XL port map( A => n91, B => n56, C => n92, Y => 
                           pc_source_out_1_port);
   U176 : NAND3XL port map( A => n165, B => n162, C => n180, Y => alu_func(0));
   U177 : NOR3XL port map( A => n181, B => c_source_out_1_port, C => n90, Y => 
                           n180);
   U178 : AOI21XL port map( A0 => n173, A1 => n174, B0 => n5, Y => alu_func(1))
                           ;
   U179 : AOI21XL port map( A0 => n169, A1 => n107, B0 => n175, Y => n174);
   U180 : NOR4BXL port map( AN => opcode(1), B => n43, C => n98, D => n166, Y 
                           => alu_func(3));
   U181 : NAND4XL port map( A => n154, B => n55, C => n167, D => n168, Y => 
                           alu_func(2));
   U182 : NOR2BXL port map( AN => n149, B => n90, Y => n167);
   U183 : NOR2XL port map( A => n42, B => n43, Y => shift_func(1));
   U184 : NOR2XL port map( A => n44, B => n43, Y => shift_func(0));
   U185 : AOI21XL port map( A0 => n45, A1 => n46, B0 => n47, Y => n44);
   U186 : AOI211XL port map( A0 => n71, A1 => n72, B0 => n2, C0 => n73, Y => 
                           rd_index(4));
   U187 : AOI22XL port map( A0 => opcode(20), A1 => n74, B0 => opcode(15), B1 
                           => n75, Y => n71);
   U188 : INVXL port map( A => opcode(12), Y => n80);
   U189 : AOI211XL port map( A0 => n82, A1 => n72, B0 => n2, C0 => n73, Y => 
                           rd_index(0));
   U190 : AOI22XL port map( A0 => opcode(16), A1 => n74, B0 => opcode(11), B1 
                           => n75, Y => n82);
   U191 : INVXL port map( A => opcode(14), Y => n77);
   U192 : NAND2XL port map( A => n56, B => n55, Y => rd_index(5));
   U193 : NOR2XL port map( A => n48, B => n49, Y => rt_index(3));
   U194 : NOR2XL port map( A => n48, B => n50, Y => rt_index(2));
   U195 : NOR2XL port map( A => n48, B => n51, Y => rt_index(1));
   U196 : NOR2XL port map( A => n48, B => n52, Y => rt_index(0));
   U197 : OAI221XL port map( A0 => n57, A1 => n62, B0 => n63, B1 => n64, C0 => 
                           n55, Y => rs_index(2));
   U198 : OAI21XL port map( A0 => n53, A1 => n54, B0 => n55, Y => n48);
   U199 : OAI21XL port map( A0 => n97, A1 => n94, B0 => n96, Y => mult_func(1))
                           ;
   U200 : OAI21XL port map( A0 => n100, A1 => n94, B0 => n96, Y => mult_func(0)
                           );
   U201 : OAI21XL port map( A0 => n5, A1 => n109, B0 => n110, Y => 
                           mem_source_out_3_port);
   U202 : NAND4BXL port map( AN => pc_source_out_0_port, B => n68, C => n86, D 
                           => n136, Y => branch_func(1));
   U203 : OAI21XL port map( A0 => exception_out_port, A1 => n163, B0 => n1, Y 
                           => b_source_out(0));
   U204 : OAI21XL port map( A0 => n132, A1 => n99, B0 => n133, Y => n126);
   U205 : OAI221XL port map( A0 => n76, A1 => n51, B0 => n80, B1 => n78, C0 => 
                           n79, Y => rd_index(1));
   U206 : OAI221XL port map( A0 => n76, A1 => n50, B0 => n62, B1 => n78, C0 => 
                           n79, Y => rd_index(2));
   U207 : OAI221XL port map( A0 => n76, A1 => n49, B0 => n77, B1 => n78, C0 => 
                           n79, Y => rd_index(3));

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity mem_ctrl is

   port( clk, reset_in, pause_in, nullify_op : in std_logic;  address_pc : in 
         std_logic_vector (31 downto 2);  opcode_out : out std_logic_vector (31
         downto 0);  address_in : in std_logic_vector (31 downto 0);  
         mem_source : in std_logic_vector (3 downto 0);  data_write : in 
         std_logic_vector (31 downto 0);  data_read : out std_logic_vector (31 
         downto 0);  pause_out : out std_logic;  address_next : out 
         std_logic_vector (31 downto 2);  byte_we_next : out std_logic_vector 
         (3 downto 0);  address : out std_logic_vector (31 downto 2);  byte_we 
         : out std_logic_vector (3 downto 0);  data_w : out std_logic_vector 
         (31 downto 0);  data_r : in std_logic_vector (31 downto 0));

end mem_ctrl;

architecture SYN_logic of mem_ctrl is

   component OAI221XL
      port( A0, A1, B0, B1, C0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB2XL
      port( B0, B1, A0N, A1N : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI32XL
      port( A0, A1, A2, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component AND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR4BXL
      port( AN, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component DFFRHQXL
      port( D, CK, RN : in std_logic;  Q : out std_logic);
   end component;
   
   signal opcode_out_31_port, opcode_out_30_port, opcode_out_29_port, 
      opcode_out_28_port, opcode_out_27_port, opcode_out_26_port, 
      opcode_out_25_port, opcode_out_24_port, opcode_out_23_port, 
      opcode_out_22_port, opcode_out_21_port, opcode_out_20_port, 
      opcode_out_19_port, opcode_out_18_port, opcode_out_17_port, 
      opcode_out_16_port, opcode_out_15_port, opcode_out_14_port, 
      opcode_out_13_port, opcode_out_12_port, opcode_out_11_port, 
      opcode_out_10_port, opcode_out_9_port, opcode_out_8_port, 
      opcode_out_7_port, opcode_out_6_port, opcode_out_5_port, 
      opcode_out_4_port, opcode_out_3_port, opcode_out_2_port, 
      opcode_out_1_port, opcode_out_0_port, byte_we_next_3_port, 
      byte_we_next_2_port, byte_we_next_1_port, byte_we_next_0_port, 
      address_31_port, address_30_port, address_29_port, address_28_port, 
      address_27_port, address_26_port, address_25_port, address_24_port, 
      address_23_port, address_22_port, address_21_port, address_20_port, 
      address_19_port, address_18_port, address_17_port, address_16_port, 
      address_15_port, address_14_port, address_13_port, address_12_port, 
      address_11_port, address_10_port, address_9_port, address_8_port, 
      address_7_port, address_6_port, address_5_port, address_4_port, 
      address_3_port, address_2_port, byte_we_3_port, byte_we_2_port, 
      byte_we_1_port, byte_we_0_port, mem_state_reg, next_opcode_reg_31_port, 
      next_opcode_reg_30_port, next_opcode_reg_29_port, next_opcode_reg_28_port
      , next_opcode_reg_27_port, next_opcode_reg_26_port, 
      next_opcode_reg_25_port, next_opcode_reg_24_port, next_opcode_reg_23_port
      , next_opcode_reg_22_port, next_opcode_reg_21_port, 
      next_opcode_reg_20_port, next_opcode_reg_19_port, next_opcode_reg_18_port
      , next_opcode_reg_17_port, next_opcode_reg_16_port, 
      next_opcode_reg_15_port, next_opcode_reg_14_port, next_opcode_reg_13_port
      , next_opcode_reg_12_port, next_opcode_reg_11_port, 
      next_opcode_reg_10_port, next_opcode_reg_9_port, next_opcode_reg_8_port, 
      next_opcode_reg_7_port, next_opcode_reg_6_port, next_opcode_reg_5_port, 
      next_opcode_reg_4_port, next_opcode_reg_3_port, next_opcode_reg_2_port, 
      next_opcode_reg_1_port, next_opcode_reg_0_port, n2, n4, n5, n6, n7, n8, 
      n9, n10, n11, n12, n13, n14, n15, n16, n17, n18, n19, n20, n21, n22, n23,
      n24, n25, n26, n27, n28, n29, n30, n31, n32, n33, n34, n35, n36, n38, n39
      , n40, n41, n42, n43, n44, n45, n46, n47, n48, n49, n50, n51, n52, n53, 
      n54, n55, n56, n57, n58, n59, n60, n61, n62, n63, n64, n65, n66, n67, n68
      , n69, n70, n71, n72, n73, n74, n75, n76, n77, n78, n79, n80, n81, n82, 
      n83, n84, n85, n86, n87, n88, n89, n90, n91, n92, n93, n94, n95, n96, n97
      , n98, n99, n100, n101, n103, n104, n105, n106, n107, n108, n109, n110, 
      n112, n113, n114, n115, n116, n117, n118, n119, n120, n122, n123, n124, 
      n126, n127, n128, n129, n130, n131, n132, n133, n134, n135, n136, n137, 
      n138, n139, n140, n141, n142, n143, n144, n145, n146, n147, n148, n149, 
      n150, n151, n152, n153, n154, n156, n157, n158, n159, n160, n161, n162, 
      n163, n164, n165, n166, n167, n168, n169, n170, n171, n173, n174, n175, 
      n176, n177, n178, n179, n181, n182, n183, n184, n185, n186, n187, n188, 
      n189, n190, n191, n192, n193, n194, n195, n196, n197, n198, n199, n200, 
      n201, n202, n203, n204, n205, n206, n207, n208, n209, n210, n211, n212, 
      n213, n214, n215, n216, n217, n218, n219, n220, n221, n222, n223, n224, 
      n225, n226, n227, n228, n229, n230, n231, n232, n233, n234, n235, n236, 
      n237, n238, n239, n240, n241, n242, n243, n244, n245, n246, n247, n248, 
      n249, n250, n251, n252, n253, n254, n255, n256, n257, n258, n259, n260, 
      n261, n262, n263, n264, n265, n266, n267, n268, n269, n270, n271, n272, 
      n273, n274, n275, n276, n277, n278, n279, n281, n1, n3, n37, n102, n111, 
      n121, n125, n155, n172, n180, n280, n282, n283, n284, n285, n286, n287, 
      n288, n289 : std_logic;

begin
   opcode_out <= ( opcode_out_31_port, opcode_out_30_port, opcode_out_29_port, 
      opcode_out_28_port, opcode_out_27_port, opcode_out_26_port, 
      opcode_out_25_port, opcode_out_24_port, opcode_out_23_port, 
      opcode_out_22_port, opcode_out_21_port, opcode_out_20_port, 
      opcode_out_19_port, opcode_out_18_port, opcode_out_17_port, 
      opcode_out_16_port, opcode_out_15_port, opcode_out_14_port, 
      opcode_out_13_port, opcode_out_12_port, opcode_out_11_port, 
      opcode_out_10_port, opcode_out_9_port, opcode_out_8_port, 
      opcode_out_7_port, opcode_out_6_port, opcode_out_5_port, 
      opcode_out_4_port, opcode_out_3_port, opcode_out_2_port, 
      opcode_out_1_port, opcode_out_0_port );
   byte_we_next <= ( byte_we_next_3_port, byte_we_next_2_port, 
      byte_we_next_1_port, byte_we_next_0_port );
   address <= ( address_31_port, address_30_port, address_29_port, 
      address_28_port, address_27_port, address_26_port, address_25_port, 
      address_24_port, address_23_port, address_22_port, address_21_port, 
      address_20_port, address_19_port, address_18_port, address_17_port, 
      address_16_port, address_15_port, address_14_port, address_13_port, 
      address_12_port, address_11_port, address_10_port, address_9_port, 
      address_8_port, address_7_port, address_6_port, address_5_port, 
      address_4_port, address_3_port, address_2_port );
   byte_we <= ( byte_we_3_port, byte_we_2_port, byte_we_1_port, byte_we_0_port 
      );
   
   address_reg_regx31x : DFFRHQXL port map( D => n211, CK => clk, RN => n287, Q
                           => address_31_port);
   address_reg_regx30x : DFFRHQXL port map( D => n210, CK => clk, RN => n286, Q
                           => address_30_port);
   address_reg_regx29x : DFFRHQXL port map( D => n209, CK => clk, RN => n287, Q
                           => address_29_port);
   address_reg_regx28x : DFFRHQXL port map( D => n208, CK => clk, RN => n286, Q
                           => address_28_port);
   address_reg_regx27x : DFFRHQXL port map( D => n207, CK => clk, RN => n287, Q
                           => address_27_port);
   address_reg_regx26x : DFFRHQXL port map( D => n206, CK => clk, RN => n287, Q
                           => address_26_port);
   address_reg_regx25x : DFFRHQXL port map( D => n205, CK => clk, RN => n287, Q
                           => address_25_port);
   address_reg_regx24x : DFFRHQXL port map( D => n204, CK => clk, RN => n287, Q
                           => address_24_port);
   address_reg_regx23x : DFFRHQXL port map( D => n203, CK => clk, RN => n287, Q
                           => address_23_port);
   address_reg_regx22x : DFFRHQXL port map( D => n202, CK => clk, RN => n287, Q
                           => address_22_port);
   address_reg_regx21x : DFFRHQXL port map( D => n201, CK => clk, RN => n287, Q
                           => address_21_port);
   address_reg_regx20x : DFFRHQXL port map( D => n200, CK => clk, RN => n287, Q
                           => address_20_port);
   address_reg_regx19x : DFFRHQXL port map( D => n199, CK => clk, RN => n287, Q
                           => address_19_port);
   address_reg_regx18x : DFFRHQXL port map( D => n198, CK => clk, RN => n287, Q
                           => address_18_port);
   address_reg_regx17x : DFFRHQXL port map( D => n197, CK => clk, RN => n287, Q
                           => address_17_port);
   address_reg_regx16x : DFFRHQXL port map( D => n196, CK => clk, RN => n287, Q
                           => address_16_port);
   address_reg_regx15x : DFFRHQXL port map( D => n195, CK => clk, RN => n287, Q
                           => address_15_port);
   address_reg_regx14x : DFFRHQXL port map( D => n194, CK => clk, RN => n286, Q
                           => address_14_port);
   address_reg_regx13x : DFFRHQXL port map( D => n193, CK => clk, RN => n286, Q
                           => address_13_port);
   address_reg_regx12x : DFFRHQXL port map( D => n192, CK => clk, RN => n286, Q
                           => address_12_port);
   address_reg_regx11x : DFFRHQXL port map( D => n191, CK => clk, RN => n286, Q
                           => address_11_port);
   address_reg_regx10x : DFFRHQXL port map( D => n190, CK => clk, RN => n286, Q
                           => address_10_port);
   address_reg_regx9x : DFFRHQXL port map( D => n189, CK => clk, RN => n286, Q 
                           => address_9_port);
   address_reg_regx8x : DFFRHQXL port map( D => n188, CK => clk, RN => n286, Q 
                           => address_8_port);
   address_reg_regx7x : DFFRHQXL port map( D => n187, CK => clk, RN => n286, Q 
                           => address_7_port);
   address_reg_regx6x : DFFRHQXL port map( D => n186, CK => clk, RN => n286, Q 
                           => address_6_port);
   address_reg_regx5x : DFFRHQXL port map( D => n185, CK => clk, RN => n286, Q 
                           => address_5_port);
   address_reg_regx4x : DFFRHQXL port map( D => n184, CK => clk, RN => n286, Q 
                           => address_4_port);
   address_reg_regx3x : DFFRHQXL port map( D => n183, CK => clk, RN => n286, Q 
                           => address_3_port);
   address_reg_regx2x : DFFRHQXL port map( D => n182, CK => clk, RN => n286, Q 
                           => address_2_port);
   byte_we_reg_regx3x : DFFRHQXL port map( D => n279, CK => clk, RN => n286, Q 
                           => byte_we_3_port);
   byte_we_reg_regx2x : DFFRHQXL port map( D => n278, CK => clk, RN => n287, Q 
                           => byte_we_2_port);
   byte_we_reg_regx1x : DFFRHQXL port map( D => n277, CK => clk, RN => n286, Q 
                           => byte_we_1_port);
   byte_we_reg_regx0x : DFFRHQXL port map( D => n276, CK => clk, RN => n286, Q 
                           => byte_we_0_port);
   next_opcode_reg_regx31x : DFFRHQXL port map( D => n275, CK => clk, RN => 
                           n286, Q => next_opcode_reg_31_port);
   next_opcode_reg_regx30x : DFFRHQXL port map( D => n273, CK => clk, RN => 
                           n287, Q => next_opcode_reg_30_port);
   next_opcode_reg_regx29x : DFFRHQXL port map( D => n271, CK => clk, RN => 
                           n287, Q => next_opcode_reg_29_port);
   next_opcode_reg_regx28x : DFFRHQXL port map( D => n269, CK => clk, RN => 
                           n287, Q => next_opcode_reg_28_port);
   next_opcode_reg_regx27x : DFFRHQXL port map( D => n267, CK => clk, RN => 
                           n286, Q => next_opcode_reg_27_port);
   next_opcode_reg_regx26x : DFFRHQXL port map( D => n265, CK => clk, RN => 
                           n287, Q => next_opcode_reg_26_port);
   next_opcode_reg_regx25x : DFFRHQXL port map( D => n263, CK => clk, RN => 
                           n286, Q => next_opcode_reg_25_port);
   next_opcode_reg_regx24x : DFFRHQXL port map( D => n261, CK => clk, RN => 
                           n286, Q => next_opcode_reg_24_port);
   next_opcode_reg_regx23x : DFFRHQXL port map( D => n259, CK => clk, RN => 
                           n287, Q => next_opcode_reg_23_port);
   next_opcode_reg_regx22x : DFFRHQXL port map( D => n257, CK => clk, RN => 
                           n287, Q => next_opcode_reg_22_port);
   next_opcode_reg_regx21x : DFFRHQXL port map( D => n255, CK => clk, RN => 
                           n286, Q => next_opcode_reg_21_port);
   next_opcode_reg_regx20x : DFFRHQXL port map( D => n253, CK => clk, RN => 
                           n287, Q => next_opcode_reg_20_port);
   next_opcode_reg_regx19x : DFFRHQXL port map( D => n251, CK => clk, RN => 
                           n286, Q => next_opcode_reg_19_port);
   next_opcode_reg_regx18x : DFFRHQXL port map( D => n249, CK => clk, RN => 
                           n286, Q => next_opcode_reg_18_port);
   next_opcode_reg_regx17x : DFFRHQXL port map( D => n247, CK => clk, RN => 
                           n287, Q => next_opcode_reg_17_port);
   next_opcode_reg_regx16x : DFFRHQXL port map( D => n245, CK => clk, RN => 
                           n286, Q => next_opcode_reg_16_port);
   next_opcode_reg_regx15x : DFFRHQXL port map( D => n243, CK => clk, RN => 
                           n287, Q => next_opcode_reg_15_port);
   next_opcode_reg_regx14x : DFFRHQXL port map( D => n241, CK => clk, RN => 
                           n286, Q => next_opcode_reg_14_port);
   next_opcode_reg_regx13x : DFFRHQXL port map( D => n239, CK => clk, RN => 
                           n286, Q => next_opcode_reg_13_port);
   next_opcode_reg_regx12x : DFFRHQXL port map( D => n237, CK => clk, RN => 
                           n287, Q => next_opcode_reg_12_port);
   next_opcode_reg_regx11x : DFFRHQXL port map( D => n235, CK => clk, RN => 
                           n287, Q => next_opcode_reg_11_port);
   next_opcode_reg_regx10x : DFFRHQXL port map( D => n233, CK => clk, RN => 
                           n286, Q => next_opcode_reg_10_port);
   next_opcode_reg_regx9x : DFFRHQXL port map( D => n231, CK => clk, RN => n286
                           , Q => next_opcode_reg_9_port);
   next_opcode_reg_regx8x : DFFRHQXL port map( D => n229, CK => clk, RN => n287
                           , Q => next_opcode_reg_8_port);
   next_opcode_reg_regx7x : DFFRHQXL port map( D => n227, CK => clk, RN => n286
                           , Q => next_opcode_reg_7_port);
   next_opcode_reg_regx6x : DFFRHQXL port map( D => n225, CK => clk, RN => n286
                           , Q => next_opcode_reg_6_port);
   next_opcode_reg_regx5x : DFFRHQXL port map( D => n223, CK => clk, RN => n287
                           , Q => next_opcode_reg_5_port);
   next_opcode_reg_regx4x : DFFRHQXL port map( D => n221, CK => clk, RN => n287
                           , Q => next_opcode_reg_4_port);
   next_opcode_reg_regx3x : DFFRHQXL port map( D => n219, CK => clk, RN => n287
                           , Q => next_opcode_reg_3_port);
   next_opcode_reg_regx2x : DFFRHQXL port map( D => n217, CK => clk, RN => n286
                           , Q => next_opcode_reg_2_port);
   next_opcode_reg_regx1x : DFFRHQXL port map( D => n215, CK => clk, RN => n287
                           , Q => next_opcode_reg_1_port);
   next_opcode_reg_regx0x : DFFRHQXL port map( D => n213, CK => clk, RN => n286
                           , Q => next_opcode_reg_0_port);
   opcode_reg_regx10x : DFFRHQXL port map( D => n232, CK => clk, RN => n286, Q 
                           => opcode_out_10_port);
   opcode_reg_regx9x : DFFRHQXL port map( D => n230, CK => clk, RN => n287, Q 
                           => opcode_out_9_port);
   opcode_reg_regx8x : DFFRHQXL port map( D => n228, CK => clk, RN => n286, Q 
                           => opcode_out_8_port);
   opcode_reg_regx7x : DFFRHQXL port map( D => n226, CK => clk, RN => n287, Q 
                           => opcode_out_7_port);
   opcode_reg_regx6x : DFFRHQXL port map( D => n224, CK => clk, RN => n286, Q 
                           => opcode_out_6_port);
   mem_state_reg_reg : DFFRHQXL port map( D => n281, CK => clk, RN => n286, Q 
                           => mem_state_reg);
   opcode_reg_regx25x : DFFRHQXL port map( D => n262, CK => clk, RN => n287, Q 
                           => opcode_out_25_port);
   opcode_reg_regx24x : DFFRHQXL port map( D => n260, CK => clk, RN => n287, Q 
                           => opcode_out_24_port);
   opcode_reg_regx22x : DFFRHQXL port map( D => n256, CK => clk, RN => n287, Q 
                           => opcode_out_22_port);
   opcode_reg_regx17x : DFFRHQXL port map( D => n246, CK => clk, RN => n287, Q 
                           => opcode_out_17_port);
   opcode_reg_regx18x : DFFRHQXL port map( D => n248, CK => clk, RN => n287, Q 
                           => opcode_out_18_port);
   opcode_reg_regx16x : DFFRHQXL port map( D => n244, CK => clk, RN => n286, Q 
                           => opcode_out_16_port);
   opcode_reg_regx12x : DFFRHQXL port map( D => n236, CK => clk, RN => n287, Q 
                           => opcode_out_12_port);
   opcode_reg_regx15x : DFFRHQXL port map( D => n242, CK => clk, RN => n287, Q 
                           => opcode_out_15_port);
   opcode_reg_regx21x : DFFRHQXL port map( D => n254, CK => clk, RN => n286, Q 
                           => opcode_out_21_port);
   opcode_reg_regx20x : DFFRHQXL port map( D => n252, CK => clk, RN => n286, Q 
                           => opcode_out_20_port);
   opcode_reg_regx19x : DFFRHQXL port map( D => n250, CK => clk, RN => n287, Q 
                           => opcode_out_19_port);
   opcode_reg_regx23x : DFFRHQXL port map( D => n258, CK => clk, RN => n286, Q 
                           => opcode_out_23_port);
   opcode_reg_regx13x : DFFRHQXL port map( D => n238, CK => clk, RN => n286, Q 
                           => opcode_out_13_port);
   opcode_reg_regx11x : DFFRHQXL port map( D => n234, CK => clk, RN => n287, Q 
                           => opcode_out_11_port);
   opcode_reg_regx14x : DFFRHQXL port map( D => n240, CK => clk, RN => n286, Q 
                           => opcode_out_14_port);
   opcode_reg_regx0x : DFFRHQXL port map( D => n212, CK => clk, RN => n287, Q 
                           => opcode_out_0_port);
   opcode_reg_regx2x : DFFRHQXL port map( D => n216, CK => clk, RN => n286, Q 
                           => opcode_out_2_port);
   opcode_reg_regx1x : DFFRHQXL port map( D => n214, CK => clk, RN => n287, Q 
                           => opcode_out_1_port);
   opcode_reg_regx26x : DFFRHQXL port map( D => n264, CK => clk, RN => n286, Q 
                           => opcode_out_26_port);
   opcode_reg_regx5x : DFFRHQXL port map( D => n222, CK => clk, RN => n287, Q 
                           => opcode_out_5_port);
   opcode_reg_regx3x : DFFRHQXL port map( D => n218, CK => clk, RN => n286, Q 
                           => opcode_out_3_port);
   opcode_reg_regx4x : DFFRHQXL port map( D => n220, CK => clk, RN => n287, Q 
                           => opcode_out_4_port);
   opcode_reg_regx31x : DFFRHQXL port map( D => n274, CK => clk, RN => n286, Q 
                           => opcode_out_31_port);
   opcode_reg_regx30x : DFFRHQXL port map( D => n272, CK => clk, RN => n287, Q 
                           => opcode_out_30_port);
   opcode_reg_regx27x : DFFRHQXL port map( D => n266, CK => clk, RN => n286, Q 
                           => opcode_out_27_port);
   opcode_reg_regx29x : DFFRHQXL port map( D => n270, CK => clk, RN => n286, Q 
                           => opcode_out_29_port);
   opcode_reg_regx28x : DFFRHQXL port map( D => n268, CK => clk, RN => n287, Q 
                           => opcode_out_28_port);
   U3 : AND2XL port map( A => n173, B => mem_source(0), Y => n1);
   U4 : AND2XL port map( A => n101, B => n289, Y => n3);
   U5 : INVXL port map( A => mem_source(3), Y => n175);
   U6 : INVXL port map( A => n146, Y => n169);
   U7 : NOR3XL port map( A => mem_source(0), B => mem_source(2), C => n175, Y 
                           => n160);
   U8 : NAND2XL port map( A => n100, B => n181, Y => n108);
   U9 : INVXL port map( A => n2, Y => n100);
   U10 : INVXL port map( A => mem_source(0), Y => n174);
   U11 : INVXL port map( A => n108, Y => n180);
   U12 : NOR3XL port map( A => mem_source(1), B => mem_source(3), C => n176, Y 
                           => n173);
   U13 : INVXL port map( A => mem_source(2), Y => n176);
   U14 : INVXL port map( A => address_in(1), Y => n162);
   U15 : NOR3XL port map( A => n174, B => mem_source(1), C => n175, Y => n140);
   U16 : NOR3XL port map( A => address_in(0), B => address_in(1), C => n169, Y 
                           => n151);
   U17 : INVXL port map( A => n170, Y => n149);
   U18 : OAI32XL port map( A0 => n103, A1 => address_in(1), A2 => address_in(0)
                           , B0 => n84, B1 => n167, Y => n147);
   U19 : NOR3XL port map( A => n175, B => mem_source(0), C => n176, Y => n146);
   U20 : NOR2XL port map( A => n169, B => n165, Y => n148);
   U21 : INVXL port map( A => n171, Y => n144);
   U22 : NAND3XL port map( A => n146, B => n163, C => mem_source(1), Y => n143)
                           ;
   U23 : INVXL port map( A => n147, Y => n166);
   U24 : NAND2XL port map( A => n160, B => n162, Y => n142);
   U25 : INVXL port map( A => n125, Y => n141);
   U26 : OAI2BB1XL port map( A0N => address_in(1), A1N => n160, B0 => n37, Y =>
                           n125);
   U27 : NAND2XL port map( A => n173, B => n174, Y => n37);
   U28 : AOI31XL port map( A0 => n159, A1 => mem_source(1), A2 => n160, B0 => 
                           n161, Y => n156);
   U29 : AOI22XL port map( A0 => n162, A1 => n103, B0 => n68, B1 => 
                           address_in(1), Y => n159);
   U30 : INVXL port map( A => n143, Y => n161);
   U31 : NOR4BXL port map( AN => n100, B => n101, C => nullify_op, D => n288, Y
                           => n36);
   U32 : NOR4BXL port map( AN => n174, B => mem_source(2), C => mem_source(3), 
                           D => mem_source(1), Y => n2);
   U33 : INVXL port map( A => pause_in, Y => n289);
   U34 : NOR2XL port map( A => n2, B => n284, Y => pause_out);
   U35 : AND2XL port map( A => n109, B => n102, Y => n115);
   U36 : NOR2BXL port map( AN => n111, B => n1, Y => n109);
   U37 : NAND2XL port map( A => mem_source(2), B => n140, Y => n102);
   U38 : INVXL port map( A => data_write(0), Y => n114);
   U39 : INVXL port map( A => data_write(1), Y => n112);
   U40 : INVXL port map( A => data_write(2), Y => n128);
   U41 : INVXL port map( A => data_write(3), Y => n120);
   U42 : INVXL port map( A => data_write(4), Y => n119);
   U43 : INVXL port map( A => data_write(5), Y => n118);
   U44 : INVXL port map( A => data_write(6), Y => n117);
   U45 : AND2XL port map( A => n111, B => n102, Y => n139);
   U46 : INVXL port map( A => data_write(7), Y => n116);
   U47 : INVXL port map( A => data_write(8), Y => n113);
   U48 : INVXL port map( A => data_write(9), Y => n110);
   U49 : INVXL port map( A => data_write(10), Y => n135);
   U50 : INVXL port map( A => data_write(11), Y => n133);
   U51 : INVXL port map( A => data_write(12), Y => n131);
   U52 : INVXL port map( A => data_write(13), Y => n129);
   U53 : INVXL port map( A => data_write(14), Y => n126);
   U54 : INVXL port map( A => n102, Y => n124);
   U55 : NAND2XL port map( A => n140, B => n176, Y => n111);
   U56 : INVXL port map( A => data_write(15), Y => n122);
   U57 : NAND2XL port map( A => address_in(1), B => address_in(0), Y => n164);
   U58 : OR2XL port map( A => address_in(0), B => n162, Y => n165);
   U59 : AOI2BB1XL port map( A0N => n162, A1N => n111, B0 => n1, Y => n179);
   U60 : NAND2XL port map( A => address_in(0), B => n162, Y => n167);
   U61 : NAND2BXL port map( AN => n285, B => n140, Y => n177);
   U62 : AOI2BB1XL port map( A0N => address_in(1), A1N => n111, B0 => n1, Y => 
                           n178);
   U63 : AOI22XL port map( A0 => n151, A1 => data_r(24), B0 => n148, B1 => 
                           data_r(8), Y => n168);
   U64 : AOI22XL port map( A0 => n151, A1 => data_r(25), B0 => n148, B1 => 
                           data_r(9), Y => n158);
   U65 : AOI22XL port map( A0 => n151, A1 => data_r(26), B0 => n148, B1 => 
                           data_r(10), Y => n157);
   U66 : AOI22XL port map( A0 => n151, A1 => data_r(27), B0 => n148, B1 => 
                           data_r(11), Y => n154);
   U67 : AOI22XL port map( A0 => n151, A1 => data_r(28), B0 => n148, B1 => 
                           data_r(12), Y => n153);
   U68 : AOI22XL port map( A0 => n151, A1 => data_r(29), B0 => n148, B1 => 
                           data_r(13), Y => n152);
   U69 : AOI22XL port map( A0 => n151, A1 => data_r(30), B0 => n148, B1 => 
                           data_r(14), Y => n150);
   U70 : AOI22XL port map( A0 => n146, A1 => n147, B0 => n148, B1 => data_r(15)
                           , Y => n145);
   U71 : INVXL port map( A => n121, Y => n280);
   U72 : INVXL port map( A => mem_state_reg, Y => n101);
   U73 : OAI31XL port map( A0 => n2, A1 => nullify_op, A2 => mem_state_reg, B0 
                           => n289, Y => n34);
   U74 : OAI31XL port map( A0 => n2, A1 => nullify_op, A2 => mem_state_reg, B0 
                           => n289, Y => n155);
   U75 : INVXL port map( A => n121, Y => n282);
   U76 : OAI31XL port map( A0 => n2, A1 => nullify_op, A2 => mem_state_reg, B0 
                           => n289, Y => n172);
   U77 : INVXL port map( A => data_r(0), Y => n38);
   U78 : INVXL port map( A => data_r(1), Y => n40);
   U79 : INVXL port map( A => data_r(2), Y => n42);
   U80 : INVXL port map( A => data_r(3), Y => n44);
   U81 : INVXL port map( A => data_r(4), Y => n46);
   U82 : INVXL port map( A => data_r(5), Y => n48);
   U83 : INVXL port map( A => data_r(6), Y => n50);
   U84 : INVXL port map( A => data_r(7), Y => n52);
   U85 : INVXL port map( A => data_r(8), Y => n54);
   U86 : INVXL port map( A => data_r(9), Y => n56);
   U87 : INVXL port map( A => data_r(10), Y => n58);
   U88 : INVXL port map( A => data_r(11), Y => n60);
   U89 : INVXL port map( A => data_r(12), Y => n62);
   U90 : INVXL port map( A => data_r(13), Y => n64);
   U91 : INVXL port map( A => data_r(14), Y => n66);
   U92 : INVXL port map( A => data_r(15), Y => n68);
   U93 : INVXL port map( A => data_r(16), Y => n70);
   U94 : INVXL port map( A => data_r(17), Y => n72);
   U95 : INVXL port map( A => data_r(18), Y => n74);
   U96 : INVXL port map( A => data_r(19), Y => n76);
   U97 : INVXL port map( A => data_r(20), Y => n78);
   U98 : INVXL port map( A => data_r(21), Y => n80);
   U99 : INVXL port map( A => data_r(22), Y => n82);
   U100 : INVXL port map( A => data_r(23), Y => n84);
   U101 : INVXL port map( A => data_r(24), Y => n86);
   U102 : INVXL port map( A => data_r(25), Y => n88);
   U103 : INVXL port map( A => data_r(26), Y => n90);
   U104 : INVXL port map( A => data_r(27), Y => n92);
   U105 : INVXL port map( A => data_r(28), Y => n94);
   U106 : INVXL port map( A => data_r(29), Y => n96);
   U107 : INVXL port map( A => n3, Y => n284);
   U108 : INVXL port map( A => data_r(30), Y => n98);
   U109 : INVXL port map( A => data_r(31), Y => n103);
   U110 : INVXL port map( A => n3, Y => n285);
   U111 : INVXL port map( A => n3, Y => n283);
   U112 : AOI22XL port map( A0 => address_in(2), A1 => n180, B0 => 
                           address_pc(2), B1 => n108, Y => n4);
   U113 : AOI22XL port map( A0 => address_in(3), A1 => n180, B0 => 
                           address_pc(3), B1 => n108, Y => n5);
   U114 : AOI22XL port map( A0 => address_in(4), A1 => n180, B0 => 
                           address_pc(4), B1 => n108, Y => n6);
   U115 : AOI22XL port map( A0 => address_in(5), A1 => n180, B0 => 
                           address_pc(5), B1 => n108, Y => n7);
   U116 : AOI22XL port map( A0 => address_in(6), A1 => n180, B0 => 
                           address_pc(6), B1 => n108, Y => n8);
   U117 : AOI22XL port map( A0 => address_in(7), A1 => n180, B0 => 
                           address_pc(7), B1 => n108, Y => n9);
   U118 : AOI22XL port map( A0 => address_in(8), A1 => n180, B0 => 
                           address_pc(8), B1 => n108, Y => n10);
   U119 : AOI22XL port map( A0 => address_in(9), A1 => n180, B0 => 
                           address_pc(9), B1 => n108, Y => n11);
   U120 : AOI22XL port map( A0 => address_in(10), A1 => n180, B0 => 
                           address_pc(10), B1 => n108, Y => n12);
   U121 : AOI22XL port map( A0 => address_in(11), A1 => n180, B0 => 
                           address_pc(11), B1 => n108, Y => n13);
   U122 : AOI22XL port map( A0 => address_in(12), A1 => n180, B0 => 
                           address_pc(12), B1 => n108, Y => n14);
   U123 : AOI22XL port map( A0 => address_in(13), A1 => n180, B0 => 
                           address_pc(13), B1 => n108, Y => n15);
   U124 : AOI22XL port map( A0 => address_in(14), A1 => n180, B0 => 
                           address_pc(14), B1 => n108, Y => n16);
   U125 : AOI22XL port map( A0 => address_in(15), A1 => n180, B0 => 
                           address_pc(15), B1 => n108, Y => n17);
   U126 : AOI22XL port map( A0 => address_in(16), A1 => n180, B0 => 
                           address_pc(16), B1 => n108, Y => n18);
   U127 : AOI22XL port map( A0 => address_in(17), A1 => n180, B0 => 
                           address_pc(17), B1 => n108, Y => n19);
   U128 : AOI22XL port map( A0 => address_in(18), A1 => n180, B0 => 
                           address_pc(18), B1 => n108, Y => n20);
   U129 : AOI22XL port map( A0 => address_in(19), A1 => n180, B0 => 
                           address_pc(19), B1 => n108, Y => n21);
   U130 : AOI22XL port map( A0 => address_in(20), A1 => n180, B0 => 
                           address_pc(20), B1 => n108, Y => n22);
   U131 : AOI22XL port map( A0 => address_in(21), A1 => n180, B0 => 
                           address_pc(21), B1 => n108, Y => n23);
   U132 : AOI22XL port map( A0 => address_in(22), A1 => n180, B0 => 
                           address_pc(22), B1 => n108, Y => n24);
   U133 : AOI22XL port map( A0 => address_in(23), A1 => n180, B0 => 
                           address_pc(23), B1 => n108, Y => n25);
   U134 : AOI22XL port map( A0 => address_in(24), A1 => n180, B0 => 
                           address_pc(24), B1 => n108, Y => n26);
   U135 : AOI22XL port map( A0 => address_in(25), A1 => n180, B0 => 
                           address_pc(25), B1 => n108, Y => n27);
   U136 : AOI22XL port map( A0 => address_in(26), A1 => n180, B0 => 
                           address_pc(26), B1 => n108, Y => n28);
   U137 : AOI22XL port map( A0 => address_in(27), A1 => n180, B0 => 
                           address_pc(27), B1 => n108, Y => n29);
   U138 : AOI22XL port map( A0 => address_in(28), A1 => n180, B0 => 
                           address_pc(28), B1 => n108, Y => n30);
   U139 : AOI22XL port map( A0 => address_in(29), A1 => n180, B0 => 
                           address_pc(29), B1 => n108, Y => n31);
   U140 : AOI22XL port map( A0 => address_in(30), A1 => n180, B0 => 
                           address_pc(30), B1 => n108, Y => n32);
   U141 : INVXL port map( A => n289, Y => n288);
   U142 : AOI22XL port map( A0 => address_in(31), A1 => n180, B0 => 
                           address_pc(31), B1 => n108, Y => n33);
   U143 : NOR2XL port map( A => n115, B => n114, Y => data_w(0));
   U144 : NOR2XL port map( A => n115, B => n112, Y => data_w(1));
   U145 : NOR2XL port map( A => n115, B => n128, Y => data_w(2));
   U146 : NOR2XL port map( A => n115, B => n120, Y => data_w(3));
   U147 : NOR2XL port map( A => n115, B => n119, Y => data_w(4));
   U148 : NOR2XL port map( A => n115, B => n118, Y => data_w(5));
   U149 : NOR2XL port map( A => n115, B => n117, Y => data_w(6));
   U150 : NOR2XL port map( A => n115, B => n116, Y => data_w(7));
   U151 : OAI22XL port map( A0 => n109, A1 => n113, B0 => n102, B1 => n114, Y 
                           => data_w(8));
   U152 : OAI22XL port map( A0 => n109, A1 => n110, B0 => n102, B1 => n112, Y 
                           => data_w(9));
   U153 : OAI22XL port map( A0 => n109, A1 => n135, B0 => n102, B1 => n128, Y 
                           => data_w(10));
   U154 : OAI22XL port map( A0 => n109, A1 => n133, B0 => n102, B1 => n120, Y 
                           => data_w(11));
   U155 : OAI22XL port map( A0 => n109, A1 => n131, B0 => n102, B1 => n119, Y 
                           => data_w(12));
   U156 : OAI22XL port map( A0 => n109, A1 => n129, B0 => n102, B1 => n118, Y 
                           => data_w(13));
   U157 : OAI22XL port map( A0 => n109, A1 => n126, B0 => n102, B1 => n117, Y 
                           => data_w(14));
   U158 : OAI22XL port map( A0 => n109, A1 => n122, B0 => n102, B1 => n116, Y 
                           => data_w(15));
   U159 : OAI2BB2XL port map( B0 => n139, B1 => n114, A0N => data_write(16), 
                           A1N => n1, Y => data_w(16));
   U160 : OAI2BB2XL port map( B0 => n139, B1 => n112, A0N => data_write(17), 
                           A1N => n1, Y => data_w(17));
   U161 : OAI2BB2XL port map( B0 => n139, B1 => n128, A0N => data_write(18), 
                           A1N => n1, Y => data_w(18));
   U162 : OAI2BB2XL port map( B0 => n139, B1 => n120, A0N => data_write(19), 
                           A1N => n1, Y => data_w(19));
   U163 : OAI2BB2XL port map( B0 => n139, B1 => n119, A0N => data_write(20), 
                           A1N => n1, Y => data_w(20));
   U164 : OAI2BB2XL port map( B0 => n139, B1 => n118, A0N => data_write(21), 
                           A1N => n1, Y => data_w(21));
   U165 : OAI2BB2XL port map( B0 => n139, B1 => n117, A0N => data_write(22), 
                           A1N => n1, Y => data_w(22));
   U166 : OAI2BB2XL port map( B0 => n139, B1 => n116, A0N => data_write(23), 
                           A1N => n1, Y => data_w(23));
   U167 : AOI22XL port map( A0 => data_write(0), A1 => n124, B0 => 
                           data_write(24), B1 => n1, Y => n138);
   U168 : AOI22XL port map( A0 => data_write(1), A1 => n124, B0 => 
                           data_write(25), B1 => n1, Y => n137);
   U169 : AOI22XL port map( A0 => data_write(2), A1 => n124, B0 => 
                           data_write(26), B1 => n1, Y => n136);
   U170 : AOI22XL port map( A0 => data_write(3), A1 => n124, B0 => 
                           data_write(27), B1 => n1, Y => n134);
   U171 : AOI22XL port map( A0 => data_write(4), A1 => n124, B0 => 
                           data_write(28), B1 => n1, Y => n132);
   U172 : AOI22XL port map( A0 => data_write(5), A1 => n124, B0 => 
                           data_write(29), B1 => n1, Y => n130);
   U173 : AOI22XL port map( A0 => data_write(6), A1 => n124, B0 => 
                           data_write(30), B1 => n1, Y => n127);
   U174 : AOI22XL port map( A0 => data_write(7), A1 => n124, B0 => 
                           data_write(31), B1 => n1, Y => n123);
   U175 : OAI22XL port map( A0 => n179, A1 => n284, B0 => n177, B1 => n164, Y 
                           => byte_we_next_0_port);
   U176 : OAI22XL port map( A0 => n179, A1 => n284, B0 => n177, B1 => n165, Y 
                           => byte_we_next_1_port);
   U177 : OAI22XL port map( A0 => n178, A1 => n284, B0 => n177, B1 => n167, Y 
                           => byte_we_next_2_port);
   U178 : OAI32XL port map( A0 => n177, A1 => address_in(1), A2 => 
                           address_in(0), B0 => n178, B1 => n285, Y => 
                           byte_we_next_3_port);
   U179 : INVXL port map( A => n4, Y => address_next(2));
   U180 : INVXL port map( A => n5, Y => address_next(3));
   U181 : INVXL port map( A => n6, Y => address_next(4));
   U182 : INVXL port map( A => n7, Y => address_next(5));
   U183 : INVXL port map( A => n8, Y => address_next(6));
   U184 : INVXL port map( A => n9, Y => address_next(7));
   U185 : INVXL port map( A => n10, Y => address_next(8));
   U186 : INVXL port map( A => n11, Y => address_next(9));
   U187 : INVXL port map( A => n12, Y => address_next(10));
   U188 : INVXL port map( A => n13, Y => address_next(11));
   U189 : INVXL port map( A => n14, Y => address_next(12));
   U190 : INVXL port map( A => n15, Y => address_next(13));
   U191 : INVXL port map( A => n16, Y => address_next(14));
   U192 : INVXL port map( A => n17, Y => address_next(15));
   U193 : INVXL port map( A => n18, Y => address_next(16));
   U194 : INVXL port map( A => n19, Y => address_next(17));
   U195 : INVXL port map( A => n20, Y => address_next(18));
   U196 : INVXL port map( A => n21, Y => address_next(19));
   U197 : INVXL port map( A => n22, Y => address_next(20));
   U198 : INVXL port map( A => n23, Y => address_next(21));
   U199 : INVXL port map( A => n24, Y => address_next(22));
   U200 : INVXL port map( A => n25, Y => address_next(23));
   U201 : INVXL port map( A => n26, Y => address_next(24));
   U202 : INVXL port map( A => n27, Y => address_next(25));
   U203 : INVXL port map( A => n28, Y => address_next(26));
   U204 : INVXL port map( A => n29, Y => address_next(27));
   U205 : INVXL port map( A => n30, Y => address_next(28));
   U206 : INVXL port map( A => n31, Y => address_next(29));
   U207 : INVXL port map( A => n32, Y => address_next(30));
   U208 : INVXL port map( A => n33, Y => address_next(31));
   U209 : OAI2BB1XL port map( A0N => opcode_out_28_port, A1N => n172, B0 => n93
                           , Y => n268);
   U210 : AOI22XL port map( A0 => next_opcode_reg_28_port, A1 => n36, B0 => 
                           data_r(28), B1 => n280, Y => n93);
   U211 : OAI2BB1XL port map( A0N => opcode_out_29_port, A1N => n34, B0 => n95,
                           Y => n270);
   U212 : AOI22XL port map( A0 => next_opcode_reg_29_port, A1 => n36, B0 => 
                           data_r(29), B1 => n280, Y => n95);
   U213 : OAI2BB1XL port map( A0N => opcode_out_27_port, A1N => n155, B0 => n91
                           , Y => n266);
   U214 : AOI22XL port map( A0 => next_opcode_reg_27_port, A1 => n36, B0 => 
                           data_r(27), B1 => n280, Y => n91);
   U215 : OAI2BB1XL port map( A0N => opcode_out_30_port, A1N => n155, B0 => n97
                           , Y => n272);
   U216 : AOI22XL port map( A0 => next_opcode_reg_30_port, A1 => n36, B0 => 
                           data_r(30), B1 => n280, Y => n97);
   U217 : OAI2BB1XL port map( A0N => opcode_out_31_port, A1N => n172, B0 => n99
                           , Y => n274);
   U218 : AOI22XL port map( A0 => next_opcode_reg_31_port, A1 => n36, B0 => 
                           data_r(31), B1 => n280, Y => n99);
   U219 : OAI2BB1XL port map( A0N => opcode_out_4_port, A1N => n172, B0 => n45,
                           Y => n220);
   U220 : AOI22XL port map( A0 => next_opcode_reg_4_port, A1 => n36, B0 => 
                           data_r(4), B1 => n280, Y => n45);
   U221 : OAI2BB1XL port map( A0N => opcode_out_3_port, A1N => n155, B0 => n43,
                           Y => n218);
   U222 : AOI22XL port map( A0 => next_opcode_reg_3_port, A1 => n36, B0 => 
                           data_r(3), B1 => n282, Y => n43);
   U223 : OAI2BB1XL port map( A0N => opcode_out_5_port, A1N => n34, B0 => n47, 
                           Y => n222);
   U224 : AOI22XL port map( A0 => next_opcode_reg_5_port, A1 => n36, B0 => 
                           data_r(5), B1 => n280, Y => n47);
   U225 : OAI2BB1XL port map( A0N => opcode_out_26_port, A1N => n34, B0 => n89,
                           Y => n264);
   U226 : AOI22XL port map( A0 => next_opcode_reg_26_port, A1 => n36, B0 => 
                           data_r(26), B1 => n280, Y => n89);
   U227 : OAI2BB1XL port map( A0N => opcode_out_1_port, A1N => n172, B0 => n39,
                           Y => n214);
   U228 : AOI22XL port map( A0 => next_opcode_reg_1_port, A1 => n36, B0 => 
                           data_r(1), B1 => n280, Y => n39);
   U229 : OAI2BB1XL port map( A0N => opcode_out_2_port, A1N => n34, B0 => n41, 
                           Y => n216);
   U230 : AOI22XL port map( A0 => next_opcode_reg_2_port, A1 => n36, B0 => 
                           data_r(2), B1 => n282, Y => n41);
   U231 : OAI2BB1XL port map( A0N => opcode_out_0_port, A1N => n155, B0 => n35,
                           Y => n212);
   U232 : AOI22XL port map( A0 => next_opcode_reg_0_port, A1 => n36, B0 => 
                           data_r(0), B1 => n282, Y => n35);
   U233 : OAI2BB1XL port map( A0N => opcode_out_14_port, A1N => n34, B0 => n65,
                           Y => n240);
   U234 : AOI22XL port map( A0 => next_opcode_reg_14_port, A1 => n36, B0 => 
                           data_r(14), B1 => n282, Y => n65);
   U235 : OAI2BB1XL port map( A0N => opcode_out_11_port, A1N => n34, B0 => n59,
                           Y => n234);
   U236 : AOI22XL port map( A0 => next_opcode_reg_11_port, A1 => n36, B0 => 
                           data_r(11), B1 => n282, Y => n59);
   U237 : OAI2BB1XL port map( A0N => opcode_out_13_port, A1N => n172, B0 => n63
                           , Y => n238);
   U238 : AOI22XL port map( A0 => next_opcode_reg_13_port, A1 => n36, B0 => 
                           data_r(13), B1 => n282, Y => n63);
   U239 : OAI2BB1XL port map( A0N => opcode_out_23_port, A1N => n34, B0 => n83,
                           Y => n258);
   U240 : AOI22XL port map( A0 => next_opcode_reg_23_port, A1 => n36, B0 => 
                           data_r(23), B1 => n280, Y => n83);
   U241 : OAI2BB1XL port map( A0N => opcode_out_19_port, A1N => n172, B0 => n75
                           , Y => n250);
   U242 : AOI22XL port map( A0 => next_opcode_reg_19_port, A1 => n36, B0 => 
                           data_r(19), B1 => n282, Y => n75);
   U243 : OAI2BB1XL port map( A0N => opcode_out_20_port, A1N => n34, B0 => n77,
                           Y => n252);
   U244 : AOI22XL port map( A0 => next_opcode_reg_20_port, A1 => n36, B0 => 
                           data_r(20), B1 => n280, Y => n77);
   U245 : OAI2BB1XL port map( A0N => opcode_out_21_port, A1N => n155, B0 => n79
                           , Y => n254);
   U246 : AOI22XL port map( A0 => next_opcode_reg_21_port, A1 => n36, B0 => 
                           data_r(21), B1 => n280, Y => n79);
   U247 : OAI2BB1XL port map( A0N => opcode_out_15_port, A1N => n155, B0 => n67
                           , Y => n242);
   U248 : AOI22XL port map( A0 => next_opcode_reg_15_port, A1 => n36, B0 => 
                           data_r(15), B1 => n282, Y => n67);
   U249 : OAI2BB1XL port map( A0N => opcode_out_12_port, A1N => n155, B0 => n61
                           , Y => n236);
   U250 : AOI22XL port map( A0 => next_opcode_reg_12_port, A1 => n36, B0 => 
                           data_r(12), B1 => n282, Y => n61);
   U251 : OAI2BB1XL port map( A0N => opcode_out_16_port, A1N => n172, B0 => n69
                           , Y => n244);
   U252 : AOI22XL port map( A0 => next_opcode_reg_16_port, A1 => n36, B0 => 
                           data_r(16), B1 => n282, Y => n69);
   U253 : OAI2BB1XL port map( A0N => opcode_out_18_port, A1N => n155, B0 => n73
                           , Y => n248);
   U254 : AOI22XL port map( A0 => next_opcode_reg_18_port, A1 => n36, B0 => 
                           data_r(18), B1 => n282, Y => n73);
   U255 : OAI2BB1XL port map( A0N => opcode_out_17_port, A1N => n34, B0 => n71,
                           Y => n246);
   U256 : AOI22XL port map( A0 => next_opcode_reg_17_port, A1 => n36, B0 => 
                           data_r(17), B1 => n282, Y => n71);
   U257 : OAI2BB1XL port map( A0N => opcode_out_22_port, A1N => n172, B0 => n81
                           , Y => n256);
   U258 : AOI22XL port map( A0 => next_opcode_reg_22_port, A1 => n36, B0 => 
                           data_r(22), B1 => n280, Y => n81);
   U259 : OAI2BB1XL port map( A0N => opcode_out_24_port, A1N => n155, B0 => n85
                           , Y => n260);
   U260 : AOI22XL port map( A0 => next_opcode_reg_24_port, A1 => n36, B0 => 
                           data_r(24), B1 => n280, Y => n85);
   U261 : OAI2BB1XL port map( A0N => opcode_out_25_port, A1N => n172, B0 => n87
                           , Y => n262);
   U262 : AOI22XL port map( A0 => next_opcode_reg_25_port, A1 => n36, B0 => 
                           data_r(25), B1 => n280, Y => n87);
   U263 : OAI2BB1XL port map( A0N => opcode_out_6_port, A1N => n155, B0 => n49,
                           Y => n224);
   U264 : AOI22XL port map( A0 => next_opcode_reg_6_port, A1 => n36, B0 => 
                           data_r(6), B1 => n282, Y => n49);
   U265 : OAI2BB1XL port map( A0N => opcode_out_7_port, A1N => n172, B0 => n51,
                           Y => n226);
   U266 : AOI22XL port map( A0 => next_opcode_reg_7_port, A1 => n36, B0 => 
                           data_r(7), B1 => n280, Y => n51);
   U267 : OAI2BB1XL port map( A0N => opcode_out_8_port, A1N => n34, B0 => n53, 
                           Y => n228);
   U268 : AOI22XL port map( A0 => next_opcode_reg_8_port, A1 => n36, B0 => 
                           data_r(8), B1 => n282, Y => n53);
   U269 : OAI2BB1XL port map( A0N => opcode_out_9_port, A1N => n155, B0 => n55,
                           Y => n230);
   U270 : AOI22XL port map( A0 => next_opcode_reg_9_port, A1 => n36, B0 => 
                           data_r(9), B1 => n282, Y => n55);
   U271 : OAI2BB1XL port map( A0N => opcode_out_10_port, A1N => n172, B0 => n57
                           , Y => n232);
   U272 : AOI22XL port map( A0 => next_opcode_reg_10_port, A1 => n36, B0 => 
                           data_r(10), B1 => n282, Y => n57);
   U273 : OAI2BB2XL port map( B0 => n283, B1 => n38, A0N => n285, A1N => 
                           next_opcode_reg_0_port, Y => n213);
   U274 : OAI2BB2XL port map( B0 => n283, B1 => n40, A0N => n285, A1N => 
                           next_opcode_reg_1_port, Y => n215);
   U275 : OAI2BB2XL port map( B0 => n283, B1 => n42, A0N => n285, A1N => 
                           next_opcode_reg_2_port, Y => n217);
   U276 : OAI2BB2XL port map( B0 => n283, B1 => n44, A0N => n285, A1N => 
                           next_opcode_reg_3_port, Y => n219);
   U277 : OAI2BB2XL port map( B0 => n283, B1 => n46, A0N => n285, A1N => 
                           next_opcode_reg_4_port, Y => n221);
   U278 : OAI2BB2XL port map( B0 => n283, B1 => n48, A0N => n285, A1N => 
                           next_opcode_reg_5_port, Y => n223);
   U279 : OAI2BB2XL port map( B0 => n283, B1 => n50, A0N => n285, A1N => 
                           next_opcode_reg_6_port, Y => n225);
   U280 : OAI2BB2XL port map( B0 => n283, B1 => n52, A0N => n285, A1N => 
                           next_opcode_reg_7_port, Y => n227);
   U281 : OAI2BB2XL port map( B0 => n283, B1 => n54, A0N => n285, A1N => 
                           next_opcode_reg_8_port, Y => n229);
   U282 : OAI2BB2XL port map( B0 => n283, B1 => n56, A0N => n285, A1N => 
                           next_opcode_reg_9_port, Y => n231);
   U283 : OAI2BB2XL port map( B0 => n283, B1 => n58, A0N => n285, A1N => 
                           next_opcode_reg_10_port, Y => n233);
   U284 : OAI2BB2XL port map( B0 => n283, B1 => n60, A0N => n285, A1N => 
                           next_opcode_reg_11_port, Y => n235);
   U285 : OAI2BB2XL port map( B0 => n283, B1 => n62, A0N => n285, A1N => 
                           next_opcode_reg_12_port, Y => n237);
   U286 : OAI2BB2XL port map( B0 => n283, B1 => n64, A0N => n285, A1N => 
                           next_opcode_reg_13_port, Y => n239);
   U287 : OAI2BB2XL port map( B0 => n283, B1 => n66, A0N => n285, A1N => 
                           next_opcode_reg_14_port, Y => n241);
   U288 : OAI2BB2XL port map( B0 => n283, B1 => n68, A0N => n285, A1N => 
                           next_opcode_reg_15_port, Y => n243);
   U289 : OAI2BB2XL port map( B0 => n283, B1 => n70, A0N => n285, A1N => 
                           next_opcode_reg_16_port, Y => n245);
   U290 : OAI2BB2XL port map( B0 => n283, B1 => n72, A0N => n285, A1N => 
                           next_opcode_reg_17_port, Y => n247);
   U291 : OAI2BB2XL port map( B0 => n283, B1 => n74, A0N => n285, A1N => 
                           next_opcode_reg_18_port, Y => n249);
   U292 : OAI2BB2XL port map( B0 => n283, B1 => n76, A0N => n284, A1N => 
                           next_opcode_reg_19_port, Y => n251);
   U293 : OAI2BB2XL port map( B0 => n283, B1 => n78, A0N => n284, A1N => 
                           next_opcode_reg_20_port, Y => n253);
   U294 : OAI2BB2XL port map( B0 => n283, B1 => n80, A0N => n284, A1N => 
                           next_opcode_reg_21_port, Y => n255);
   U295 : OAI2BB2XL port map( B0 => n283, B1 => n82, A0N => n285, A1N => 
                           next_opcode_reg_22_port, Y => n257);
   U296 : OAI2BB2XL port map( B0 => n283, B1 => n84, A0N => n284, A1N => 
                           next_opcode_reg_23_port, Y => n259);
   U297 : OAI2BB2XL port map( B0 => n284, B1 => n86, A0N => n285, A1N => 
                           next_opcode_reg_24_port, Y => n261);
   U298 : OAI2BB2XL port map( B0 => n284, B1 => n88, A0N => n285, A1N => 
                           next_opcode_reg_25_port, Y => n263);
   U299 : OAI2BB2XL port map( B0 => n284, B1 => n90, A0N => n285, A1N => 
                           next_opcode_reg_26_port, Y => n265);
   U300 : OAI2BB2XL port map( B0 => n284, B1 => n92, A0N => n285, A1N => 
                           next_opcode_reg_27_port, Y => n267);
   U301 : OAI2BB2XL port map( B0 => n284, B1 => n94, A0N => n285, A1N => 
                           next_opcode_reg_28_port, Y => n269);
   U302 : OAI2BB2XL port map( B0 => n284, B1 => n96, A0N => n285, A1N => 
                           next_opcode_reg_29_port, Y => n271);
   U303 : OAI2BB2XL port map( B0 => n283, B1 => n98, A0N => n285, A1N => 
                           next_opcode_reg_30_port, Y => n273);
   U304 : OAI2BB2XL port map( B0 => n283, B1 => n103, A0N => n285, A1N => 
                           next_opcode_reg_31_port, Y => n275);
   U305 : INVXL port map( A => n104, Y => n276);
   U306 : AOI21XL port map( A0 => byte_we_0_port, A1 => n288, B0 => 
                           byte_we_next_0_port, Y => n104);
   U307 : INVXL port map( A => n105, Y => n277);
   U308 : AOI21XL port map( A0 => byte_we_1_port, A1 => n288, B0 => 
                           byte_we_next_1_port, Y => n105);
   U309 : INVXL port map( A => n106, Y => n278);
   U310 : AOI21XL port map( A0 => byte_we_2_port, A1 => n288, B0 => 
                           byte_we_next_2_port, Y => n106);
   U311 : INVXL port map( A => n107, Y => n279);
   U312 : AOI21XL port map( A0 => byte_we_3_port, A1 => n288, B0 => 
                           byte_we_next_3_port, Y => n107);
   U313 : OAI2BB2XL port map( B0 => n288, B1 => n4, A0N => address_2_port, A1N 
                           => n288, Y => n182);
   U314 : OAI2BB2XL port map( B0 => n288, B1 => n5, A0N => address_3_port, A1N 
                           => n288, Y => n183);
   U315 : OAI2BB2XL port map( B0 => n288, B1 => n6, A0N => address_4_port, A1N 
                           => n288, Y => n184);
   U316 : OAI2BB2XL port map( B0 => n288, B1 => n7, A0N => address_5_port, A1N 
                           => n288, Y => n185);
   U317 : OAI2BB2XL port map( B0 => n288, B1 => n8, A0N => address_6_port, A1N 
                           => n288, Y => n186);
   U318 : OAI2BB2XL port map( B0 => n288, B1 => n9, A0N => address_7_port, A1N 
                           => n288, Y => n187);
   U319 : OAI2BB2XL port map( B0 => n288, B1 => n10, A0N => address_8_port, A1N
                           => n288, Y => n188);
   U320 : OAI2BB2XL port map( B0 => n288, B1 => n11, A0N => address_9_port, A1N
                           => n288, Y => n189);
   U321 : OAI2BB2XL port map( B0 => n288, B1 => n12, A0N => address_10_port, 
                           A1N => n288, Y => n190);
   U322 : OAI2BB2XL port map( B0 => n288, B1 => n13, A0N => address_11_port, 
                           A1N => n288, Y => n191);
   U323 : OAI2BB2XL port map( B0 => pause_in, B1 => n14, A0N => address_12_port
                           , A1N => n288, Y => n192);
   U324 : OAI2BB2XL port map( B0 => pause_in, B1 => n15, A0N => address_13_port
                           , A1N => n288, Y => n193);
   U325 : INVXL port map( A => reset_in, Y => n286);
   U326 : OAI2BB2XL port map( B0 => pause_in, B1 => n16, A0N => address_14_port
                           , A1N => n288, Y => n194);
   U327 : OAI2BB2XL port map( B0 => n288, B1 => n17, A0N => address_15_port, 
                           A1N => n288, Y => n195);
   U328 : OAI2BB2XL port map( B0 => n288, B1 => n18, A0N => address_16_port, 
                           A1N => n288, Y => n196);
   U329 : OAI2BB2XL port map( B0 => n288, B1 => n19, A0N => address_17_port, 
                           A1N => n288, Y => n197);
   U330 : OAI2BB2XL port map( B0 => n288, B1 => n20, A0N => address_18_port, 
                           A1N => n288, Y => n198);
   U331 : OAI2BB2XL port map( B0 => pause_in, B1 => n21, A0N => address_19_port
                           , A1N => n288, Y => n199);
   U332 : OAI2BB2XL port map( B0 => n288, B1 => n22, A0N => address_20_port, 
                           A1N => n288, Y => n200);
   U333 : OAI2BB2XL port map( B0 => pause_in, B1 => n23, A0N => address_21_port
                           , A1N => n288, Y => n201);
   U334 : OAI2BB2XL port map( B0 => pause_in, B1 => n24, A0N => address_22_port
                           , A1N => n288, Y => n202);
   U335 : OAI2BB2XL port map( B0 => n288, B1 => n25, A0N => address_23_port, 
                           A1N => n288, Y => n203);
   U336 : OAI2BB2XL port map( B0 => pause_in, B1 => n26, A0N => address_24_port
                           , A1N => n288, Y => n204);
   U337 : OAI2BB2XL port map( B0 => pause_in, B1 => n27, A0N => address_25_port
                           , A1N => n288, Y => n205);
   U338 : OAI2BB2XL port map( B0 => n288, B1 => n28, A0N => address_26_port, 
                           A1N => n288, Y => n206);
   U339 : INVXL port map( A => reset_in, Y => n287);
   U340 : OAI2BB2XL port map( B0 => n288, B1 => n29, A0N => address_27_port, 
                           A1N => n288, Y => n207);
   U341 : OAI2BB2XL port map( B0 => pause_in, B1 => n30, A0N => address_28_port
                           , A1N => n288, Y => n208);
   U342 : OAI2BB2XL port map( B0 => n288, B1 => n31, A0N => address_29_port, 
                           A1N => n288, Y => n209);
   U343 : OAI2BB2XL port map( B0 => pause_in, B1 => n32, A0N => address_30_port
                           , A1N => n288, Y => n210);
   U344 : OAI2BB2XL port map( B0 => n288, B1 => n33, A0N => address_31_port, 
                           A1N => n288, Y => n211);
   U345 : OAI21XL port map( A0 => n167, A1 => n169, B0 => n142, Y => n170);
   U346 : OAI21XL port map( A0 => n164, A1 => n169, B0 => n141, Y => n171);
   U347 : OAI221XL port map( A0 => n52, A1 => n164, B0 => n68, B1 => n165, C0 
                           => n166, Y => n163);
   U348 : OAI21XL port map( A0 => n72, A1 => n37, B0 => n156, Y => 
                           data_read(17));
   U349 : OAI21XL port map( A0 => n74, A1 => n37, B0 => n156, Y => 
                           data_read(18));
   U350 : OAI21XL port map( A0 => n76, A1 => n37, B0 => n156, Y => 
                           data_read(19));
   U351 : OAI21XL port map( A0 => n78, A1 => n37, B0 => n156, Y => 
                           data_read(20));
   U352 : OAI21XL port map( A0 => n80, A1 => n37, B0 => n156, Y => 
                           data_read(21));
   U353 : OAI21XL port map( A0 => n82, A1 => n37, B0 => n156, Y => 
                           data_read(22));
   U354 : OAI21XL port map( A0 => n84, A1 => n37, B0 => n156, Y => 
                           data_read(23));
   U355 : OAI21XL port map( A0 => n86, A1 => n37, B0 => n156, Y => 
                           data_read(24));
   U356 : OAI21XL port map( A0 => n88, A1 => n37, B0 => n156, Y => 
                           data_read(25));
   U357 : OAI21XL port map( A0 => n90, A1 => n37, B0 => n156, Y => 
                           data_read(26));
   U358 : OAI21XL port map( A0 => n92, A1 => n37, B0 => n156, Y => 
                           data_read(27));
   U359 : OAI21XL port map( A0 => n94, A1 => n37, B0 => n156, Y => 
                           data_read(28));
   U360 : OAI21XL port map( A0 => n96, A1 => n37, B0 => n156, Y => 
                           data_read(29));
   U361 : OAI21XL port map( A0 => n98, A1 => n37, B0 => n156, Y => 
                           data_read(30));
   U362 : OAI21XL port map( A0 => n103, A1 => n37, B0 => n156, Y => 
                           data_read(31));
   U363 : OAI21XL port map( A0 => n70, A1 => n37, B0 => n156, Y => 
                           data_read(16));
   U364 : OAI221XL port map( A0 => n141, A1 => n54, B0 => n86, B1 => n142, C0 
                           => n143, Y => data_read(8));
   U365 : OAI221XL port map( A0 => n141, A1 => n56, B0 => n88, B1 => n142, C0 
                           => n143, Y => data_read(9));
   U366 : OAI221XL port map( A0 => n141, A1 => n58, B0 => n90, B1 => n142, C0 
                           => n143, Y => data_read(10));
   U367 : OAI221XL port map( A0 => n141, A1 => n60, B0 => n92, B1 => n142, C0 
                           => n143, Y => data_read(11));
   U368 : OAI221XL port map( A0 => n141, A1 => n62, B0 => n94, B1 => n142, C0 
                           => n143, Y => data_read(12));
   U369 : OAI221XL port map( A0 => n141, A1 => n64, B0 => n96, B1 => n142, C0 
                           => n143, Y => data_read(13));
   U370 : OAI221XL port map( A0 => n141, A1 => n66, B0 => n98, B1 => n142, C0 
                           => n143, Y => data_read(14));
   U371 : OAI221XL port map( A0 => n141, A1 => n68, B0 => n103, B1 => n142, C0 
                           => n143, Y => data_read(15));
   U372 : OR3XL port map( A => nullify_op, B => n288, C => n100, Y => n121);
   U373 : OAI21XL port map( A0 => n101, A1 => n289, B0 => n284, Y => n181);
   U374 : OAI21XL port map( A0 => n101, A1 => n289, B0 => n108, Y => n281);
   U375 : OAI21XL port map( A0 => n111, A1 => n113, B0 => n138, Y => data_w(24)
                           );
   U376 : OAI21XL port map( A0 => n111, A1 => n110, B0 => n137, Y => data_w(25)
                           );
   U377 : OAI21XL port map( A0 => n111, A1 => n135, B0 => n136, Y => data_w(26)
                           );
   U378 : OAI21XL port map( A0 => n111, A1 => n133, B0 => n134, Y => data_w(27)
                           );
   U379 : OAI21XL port map( A0 => n111, A1 => n131, B0 => n132, Y => data_w(28)
                           );
   U380 : OAI21XL port map( A0 => n111, A1 => n129, B0 => n130, Y => data_w(29)
                           );
   U381 : OAI21XL port map( A0 => n111, A1 => n126, B0 => n127, Y => data_w(30)
                           );
   U382 : OAI21XL port map( A0 => n111, A1 => n122, B0 => n123, Y => data_w(31)
                           );
   U383 : OAI221XL port map( A0 => n144, A1 => n38, B0 => n149, B1 => n70, C0 
                           => n168, Y => data_read(0));
   U384 : OAI221XL port map( A0 => n144, A1 => n40, B0 => n149, B1 => n72, C0 
                           => n158, Y => data_read(1));
   U385 : OAI221XL port map( A0 => n144, A1 => n42, B0 => n149, B1 => n74, C0 
                           => n157, Y => data_read(2));
   U386 : OAI221XL port map( A0 => n144, A1 => n44, B0 => n149, B1 => n76, C0 
                           => n154, Y => data_read(3));
   U387 : OAI221XL port map( A0 => n144, A1 => n46, B0 => n149, B1 => n78, C0 
                           => n153, Y => data_read(4));
   U388 : OAI221XL port map( A0 => n144, A1 => n48, B0 => n149, B1 => n80, C0 
                           => n152, Y => data_read(5));
   U389 : OAI221XL port map( A0 => n144, A1 => n50, B0 => n149, B1 => n82, C0 
                           => n150, Y => data_read(6));
   U390 : OAI221XL port map( A0 => n144, A1 => n52, B0 => n84, B1 => n142, C0 
                           => n145, Y => data_read(7));

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity pc_next is

   port( clk, reset_in : in std_logic;  pc_new : in std_logic_vector (31 downto
         2);  take_branch, pause_in : in std_logic;  opcode25_0 : in 
         std_logic_vector (25 downto 0);  pc_source : in std_logic_vector (1 
         downto 0);  pc_future, pc_current, pc_plus4 : out std_logic_vector (31
         downto 2));

end pc_next;

architecture SYN_logic of pc_next is

   component OAI2BB2X1
      port( B0, B1, A0N, A1N : in std_logic;  Y : out std_logic);
   end component;
   
   component OR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1XL
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI222XL
      port( A0, A1, B0, B1, C0, C1 : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component XNOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component XOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB2XL
      port( B0, B1, A0N, A1N : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI22XL
      port( A0, A1, B0, B1 : in std_logic;  Y : out std_logic);
   end component;
   
   component OR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI2BB2XL
      port( B0, B1, A0N, A1N : in std_logic;  Y : out std_logic);
   end component;
   
   component DFFRHQXL
      port( D, CK, RN : in std_logic;  Q : out std_logic);
   end component;
   
   signal pc_current_31_port, pc_current_30_port, pc_current_29_port, 
      pc_current_28_port, pc_current_27_port, pc_current_26_port, 
      pc_current_25_port, pc_current_24_port, pc_current_23_port, 
      pc_current_22_port, pc_current_21_port, pc_current_20_port, 
      pc_current_19_port, pc_current_18_port, pc_current_17_port, 
      pc_current_16_port, pc_current_15_port, pc_current_14_port, 
      pc_current_13_port, pc_current_12_port, pc_current_11_port, 
      pc_current_10_port, pc_current_9_port, pc_current_8_port, 
      pc_current_7_port, pc_current_6_port, pc_current_5_port, 
      pc_current_4_port, pc_current_3_port, pc_current_2_port, pc_plus4_31_port
      , pc_plus4_30_port, pc_plus4_29_port, pc_plus4_28_port, pc_plus4_27_port,
      pc_plus4_26_port, pc_plus4_25_port, pc_plus4_24_port, pc_plus4_23_port, 
      pc_plus4_22_port, pc_plus4_21_port, pc_plus4_20_port, pc_plus4_19_port, 
      pc_plus4_18_port, pc_plus4_17_port, pc_plus4_16_port, pc_plus4_15_port, 
      pc_plus4_14_port, pc_plus4_13_port, pc_plus4_12_port, pc_plus4_11_port, 
      pc_plus4_10_port, pc_plus4_9_port, pc_plus4_8_port, pc_plus4_7_port, 
      pc_plus4_6_port, pc_plus4_5_port, pc_plus4_4_port, pc_plus4_3_port, 
      pc_plus4_2_port, N102, N103, N104, N105, N106, N107, N108, N109, N110, 
      N111, N112, N113, N114, N115, N116, N117, N118, N119, N120, N121, N122, 
      N123, N124, N125, N126, N127, N128, N129, N130, N131, n2, n4, n5, n6, n7,
      n8, n9, n10, n11, n12, n13, n14, n15, n16, n17, n18, n19, n20, n21, n22, 
      n23, n24, n25, n26, n27, n28, n29, n30, n31, n32, n34, n36, n37, n38, n39
      , n40, n41, n42, n43, n44, n46, n47, n48, n49, n50, n51, n52, n53, n54, 
      n55, n56, n57, n58, n59, n60, n61, n62, n63, n64, n65, n66, n67, n68, n69
      , n70, n71, n72, n73, n74, n75, n76, n77, n78, n79, n80, n81, n82, n83, 
      n84, n85, n86, n87, n88, n89, n90, n91, n92, n93, n94, n95, n96, n97, n98
      , n99, n100, n101, n1020, n1030, n1040, n1050, n1060, n1070, n1080, n1090
      , n1100, n1110, n1120, n1130, n1140, n1150, n1160, n1170, n1180, n1190, 
      n1200, n1210, n1, n3, n33, n35, n45, n1220, n1230 : std_logic;

begin
   pc_current <= ( pc_current_31_port, pc_current_30_port, pc_current_29_port, 
      pc_current_28_port, pc_current_27_port, pc_current_26_port, 
      pc_current_25_port, pc_current_24_port, pc_current_23_port, 
      pc_current_22_port, pc_current_21_port, pc_current_20_port, 
      pc_current_19_port, pc_current_18_port, pc_current_17_port, 
      pc_current_16_port, pc_current_15_port, pc_current_14_port, 
      pc_current_13_port, pc_current_12_port, pc_current_11_port, 
      pc_current_10_port, pc_current_9_port, pc_current_8_port, 
      pc_current_7_port, pc_current_6_port, pc_current_5_port, 
      pc_current_4_port, pc_current_3_port, pc_current_2_port );
   pc_plus4 <= ( pc_plus4_31_port, pc_plus4_30_port, pc_plus4_29_port, 
      pc_plus4_28_port, pc_plus4_27_port, pc_plus4_26_port, pc_plus4_25_port, 
      pc_plus4_24_port, pc_plus4_23_port, pc_plus4_22_port, pc_plus4_21_port, 
      pc_plus4_20_port, pc_plus4_19_port, pc_plus4_18_port, pc_plus4_17_port, 
      pc_plus4_16_port, pc_plus4_15_port, pc_plus4_14_port, pc_plus4_13_port, 
      pc_plus4_12_port, pc_plus4_11_port, pc_plus4_10_port, pc_plus4_9_port, 
      pc_plus4_8_port, pc_plus4_7_port, pc_plus4_6_port, pc_plus4_5_port, 
      pc_plus4_4_port, pc_plus4_3_port, pc_plus4_2_port );
   
   pc_reg_regx31x : DFFRHQXL port map( D => N131, CK => clk, RN => n1230, Q => 
                           pc_current_31_port);
   pc_reg_regx29x : DFFRHQXL port map( D => N129, CK => clk, RN => n1230, Q => 
                           pc_current_29_port);
   pc_reg_regx26x : DFFRHQXL port map( D => N126, CK => clk, RN => n1230, Q => 
                           pc_current_26_port);
   pc_reg_regx28x : DFFRHQXL port map( D => N128, CK => clk, RN => n1230, Q => 
                           pc_current_28_port);
   pc_reg_regx25x : DFFRHQXL port map( D => N125, CK => clk, RN => n1230, Q => 
                           pc_current_25_port);
   pc_reg_regx30x : DFFRHQXL port map( D => N130, CK => clk, RN => n1230, Q => 
                           pc_current_30_port);
   pc_reg_regx27x : DFFRHQXL port map( D => N127, CK => clk, RN => n1230, Q => 
                           pc_current_27_port);
   pc_reg_regx18x : DFFRHQXL port map( D => N118, CK => clk, RN => n1230, Q => 
                           pc_current_18_port);
   pc_reg_regx22x : DFFRHQXL port map( D => N122, CK => clk, RN => n1230, Q => 
                           pc_current_22_port);
   pc_reg_regx21x : DFFRHQXL port map( D => N121, CK => clk, RN => n1230, Q => 
                           pc_current_21_port);
   pc_reg_regx20x : DFFRHQXL port map( D => N120, CK => clk, RN => n1220, Q => 
                           pc_current_20_port);
   pc_reg_regx24x : DFFRHQXL port map( D => N124, CK => clk, RN => n1230, Q => 
                           pc_current_24_port);
   pc_reg_regx19x : DFFRHQXL port map( D => N119, CK => clk, RN => n1230, Q => 
                           pc_current_19_port);
   pc_reg_regx23x : DFFRHQXL port map( D => N123, CK => clk, RN => n1230, Q => 
                           pc_current_23_port);
   pc_reg_regx14x : DFFRHQXL port map( D => N114, CK => clk, RN => n1230, Q => 
                           pc_current_14_port);
   pc_reg_regx13x : DFFRHQXL port map( D => N113, CK => clk, RN => n1220, Q => 
                           pc_current_13_port);
   pc_reg_regx17x : DFFRHQXL port map( D => N117, CK => clk, RN => n1230, Q => 
                           pc_current_17_port);
   pc_reg_regx12x : DFFRHQXL port map( D => N112, CK => clk, RN => n1220, Q => 
                           pc_current_12_port);
   pc_reg_regx16x : DFFRHQXL port map( D => N116, CK => clk, RN => n1230, Q => 
                           pc_current_16_port);
   pc_reg_regx11x : DFFRHQXL port map( D => N111, CK => clk, RN => n1230, Q => 
                           pc_current_11_port);
   pc_reg_regx15x : DFFRHQXL port map( D => N115, CK => clk, RN => n1230, Q => 
                           pc_current_15_port);
   pc_reg_regx6x : DFFRHQXL port map( D => N106, CK => clk, RN => n1220, Q => 
                           pc_current_6_port);
   pc_reg_regx10x : DFFRHQXL port map( D => N110, CK => clk, RN => n1220, Q => 
                           pc_current_10_port);
   pc_reg_regx2x : DFFRHQXL port map( D => N102, CK => clk, RN => n1230, Q => 
                           pc_current_2_port);
   pc_reg_regx4x : DFFRHQXL port map( D => N104, CK => clk, RN => n1230, Q => 
                           pc_current_4_port);
   pc_reg_regx5x : DFFRHQXL port map( D => N105, CK => clk, RN => n1220, Q => 
                           pc_current_5_port);
   pc_reg_regx9x : DFFRHQXL port map( D => N109, CK => clk, RN => n1230, Q => 
                           pc_current_9_port);
   pc_reg_regx3x : DFFRHQXL port map( D => N103, CK => clk, RN => n1230, Q => 
                           pc_current_3_port);
   pc_reg_regx8x : DFFRHQXL port map( D => N108, CK => clk, RN => n1230, Q => 
                           pc_current_8_port);
   pc_reg_regx7x : DFFRHQXL port map( D => N107, CK => clk, RN => n1230, Q => 
                           pc_current_7_port);
   U3 : AOI2BB2XL port map( B0 => pc_source(1), B1 => n1200, A0N => 
                           pc_source(0), A1N => pc_source(1), Y => n1190);
   U4 : NOR2XL port map( A => pause_in, B => n1190, Y => n1);
   U5 : INVXL port map( A => pc_source(1), Y => n1210);
   U6 : INVXL port map( A => take_branch, Y => n1200);
   U7 : NAND3XL port map( A => pc_current_27_port, B => n40, C => 
                           pc_current_28_port, Y => n39);
   U8 : INVXL port map( A => reset_in, Y => n1220);
   U9 : INVXL port map( A => pc_current_29_port, Y => n12);
   U10 : NOR3XL port map( A => n7, B => n1060, C => n6, Y => n100);
   U11 : NAND3XL port map( A => pc_current_7_port, B => n100, C => 
                           pc_current_8_port, Y => n94);
   U12 : INVXL port map( A => pc_current_9_port, Y => n2);
   U13 : INVXL port map( A => pc_current_10_port, Y => n32);
   U14 : NAND3XL port map( A => pc_current_3_port, B => pc_current_2_port, C =>
                           pc_current_4_port, Y => n1060);
   U15 : INVXL port map( A => pc_current_5_port, Y => n7);
   U16 : INVXL port map( A => pc_current_6_port, Y => n6);
   U17 : NOR3XL port map( A => n29, B => n82, C => n28, Y => n76);
   U18 : NOR3XL port map( A => n94, B => n2, C => n32, Y => n88);
   U19 : NAND3XL port map( A => pc_current_11_port, B => n88, C => 
                           pc_current_12_port, Y => n82);
   U20 : INVXL port map( A => pc_current_13_port, Y => n29);
   U21 : INVXL port map( A => pc_current_14_port, Y => n28);
   U22 : NOR3XL port map( A => n21, B => n58, C => n20, Y => n52);
   U23 : NOR3XL port map( A => n25, B => n70, C => n24, Y => n64);
   U24 : NAND3XL port map( A => pc_current_19_port, B => n64, C => 
                           pc_current_20_port, Y => n58);
   U25 : INVXL port map( A => pc_current_21_port, Y => n21);
   U26 : INVXL port map( A => pc_current_22_port, Y => n20);
   U27 : NAND3XL port map( A => pc_current_15_port, B => n76, C => 
                           pc_current_16_port, Y => n70);
   U28 : INVXL port map( A => pc_current_17_port, Y => n25);
   U29 : INVXL port map( A => pc_current_18_port, Y => n24);
   U30 : NOR3XL port map( A => n17, B => n46, C => n16, Y => n40);
   U31 : NOR2XL port map( A => n12, B => n39, Y => n37);
   U32 : NAND2XL port map( A => pc_source(0), B => n1210, Y => n41);
   U33 : XNOR2XL port map( A => n42, B => pc_current_28_port, Y => 
                           pc_plus4_28_port);
   U34 : NAND2XL port map( A => pc_current_27_port, B => n40, Y => n42);
   U35 : NAND3XL port map( A => pc_current_23_port, B => n52, C => 
                           pc_current_24_port, Y => n46);
   U36 : INVXL port map( A => pc_current_25_port, Y => n17);
   U37 : INVXL port map( A => pc_current_26_port, Y => n16);
   U38 : OR2XL port map( A => n41, B => pause_in, Y => n3);
   U39 : XOR2XL port map( A => n39, B => n12, Y => pc_plus4_29_port);
   U40 : INVXL port map( A => reset_in, Y => n1230);
   U41 : XNOR2XL port map( A => n36, B => pc_current_31_port, Y => 
                           pc_plus4_31_port);
   U42 : NAND2XL port map( A => pc_current_30_port, B => n37, Y => n36);
   U43 : INVXL port map( A => n9, Y => pc_future(3));
   U44 : AOI22XL port map( A0 => pc_current_3_port, A1 => reset_in, B0 => n1230
                           , B1 => N103, Y => n9);
   U45 : INVXL port map( A => n8, Y => pc_future(4));
   U46 : AOI22XL port map( A0 => pc_current_4_port, A1 => reset_in, B0 => n1220
                           , B1 => N104, Y => n8);
   U47 : OAI2BB2XL port map( B0 => n7, B1 => n1220, A0N => n1230, A1N => N105, 
                           Y => pc_future(5));
   U48 : OAI2BB2XL port map( B0 => n6, B1 => n1220, A0N => n1230, A1N => N106, 
                           Y => pc_future(6));
   U49 : INVXL port map( A => n5, Y => pc_future(7));
   U50 : AOI22XL port map( A0 => pc_current_7_port, A1 => reset_in, B0 => n1230
                           , B1 => N107, Y => n5);
   U51 : INVXL port map( A => n4, Y => pc_future(8));
   U52 : AOI22XL port map( A0 => pc_current_8_port, A1 => reset_in, B0 => n1230
                           , B1 => N108, Y => n4);
   U53 : OAI2BB2XL port map( B0 => n2, B1 => n1220, A0N => n1230, A1N => N109, 
                           Y => pc_future(9));
   U54 : OAI2BB2XL port map( B0 => n32, B1 => n1220, A0N => n1230, A1N => N110,
                           Y => pc_future(10));
   U55 : INVXL port map( A => n31, Y => pc_future(11));
   U56 : AOI22XL port map( A0 => pc_current_11_port, A1 => reset_in, B0 => 
                           n1230, B1 => N111, Y => n31);
   U57 : INVXL port map( A => n30, Y => pc_future(12));
   U58 : AOI22XL port map( A0 => pc_current_12_port, A1 => reset_in, B0 => 
                           n1230, B1 => N112, Y => n30);
   U59 : OAI2BB2XL port map( B0 => n29, B1 => n1230, A0N => n1230, A1N => N113,
                           Y => pc_future(13));
   U60 : OAI2BB2XL port map( B0 => n28, B1 => n1220, A0N => n1230, A1N => N114,
                           Y => pc_future(14));
   U61 : INVXL port map( A => n27, Y => pc_future(15));
   U62 : AOI22XL port map( A0 => pc_current_15_port, A1 => reset_in, B0 => 
                           n1220, B1 => N115, Y => n27);
   U63 : INVXL port map( A => n26, Y => pc_future(16));
   U64 : AOI22XL port map( A0 => pc_current_16_port, A1 => reset_in, B0 => 
                           n1230, B1 => N116, Y => n26);
   U65 : OAI2BB2XL port map( B0 => n25, B1 => n1220, A0N => n1230, A1N => N117,
                           Y => pc_future(17));
   U66 : OAI2BB2XL port map( B0 => n24, B1 => n1220, A0N => n1230, A1N => N118,
                           Y => pc_future(18));
   U67 : INVXL port map( A => n23, Y => pc_future(19));
   U68 : AOI22XL port map( A0 => pc_current_19_port, A1 => reset_in, B0 => 
                           n1230, B1 => N119, Y => n23);
   U69 : INVXL port map( A => n22, Y => pc_future(20));
   U70 : AOI22XL port map( A0 => pc_current_20_port, A1 => reset_in, B0 => 
                           n1220, B1 => N120, Y => n22);
   U71 : OAI2BB2XL port map( B0 => n21, B1 => n1220, A0N => n1230, A1N => N121,
                           Y => pc_future(21));
   U72 : OAI2BB2XL port map( B0 => n20, B1 => n1220, A0N => n1230, A1N => N122,
                           Y => pc_future(22));
   U73 : INVXL port map( A => n19, Y => pc_future(23));
   U74 : AOI22XL port map( A0 => pc_current_23_port, A1 => reset_in, B0 => 
                           n1230, B1 => N123, Y => n19);
   U75 : INVXL port map( A => n18, Y => pc_future(24));
   U76 : AOI22XL port map( A0 => pc_current_24_port, A1 => reset_in, B0 => 
                           n1230, B1 => N124, Y => n18);
   U77 : OAI2BB2XL port map( B0 => n17, B1 => n1220, A0N => n1230, A1N => N125,
                           Y => pc_future(25));
   U78 : OAI2BB2XL port map( B0 => n16, B1 => n1220, A0N => n1230, A1N => N126,
                           Y => pc_future(26));
   U79 : INVXL port map( A => n15, Y => pc_future(27));
   U80 : AOI22XL port map( A0 => pc_current_27_port, A1 => reset_in, B0 => 
                           n1230, B1 => N127, Y => n15);
   U81 : OAI2BB2XL port map( B0 => reset_in, B1 => n14, A0N => 
                           pc_current_28_port, A1N => reset_in, Y => 
                           pc_future(28));
   U82 : OAI22XL port map( A0 => n12, A1 => n1230, B0 => reset_in, B1 => n13, Y
                           => pc_future(29));
   U83 : INVXL port map( A => n11, Y => pc_future(30));
   U84 : AOI22XL port map( A0 => pc_current_30_port, A1 => reset_in, B0 => 
                           n1230, B1 => N130, Y => n11);
   U85 : OAI2BB2XL port map( B0 => reset_in, B1 => n10, A0N => 
                           pc_current_31_port, A1N => reset_in, Y => 
                           pc_future(31));
   U86 : XOR2XL port map( A => pc_current_7_port, B => n100, Y => 
                           pc_plus4_7_port);
   U87 : XNOR2XL port map( A => n1030, B => pc_current_8_port, Y => 
                           pc_plus4_8_port);
   U88 : NAND2XL port map( A => pc_current_7_port, B => n100, Y => n1030);
   U89 : XNOR2XL port map( A => pc_current_3_port, B => pc_plus4_2_port, Y => 
                           pc_plus4_3_port);
   U90 : XNOR2XL port map( A => n94, B => pc_current_9_port, Y => 
                           pc_plus4_9_port);
   U91 : XNOR2XL port map( A => n1060, B => pc_current_5_port, Y => 
                           pc_plus4_5_port);
   U92 : XNOR2XL port map( A => n1140, B => pc_current_4_port, Y => 
                           pc_plus4_4_port);
   U93 : NAND2XL port map( A => pc_current_3_port, B => pc_current_2_port, Y =>
                           n1140);
   U94 : INVXL port map( A => pc_current_2_port, Y => pc_plus4_2_port);
   U95 : XNOR2XL port map( A => n32, B => n97, Y => pc_plus4_10_port);
   U96 : NOR2XL port map( A => n2, B => n94, Y => n97);
   U97 : XNOR2XL port map( A => n6, B => n1090, Y => pc_plus4_6_port);
   U98 : NOR2XL port map( A => n1060, B => n7, Y => n1090);
   U99 : XOR2XL port map( A => pc_current_15_port, B => n76, Y => 
                           pc_plus4_15_port);
   U100 : XOR2XL port map( A => pc_current_11_port, B => n88, Y => 
                           pc_plus4_11_port);
   U101 : XNOR2XL port map( A => n79, B => pc_current_16_port, Y => 
                           pc_plus4_16_port);
   U102 : NAND2XL port map( A => pc_current_15_port, B => n76, Y => n79);
   U103 : XNOR2XL port map( A => n91, B => pc_current_12_port, Y => 
                           pc_plus4_12_port);
   U104 : NAND2XL port map( A => pc_current_11_port, B => n88, Y => n91);
   U105 : XNOR2XL port map( A => n70, B => pc_current_17_port, Y => 
                           pc_plus4_17_port);
   U106 : XNOR2XL port map( A => n82, B => pc_current_13_port, Y => 
                           pc_plus4_13_port);
   U107 : XNOR2XL port map( A => n28, B => n85, Y => pc_plus4_14_port);
   U108 : NOR2XL port map( A => n82, B => n29, Y => n85);
   U109 : INVXL port map( A => n3, Y => n35);
   U110 : XOR2XL port map( A => pc_current_23_port, B => n52, Y => 
                           pc_plus4_23_port);
   U111 : XOR2XL port map( A => pc_current_19_port, B => n64, Y => 
                           pc_plus4_19_port);
   U112 : XNOR2XL port map( A => n55, B => pc_current_24_port, Y => 
                           pc_plus4_24_port);
   U113 : NAND2XL port map( A => pc_current_23_port, B => n52, Y => n55);
   U114 : XNOR2XL port map( A => n67, B => pc_current_20_port, Y => 
                           pc_plus4_20_port);
   U115 : NAND2XL port map( A => pc_current_19_port, B => n64, Y => n67);
   U116 : XNOR2XL port map( A => n58, B => pc_current_21_port, Y => 
                           pc_plus4_21_port);
   U117 : XNOR2XL port map( A => n20, B => n61, Y => pc_plus4_22_port);
   U118 : NOR2XL port map( A => n58, B => n21, Y => n61);
   U119 : XNOR2XL port map( A => n24, B => n73, Y => pc_plus4_18_port);
   U120 : NOR2XL port map( A => n70, B => n25, Y => n73);
   U121 : XOR2XL port map( A => pc_current_27_port, B => n40, Y => 
                           pc_plus4_27_port);
   U122 : XOR2XL port map( A => pc_current_30_port, B => n37, Y => 
                           pc_plus4_30_port);
   U123 : NAND2BXL port map( AN => pause_in, B => n41, Y => n34);
   U124 : XNOR2XL port map( A => n46, B => pc_current_25_port, Y => 
                           pc_plus4_25_port);
   U125 : INVXL port map( A => n33, Y => n45);
   U126 : AOI222XL port map( A0 => pc_plus4_28_port, A1 => n1, B0 => n34, B1 =>
                           pc_current_28_port, C0 => n45, C1 => pc_new(28), Y 
                           => n14);
   U127 : XNOR2XL port map( A => n16, B => n49, Y => pc_plus4_26_port);
   U128 : NOR2XL port map( A => n46, B => n17, Y => n49);
   U129 : AOI222XL port map( A0 => pc_plus4_29_port, A1 => n1, B0 => n34, B1 =>
                           pc_current_29_port, C0 => n45, C1 => pc_new(29), Y 
                           => n13);
   U130 : AOI222XL port map( A0 => pc_plus4_31_port, A1 => n1, B0 => n34, B1 =>
                           pc_current_31_port, C0 => n45, C1 => pc_new(31), Y 
                           => n10);
   U131 : NAND2XL port map( A => n1040, B => n1050, Y => N107);
   U132 : AOI22XL port map( A0 => pc_new(7), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_7_port, Y => n1040);
   U133 : AOI22XL port map( A0 => opcode25_0(5), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_7_port, Y => n1050);
   U134 : NAND2XL port map( A => n101, B => n1020, Y => N108);
   U135 : AOI22XL port map( A0 => opcode25_0(6), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_8_port, Y => n1020);
   U136 : AOI22XL port map( A0 => pc_new(8), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_8_port, Y => n101);
   U137 : NAND2XL port map( A => n1150, B => n1160, Y => N103);
   U138 : AOI22XL port map( A0 => pc_new(3), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_3_port, Y => n1150);
   U139 : AOI22XL port map( A0 => opcode25_0(1), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_3_port, Y => n1160);
   U140 : NAND2XL port map( A => n98, B => n99, Y => N109);
   U141 : AOI22XL port map( A0 => opcode25_0(7), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_9_port, Y => n99);
   U142 : AOI22XL port map( A0 => pc_new(9), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_9_port, Y => n98);
   U143 : NAND2XL port map( A => n1100, B => n1110, Y => N105);
   U144 : AOI22XL port map( A0 => pc_new(5), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_5_port, Y => n1100);
   U145 : AOI22XL port map( A0 => opcode25_0(3), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_5_port, Y => n1110);
   U146 : NAND2XL port map( A => n1120, B => n1130, Y => N104);
   U147 : AOI22XL port map( A0 => pc_new(4), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_4_port, Y => n1120);
   U148 : AOI22XL port map( A0 => opcode25_0(2), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_4_port, Y => n1130);
   U149 : NAND2XL port map( A => n1170, B => n1180, Y => N102);
   U150 : AOI22XL port map( A0 => pc_new(2), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_2_port, Y => n1170);
   U151 : AOI22XL port map( A0 => opcode25_0(0), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_2_port, Y => n1180);
   U152 : NAND2XL port map( A => n95, B => n96, Y => N110);
   U153 : AOI22XL port map( A0 => opcode25_0(8), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_10_port, Y => n96);
   U154 : AOI22XL port map( A0 => pc_new(10), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_10_port, Y => n95);
   U155 : NAND2XL port map( A => n1070, B => n1080, Y => N106);
   U156 : AOI22XL port map( A0 => pc_new(6), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_6_port, Y => n1070);
   U157 : AOI22XL port map( A0 => opcode25_0(4), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_6_port, Y => n1080);
   U158 : NAND2XL port map( A => n80, B => n81, Y => N115);
   U159 : AOI22XL port map( A0 => opcode25_0(13), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_15_port, Y => n81);
   U160 : AOI22XL port map( A0 => pc_new(15), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_15_port, Y => n80);
   U161 : NAND2XL port map( A => n92, B => n93, Y => N111);
   U162 : AOI22XL port map( A0 => opcode25_0(9), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_11_port, Y => n93);
   U163 : AOI22XL port map( A0 => pc_new(11), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_11_port, Y => n92);
   U164 : NAND2XL port map( A => n77, B => n78, Y => N116);
   U165 : AOI22XL port map( A0 => opcode25_0(14), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_16_port, Y => n78);
   U166 : AOI22XL port map( A0 => pc_new(16), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_16_port, Y => n77);
   U167 : NAND2XL port map( A => n89, B => n90, Y => N112);
   U168 : AOI22XL port map( A0 => opcode25_0(10), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_12_port, Y => n90);
   U169 : AOI22XL port map( A0 => pc_new(12), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_12_port, Y => n89);
   U170 : NAND2XL port map( A => n74, B => n75, Y => N117);
   U171 : AOI22XL port map( A0 => opcode25_0(15), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_17_port, Y => n75);
   U172 : AOI22XL port map( A0 => pc_new(17), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_17_port, Y => n74);
   U173 : NAND2XL port map( A => n86, B => n87, Y => N113);
   U174 : AOI22XL port map( A0 => opcode25_0(11), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_13_port, Y => n87);
   U175 : AOI22XL port map( A0 => pc_new(13), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_13_port, Y => n86);
   U176 : NAND2XL port map( A => n83, B => n84, Y => N114);
   U177 : AOI22XL port map( A0 => opcode25_0(12), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_14_port, Y => n84);
   U178 : AOI22XL port map( A0 => pc_new(14), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_14_port, Y => n83);
   U179 : NAND2XL port map( A => n56, B => n57, Y => N123);
   U180 : AOI22XL port map( A0 => opcode25_0(21), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_23_port, Y => n57);
   U181 : AOI22XL port map( A0 => pc_new(23), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_23_port, Y => n56);
   U182 : NAND2XL port map( A => n68, B => n69, Y => N119);
   U183 : AOI22XL port map( A0 => opcode25_0(17), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_19_port, Y => n69);
   U184 : AOI22XL port map( A0 => pc_new(19), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_19_port, Y => n68);
   U185 : NAND2XL port map( A => n53, B => n54, Y => N124);
   U186 : AOI22XL port map( A0 => opcode25_0(22), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_24_port, Y => n54);
   U187 : AOI22XL port map( A0 => pc_new(24), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_24_port, Y => n53);
   U188 : NAND2XL port map( A => n65, B => n66, Y => N120);
   U189 : AOI22XL port map( A0 => opcode25_0(18), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_20_port, Y => n66);
   U190 : AOI22XL port map( A0 => pc_new(20), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_20_port, Y => n65);
   U191 : NAND2XL port map( A => n62, B => n63, Y => N121);
   U192 : AOI22XL port map( A0 => opcode25_0(19), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_21_port, Y => n63);
   U193 : AOI22XL port map( A0 => pc_new(21), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_21_port, Y => n62);
   U194 : NAND2XL port map( A => n59, B => n60, Y => N122);
   U195 : AOI22XL port map( A0 => opcode25_0(20), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_22_port, Y => n60);
   U196 : AOI22XL port map( A0 => pc_new(22), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_22_port, Y => n59);
   U197 : NAND2XL port map( A => n71, B => n72, Y => N118);
   U198 : AOI22XL port map( A0 => opcode25_0(16), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_18_port, Y => n72);
   U199 : AOI22XL port map( A0 => pc_new(18), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_18_port, Y => n71);
   U200 : NAND2XL port map( A => n43, B => n44, Y => N127);
   U201 : AOI22XL port map( A0 => opcode25_0(25), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_27_port, Y => n44);
   U202 : AOI22XL port map( A0 => pc_new(27), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_27_port, Y => n43);
   U203 : OAI2BB1XL port map( A0N => pc_new(30), A1N => n45, B0 => n38, Y => 
                           N130);
   U204 : AOI22XL port map( A0 => pc_current_30_port, A1 => n34, B0 => n1, B1 
                           => pc_plus4_30_port, Y => n38);
   U205 : NAND2XL port map( A => n50, B => n51, Y => N125);
   U206 : AOI22XL port map( A0 => opcode25_0(23), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_25_port, Y => n51);
   U207 : AOI22XL port map( A0 => pc_new(25), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_25_port, Y => n50);
   U208 : INVXL port map( A => n14, Y => N128);
   U209 : NAND2XL port map( A => n47, B => n48, Y => N126);
   U210 : AOI22XL port map( A0 => opcode25_0(24), A1 => n35, B0 => n1, B1 => 
                           pc_plus4_26_port, Y => n48);
   U211 : AOI22XL port map( A0 => pc_new(26), A1 => n45, B0 => pause_in, B1 => 
                           pc_current_26_port, Y => n47);
   U212 : INVXL port map( A => n13, Y => N129);
   U213 : INVXL port map( A => n10, Y => N131);
   U214 : OR3XL port map( A => n1210, B => pause_in, C => n1200, Y => n33);
   U215 : OAI2BB2X1 port map( B0 => pc_plus4_2_port, B1 => n1220, A0N => n1220,
                           A1N => N102, Y => pc_future(2));

end SYN_logic;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_mlite_cpu.all;

entity mlite_cpu is

   port( clk, reset_in, intr_in : in std_logic;  address_next : out 
         std_logic_vector (31 downto 2);  byte_we_next : out std_logic_vector 
         (3 downto 0);  address : out std_logic_vector (31 downto 2);  byte_we 
         : out std_logic_vector (3 downto 0);  data_w : out std_logic_vector 
         (31 downto 0);  data_r : in std_logic_vector (31 downto 0);  mem_pause
         : in std_logic);

end mlite_cpu;

architecture SYN_logic of mlite_cpu is

   component ADDHXL
      port( A, B : in std_logic;  CO, S : out std_logic);
   end component;
   
   component OR3XL
      port( A, B, C : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI2BB1X1
      port( A0N, A1N, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component XNOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component INVXL
      port( A : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND2BXL
      port( AN, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NOR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component NAND4XL
      port( A, B, C, D : in std_logic;  Y : out std_logic);
   end component;
   
   component AOI21XL
      port( A0, A1, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component OR2XL
      port( A, B : in std_logic;  Y : out std_logic);
   end component;
   
   component OAI31XL
      port( A0, A1, A2, B0 : in std_logic;  Y : out std_logic);
   end component;
   
   component DFFRHQXL
      port( D, CK, RN : in std_logic;  Q : out std_logic);
   end component;
   
   component pipeline
      port( clk, reset : in std_logic;  a_bus : in std_logic_vector (31 downto 
            0);  a_busD : out std_logic_vector (31 downto 0);  b_bus : in 
            std_logic_vector (31 downto 0);  b_busD : out std_logic_vector (31 
            downto 0);  alu_func : in std_logic_vector (3 downto 0);  alu_funcD
            : out std_logic_vector (3 downto 0);  shift_func : in 
            std_logic_vector (1 downto 0);  shift_funcD : out std_logic_vector 
            (1 downto 0);  mult_func : in std_logic_vector (3 downto 0);  
            mult_funcD : out std_logic_vector (3 downto 0);  reg_dest : in 
            std_logic_vector (31 downto 0);  reg_destD : out std_logic_vector 
            (31 downto 0);  rd_index : in std_logic_vector (5 downto 0);  
            rd_indexD : out std_logic_vector (5 downto 0);  rs_index, rt_index 
            : in std_logic_vector (5 downto 0);  pc_source : in 
            std_logic_vector (1 downto 0);  mem_source : in std_logic_vector (3
            downto 0);  a_source, b_source : in std_logic_vector (1 downto 0); 
            c_source : in std_logic_vector (2 downto 0);  c_bus : in 
            std_logic_vector (31 downto 0);  pause_any : in std_logic;  
            pause_pipeline : out std_logic);
   end component;
   
   component mult_mult_typeDEFAULT
      port( clk, reset_in : in std_logic;  a, b : in std_logic_vector (31 
            downto 0);  mult_func : in std_logic_vector (3 downto 0);  c_mult :
            out std_logic_vector (31 downto 0);  pause_out : out std_logic);
   end component;
   
   component shifter_shifter_typeDEFAULT
      port( value : in std_logic_vector (31 downto 0);  shift_amount : in 
            std_logic_vector (4 downto 0);  shift_func : in std_logic_vector (1
            downto 0);  c_shift : out std_logic_vector (31 downto 0));
   end component;
   
   component alu_alu_typeDEFAULT
      port( a_in, b_in : in std_logic_vector (31 downto 0);  alu_function : in 
            std_logic_vector (3 downto 0);  c_alu : out std_logic_vector (31 
            downto 0));
   end component;
   
   component bus_mux
      port( imm_in : in std_logic_vector (15 downto 0);  reg_source : in 
            std_logic_vector (31 downto 0);  a_mux : in std_logic_vector (1 
            downto 0);  a_out : out std_logic_vector (31 downto 0);  reg_target
            : in std_logic_vector (31 downto 0);  b_mux : in std_logic_vector 
            (1 downto 0);  b_out : out std_logic_vector (31 downto 0);  c_bus, 
            c_memory : in std_logic_vector (31 downto 0);  c_pc, c_pc_plus4 : 
            in std_logic_vector (31 downto 2);  c_mux : in std_logic_vector (2 
            downto 0);  reg_dest_out : out std_logic_vector (31 downto 0);  
            branch_func : in std_logic_vector (2 downto 0);  take_branch : out 
            std_logic);
   end component;
   
   component reg_bank_entity
      port( clk, reset_in, pause : in std_logic;  rs_index, rt_index, rd_index 
            : in std_logic_vector (5 downto 0);  reg_source_out, reg_target_out
            : out std_logic_vector (31 downto 0);  reg_dest_new : in 
            std_logic_vector (31 downto 0);  intr_enable : out std_logic);
   end component;
   
   component control
      port( opcode : in std_logic_vector (31 downto 0);  intr_signal : in 
            std_logic;  rs_index, rt_index, rd_index : out std_logic_vector (5 
            downto 0);  imm_out : out std_logic_vector (15 downto 0);  alu_func
            : out std_logic_vector (3 downto 0);  shift_func : out 
            std_logic_vector (1 downto 0);  mult_func : out std_logic_vector (3
            downto 0);  branch_func : out std_logic_vector (2 downto 0);  
            a_source_out, b_source_out : out std_logic_vector (1 downto 0);  
            c_source_out : out std_logic_vector (2 downto 0);  pc_source_out : 
            out std_logic_vector (1 downto 0);  mem_source_out : out 
            std_logic_vector (3 downto 0);  exception_out : out std_logic);
   end component;
   
   component mem_ctrl
      port( clk, reset_in, pause_in, nullify_op : in std_logic;  address_pc : 
            in std_logic_vector (31 downto 2);  opcode_out : out 
            std_logic_vector (31 downto 0);  address_in : in std_logic_vector 
            (31 downto 0);  mem_source : in std_logic_vector (3 downto 0);  
            data_write : in std_logic_vector (31 downto 0);  data_read : out 
            std_logic_vector (31 downto 0);  pause_out : out std_logic;  
            address_next : out std_logic_vector (31 downto 2);  byte_we_next : 
            out std_logic_vector (3 downto 0);  address : out std_logic_vector 
            (31 downto 2);  byte_we : out std_logic_vector (3 downto 0);  
            data_w : out std_logic_vector (31 downto 0);  data_r : in 
            std_logic_vector (31 downto 0));
   end component;
   
   component pc_next
      port( clk, reset_in : in std_logic;  pc_new : in std_logic_vector (31 
            downto 2);  take_branch, pause_in : in std_logic;  opcode25_0 : in 
            std_logic_vector (25 downto 0);  pc_source : in std_logic_vector (1
            downto 0);  pc_future, pc_current, pc_plus4 : out std_logic_vector 
            (31 downto 2));
   end component;
   
   signal pause_ctrl, pause_mult, pause_pipeline, pause_any, pause_non_ctrl, 
      pause_bank, pc_source_1_port, pc_source_0_port, take_branch, intr_signal,
      exception_sig, nullify_op, c_alu_31_port, c_alu_30_port, c_alu_29_port, 
      c_alu_28_port, c_alu_27_port, c_alu_26_port, c_alu_25_port, c_alu_24_port
      , c_alu_23_port, c_alu_22_port, c_alu_21_port, c_alu_20_port, 
      c_alu_19_port, c_alu_18_port, c_alu_17_port, c_alu_16_port, c_alu_15_port
      , c_alu_14_port, c_alu_13_port, c_alu_12_port, c_alu_11_port, 
      c_alu_10_port, c_alu_9_port, c_alu_8_port, c_alu_7_port, c_alu_6_port, 
      c_alu_5_port, c_alu_4_port, c_alu_3_port, c_alu_2_port, c_alu_1_port, 
      c_alu_0_port, c_shift_31_port, c_shift_30_port, c_shift_29_port, 
      c_shift_28_port, c_shift_27_port, c_shift_26_port, c_shift_25_port, 
      c_shift_24_port, c_shift_23_port, c_shift_22_port, c_shift_21_port, 
      c_shift_20_port, c_shift_19_port, c_shift_18_port, c_shift_17_port, 
      c_shift_16_port, c_shift_15_port, c_shift_14_port, c_shift_13_port, 
      c_shift_12_port, c_shift_11_port, c_shift_10_port, c_shift_9_port, 
      c_shift_8_port, c_shift_7_port, c_shift_6_port, c_shift_5_port, 
      c_shift_4_port, c_shift_3_port, c_shift_2_port, c_shift_1_port, 
      c_shift_0_port, c_mult_31_port, c_mult_30_port, c_mult_29_port, 
      c_mult_28_port, c_mult_27_port, c_mult_26_port, c_mult_25_port, 
      c_mult_24_port, c_mult_23_port, c_mult_22_port, c_mult_21_port, 
      c_mult_20_port, c_mult_19_port, c_mult_18_port, c_mult_17_port, 
      c_mult_16_port, c_mult_15_port, c_mult_14_port, c_mult_13_port, 
      c_mult_12_port, c_mult_11_port, c_mult_10_port, c_mult_9_port, 
      c_mult_8_port, c_mult_7_port, c_mult_6_port, c_mult_5_port, c_mult_4_port
      , c_mult_3_port, c_mult_2_port, c_mult_1_port, c_mult_0_port, 
      c_bus_31_port, c_bus_30_port, c_bus_29_port, c_bus_28_port, c_bus_27_port
      , c_bus_26_port, c_bus_25_port, c_bus_24_port, c_bus_23_port, 
      c_bus_22_port, c_bus_21_port, c_bus_20_port, c_bus_19_port, c_bus_18_port
      , c_bus_17_port, c_bus_16_port, c_bus_15_port, c_bus_14_port, 
      c_bus_13_port, c_bus_12_port, c_bus_11_port, c_bus_10_port, c_bus_9_port,
      c_bus_8_port, c_bus_7_port, c_bus_6_port, c_bus_5_port, c_bus_4_port, 
      c_bus_3_port, c_bus_2_port, c_bus_1_port, c_bus_0_port, reset_reg_3_port,
      reset_reg_2_port, reset_reg_1_port, reset_reg_0_port, reset, N11, N12, 
      intr_enable, opcode_31_port, opcode_30_port, opcode_29_port, 
      opcode_28_port, opcode_27_port, opcode_26_port, opcode_25_port, 
      opcode_24_port, opcode_23_port, opcode_22_port, opcode_21_port, 
      opcode_20_port, opcode_19_port, opcode_18_port, opcode_17_port, 
      opcode_16_port, opcode_15_port, opcode_14_port, opcode_13_port, 
      opcode_12_port, opcode_11_port, opcode_10_port, opcode_9_port, 
      opcode_8_port, opcode_7_port, opcode_6_port, opcode_5_port, opcode_4_port
      , opcode_3_port, opcode_2_port, opcode_1_port, opcode_0_port, 
      pc_future_31_port, pc_future_30_port, pc_future_29_port, 
      pc_future_28_port, pc_future_27_port, pc_future_26_port, 
      pc_future_25_port, pc_future_24_port, pc_future_23_port, 
      pc_future_22_port, pc_future_21_port, pc_future_20_port, 
      pc_future_19_port, pc_future_18_port, pc_future_17_port, 
      pc_future_16_port, pc_future_15_port, pc_future_14_port, 
      pc_future_13_port, pc_future_12_port, pc_future_11_port, 
      pc_future_10_port, pc_future_9_port, pc_future_8_port, pc_future_7_port, 
      pc_future_6_port, pc_future_5_port, pc_future_4_port, pc_future_3_port, 
      pc_future_2_port, pc_current_31_port, pc_current_30_port, 
      pc_current_29_port, pc_current_28_port, pc_current_27_port, 
      pc_current_26_port, pc_current_25_port, pc_current_24_port, 
      pc_current_23_port, pc_current_22_port, pc_current_21_port, 
      pc_current_20_port, pc_current_19_port, pc_current_18_port, 
      pc_current_17_port, pc_current_16_port, pc_current_15_port, 
      pc_current_14_port, pc_current_13_port, pc_current_12_port, 
      pc_current_11_port, pc_current_10_port, pc_current_9_port, 
      pc_current_8_port, pc_current_7_port, pc_current_6_port, 
      pc_current_5_port, pc_current_4_port, pc_current_3_port, 
      pc_current_2_port, pc_plus4_31_port, pc_plus4_30_port, pc_plus4_29_port, 
      pc_plus4_28_port, pc_plus4_27_port, pc_plus4_26_port, pc_plus4_25_port, 
      pc_plus4_24_port, pc_plus4_23_port, pc_plus4_22_port, pc_plus4_21_port, 
      pc_plus4_20_port, pc_plus4_19_port, pc_plus4_18_port, pc_plus4_17_port, 
      pc_plus4_16_port, pc_plus4_15_port, pc_plus4_14_port, pc_plus4_13_port, 
      pc_plus4_12_port, pc_plus4_11_port, pc_plus4_10_port, pc_plus4_9_port, 
      pc_plus4_8_port, pc_plus4_7_port, pc_plus4_6_port, pc_plus4_5_port, 
      pc_plus4_4_port, pc_plus4_3_port, pc_plus4_2_port, mem_source_3_port, 
      mem_source_2_port, mem_source_1_port, mem_source_0_port, 
      reg_target_31_port, reg_target_30_port, reg_target_29_port, 
      reg_target_28_port, reg_target_27_port, reg_target_26_port, 
      reg_target_25_port, reg_target_24_port, reg_target_23_port, 
      reg_target_22_port, reg_target_21_port, reg_target_20_port, 
      reg_target_19_port, reg_target_18_port, reg_target_17_port, 
      reg_target_16_port, reg_target_15_port, reg_target_14_port, 
      reg_target_13_port, reg_target_12_port, reg_target_11_port, 
      reg_target_10_port, reg_target_9_port, reg_target_8_port, 
      reg_target_7_port, reg_target_6_port, reg_target_5_port, 
      reg_target_4_port, reg_target_3_port, reg_target_2_port, 
      reg_target_1_port, reg_target_0_port, c_memory_31_port, c_memory_30_port,
      c_memory_29_port, c_memory_28_port, c_memory_27_port, c_memory_26_port, 
      c_memory_25_port, c_memory_24_port, c_memory_23_port, c_memory_22_port, 
      c_memory_21_port, c_memory_20_port, c_memory_19_port, c_memory_18_port, 
      c_memory_17_port, c_memory_16_port, c_memory_15_port, c_memory_14_port, 
      c_memory_13_port, c_memory_12_port, c_memory_11_port, c_memory_10_port, 
      c_memory_9_port, c_memory_8_port, c_memory_7_port, c_memory_6_port, 
      c_memory_5_port, c_memory_4_port, c_memory_3_port, c_memory_2_port, 
      c_memory_1_port, c_memory_0_port, rs_index_5_port, rs_index_4_port, 
      rs_index_3_port, rs_index_2_port, rs_index_1_port, rs_index_0_port, 
      rt_index_5_port, rt_index_4_port, rt_index_3_port, rt_index_2_port, 
      rt_index_1_port, rt_index_0_port, rd_index_5_port, rd_index_4_port, 
      rd_index_3_port, rd_index_2_port, rd_index_1_port, rd_index_0_port, 
      imm_15_port, imm_14_port, imm_13_port, imm_12_port, imm_11_port, 
      imm_10_port, imm_9_port, imm_8_port, imm_7_port, imm_6_port, imm_5_port, 
      imm_4_port, imm_3_port, imm_2_port, imm_1_port, imm_0_port, 
      alu_func_3_port, alu_func_2_port, alu_func_1_port, alu_func_0_port, 
      shift_func_1_port, shift_func_0_port, mult_func_3_port, mult_func_2_port,
      mult_func_1_port, mult_func_0_port, branch_func_2_port, 
      branch_func_1_port, branch_func_0_port, a_source_1_port, a_source_0_port,
      b_source_1_port, b_source_0_port, c_source_2_port, c_source_1_port, 
      c_source_0_port, rd_indexD_5_port, rd_indexD_4_port, rd_indexD_3_port, 
      rd_indexD_2_port, rd_indexD_1_port, rd_indexD_0_port, reg_source_31_port,
      reg_source_30_port, reg_source_29_port, reg_source_28_port, 
      reg_source_27_port, reg_source_26_port, reg_source_25_port, 
      reg_source_24_port, reg_source_23_port, reg_source_22_port, 
      reg_source_21_port, reg_source_20_port, reg_source_19_port, 
      reg_source_18_port, reg_source_17_port, reg_source_16_port, 
      reg_source_15_port, reg_source_14_port, reg_source_13_port, 
      reg_source_12_port, reg_source_11_port, reg_source_10_port, 
      reg_source_9_port, reg_source_8_port, reg_source_7_port, 
      reg_source_6_port, reg_source_5_port, reg_source_4_port, 
      reg_source_3_port, reg_source_2_port, reg_source_1_port, 
      reg_source_0_port, reg_destD_31_port, reg_destD_30_port, 
      reg_destD_29_port, reg_destD_28_port, reg_destD_27_port, 
      reg_destD_26_port, reg_destD_25_port, reg_destD_24_port, 
      reg_destD_23_port, reg_destD_22_port, reg_destD_21_port, 
      reg_destD_20_port, reg_destD_19_port, reg_destD_18_port, 
      reg_destD_17_port, reg_destD_16_port, reg_destD_15_port, 
      reg_destD_14_port, reg_destD_13_port, reg_destD_12_port, 
      reg_destD_11_port, reg_destD_10_port, reg_destD_9_port, reg_destD_8_port,
      reg_destD_7_port, reg_destD_6_port, reg_destD_5_port, reg_destD_4_port, 
      reg_destD_3_port, reg_destD_2_port, reg_destD_1_port, reg_destD_0_port, 
      a_bus_31_port, a_bus_30_port, a_bus_29_port, a_bus_28_port, a_bus_27_port
      , a_bus_26_port, a_bus_25_port, a_bus_24_port, a_bus_23_port, 
      a_bus_22_port, a_bus_21_port, a_bus_20_port, a_bus_19_port, a_bus_18_port
      , a_bus_17_port, a_bus_16_port, a_bus_15_port, a_bus_14_port, 
      a_bus_13_port, a_bus_12_port, a_bus_11_port, a_bus_10_port, a_bus_9_port,
      a_bus_8_port, a_bus_7_port, a_bus_6_port, a_bus_5_port, a_bus_4_port, 
      a_bus_3_port, a_bus_2_port, a_bus_1_port, a_bus_0_port, b_bus_31_port, 
      b_bus_30_port, b_bus_29_port, b_bus_28_port, b_bus_27_port, b_bus_26_port
      , b_bus_25_port, b_bus_24_port, b_bus_23_port, b_bus_22_port, 
      b_bus_21_port, b_bus_20_port, b_bus_19_port, b_bus_18_port, b_bus_17_port
      , b_bus_16_port, b_bus_15_port, b_bus_14_port, b_bus_13_port, 
      b_bus_12_port, b_bus_11_port, b_bus_10_port, b_bus_9_port, b_bus_8_port, 
      b_bus_7_port, b_bus_6_port, b_bus_5_port, b_bus_4_port, b_bus_3_port, 
      b_bus_2_port, b_bus_1_port, b_bus_0_port, reg_dest_31_port, 
      reg_dest_30_port, reg_dest_29_port, reg_dest_28_port, reg_dest_27_port, 
      reg_dest_26_port, reg_dest_25_port, reg_dest_24_port, reg_dest_23_port, 
      reg_dest_22_port, reg_dest_21_port, reg_dest_20_port, reg_dest_19_port, 
      reg_dest_18_port, reg_dest_17_port, reg_dest_16_port, reg_dest_15_port, 
      reg_dest_14_port, reg_dest_13_port, reg_dest_12_port, reg_dest_11_port, 
      reg_dest_10_port, reg_dest_9_port, reg_dest_8_port, reg_dest_7_port, 
      reg_dest_6_port, reg_dest_5_port, reg_dest_4_port, reg_dest_3_port, 
      reg_dest_2_port, reg_dest_1_port, reg_dest_0_port, a_busD_31_port, 
      a_busD_30_port, a_busD_29_port, a_busD_28_port, a_busD_27_port, 
      a_busD_26_port, a_busD_25_port, a_busD_24_port, a_busD_23_port, 
      a_busD_22_port, a_busD_21_port, a_busD_20_port, a_busD_19_port, 
      a_busD_18_port, a_busD_17_port, a_busD_16_port, a_busD_15_port, 
      a_busD_14_port, a_busD_13_port, a_busD_12_port, a_busD_11_port, 
      a_busD_10_port, a_busD_9_port, a_busD_8_port, a_busD_7_port, 
      a_busD_6_port, a_busD_5_port, a_busD_4_port, a_busD_3_port, a_busD_2_port
      , a_busD_1_port, a_busD_0_port, b_busD_31_port, b_busD_30_port, 
      b_busD_29_port, b_busD_28_port, b_busD_27_port, b_busD_26_port, 
      b_busD_25_port, b_busD_24_port, b_busD_23_port, b_busD_22_port, 
      b_busD_21_port, b_busD_20_port, b_busD_19_port, b_busD_18_port, 
      b_busD_17_port, b_busD_16_port, b_busD_15_port, b_busD_14_port, 
      b_busD_13_port, b_busD_12_port, b_busD_11_port, b_busD_10_port, 
      b_busD_9_port, b_busD_8_port, b_busD_7_port, b_busD_6_port, b_busD_5_port
      , b_busD_4_port, b_busD_3_port, b_busD_2_port, b_busD_1_port, 
      b_busD_0_port, alu_funcD_3_port, alu_funcD_2_port, alu_funcD_1_port, 
      alu_funcD_0_port, shift_funcD_1_port, shift_funcD_0_port, 
      mult_funcD_3_port, mult_funcD_2_port, mult_funcD_1_port, 
      mult_funcD_0_port, n1, n2, n3, n4, n5, n6, n7, n8, n9, n10, n110, n120, 
      n13, n14, n15, add_165xcarryx2x, add_165xcarryx3x, n16, n_1002 : 
      std_logic;

begin
   
   u1_pc_next : pc_next port map( clk => clk, reset_in => reset, pc_new(31) => 
                           c_bus_31_port, pc_new(30) => c_bus_30_port, 
                           pc_new(29) => c_bus_29_port, pc_new(28) => 
                           c_bus_28_port, pc_new(27) => c_bus_27_port, 
                           pc_new(26) => c_bus_26_port, pc_new(25) => 
                           c_bus_25_port, pc_new(24) => c_bus_24_port, 
                           pc_new(23) => c_bus_23_port, pc_new(22) => 
                           c_bus_22_port, pc_new(21) => c_bus_21_port, 
                           pc_new(20) => c_bus_20_port, pc_new(19) => 
                           c_bus_19_port, pc_new(18) => c_bus_18_port, 
                           pc_new(17) => c_bus_17_port, pc_new(16) => 
                           c_bus_16_port, pc_new(15) => c_bus_15_port, 
                           pc_new(14) => c_bus_14_port, pc_new(13) => 
                           c_bus_13_port, pc_new(12) => c_bus_12_port, 
                           pc_new(11) => c_bus_11_port, pc_new(10) => 
                           c_bus_10_port, pc_new(9) => c_bus_9_port, pc_new(8) 
                           => c_bus_8_port, pc_new(7) => c_bus_7_port, 
                           pc_new(6) => c_bus_6_port, pc_new(5) => c_bus_5_port
                           , pc_new(4) => c_bus_4_port, pc_new(3) => 
                           c_bus_3_port, pc_new(2) => c_bus_2_port, take_branch
                           => take_branch, pause_in => pause_any, 
                           opcode25_0(25) => opcode_25_port, opcode25_0(24) => 
                           opcode_24_port, opcode25_0(23) => opcode_23_port, 
                           opcode25_0(22) => opcode_22_port, opcode25_0(21) => 
                           opcode_21_port, opcode25_0(20) => opcode_20_port, 
                           opcode25_0(19) => opcode_19_port, opcode25_0(18) => 
                           opcode_18_port, opcode25_0(17) => opcode_17_port, 
                           opcode25_0(16) => opcode_16_port, opcode25_0(15) => 
                           opcode_15_port, opcode25_0(14) => opcode_14_port, 
                           opcode25_0(13) => opcode_13_port, opcode25_0(12) => 
                           opcode_12_port, opcode25_0(11) => opcode_11_port, 
                           opcode25_0(10) => opcode_10_port, opcode25_0(9) => 
                           opcode_9_port, opcode25_0(8) => opcode_8_port, 
                           opcode25_0(7) => opcode_7_port, opcode25_0(6) => 
                           opcode_6_port, opcode25_0(5) => opcode_5_port, 
                           opcode25_0(4) => opcode_4_port, opcode25_0(3) => 
                           opcode_3_port, opcode25_0(2) => opcode_2_port, 
                           opcode25_0(1) => opcode_1_port, opcode25_0(0) => 
                           opcode_0_port, pc_source(1) => pc_source_1_port, 
                           pc_source(0) => pc_source_0_port, pc_future(31) => 
                           pc_future_31_port, pc_future(30) => 
                           pc_future_30_port, pc_future(29) => 
                           pc_future_29_port, pc_future(28) => 
                           pc_future_28_port, pc_future(27) => 
                           pc_future_27_port, pc_future(26) => 
                           pc_future_26_port, pc_future(25) => 
                           pc_future_25_port, pc_future(24) => 
                           pc_future_24_port, pc_future(23) => 
                           pc_future_23_port, pc_future(22) => 
                           pc_future_22_port, pc_future(21) => 
                           pc_future_21_port, pc_future(20) => 
                           pc_future_20_port, pc_future(19) => 
                           pc_future_19_port, pc_future(18) => 
                           pc_future_18_port, pc_future(17) => 
                           pc_future_17_port, pc_future(16) => 
                           pc_future_16_port, pc_future(15) => 
                           pc_future_15_port, pc_future(14) => 
                           pc_future_14_port, pc_future(13) => 
                           pc_future_13_port, pc_future(12) => 
                           pc_future_12_port, pc_future(11) => 
                           pc_future_11_port, pc_future(10) => 
                           pc_future_10_port, pc_future(9) => pc_future_9_port,
                           pc_future(8) => pc_future_8_port, pc_future(7) => 
                           pc_future_7_port, pc_future(6) => pc_future_6_port, 
                           pc_future(5) => pc_future_5_port, pc_future(4) => 
                           pc_future_4_port, pc_future(3) => pc_future_3_port, 
                           pc_future(2) => pc_future_2_port, pc_current(31) => 
                           pc_current_31_port, pc_current(30) => 
                           pc_current_30_port, pc_current(29) => 
                           pc_current_29_port, pc_current(28) => 
                           pc_current_28_port, pc_current(27) => 
                           pc_current_27_port, pc_current(26) => 
                           pc_current_26_port, pc_current(25) => 
                           pc_current_25_port, pc_current(24) => 
                           pc_current_24_port, pc_current(23) => 
                           pc_current_23_port, pc_current(22) => 
                           pc_current_22_port, pc_current(21) => 
                           pc_current_21_port, pc_current(20) => 
                           pc_current_20_port, pc_current(19) => 
                           pc_current_19_port, pc_current(18) => 
                           pc_current_18_port, pc_current(17) => 
                           pc_current_17_port, pc_current(16) => 
                           pc_current_16_port, pc_current(15) => 
                           pc_current_15_port, pc_current(14) => 
                           pc_current_14_port, pc_current(13) => 
                           pc_current_13_port, pc_current(12) => 
                           pc_current_12_port, pc_current(11) => 
                           pc_current_11_port, pc_current(10) => 
                           pc_current_10_port, pc_current(9) => 
                           pc_current_9_port, pc_current(8) => 
                           pc_current_8_port, pc_current(7) => 
                           pc_current_7_port, pc_current(6) => 
                           pc_current_6_port, pc_current(5) => 
                           pc_current_5_port, pc_current(4) => 
                           pc_current_4_port, pc_current(3) => 
                           pc_current_3_port, pc_current(2) => 
                           pc_current_2_port, pc_plus4(31) => pc_plus4_31_port,
                           pc_plus4(30) => pc_plus4_30_port, pc_plus4(29) => 
                           pc_plus4_29_port, pc_plus4(28) => pc_plus4_28_port, 
                           pc_plus4(27) => pc_plus4_27_port, pc_plus4(26) => 
                           pc_plus4_26_port, pc_plus4(25) => pc_plus4_25_port, 
                           pc_plus4(24) => pc_plus4_24_port, pc_plus4(23) => 
                           pc_plus4_23_port, pc_plus4(22) => pc_plus4_22_port, 
                           pc_plus4(21) => pc_plus4_21_port, pc_plus4(20) => 
                           pc_plus4_20_port, pc_plus4(19) => pc_plus4_19_port, 
                           pc_plus4(18) => pc_plus4_18_port, pc_plus4(17) => 
                           pc_plus4_17_port, pc_plus4(16) => pc_plus4_16_port, 
                           pc_plus4(15) => pc_plus4_15_port, pc_plus4(14) => 
                           pc_plus4_14_port, pc_plus4(13) => pc_plus4_13_port, 
                           pc_plus4(12) => pc_plus4_12_port, pc_plus4(11) => 
                           pc_plus4_11_port, pc_plus4(10) => pc_plus4_10_port, 
                           pc_plus4(9) => pc_plus4_9_port, pc_plus4(8) => 
                           pc_plus4_8_port, pc_plus4(7) => pc_plus4_7_port, 
                           pc_plus4(6) => pc_plus4_6_port, pc_plus4(5) => 
                           pc_plus4_5_port, pc_plus4(4) => pc_plus4_4_port, 
                           pc_plus4(3) => pc_plus4_3_port, pc_plus4(2) => 
                           pc_plus4_2_port);
   u2_mem_ctrl : mem_ctrl port map( clk => clk, reset_in => reset, pause_in => 
                           pause_non_ctrl, nullify_op => nullify_op, 
                           address_pc(31) => pc_future_31_port, address_pc(30) 
                           => pc_future_30_port, address_pc(29) => 
                           pc_future_29_port, address_pc(28) => 
                           pc_future_28_port, address_pc(27) => 
                           pc_future_27_port, address_pc(26) => 
                           pc_future_26_port, address_pc(25) => 
                           pc_future_25_port, address_pc(24) => 
                           pc_future_24_port, address_pc(23) => 
                           pc_future_23_port, address_pc(22) => 
                           pc_future_22_port, address_pc(21) => 
                           pc_future_21_port, address_pc(20) => 
                           pc_future_20_port, address_pc(19) => 
                           pc_future_19_port, address_pc(18) => 
                           pc_future_18_port, address_pc(17) => 
                           pc_future_17_port, address_pc(16) => 
                           pc_future_16_port, address_pc(15) => 
                           pc_future_15_port, address_pc(14) => 
                           pc_future_14_port, address_pc(13) => 
                           pc_future_13_port, address_pc(12) => 
                           pc_future_12_port, address_pc(11) => 
                           pc_future_11_port, address_pc(10) => 
                           pc_future_10_port, address_pc(9) => pc_future_9_port
                           , address_pc(8) => pc_future_8_port, address_pc(7) 
                           => pc_future_7_port, address_pc(6) => 
                           pc_future_6_port, address_pc(5) => pc_future_5_port,
                           address_pc(4) => pc_future_4_port, address_pc(3) => 
                           pc_future_3_port, address_pc(2) => pc_future_2_port,
                           opcode_out(31) => opcode_31_port, opcode_out(30) => 
                           opcode_30_port, opcode_out(29) => opcode_29_port, 
                           opcode_out(28) => opcode_28_port, opcode_out(27) => 
                           opcode_27_port, opcode_out(26) => opcode_26_port, 
                           opcode_out(25) => opcode_25_port, opcode_out(24) => 
                           opcode_24_port, opcode_out(23) => opcode_23_port, 
                           opcode_out(22) => opcode_22_port, opcode_out(21) => 
                           opcode_21_port, opcode_out(20) => opcode_20_port, 
                           opcode_out(19) => opcode_19_port, opcode_out(18) => 
                           opcode_18_port, opcode_out(17) => opcode_17_port, 
                           opcode_out(16) => opcode_16_port, opcode_out(15) => 
                           opcode_15_port, opcode_out(14) => opcode_14_port, 
                           opcode_out(13) => opcode_13_port, opcode_out(12) => 
                           opcode_12_port, opcode_out(11) => opcode_11_port, 
                           opcode_out(10) => opcode_10_port, opcode_out(9) => 
                           opcode_9_port, opcode_out(8) => opcode_8_port, 
                           opcode_out(7) => opcode_7_port, opcode_out(6) => 
                           opcode_6_port, opcode_out(5) => opcode_5_port, 
                           opcode_out(4) => opcode_4_port, opcode_out(3) => 
                           opcode_3_port, opcode_out(2) => opcode_2_port, 
                           opcode_out(1) => opcode_1_port, opcode_out(0) => 
                           opcode_0_port, address_in(31) => c_bus_31_port, 
                           address_in(30) => c_bus_30_port, address_in(29) => 
                           c_bus_29_port, address_in(28) => c_bus_28_port, 
                           address_in(27) => c_bus_27_port, address_in(26) => 
                           c_bus_26_port, address_in(25) => c_bus_25_port, 
                           address_in(24) => c_bus_24_port, address_in(23) => 
                           c_bus_23_port, address_in(22) => c_bus_22_port, 
                           address_in(21) => c_bus_21_port, address_in(20) => 
                           c_bus_20_port, address_in(19) => c_bus_19_port, 
                           address_in(18) => c_bus_18_port, address_in(17) => 
                           c_bus_17_port, address_in(16) => c_bus_16_port, 
                           address_in(15) => c_bus_15_port, address_in(14) => 
                           c_bus_14_port, address_in(13) => c_bus_13_port, 
                           address_in(12) => c_bus_12_port, address_in(11) => 
                           c_bus_11_port, address_in(10) => c_bus_10_port, 
                           address_in(9) => c_bus_9_port, address_in(8) => 
                           c_bus_8_port, address_in(7) => c_bus_7_port, 
                           address_in(6) => c_bus_6_port, address_in(5) => 
                           c_bus_5_port, address_in(4) => c_bus_4_port, 
                           address_in(3) => c_bus_3_port, address_in(2) => 
                           c_bus_2_port, address_in(1) => c_bus_1_port, 
                           address_in(0) => c_bus_0_port, mem_source(3) => 
                           mem_source_3_port, mem_source(2) => 
                           mem_source_2_port, mem_source(1) => 
                           mem_source_1_port, mem_source(0) => 
                           mem_source_0_port, data_write(31) => 
                           reg_target_31_port, data_write(30) => 
                           reg_target_30_port, data_write(29) => 
                           reg_target_29_port, data_write(28) => 
                           reg_target_28_port, data_write(27) => 
                           reg_target_27_port, data_write(26) => 
                           reg_target_26_port, data_write(25) => 
                           reg_target_25_port, data_write(24) => 
                           reg_target_24_port, data_write(23) => 
                           reg_target_23_port, data_write(22) => 
                           reg_target_22_port, data_write(21) => 
                           reg_target_21_port, data_write(20) => 
                           reg_target_20_port, data_write(19) => 
                           reg_target_19_port, data_write(18) => 
                           reg_target_18_port, data_write(17) => 
                           reg_target_17_port, data_write(16) => 
                           reg_target_16_port, data_write(15) => 
                           reg_target_15_port, data_write(14) => 
                           reg_target_14_port, data_write(13) => 
                           reg_target_13_port, data_write(12) => 
                           reg_target_12_port, data_write(11) => 
                           reg_target_11_port, data_write(10) => 
                           reg_target_10_port, data_write(9) => 
                           reg_target_9_port, data_write(8) => 
                           reg_target_8_port, data_write(7) => 
                           reg_target_7_port, data_write(6) => 
                           reg_target_6_port, data_write(5) => 
                           reg_target_5_port, data_write(4) => 
                           reg_target_4_port, data_write(3) => 
                           reg_target_3_port, data_write(2) => 
                           reg_target_2_port, data_write(1) => 
                           reg_target_1_port, data_write(0) => 
                           reg_target_0_port, data_read(31) => c_memory_31_port
                           , data_read(30) => c_memory_30_port, data_read(29) 
                           => c_memory_29_port, data_read(28) => 
                           c_memory_28_port, data_read(27) => c_memory_27_port,
                           data_read(26) => c_memory_26_port, data_read(25) => 
                           c_memory_25_port, data_read(24) => c_memory_24_port,
                           data_read(23) => c_memory_23_port, data_read(22) => 
                           c_memory_22_port, data_read(21) => c_memory_21_port,
                           data_read(20) => c_memory_20_port, data_read(19) => 
                           c_memory_19_port, data_read(18) => c_memory_18_port,
                           data_read(17) => c_memory_17_port, data_read(16) => 
                           c_memory_16_port, data_read(15) => c_memory_15_port,
                           data_read(14) => c_memory_14_port, data_read(13) => 
                           c_memory_13_port, data_read(12) => c_memory_12_port,
                           data_read(11) => c_memory_11_port, data_read(10) => 
                           c_memory_10_port, data_read(9) => c_memory_9_port, 
                           data_read(8) => c_memory_8_port, data_read(7) => 
                           c_memory_7_port, data_read(6) => c_memory_6_port, 
                           data_read(5) => c_memory_5_port, data_read(4) => 
                           c_memory_4_port, data_read(3) => c_memory_3_port, 
                           data_read(2) => c_memory_2_port, data_read(1) => 
                           c_memory_1_port, data_read(0) => c_memory_0_port, 
                           pause_out => pause_ctrl, address_next(31) => 
                           address_next(31), address_next(30) => 
                           address_next(30), address_next(29) => 
                           address_next(29), address_next(28) => 
                           address_next(28), address_next(27) => 
                           address_next(27), address_next(26) => 
                           address_next(26), address_next(25) => 
                           address_next(25), address_next(24) => 
                           address_next(24), address_next(23) => 
                           address_next(23), address_next(22) => 
                           address_next(22), address_next(21) => 
                           address_next(21), address_next(20) => 
                           address_next(20), address_next(19) => 
                           address_next(19), address_next(18) => 
                           address_next(18), address_next(17) => 
                           address_next(17), address_next(16) => 
                           address_next(16), address_next(15) => 
                           address_next(15), address_next(14) => 
                           address_next(14), address_next(13) => 
                           address_next(13), address_next(12) => 
                           address_next(12), address_next(11) => 
                           address_next(11), address_next(10) => 
                           address_next(10), address_next(9) => address_next(9)
                           , address_next(8) => address_next(8), 
                           address_next(7) => address_next(7), address_next(6) 
                           => address_next(6), address_next(5) => 
                           address_next(5), address_next(4) => address_next(4),
                           address_next(3) => address_next(3), address_next(2) 
                           => address_next(2), byte_we_next(3) => 
                           byte_we_next(3), byte_we_next(2) => byte_we_next(2),
                           byte_we_next(1) => byte_we_next(1), byte_we_next(0) 
                           => byte_we_next(0), address(31) => address(31), 
                           address(30) => address(30), address(29) => 
                           address(29), address(28) => address(28), address(27)
                           => address(27), address(26) => address(26), 
                           address(25) => address(25), address(24) => 
                           address(24), address(23) => address(23), address(22)
                           => address(22), address(21) => address(21), 
                           address(20) => address(20), address(19) => 
                           address(19), address(18) => address(18), address(17)
                           => address(17), address(16) => address(16), 
                           address(15) => address(15), address(14) => 
                           address(14), address(13) => address(13), address(12)
                           => address(12), address(11) => address(11), 
                           address(10) => address(10), address(9) => address(9)
                           , address(8) => address(8), address(7) => address(7)
                           , address(6) => address(6), address(5) => address(5)
                           , address(4) => address(4), address(3) => address(3)
                           , address(2) => address(2), byte_we(3) => byte_we(3)
                           , byte_we(2) => byte_we(2), byte_we(1) => byte_we(1)
                           , byte_we(0) => byte_we(0), data_w(31) => data_w(31)
                           , data_w(30) => data_w(30), data_w(29) => data_w(29)
                           , data_w(28) => data_w(28), data_w(27) => data_w(27)
                           , data_w(26) => data_w(26), data_w(25) => data_w(25)
                           , data_w(24) => data_w(24), data_w(23) => data_w(23)
                           , data_w(22) => data_w(22), data_w(21) => data_w(21)
                           , data_w(20) => data_w(20), data_w(19) => data_w(19)
                           , data_w(18) => data_w(18), data_w(17) => data_w(17)
                           , data_w(16) => data_w(16), data_w(15) => data_w(15)
                           , data_w(14) => data_w(14), data_w(13) => data_w(13)
                           , data_w(12) => data_w(12), data_w(11) => data_w(11)
                           , data_w(10) => data_w(10), data_w(9) => data_w(9), 
                           data_w(8) => data_w(8), data_w(7) => data_w(7), 
                           data_w(6) => data_w(6), data_w(5) => data_w(5), 
                           data_w(4) => data_w(4), data_w(3) => data_w(3), 
                           data_w(2) => data_w(2), data_w(1) => data_w(1), 
                           data_w(0) => data_w(0), data_r(31) => data_r(31), 
                           data_r(30) => data_r(30), data_r(29) => data_r(29), 
                           data_r(28) => data_r(28), data_r(27) => data_r(27), 
                           data_r(26) => data_r(26), data_r(25) => data_r(25), 
                           data_r(24) => data_r(24), data_r(23) => data_r(23), 
                           data_r(22) => data_r(22), data_r(21) => data_r(21), 
                           data_r(20) => data_r(20), data_r(19) => data_r(19), 
                           data_r(18) => data_r(18), data_r(17) => data_r(17), 
                           data_r(16) => data_r(16), data_r(15) => data_r(15), 
                           data_r(14) => data_r(14), data_r(13) => data_r(13), 
                           data_r(12) => data_r(12), data_r(11) => data_r(11), 
                           data_r(10) => data_r(10), data_r(9) => data_r(9), 
                           data_r(8) => data_r(8), data_r(7) => data_r(7), 
                           data_r(6) => data_r(6), data_r(5) => data_r(5), 
                           data_r(4) => data_r(4), data_r(3) => data_r(3), 
                           data_r(2) => data_r(2), data_r(1) => data_r(1), 
                           data_r(0) => data_r(0));
   u3_control : control port map( opcode(31) => opcode_31_port, opcode(30) => 
                           opcode_30_port, opcode(29) => opcode_29_port, 
                           opcode(28) => opcode_28_port, opcode(27) => 
                           opcode_27_port, opcode(26) => opcode_26_port, 
                           opcode(25) => opcode_25_port, opcode(24) => 
                           opcode_24_port, opcode(23) => opcode_23_port, 
                           opcode(22) => opcode_22_port, opcode(21) => 
                           opcode_21_port, opcode(20) => opcode_20_port, 
                           opcode(19) => opcode_19_port, opcode(18) => 
                           opcode_18_port, opcode(17) => opcode_17_port, 
                           opcode(16) => opcode_16_port, opcode(15) => 
                           opcode_15_port, opcode(14) => opcode_14_port, 
                           opcode(13) => opcode_13_port, opcode(12) => 
                           opcode_12_port, opcode(11) => opcode_11_port, 
                           opcode(10) => opcode_10_port, opcode(9) => 
                           opcode_9_port, opcode(8) => opcode_8_port, opcode(7)
                           => opcode_7_port, opcode(6) => opcode_6_port, 
                           opcode(5) => opcode_5_port, opcode(4) => 
                           opcode_4_port, opcode(3) => opcode_3_port, opcode(2)
                           => opcode_2_port, opcode(1) => opcode_1_port, 
                           opcode(0) => opcode_0_port, intr_signal => 
                           intr_signal, rs_index(5) => rs_index_5_port, 
                           rs_index(4) => rs_index_4_port, rs_index(3) => 
                           rs_index_3_port, rs_index(2) => rs_index_2_port, 
                           rs_index(1) => rs_index_1_port, rs_index(0) => 
                           rs_index_0_port, rt_index(5) => n_1002, rt_index(4) 
                           => rt_index_4_port, rt_index(3) => rt_index_3_port, 
                           rt_index(2) => rt_index_2_port, rt_index(1) => 
                           rt_index_1_port, rt_index(0) => rt_index_0_port, 
                           rd_index(5) => rd_index_5_port, rd_index(4) => 
                           rd_index_4_port, rd_index(3) => rd_index_3_port, 
                           rd_index(2) => rd_index_2_port, rd_index(1) => 
                           rd_index_1_port, rd_index(0) => rd_index_0_port, 
                           imm_out(15) => imm_15_port, imm_out(14) => 
                           imm_14_port, imm_out(13) => imm_13_port, imm_out(12)
                           => imm_12_port, imm_out(11) => imm_11_port, 
                           imm_out(10) => imm_10_port, imm_out(9) => imm_9_port
                           , imm_out(8) => imm_8_port, imm_out(7) => imm_7_port
                           , imm_out(6) => imm_6_port, imm_out(5) => imm_5_port
                           , imm_out(4) => imm_4_port, imm_out(3) => imm_3_port
                           , imm_out(2) => imm_2_port, imm_out(1) => imm_1_port
                           , imm_out(0) => imm_0_port, alu_func(3) => 
                           alu_func_3_port, alu_func(2) => alu_func_2_port, 
                           alu_func(1) => alu_func_1_port, alu_func(0) => 
                           alu_func_0_port, shift_func(1) => shift_func_1_port,
                           shift_func(0) => shift_func_0_port, mult_func(3) => 
                           mult_func_3_port, mult_func(2) => mult_func_2_port, 
                           mult_func(1) => mult_func_1_port, mult_func(0) => 
                           mult_func_0_port, branch_func(2) => 
                           branch_func_2_port, branch_func(1) => 
                           branch_func_1_port, branch_func(0) => 
                           branch_func_0_port, a_source_out(1) => 
                           a_source_1_port, a_source_out(0) => a_source_0_port,
                           b_source_out(1) => b_source_1_port, b_source_out(0) 
                           => b_source_0_port, c_source_out(2) => 
                           c_source_2_port, c_source_out(1) => c_source_1_port,
                           c_source_out(0) => c_source_0_port, pc_source_out(1)
                           => pc_source_1_port, pc_source_out(0) => 
                           pc_source_0_port, mem_source_out(3) => 
                           mem_source_3_port, mem_source_out(2) => 
                           mem_source_2_port, mem_source_out(1) => 
                           mem_source_1_port, mem_source_out(0) => 
                           mem_source_0_port, exception_out => exception_sig);
   u4_reg_bank : reg_bank_entity port map( clk => clk, reset_in => reset, pause
                           => pause_bank, rs_index(5) => rs_index_5_port, 
                           rs_index(4) => rs_index_4_port, rs_index(3) => 
                           rs_index_3_port, rs_index(2) => rs_index_2_port, 
                           rs_index(1) => rs_index_1_port, rs_index(0) => 
                           rs_index_0_port, rt_index(5) => rt_index_5_port, 
                           rt_index(4) => rt_index_4_port, rt_index(3) => 
                           rt_index_3_port, rt_index(2) => rt_index_2_port, 
                           rt_index(1) => rt_index_1_port, rt_index(0) => 
                           rt_index_0_port, rd_index(5) => rd_indexD_5_port, 
                           rd_index(4) => rd_indexD_4_port, rd_index(3) => 
                           rd_indexD_3_port, rd_index(2) => rd_indexD_2_port, 
                           rd_index(1) => rd_indexD_1_port, rd_index(0) => 
                           rd_indexD_0_port, reg_source_out(31) => 
                           reg_source_31_port, reg_source_out(30) => 
                           reg_source_30_port, reg_source_out(29) => 
                           reg_source_29_port, reg_source_out(28) => 
                           reg_source_28_port, reg_source_out(27) => 
                           reg_source_27_port, reg_source_out(26) => 
                           reg_source_26_port, reg_source_out(25) => 
                           reg_source_25_port, reg_source_out(24) => 
                           reg_source_24_port, reg_source_out(23) => 
                           reg_source_23_port, reg_source_out(22) => 
                           reg_source_22_port, reg_source_out(21) => 
                           reg_source_21_port, reg_source_out(20) => 
                           reg_source_20_port, reg_source_out(19) => 
                           reg_source_19_port, reg_source_out(18) => 
                           reg_source_18_port, reg_source_out(17) => 
                           reg_source_17_port, reg_source_out(16) => 
                           reg_source_16_port, reg_source_out(15) => 
                           reg_source_15_port, reg_source_out(14) => 
                           reg_source_14_port, reg_source_out(13) => 
                           reg_source_13_port, reg_source_out(12) => 
                           reg_source_12_port, reg_source_out(11) => 
                           reg_source_11_port, reg_source_out(10) => 
                           reg_source_10_port, reg_source_out(9) => 
                           reg_source_9_port, reg_source_out(8) => 
                           reg_source_8_port, reg_source_out(7) => 
                           reg_source_7_port, reg_source_out(6) => 
                           reg_source_6_port, reg_source_out(5) => 
                           reg_source_5_port, reg_source_out(4) => 
                           reg_source_4_port, reg_source_out(3) => 
                           reg_source_3_port, reg_source_out(2) => 
                           reg_source_2_port, reg_source_out(1) => 
                           reg_source_1_port, reg_source_out(0) => 
                           reg_source_0_port, reg_target_out(31) => 
                           reg_target_31_port, reg_target_out(30) => 
                           reg_target_30_port, reg_target_out(29) => 
                           reg_target_29_port, reg_target_out(28) => 
                           reg_target_28_port, reg_target_out(27) => 
                           reg_target_27_port, reg_target_out(26) => 
                           reg_target_26_port, reg_target_out(25) => 
                           reg_target_25_port, reg_target_out(24) => 
                           reg_target_24_port, reg_target_out(23) => 
                           reg_target_23_port, reg_target_out(22) => 
                           reg_target_22_port, reg_target_out(21) => 
                           reg_target_21_port, reg_target_out(20) => 
                           reg_target_20_port, reg_target_out(19) => 
                           reg_target_19_port, reg_target_out(18) => 
                           reg_target_18_port, reg_target_out(17) => 
                           reg_target_17_port, reg_target_out(16) => 
                           reg_target_16_port, reg_target_out(15) => 
                           reg_target_15_port, reg_target_out(14) => 
                           reg_target_14_port, reg_target_out(13) => 
                           reg_target_13_port, reg_target_out(12) => 
                           reg_target_12_port, reg_target_out(11) => 
                           reg_target_11_port, reg_target_out(10) => 
                           reg_target_10_port, reg_target_out(9) => 
                           reg_target_9_port, reg_target_out(8) => 
                           reg_target_8_port, reg_target_out(7) => 
                           reg_target_7_port, reg_target_out(6) => 
                           reg_target_6_port, reg_target_out(5) => 
                           reg_target_5_port, reg_target_out(4) => 
                           reg_target_4_port, reg_target_out(3) => 
                           reg_target_3_port, reg_target_out(2) => 
                           reg_target_2_port, reg_target_out(1) => 
                           reg_target_1_port, reg_target_out(0) => 
                           reg_target_0_port, reg_dest_new(31) => 
                           reg_destD_31_port, reg_dest_new(30) => 
                           reg_destD_30_port, reg_dest_new(29) => 
                           reg_destD_29_port, reg_dest_new(28) => 
                           reg_destD_28_port, reg_dest_new(27) => 
                           reg_destD_27_port, reg_dest_new(26) => 
                           reg_destD_26_port, reg_dest_new(25) => 
                           reg_destD_25_port, reg_dest_new(24) => 
                           reg_destD_24_port, reg_dest_new(23) => 
                           reg_destD_23_port, reg_dest_new(22) => 
                           reg_destD_22_port, reg_dest_new(21) => 
                           reg_destD_21_port, reg_dest_new(20) => 
                           reg_destD_20_port, reg_dest_new(19) => 
                           reg_destD_19_port, reg_dest_new(18) => 
                           reg_destD_18_port, reg_dest_new(17) => 
                           reg_destD_17_port, reg_dest_new(16) => 
                           reg_destD_16_port, reg_dest_new(15) => 
                           reg_destD_15_port, reg_dest_new(14) => 
                           reg_destD_14_port, reg_dest_new(13) => 
                           reg_destD_13_port, reg_dest_new(12) => 
                           reg_destD_12_port, reg_dest_new(11) => 
                           reg_destD_11_port, reg_dest_new(10) => 
                           reg_destD_10_port, reg_dest_new(9) => 
                           reg_destD_9_port, reg_dest_new(8) => 
                           reg_destD_8_port, reg_dest_new(7) => 
                           reg_destD_7_port, reg_dest_new(6) => 
                           reg_destD_6_port, reg_dest_new(5) => 
                           reg_destD_5_port, reg_dest_new(4) => 
                           reg_destD_4_port, reg_dest_new(3) => 
                           reg_destD_3_port, reg_dest_new(2) => 
                           reg_destD_2_port, reg_dest_new(1) => 
                           reg_destD_1_port, reg_dest_new(0) => 
                           reg_destD_0_port, intr_enable => intr_enable);
   u5_bus_mux : bus_mux port map( imm_in(15) => imm_15_port, imm_in(14) => 
                           imm_14_port, imm_in(13) => imm_13_port, imm_in(12) 
                           => imm_12_port, imm_in(11) => imm_11_port, 
                           imm_in(10) => imm_10_port, imm_in(9) => imm_9_port, 
                           imm_in(8) => imm_8_port, imm_in(7) => imm_7_port, 
                           imm_in(6) => imm_6_port, imm_in(5) => imm_5_port, 
                           imm_in(4) => imm_4_port, imm_in(3) => imm_3_port, 
                           imm_in(2) => imm_2_port, imm_in(1) => imm_1_port, 
                           imm_in(0) => imm_0_port, reg_source(31) => 
                           reg_source_31_port, reg_source(30) => 
                           reg_source_30_port, reg_source(29) => 
                           reg_source_29_port, reg_source(28) => 
                           reg_source_28_port, reg_source(27) => 
                           reg_source_27_port, reg_source(26) => 
                           reg_source_26_port, reg_source(25) => 
                           reg_source_25_port, reg_source(24) => 
                           reg_source_24_port, reg_source(23) => 
                           reg_source_23_port, reg_source(22) => 
                           reg_source_22_port, reg_source(21) => 
                           reg_source_21_port, reg_source(20) => 
                           reg_source_20_port, reg_source(19) => 
                           reg_source_19_port, reg_source(18) => 
                           reg_source_18_port, reg_source(17) => 
                           reg_source_17_port, reg_source(16) => 
                           reg_source_16_port, reg_source(15) => 
                           reg_source_15_port, reg_source(14) => 
                           reg_source_14_port, reg_source(13) => 
                           reg_source_13_port, reg_source(12) => 
                           reg_source_12_port, reg_source(11) => 
                           reg_source_11_port, reg_source(10) => 
                           reg_source_10_port, reg_source(9) => 
                           reg_source_9_port, reg_source(8) => 
                           reg_source_8_port, reg_source(7) => 
                           reg_source_7_port, reg_source(6) => 
                           reg_source_6_port, reg_source(5) => 
                           reg_source_5_port, reg_source(4) => 
                           reg_source_4_port, reg_source(3) => 
                           reg_source_3_port, reg_source(2) => 
                           reg_source_2_port, reg_source(1) => 
                           reg_source_1_port, reg_source(0) => 
                           reg_source_0_port, a_mux(1) => a_source_1_port, 
                           a_mux(0) => a_source_0_port, a_out(31) => 
                           a_bus_31_port, a_out(30) => a_bus_30_port, a_out(29)
                           => a_bus_29_port, a_out(28) => a_bus_28_port, 
                           a_out(27) => a_bus_27_port, a_out(26) => 
                           a_bus_26_port, a_out(25) => a_bus_25_port, a_out(24)
                           => a_bus_24_port, a_out(23) => a_bus_23_port, 
                           a_out(22) => a_bus_22_port, a_out(21) => 
                           a_bus_21_port, a_out(20) => a_bus_20_port, a_out(19)
                           => a_bus_19_port, a_out(18) => a_bus_18_port, 
                           a_out(17) => a_bus_17_port, a_out(16) => 
                           a_bus_16_port, a_out(15) => a_bus_15_port, a_out(14)
                           => a_bus_14_port, a_out(13) => a_bus_13_port, 
                           a_out(12) => a_bus_12_port, a_out(11) => 
                           a_bus_11_port, a_out(10) => a_bus_10_port, a_out(9) 
                           => a_bus_9_port, a_out(8) => a_bus_8_port, a_out(7) 
                           => a_bus_7_port, a_out(6) => a_bus_6_port, a_out(5) 
                           => a_bus_5_port, a_out(4) => a_bus_4_port, a_out(3) 
                           => a_bus_3_port, a_out(2) => a_bus_2_port, a_out(1) 
                           => a_bus_1_port, a_out(0) => a_bus_0_port, 
                           reg_target(31) => reg_target_31_port, reg_target(30)
                           => reg_target_30_port, reg_target(29) => 
                           reg_target_29_port, reg_target(28) => 
                           reg_target_28_port, reg_target(27) => 
                           reg_target_27_port, reg_target(26) => 
                           reg_target_26_port, reg_target(25) => 
                           reg_target_25_port, reg_target(24) => 
                           reg_target_24_port, reg_target(23) => 
                           reg_target_23_port, reg_target(22) => 
                           reg_target_22_port, reg_target(21) => 
                           reg_target_21_port, reg_target(20) => 
                           reg_target_20_port, reg_target(19) => 
                           reg_target_19_port, reg_target(18) => 
                           reg_target_18_port, reg_target(17) => 
                           reg_target_17_port, reg_target(16) => 
                           reg_target_16_port, reg_target(15) => 
                           reg_target_15_port, reg_target(14) => 
                           reg_target_14_port, reg_target(13) => 
                           reg_target_13_port, reg_target(12) => 
                           reg_target_12_port, reg_target(11) => 
                           reg_target_11_port, reg_target(10) => 
                           reg_target_10_port, reg_target(9) => 
                           reg_target_9_port, reg_target(8) => 
                           reg_target_8_port, reg_target(7) => 
                           reg_target_7_port, reg_target(6) => 
                           reg_target_6_port, reg_target(5) => 
                           reg_target_5_port, reg_target(4) => 
                           reg_target_4_port, reg_target(3) => 
                           reg_target_3_port, reg_target(2) => 
                           reg_target_2_port, reg_target(1) => 
                           reg_target_1_port, reg_target(0) => 
                           reg_target_0_port, b_mux(1) => b_source_1_port, 
                           b_mux(0) => b_source_0_port, b_out(31) => 
                           b_bus_31_port, b_out(30) => b_bus_30_port, b_out(29)
                           => b_bus_29_port, b_out(28) => b_bus_28_port, 
                           b_out(27) => b_bus_27_port, b_out(26) => 
                           b_bus_26_port, b_out(25) => b_bus_25_port, b_out(24)
                           => b_bus_24_port, b_out(23) => b_bus_23_port, 
                           b_out(22) => b_bus_22_port, b_out(21) => 
                           b_bus_21_port, b_out(20) => b_bus_20_port, b_out(19)
                           => b_bus_19_port, b_out(18) => b_bus_18_port, 
                           b_out(17) => b_bus_17_port, b_out(16) => 
                           b_bus_16_port, b_out(15) => b_bus_15_port, b_out(14)
                           => b_bus_14_port, b_out(13) => b_bus_13_port, 
                           b_out(12) => b_bus_12_port, b_out(11) => 
                           b_bus_11_port, b_out(10) => b_bus_10_port, b_out(9) 
                           => b_bus_9_port, b_out(8) => b_bus_8_port, b_out(7) 
                           => b_bus_7_port, b_out(6) => b_bus_6_port, b_out(5) 
                           => b_bus_5_port, b_out(4) => b_bus_4_port, b_out(3) 
                           => b_bus_3_port, b_out(2) => b_bus_2_port, b_out(1) 
                           => b_bus_1_port, b_out(0) => b_bus_0_port, c_bus(31)
                           => c_bus_31_port, c_bus(30) => c_bus_30_port, 
                           c_bus(29) => c_bus_29_port, c_bus(28) => 
                           c_bus_28_port, c_bus(27) => c_bus_27_port, c_bus(26)
                           => c_bus_26_port, c_bus(25) => c_bus_25_port, 
                           c_bus(24) => c_bus_24_port, c_bus(23) => 
                           c_bus_23_port, c_bus(22) => c_bus_22_port, c_bus(21)
                           => c_bus_21_port, c_bus(20) => c_bus_20_port, 
                           c_bus(19) => c_bus_19_port, c_bus(18) => 
                           c_bus_18_port, c_bus(17) => c_bus_17_port, c_bus(16)
                           => c_bus_16_port, c_bus(15) => c_bus_15_port, 
                           c_bus(14) => c_bus_14_port, c_bus(13) => 
                           c_bus_13_port, c_bus(12) => c_bus_12_port, c_bus(11)
                           => c_bus_11_port, c_bus(10) => c_bus_10_port, 
                           c_bus(9) => c_bus_9_port, c_bus(8) => c_bus_8_port, 
                           c_bus(7) => c_bus_7_port, c_bus(6) => c_bus_6_port, 
                           c_bus(5) => c_bus_5_port, c_bus(4) => c_bus_4_port, 
                           c_bus(3) => c_bus_3_port, c_bus(2) => c_bus_2_port, 
                           c_bus(1) => c_bus_1_port, c_bus(0) => c_bus_0_port, 
                           c_memory(31) => c_memory_31_port, c_memory(30) => 
                           c_memory_30_port, c_memory(29) => c_memory_29_port, 
                           c_memory(28) => c_memory_28_port, c_memory(27) => 
                           c_memory_27_port, c_memory(26) => c_memory_26_port, 
                           c_memory(25) => c_memory_25_port, c_memory(24) => 
                           c_memory_24_port, c_memory(23) => c_memory_23_port, 
                           c_memory(22) => c_memory_22_port, c_memory(21) => 
                           c_memory_21_port, c_memory(20) => c_memory_20_port, 
                           c_memory(19) => c_memory_19_port, c_memory(18) => 
                           c_memory_18_port, c_memory(17) => c_memory_17_port, 
                           c_memory(16) => c_memory_16_port, c_memory(15) => 
                           c_memory_15_port, c_memory(14) => c_memory_14_port, 
                           c_memory(13) => c_memory_13_port, c_memory(12) => 
                           c_memory_12_port, c_memory(11) => c_memory_11_port, 
                           c_memory(10) => c_memory_10_port, c_memory(9) => 
                           c_memory_9_port, c_memory(8) => c_memory_8_port, 
                           c_memory(7) => c_memory_7_port, c_memory(6) => 
                           c_memory_6_port, c_memory(5) => c_memory_5_port, 
                           c_memory(4) => c_memory_4_port, c_memory(3) => 
                           c_memory_3_port, c_memory(2) => c_memory_2_port, 
                           c_memory(1) => c_memory_1_port, c_memory(0) => 
                           c_memory_0_port, c_pc(31) => pc_current_31_port, 
                           c_pc(30) => pc_current_30_port, c_pc(29) => 
                           pc_current_29_port, c_pc(28) => pc_current_28_port, 
                           c_pc(27) => pc_current_27_port, c_pc(26) => 
                           pc_current_26_port, c_pc(25) => pc_current_25_port, 
                           c_pc(24) => pc_current_24_port, c_pc(23) => 
                           pc_current_23_port, c_pc(22) => pc_current_22_port, 
                           c_pc(21) => pc_current_21_port, c_pc(20) => 
                           pc_current_20_port, c_pc(19) => pc_current_19_port, 
                           c_pc(18) => pc_current_18_port, c_pc(17) => 
                           pc_current_17_port, c_pc(16) => pc_current_16_port, 
                           c_pc(15) => pc_current_15_port, c_pc(14) => 
                           pc_current_14_port, c_pc(13) => pc_current_13_port, 
                           c_pc(12) => pc_current_12_port, c_pc(11) => 
                           pc_current_11_port, c_pc(10) => pc_current_10_port, 
                           c_pc(9) => pc_current_9_port, c_pc(8) => 
                           pc_current_8_port, c_pc(7) => pc_current_7_port, 
                           c_pc(6) => pc_current_6_port, c_pc(5) => 
                           pc_current_5_port, c_pc(4) => pc_current_4_port, 
                           c_pc(3) => pc_current_3_port, c_pc(2) => 
                           pc_current_2_port, c_pc_plus4(31) => 
                           pc_plus4_31_port, c_pc_plus4(30) => pc_plus4_30_port
                           , c_pc_plus4(29) => pc_plus4_29_port, c_pc_plus4(28)
                           => pc_plus4_28_port, c_pc_plus4(27) => 
                           pc_plus4_27_port, c_pc_plus4(26) => pc_plus4_26_port
                           , c_pc_plus4(25) => pc_plus4_25_port, c_pc_plus4(24)
                           => pc_plus4_24_port, c_pc_plus4(23) => 
                           pc_plus4_23_port, c_pc_plus4(22) => pc_plus4_22_port
                           , c_pc_plus4(21) => pc_plus4_21_port, c_pc_plus4(20)
                           => pc_plus4_20_port, c_pc_plus4(19) => 
                           pc_plus4_19_port, c_pc_plus4(18) => pc_plus4_18_port
                           , c_pc_plus4(17) => pc_plus4_17_port, c_pc_plus4(16)
                           => pc_plus4_16_port, c_pc_plus4(15) => 
                           pc_plus4_15_port, c_pc_plus4(14) => pc_plus4_14_port
                           , c_pc_plus4(13) => pc_plus4_13_port, c_pc_plus4(12)
                           => pc_plus4_12_port, c_pc_plus4(11) => 
                           pc_plus4_11_port, c_pc_plus4(10) => pc_plus4_10_port
                           , c_pc_plus4(9) => pc_plus4_9_port, c_pc_plus4(8) =>
                           pc_plus4_8_port, c_pc_plus4(7) => pc_plus4_7_port, 
                           c_pc_plus4(6) => pc_plus4_6_port, c_pc_plus4(5) => 
                           pc_plus4_5_port, c_pc_plus4(4) => pc_plus4_4_port, 
                           c_pc_plus4(3) => pc_plus4_3_port, c_pc_plus4(2) => 
                           pc_plus4_2_port, c_mux(2) => c_source_2_port, 
                           c_mux(1) => c_source_1_port, c_mux(0) => 
                           c_source_0_port, reg_dest_out(31) => 
                           reg_dest_31_port, reg_dest_out(30) => 
                           reg_dest_30_port, reg_dest_out(29) => 
                           reg_dest_29_port, reg_dest_out(28) => 
                           reg_dest_28_port, reg_dest_out(27) => 
                           reg_dest_27_port, reg_dest_out(26) => 
                           reg_dest_26_port, reg_dest_out(25) => 
                           reg_dest_25_port, reg_dest_out(24) => 
                           reg_dest_24_port, reg_dest_out(23) => 
                           reg_dest_23_port, reg_dest_out(22) => 
                           reg_dest_22_port, reg_dest_out(21) => 
                           reg_dest_21_port, reg_dest_out(20) => 
                           reg_dest_20_port, reg_dest_out(19) => 
                           reg_dest_19_port, reg_dest_out(18) => 
                           reg_dest_18_port, reg_dest_out(17) => 
                           reg_dest_17_port, reg_dest_out(16) => 
                           reg_dest_16_port, reg_dest_out(15) => 
                           reg_dest_15_port, reg_dest_out(14) => 
                           reg_dest_14_port, reg_dest_out(13) => 
                           reg_dest_13_port, reg_dest_out(12) => 
                           reg_dest_12_port, reg_dest_out(11) => 
                           reg_dest_11_port, reg_dest_out(10) => 
                           reg_dest_10_port, reg_dest_out(9) => reg_dest_9_port
                           , reg_dest_out(8) => reg_dest_8_port, 
                           reg_dest_out(7) => reg_dest_7_port, reg_dest_out(6) 
                           => reg_dest_6_port, reg_dest_out(5) => 
                           reg_dest_5_port, reg_dest_out(4) => reg_dest_4_port,
                           reg_dest_out(3) => reg_dest_3_port, reg_dest_out(2) 
                           => reg_dest_2_port, reg_dest_out(1) => 
                           reg_dest_1_port, reg_dest_out(0) => reg_dest_0_port,
                           branch_func(2) => branch_func_2_port, branch_func(1)
                           => branch_func_1_port, branch_func(0) => 
                           branch_func_0_port, take_branch => take_branch);
   u6_alu : alu_alu_typeDEFAULT port map( a_in(31) => a_busD_31_port, a_in(30) 
                           => a_busD_30_port, a_in(29) => a_busD_29_port, 
                           a_in(28) => a_busD_28_port, a_in(27) => 
                           a_busD_27_port, a_in(26) => a_busD_26_port, a_in(25)
                           => a_busD_25_port, a_in(24) => a_busD_24_port, 
                           a_in(23) => a_busD_23_port, a_in(22) => 
                           a_busD_22_port, a_in(21) => a_busD_21_port, a_in(20)
                           => a_busD_20_port, a_in(19) => a_busD_19_port, 
                           a_in(18) => a_busD_18_port, a_in(17) => 
                           a_busD_17_port, a_in(16) => a_busD_16_port, a_in(15)
                           => a_busD_15_port, a_in(14) => a_busD_14_port, 
                           a_in(13) => a_busD_13_port, a_in(12) => 
                           a_busD_12_port, a_in(11) => a_busD_11_port, a_in(10)
                           => a_busD_10_port, a_in(9) => a_busD_9_port, a_in(8)
                           => a_busD_8_port, a_in(7) => a_busD_7_port, a_in(6) 
                           => a_busD_6_port, a_in(5) => a_busD_5_port, a_in(4) 
                           => a_busD_4_port, a_in(3) => a_busD_3_port, a_in(2) 
                           => a_busD_2_port, a_in(1) => a_busD_1_port, a_in(0) 
                           => a_busD_0_port, b_in(31) => b_busD_31_port, 
                           b_in(30) => b_busD_30_port, b_in(29) => 
                           b_busD_29_port, b_in(28) => b_busD_28_port, b_in(27)
                           => b_busD_27_port, b_in(26) => b_busD_26_port, 
                           b_in(25) => b_busD_25_port, b_in(24) => 
                           b_busD_24_port, b_in(23) => b_busD_23_port, b_in(22)
                           => b_busD_22_port, b_in(21) => b_busD_21_port, 
                           b_in(20) => b_busD_20_port, b_in(19) => 
                           b_busD_19_port, b_in(18) => b_busD_18_port, b_in(17)
                           => b_busD_17_port, b_in(16) => b_busD_16_port, 
                           b_in(15) => b_busD_15_port, b_in(14) => 
                           b_busD_14_port, b_in(13) => b_busD_13_port, b_in(12)
                           => b_busD_12_port, b_in(11) => b_busD_11_port, 
                           b_in(10) => b_busD_10_port, b_in(9) => b_busD_9_port
                           , b_in(8) => b_busD_8_port, b_in(7) => b_busD_7_port
                           , b_in(6) => b_busD_6_port, b_in(5) => b_busD_5_port
                           , b_in(4) => b_busD_4_port, b_in(3) => b_busD_3_port
                           , b_in(2) => b_busD_2_port, b_in(1) => b_busD_1_port
                           , b_in(0) => b_busD_0_port, alu_function(3) => 
                           alu_funcD_3_port, alu_function(2) => 
                           alu_funcD_2_port, alu_function(1) => 
                           alu_funcD_1_port, alu_function(0) => 
                           alu_funcD_0_port, c_alu(31) => c_alu_31_port, 
                           c_alu(30) => c_alu_30_port, c_alu(29) => 
                           c_alu_29_port, c_alu(28) => c_alu_28_port, c_alu(27)
                           => c_alu_27_port, c_alu(26) => c_alu_26_port, 
                           c_alu(25) => c_alu_25_port, c_alu(24) => 
                           c_alu_24_port, c_alu(23) => c_alu_23_port, c_alu(22)
                           => c_alu_22_port, c_alu(21) => c_alu_21_port, 
                           c_alu(20) => c_alu_20_port, c_alu(19) => 
                           c_alu_19_port, c_alu(18) => c_alu_18_port, c_alu(17)
                           => c_alu_17_port, c_alu(16) => c_alu_16_port, 
                           c_alu(15) => c_alu_15_port, c_alu(14) => 
                           c_alu_14_port, c_alu(13) => c_alu_13_port, c_alu(12)
                           => c_alu_12_port, c_alu(11) => c_alu_11_port, 
                           c_alu(10) => c_alu_10_port, c_alu(9) => c_alu_9_port
                           , c_alu(8) => c_alu_8_port, c_alu(7) => c_alu_7_port
                           , c_alu(6) => c_alu_6_port, c_alu(5) => c_alu_5_port
                           , c_alu(4) => c_alu_4_port, c_alu(3) => c_alu_3_port
                           , c_alu(2) => c_alu_2_port, c_alu(1) => c_alu_1_port
                           , c_alu(0) => c_alu_0_port);
   u7_shifter : shifter_shifter_typeDEFAULT port map( value(31) => 
                           b_busD_31_port, value(30) => b_busD_30_port, 
                           value(29) => b_busD_29_port, value(28) => 
                           b_busD_28_port, value(27) => b_busD_27_port, 
                           value(26) => b_busD_26_port, value(25) => 
                           b_busD_25_port, value(24) => b_busD_24_port, 
                           value(23) => b_busD_23_port, value(22) => 
                           b_busD_22_port, value(21) => b_busD_21_port, 
                           value(20) => b_busD_20_port, value(19) => 
                           b_busD_19_port, value(18) => b_busD_18_port, 
                           value(17) => b_busD_17_port, value(16) => 
                           b_busD_16_port, value(15) => b_busD_15_port, 
                           value(14) => b_busD_14_port, value(13) => 
                           b_busD_13_port, value(12) => b_busD_12_port, 
                           value(11) => b_busD_11_port, value(10) => 
                           b_busD_10_port, value(9) => b_busD_9_port, value(8) 
                           => b_busD_8_port, value(7) => b_busD_7_port, 
                           value(6) => b_busD_6_port, value(5) => b_busD_5_port
                           , value(4) => b_busD_4_port, value(3) => 
                           b_busD_3_port, value(2) => b_busD_2_port, value(1) 
                           => b_busD_1_port, value(0) => b_busD_0_port, 
                           shift_amount(4) => a_busD_4_port, shift_amount(3) =>
                           a_busD_3_port, shift_amount(2) => a_busD_2_port, 
                           shift_amount(1) => a_busD_1_port, shift_amount(0) =>
                           a_busD_0_port, shift_func(1) => shift_funcD_1_port, 
                           shift_func(0) => shift_funcD_0_port, c_shift(31) => 
                           c_shift_31_port, c_shift(30) => c_shift_30_port, 
                           c_shift(29) => c_shift_29_port, c_shift(28) => 
                           c_shift_28_port, c_shift(27) => c_shift_27_port, 
                           c_shift(26) => c_shift_26_port, c_shift(25) => 
                           c_shift_25_port, c_shift(24) => c_shift_24_port, 
                           c_shift(23) => c_shift_23_port, c_shift(22) => 
                           c_shift_22_port, c_shift(21) => c_shift_21_port, 
                           c_shift(20) => c_shift_20_port, c_shift(19) => 
                           c_shift_19_port, c_shift(18) => c_shift_18_port, 
                           c_shift(17) => c_shift_17_port, c_shift(16) => 
                           c_shift_16_port, c_shift(15) => c_shift_15_port, 
                           c_shift(14) => c_shift_14_port, c_shift(13) => 
                           c_shift_13_port, c_shift(12) => c_shift_12_port, 
                           c_shift(11) => c_shift_11_port, c_shift(10) => 
                           c_shift_10_port, c_shift(9) => c_shift_9_port, 
                           c_shift(8) => c_shift_8_port, c_shift(7) => 
                           c_shift_7_port, c_shift(6) => c_shift_6_port, 
                           c_shift(5) => c_shift_5_port, c_shift(4) => 
                           c_shift_4_port, c_shift(3) => c_shift_3_port, 
                           c_shift(2) => c_shift_2_port, c_shift(1) => 
                           c_shift_1_port, c_shift(0) => c_shift_0_port);
   u8_mult : mult_mult_typeDEFAULT port map( clk => clk, reset_in => reset, 
                           a(31) => a_busD_31_port, a(30) => a_busD_30_port, 
                           a(29) => a_busD_29_port, a(28) => a_busD_28_port, 
                           a(27) => a_busD_27_port, a(26) => a_busD_26_port, 
                           a(25) => a_busD_25_port, a(24) => a_busD_24_port, 
                           a(23) => a_busD_23_port, a(22) => a_busD_22_port, 
                           a(21) => a_busD_21_port, a(20) => a_busD_20_port, 
                           a(19) => a_busD_19_port, a(18) => a_busD_18_port, 
                           a(17) => a_busD_17_port, a(16) => a_busD_16_port, 
                           a(15) => a_busD_15_port, a(14) => a_busD_14_port, 
                           a(13) => a_busD_13_port, a(12) => a_busD_12_port, 
                           a(11) => a_busD_11_port, a(10) => a_busD_10_port, 
                           a(9) => a_busD_9_port, a(8) => a_busD_8_port, a(7) 
                           => a_busD_7_port, a(6) => a_busD_6_port, a(5) => 
                           a_busD_5_port, a(4) => a_busD_4_port, a(3) => 
                           a_busD_3_port, a(2) => a_busD_2_port, a(1) => 
                           a_busD_1_port, a(0) => a_busD_0_port, b(31) => 
                           b_busD_31_port, b(30) => b_busD_30_port, b(29) => 
                           b_busD_29_port, b(28) => b_busD_28_port, b(27) => 
                           b_busD_27_port, b(26) => b_busD_26_port, b(25) => 
                           b_busD_25_port, b(24) => b_busD_24_port, b(23) => 
                           b_busD_23_port, b(22) => b_busD_22_port, b(21) => 
                           b_busD_21_port, b(20) => b_busD_20_port, b(19) => 
                           b_busD_19_port, b(18) => b_busD_18_port, b(17) => 
                           b_busD_17_port, b(16) => b_busD_16_port, b(15) => 
                           b_busD_15_port, b(14) => b_busD_14_port, b(13) => 
                           b_busD_13_port, b(12) => b_busD_12_port, b(11) => 
                           b_busD_11_port, b(10) => b_busD_10_port, b(9) => 
                           b_busD_9_port, b(8) => b_busD_8_port, b(7) => 
                           b_busD_7_port, b(6) => b_busD_6_port, b(5) => 
                           b_busD_5_port, b(4) => b_busD_4_port, b(3) => 
                           b_busD_3_port, b(2) => b_busD_2_port, b(1) => 
                           b_busD_1_port, b(0) => b_busD_0_port, mult_func(3) 
                           => mult_funcD_3_port, mult_func(2) => 
                           mult_funcD_2_port, mult_func(1) => mult_funcD_1_port
                           , mult_func(0) => mult_funcD_0_port, c_mult(31) => 
                           c_mult_31_port, c_mult(30) => c_mult_30_port, 
                           c_mult(29) => c_mult_29_port, c_mult(28) => 
                           c_mult_28_port, c_mult(27) => c_mult_27_port, 
                           c_mult(26) => c_mult_26_port, c_mult(25) => 
                           c_mult_25_port, c_mult(24) => c_mult_24_port, 
                           c_mult(23) => c_mult_23_port, c_mult(22) => 
                           c_mult_22_port, c_mult(21) => c_mult_21_port, 
                           c_mult(20) => c_mult_20_port, c_mult(19) => 
                           c_mult_19_port, c_mult(18) => c_mult_18_port, 
                           c_mult(17) => c_mult_17_port, c_mult(16) => 
                           c_mult_16_port, c_mult(15) => c_mult_15_port, 
                           c_mult(14) => c_mult_14_port, c_mult(13) => 
                           c_mult_13_port, c_mult(12) => c_mult_12_port, 
                           c_mult(11) => c_mult_11_port, c_mult(10) => 
                           c_mult_10_port, c_mult(9) => c_mult_9_port, 
                           c_mult(8) => c_mult_8_port, c_mult(7) => 
                           c_mult_7_port, c_mult(6) => c_mult_6_port, c_mult(5)
                           => c_mult_5_port, c_mult(4) => c_mult_4_port, 
                           c_mult(3) => c_mult_3_port, c_mult(2) => 
                           c_mult_2_port, c_mult(1) => c_mult_1_port, c_mult(0)
                           => c_mult_0_port, pause_out => pause_mult);
   u9_pipeline : pipeline port map( clk => clk, reset => reset, a_bus(31) => 
                           a_bus_31_port, a_bus(30) => a_bus_30_port, a_bus(29)
                           => a_bus_29_port, a_bus(28) => a_bus_28_port, 
                           a_bus(27) => a_bus_27_port, a_bus(26) => 
                           a_bus_26_port, a_bus(25) => a_bus_25_port, a_bus(24)
                           => a_bus_24_port, a_bus(23) => a_bus_23_port, 
                           a_bus(22) => a_bus_22_port, a_bus(21) => 
                           a_bus_21_port, a_bus(20) => a_bus_20_port, a_bus(19)
                           => a_bus_19_port, a_bus(18) => a_bus_18_port, 
                           a_bus(17) => a_bus_17_port, a_bus(16) => 
                           a_bus_16_port, a_bus(15) => a_bus_15_port, a_bus(14)
                           => a_bus_14_port, a_bus(13) => a_bus_13_port, 
                           a_bus(12) => a_bus_12_port, a_bus(11) => 
                           a_bus_11_port, a_bus(10) => a_bus_10_port, a_bus(9) 
                           => a_bus_9_port, a_bus(8) => a_bus_8_port, a_bus(7) 
                           => a_bus_7_port, a_bus(6) => a_bus_6_port, a_bus(5) 
                           => a_bus_5_port, a_bus(4) => a_bus_4_port, a_bus(3) 
                           => a_bus_3_port, a_bus(2) => a_bus_2_port, a_bus(1) 
                           => a_bus_1_port, a_bus(0) => a_bus_0_port, 
                           a_busD(31) => a_busD_31_port, a_busD(30) => 
                           a_busD_30_port, a_busD(29) => a_busD_29_port, 
                           a_busD(28) => a_busD_28_port, a_busD(27) => 
                           a_busD_27_port, a_busD(26) => a_busD_26_port, 
                           a_busD(25) => a_busD_25_port, a_busD(24) => 
                           a_busD_24_port, a_busD(23) => a_busD_23_port, 
                           a_busD(22) => a_busD_22_port, a_busD(21) => 
                           a_busD_21_port, a_busD(20) => a_busD_20_port, 
                           a_busD(19) => a_busD_19_port, a_busD(18) => 
                           a_busD_18_port, a_busD(17) => a_busD_17_port, 
                           a_busD(16) => a_busD_16_port, a_busD(15) => 
                           a_busD_15_port, a_busD(14) => a_busD_14_port, 
                           a_busD(13) => a_busD_13_port, a_busD(12) => 
                           a_busD_12_port, a_busD(11) => a_busD_11_port, 
                           a_busD(10) => a_busD_10_port, a_busD(9) => 
                           a_busD_9_port, a_busD(8) => a_busD_8_port, a_busD(7)
                           => a_busD_7_port, a_busD(6) => a_busD_6_port, 
                           a_busD(5) => a_busD_5_port, a_busD(4) => 
                           a_busD_4_port, a_busD(3) => a_busD_3_port, a_busD(2)
                           => a_busD_2_port, a_busD(1) => a_busD_1_port, 
                           a_busD(0) => a_busD_0_port, b_bus(31) => 
                           b_bus_31_port, b_bus(30) => b_bus_30_port, b_bus(29)
                           => b_bus_29_port, b_bus(28) => b_bus_28_port, 
                           b_bus(27) => b_bus_27_port, b_bus(26) => 
                           b_bus_26_port, b_bus(25) => b_bus_25_port, b_bus(24)
                           => b_bus_24_port, b_bus(23) => b_bus_23_port, 
                           b_bus(22) => b_bus_22_port, b_bus(21) => 
                           b_bus_21_port, b_bus(20) => b_bus_20_port, b_bus(19)
                           => b_bus_19_port, b_bus(18) => b_bus_18_port, 
                           b_bus(17) => b_bus_17_port, b_bus(16) => 
                           b_bus_16_port, b_bus(15) => b_bus_15_port, b_bus(14)
                           => b_bus_14_port, b_bus(13) => b_bus_13_port, 
                           b_bus(12) => b_bus_12_port, b_bus(11) => 
                           b_bus_11_port, b_bus(10) => b_bus_10_port, b_bus(9) 
                           => b_bus_9_port, b_bus(8) => b_bus_8_port, b_bus(7) 
                           => b_bus_7_port, b_bus(6) => b_bus_6_port, b_bus(5) 
                           => b_bus_5_port, b_bus(4) => b_bus_4_port, b_bus(3) 
                           => b_bus_3_port, b_bus(2) => b_bus_2_port, b_bus(1) 
                           => b_bus_1_port, b_bus(0) => b_bus_0_port, 
                           b_busD(31) => b_busD_31_port, b_busD(30) => 
                           b_busD_30_port, b_busD(29) => b_busD_29_port, 
                           b_busD(28) => b_busD_28_port, b_busD(27) => 
                           b_busD_27_port, b_busD(26) => b_busD_26_port, 
                           b_busD(25) => b_busD_25_port, b_busD(24) => 
                           b_busD_24_port, b_busD(23) => b_busD_23_port, 
                           b_busD(22) => b_busD_22_port, b_busD(21) => 
                           b_busD_21_port, b_busD(20) => b_busD_20_port, 
                           b_busD(19) => b_busD_19_port, b_busD(18) => 
                           b_busD_18_port, b_busD(17) => b_busD_17_port, 
                           b_busD(16) => b_busD_16_port, b_busD(15) => 
                           b_busD_15_port, b_busD(14) => b_busD_14_port, 
                           b_busD(13) => b_busD_13_port, b_busD(12) => 
                           b_busD_12_port, b_busD(11) => b_busD_11_port, 
                           b_busD(10) => b_busD_10_port, b_busD(9) => 
                           b_busD_9_port, b_busD(8) => b_busD_8_port, b_busD(7)
                           => b_busD_7_port, b_busD(6) => b_busD_6_port, 
                           b_busD(5) => b_busD_5_port, b_busD(4) => 
                           b_busD_4_port, b_busD(3) => b_busD_3_port, b_busD(2)
                           => b_busD_2_port, b_busD(1) => b_busD_1_port, 
                           b_busD(0) => b_busD_0_port, alu_func(3) => 
                           alu_func_3_port, alu_func(2) => alu_func_2_port, 
                           alu_func(1) => alu_func_1_port, alu_func(0) => 
                           alu_func_0_port, alu_funcD(3) => alu_funcD_3_port, 
                           alu_funcD(2) => alu_funcD_2_port, alu_funcD(1) => 
                           alu_funcD_1_port, alu_funcD(0) => alu_funcD_0_port, 
                           shift_func(1) => shift_func_1_port, shift_func(0) =>
                           shift_func_0_port, shift_funcD(1) => 
                           shift_funcD_1_port, shift_funcD(0) => 
                           shift_funcD_0_port, mult_func(3) => mult_func_3_port
                           , mult_func(2) => mult_func_2_port, mult_func(1) => 
                           mult_func_1_port, mult_func(0) => mult_func_0_port, 
                           mult_funcD(3) => mult_funcD_3_port, mult_funcD(2) =>
                           mult_funcD_2_port, mult_funcD(1) => 
                           mult_funcD_1_port, mult_funcD(0) => 
                           mult_funcD_0_port, reg_dest(31) => reg_dest_31_port,
                           reg_dest(30) => reg_dest_30_port, reg_dest(29) => 
                           reg_dest_29_port, reg_dest(28) => reg_dest_28_port, 
                           reg_dest(27) => reg_dest_27_port, reg_dest(26) => 
                           reg_dest_26_port, reg_dest(25) => reg_dest_25_port, 
                           reg_dest(24) => reg_dest_24_port, reg_dest(23) => 
                           reg_dest_23_port, reg_dest(22) => reg_dest_22_port, 
                           reg_dest(21) => reg_dest_21_port, reg_dest(20) => 
                           reg_dest_20_port, reg_dest(19) => reg_dest_19_port, 
                           reg_dest(18) => reg_dest_18_port, reg_dest(17) => 
                           reg_dest_17_port, reg_dest(16) => reg_dest_16_port, 
                           reg_dest(15) => reg_dest_15_port, reg_dest(14) => 
                           reg_dest_14_port, reg_dest(13) => reg_dest_13_port, 
                           reg_dest(12) => reg_dest_12_port, reg_dest(11) => 
                           reg_dest_11_port, reg_dest(10) => reg_dest_10_port, 
                           reg_dest(9) => reg_dest_9_port, reg_dest(8) => 
                           reg_dest_8_port, reg_dest(7) => reg_dest_7_port, 
                           reg_dest(6) => reg_dest_6_port, reg_dest(5) => 
                           reg_dest_5_port, reg_dest(4) => reg_dest_4_port, 
                           reg_dest(3) => reg_dest_3_port, reg_dest(2) => 
                           reg_dest_2_port, reg_dest(1) => reg_dest_1_port, 
                           reg_dest(0) => reg_dest_0_port, reg_destD(31) => 
                           reg_destD_31_port, reg_destD(30) => 
                           reg_destD_30_port, reg_destD(29) => 
                           reg_destD_29_port, reg_destD(28) => 
                           reg_destD_28_port, reg_destD(27) => 
                           reg_destD_27_port, reg_destD(26) => 
                           reg_destD_26_port, reg_destD(25) => 
                           reg_destD_25_port, reg_destD(24) => 
                           reg_destD_24_port, reg_destD(23) => 
                           reg_destD_23_port, reg_destD(22) => 
                           reg_destD_22_port, reg_destD(21) => 
                           reg_destD_21_port, reg_destD(20) => 
                           reg_destD_20_port, reg_destD(19) => 
                           reg_destD_19_port, reg_destD(18) => 
                           reg_destD_18_port, reg_destD(17) => 
                           reg_destD_17_port, reg_destD(16) => 
                           reg_destD_16_port, reg_destD(15) => 
                           reg_destD_15_port, reg_destD(14) => 
                           reg_destD_14_port, reg_destD(13) => 
                           reg_destD_13_port, reg_destD(12) => 
                           reg_destD_12_port, reg_destD(11) => 
                           reg_destD_11_port, reg_destD(10) => 
                           reg_destD_10_port, reg_destD(9) => reg_destD_9_port,
                           reg_destD(8) => reg_destD_8_port, reg_destD(7) => 
                           reg_destD_7_port, reg_destD(6) => reg_destD_6_port, 
                           reg_destD(5) => reg_destD_5_port, reg_destD(4) => 
                           reg_destD_4_port, reg_destD(3) => reg_destD_3_port, 
                           reg_destD(2) => reg_destD_2_port, reg_destD(1) => 
                           reg_destD_1_port, reg_destD(0) => reg_destD_0_port, 
                           rd_index(5) => rd_index_5_port, rd_index(4) => 
                           rd_index_4_port, rd_index(3) => rd_index_3_port, 
                           rd_index(2) => rd_index_2_port, rd_index(1) => 
                           rd_index_1_port, rd_index(0) => rd_index_0_port, 
                           rd_indexD(5) => rd_indexD_5_port, rd_indexD(4) => 
                           rd_indexD_4_port, rd_indexD(3) => rd_indexD_3_port, 
                           rd_indexD(2) => rd_indexD_2_port, rd_indexD(1) => 
                           rd_indexD_1_port, rd_indexD(0) => rd_indexD_0_port, 
                           rs_index(5) => rs_index_5_port, rs_index(4) => 
                           rs_index_4_port, rs_index(3) => rs_index_3_port, 
                           rs_index(2) => rs_index_2_port, rs_index(1) => 
                           rs_index_1_port, rs_index(0) => rs_index_0_port, 
                           rt_index(5) => rt_index_5_port, rt_index(4) => 
                           rt_index_4_port, rt_index(3) => rt_index_3_port, 
                           rt_index(2) => rt_index_2_port, rt_index(1) => 
                           rt_index_1_port, rt_index(0) => rt_index_0_port, 
                           pc_source(1) => pc_source_1_port, pc_source(0) => 
                           pc_source_0_port, mem_source(3) => mem_source_3_port
                           , mem_source(2) => mem_source_2_port, mem_source(1) 
                           => mem_source_1_port, mem_source(0) => 
                           mem_source_0_port, a_source(1) => a_source_1_port, 
                           a_source(0) => a_source_0_port, b_source(1) => 
                           b_source_1_port, b_source(0) => b_source_0_port, 
                           c_source(2) => c_source_2_port, c_source(1) => 
                           c_source_1_port, c_source(0) => c_source_0_port, 
                           c_bus(31) => c_bus_31_port, c_bus(30) => 
                           c_bus_30_port, c_bus(29) => c_bus_29_port, c_bus(28)
                           => c_bus_28_port, c_bus(27) => c_bus_27_port, 
                           c_bus(26) => c_bus_26_port, c_bus(25) => 
                           c_bus_25_port, c_bus(24) => c_bus_24_port, c_bus(23)
                           => c_bus_23_port, c_bus(22) => c_bus_22_port, 
                           c_bus(21) => c_bus_21_port, c_bus(20) => 
                           c_bus_20_port, c_bus(19) => c_bus_19_port, c_bus(18)
                           => c_bus_18_port, c_bus(17) => c_bus_17_port, 
                           c_bus(16) => c_bus_16_port, c_bus(15) => 
                           c_bus_15_port, c_bus(14) => c_bus_14_port, c_bus(13)
                           => c_bus_13_port, c_bus(12) => c_bus_12_port, 
                           c_bus(11) => c_bus_11_port, c_bus(10) => 
                           c_bus_10_port, c_bus(9) => c_bus_9_port, c_bus(8) =>
                           c_bus_8_port, c_bus(7) => c_bus_7_port, c_bus(6) => 
                           c_bus_6_port, c_bus(5) => c_bus_5_port, c_bus(4) => 
                           c_bus_4_port, c_bus(3) => c_bus_3_port, c_bus(2) => 
                           c_bus_2_port, c_bus(1) => c_bus_1_port, c_bus(0) => 
                           c_bus_0_port, pause_any => pause_any, pause_pipeline
                           => pause_pipeline);
   reset_reg_regx3x : DFFRHQXL port map( D => n110, CK => clk, RN => n14, Q => 
                           reset_reg_3_port);
   reset_reg_regx2x : DFFRHQXL port map( D => n120, CK => clk, RN => n14, Q => 
                           reset_reg_2_port);
   reset_reg_regx1x : DFFRHQXL port map( D => n13, CK => clk, RN => n14, Q => 
                           reset_reg_1_port);
   reset_reg_regx0x : DFFRHQXL port map( D => n15, CK => clk, RN => n14, Q => 
                           reset_reg_0_port);
   intr_signal_reg : DFFRHQXL port map( D => n10, CK => clk, RN => n14, Q => 
                           intr_signal);
   U55 : NOR2XL port map( A => mem_pause, B => pause_mult, Y => n2);
   U56 : OAI31XL port map( A0 => n4, A1 => take_branch, A2 => n5, B0 => n6, Y 
                           => nullify_op);
   U57 : NOR2XL port map( A => intr_signal, B => exception_sig, Y => n6);
   U58 : INVXL port map( A => pc_source_1_port, Y => n4);
   U59 : INVXL port map( A => pc_source_0_port, Y => n5);
   U60 : OR2XL port map( A => n1, B => reset_in, Y => reset);
   U61 : AOI21XL port map( A0 => n2, A1 => n3, B0 => pause_pipeline, Y => 
                           pause_bank);
   U62 : INVXL port map( A => pause_ctrl, Y => n3);
   U63 : NAND2BXL port map( AN => pause_pipeline, B => n2, Y => pause_non_ctrl)
                           ;
   U64 : NOR2XL port map( A => pause_non_ctrl, B => pause_ctrl, Y => n9);
   U65 : INVXL port map( A => n9, Y => pause_any);
   U66 : NAND4XL port map( A => reset_reg_3_port, B => reset_reg_2_port, C => 
                           reset_reg_1_port, D => reset_reg_0_port, Y => n1);
   U67 : NAND4XL port map( A => intr_in, B => intr_enable, C => n8, D => n9, Y 
                           => n7);
   U68 : NOR2XL port map( A => pc_source_1_port, B => pc_source_0_port, Y => n8
                           );
   U69 : NAND2XL port map( A => reset_reg_0_port, B => n1, Y => n15);
   U70 : NAND2BXL port map( AN => N11, B => n1, Y => n13);
   U71 : NAND2BXL port map( AN => N12, B => n1, Y => n120);
   U72 : INVXL port map( A => reset_in, Y => n14);
   U73 : NAND2XL port map( A => n16, B => n1, Y => n110);
   U74 : XNOR2XL port map( A => add_165xcarryx3x, B => reset_reg_3_port, Y => 
                           n16);
   U75 : OR3XL port map( A => c_shift_0_port, B => c_mult_0_port, C => 
                           c_alu_0_port, Y => c_bus_0_port);
   U76 : OR3XL port map( A => c_shift_31_port, B => c_mult_31_port, C => 
                           c_alu_31_port, Y => c_bus_31_port);
   U77 : OR3XL port map( A => c_shift_29_port, B => c_mult_29_port, C => 
                           c_alu_29_port, Y => c_bus_29_port);
   U78 : OR3XL port map( A => c_shift_28_port, B => c_mult_28_port, C => 
                           c_alu_28_port, Y => c_bus_28_port);
   U79 : OR3XL port map( A => c_shift_30_port, B => c_mult_30_port, C => 
                           c_alu_30_port, Y => c_bus_30_port);
   U80 : OR3XL port map( A => c_shift_25_port, B => c_mult_25_port, C => 
                           c_alu_25_port, Y => c_bus_25_port);
   U81 : OR3XL port map( A => c_shift_26_port, B => c_mult_26_port, C => 
                           c_alu_26_port, Y => c_bus_26_port);
   U82 : OR3XL port map( A => c_shift_27_port, B => c_mult_27_port, C => 
                           c_alu_27_port, Y => c_bus_27_port);
   U83 : OR3XL port map( A => c_shift_23_port, B => c_mult_23_port, C => 
                           c_alu_23_port, Y => c_bus_23_port);
   U84 : OR3XL port map( A => c_shift_24_port, B => c_mult_24_port, C => 
                           c_alu_24_port, Y => c_bus_24_port);
   U85 : OR3XL port map( A => c_shift_19_port, B => c_mult_19_port, C => 
                           c_alu_19_port, Y => c_bus_19_port);
   U86 : OR3XL port map( A => c_shift_20_port, B => c_mult_20_port, C => 
                           c_alu_20_port, Y => c_bus_20_port);
   U87 : OR3XL port map( A => c_shift_21_port, B => c_mult_21_port, C => 
                           c_alu_21_port, Y => c_bus_21_port);
   U88 : OR3XL port map( A => c_shift_22_port, B => c_mult_22_port, C => 
                           c_alu_22_port, Y => c_bus_22_port);
   U89 : OR3XL port map( A => c_shift_17_port, B => c_mult_17_port, C => 
                           c_alu_17_port, Y => c_bus_17_port);
   U90 : OR3XL port map( A => c_shift_18_port, B => c_mult_18_port, C => 
                           c_alu_18_port, Y => c_bus_18_port);
   U91 : OR3XL port map( A => c_shift_14_port, B => c_mult_14_port, C => 
                           c_alu_14_port, Y => c_bus_14_port);
   U92 : OR3XL port map( A => c_shift_15_port, B => c_mult_15_port, C => 
                           c_alu_15_port, Y => c_bus_15_port);
   U93 : OR3XL port map( A => c_shift_16_port, B => c_mult_16_port, C => 
                           c_alu_16_port, Y => c_bus_16_port);
   U94 : OR3XL port map( A => c_shift_11_port, B => c_mult_11_port, C => 
                           c_alu_11_port, Y => c_bus_11_port);
   U95 : OR3XL port map( A => c_shift_12_port, B => c_mult_12_port, C => 
                           c_alu_12_port, Y => c_bus_12_port);
   U96 : OR3XL port map( A => c_shift_13_port, B => c_mult_13_port, C => 
                           c_alu_13_port, Y => c_bus_13_port);
   U97 : OR3XL port map( A => c_shift_10_port, B => c_mult_10_port, C => 
                           c_alu_10_port, Y => c_bus_10_port);
   U98 : OR3XL port map( A => c_shift_9_port, B => c_mult_9_port, C => 
                           c_alu_9_port, Y => c_bus_9_port);
   U99 : OR3XL port map( A => c_shift_1_port, B => c_mult_1_port, C => 
                           c_alu_1_port, Y => c_bus_1_port);
   U100 : OR3XL port map( A => c_shift_8_port, B => c_mult_8_port, C => 
                           c_alu_8_port, Y => c_bus_8_port);
   U101 : OR3XL port map( A => c_shift_6_port, B => c_mult_6_port, C => 
                           c_alu_6_port, Y => c_bus_6_port);
   U102 : OR3XL port map( A => c_shift_7_port, B => c_mult_7_port, C => 
                           c_alu_7_port, Y => c_bus_7_port);
   U103 : OR3XL port map( A => c_shift_3_port, B => c_mult_3_port, C => 
                           c_alu_3_port, Y => c_bus_3_port);
   U104 : OR3XL port map( A => c_shift_5_port, B => c_mult_5_port, C => 
                           c_alu_5_port, Y => c_bus_5_port);
   U105 : OR3XL port map( A => c_shift_4_port, B => c_mult_4_port, C => 
                           c_alu_4_port, Y => c_bus_4_port);
   U106 : OAI2BB1X1 port map( A0N => pause_any, A1N => intr_signal, B0 => n7, Y
                           => n10);
   U107 : OR3XL port map( A => c_shift_2_port, B => c_mult_2_port, C => 
                           c_alu_2_port, Y => c_bus_2_port);
   U108 : ADDHXL port map( A => reset_reg_1_port, B => reset_reg_0_port, CO => 
                           add_165xcarryx2x, S => N11);
   U109 : ADDHXL port map( A => reset_reg_2_port, B => add_165xcarryx2x, CO => 
                           add_165xcarryx3x, S => N12);
   rt_index_5_port <= '0';

end SYN_logic;
